<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
require dirname(__FILE__) . '/../../../config/config.inc.php';
require dirname(__FILE__) . '/../../../init.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCustomerImport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCartRuleImport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsTextureImport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCustomer.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsProduct.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsExport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsAdvImporterManager.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

// Considering the indexing task can be really long, we ask the PHP process to not stop before 2 hours.
ini_set('max_execution_time', 7200);

if (Tools::getIsset('simulate')) {
    $sim = Tools::getValue('simulate');
    $simulate = ((string) $sim == '1');
} else {
    $simulate = true;
}
if (Tools::getIsset('limit')) {
    $limit = Tools::getValue('limit');
} else {
    $limit = false;
}
if (Tools::getIsset('id')) {
    $id = Tools::getValue('id');
} else {
    $id = false;
}
if (Tools::getIsset('url')) {
    $url = Tools::getValue('url');
    $logfile = $url . '.log';
} else {
    $url = false;
    $logfile = null;
}
if (Tools::getIsset('tot_records')) {
    $tot_records = Tools::getValue('tot_records');
} else {
    $tot_records = 0;
}
$debug = Tools::getIsset('debug');
// MxbToolsDebug::writeLog('mxb_tools AJAX', $url, 'SIMULATE=' . (int) $simulate . ' limit=' . $limit . ' id=' . $id);

// create order PDF
if ($url == 'generateOrderCsv') {
    $id_order = Tools::getIsset('id_order') ? (int) Tools::getValue('id_order') : 0;
    if ($debug && !$id_order) {
        $id_order = Tools::getValue('id_order');
    }
    $order = new Order($id_order);
    if (!Validate::isLoadedObject($order)) {
        exit('The order cannot be found within your database.');
    }
    $filename = Tools::getIsset('filename') ? (int) Tools::getValue('filename') : 0;

    if ($debug && !$filename) {
        $filename = Tools::getValue('filename');
    }
    if (!$filename) {
        $filename = 'order-' . $order->reference . '.csv';
    }

    header('Content-type: text/csv');
    // header('Content-Disposition: attachment; filename=savethis.txt');
    $id_lang = (int) Context::getContext()->language->id;
    // see AdminMxbToolsToolsController::exportOrderedProductsCSV()
    $data = [];
    $data['size_tag'] = (Configuration::getGlobalValue('MXB_TOOLS_SIZE_PRODUCT_TAG', $id_lang) !== false
        ? Configuration::getGlobalValue('MXB_TOOLS_SIZE_PRODUCT_TAG', $id_lang)
        : '');
    $data['color_tag'] = (Configuration::getGlobalValue('MXB_TOOLS_COLOR_PRODUCT_TAG', $id_lang) !== false
        ? Configuration::getGlobalValue('MXB_TOOLS_COLOR_PRODUCT_TAG', $id_lang)
        : '');
    // $data['date_from'] = '01-01-1900';
    // $data['date_to'] = '01-01-2100';
    $data['date_from'] = MxbToolsUtils::getDateString('2020-01-01');
    $data['date_to'] = MxbToolsUtils::getDateString('2030-01-01');

    $data['id_from'] = (int) $id_order;
    $data['id_to'] = (int) $id_order;
    $data['output_format'] = 'csv';  // 'csv', 'xml'
    $data['output_type'] = 'ordered_products';    // 'orders', 'ordered_products'
    $data['id_order_state'] = 0;
    $data['id_order_state_next'] = 0;
    $data['id_order_state_list'] = '';
    $data['id_order_state_list_not'] = '';
    $data['order_prefix_from'] = '';
    $data['order_prefix_to'] = '';
    $data['force_order_numeric'] = false;
    $data['export_invoice'] = false;
    $data['export_delivery_slip'] = false;
    $data['numbering'] = (Configuration::getGlobalValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_NUMBERING') !== false
        ? (string) Configuration::getGlobalValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_NUMBERING')
        : '');
    $data['filename'] = $filename;

    $ret = MxbToolsExport::exportOrders($data);
    if (isset($ret['errors'])) {
        exit(implode(', ', $ret['errors']));
    }
    exit($ret['content']);
}

/*
 * FROM HERE: RETURNS JSON CONTENT
 */
header('Content-type: application/json');

if ($url == 'addCategoryProductsToCategory.php') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    if (Tools::getIsset('val1')) {
        $sourceCategory = Tools::getValue('val1');
    } else {
        exit(json_encode(['error' => 'Source id_category must be > 0']));
    }

    if (Tools::getIsset('val2')) {
        $targetCategory = Tools::getValue('val2');
    } else {
        exit(json_encode(['error' => 'Target id_category must be > 0']));
    }
    // MxbToolsDebug::writeLog('mxb_tools AJAX', $url, 'calling addCategoryProductsToCategory() ...');

    if (!$simulate) {
        $result = MxbToolsProduct::addCategoryProductsToCategory(
            (int) $sourceCategory,
            (int) $targetCategory,
            (int) $limit,
            (int) $id
        );
        $result['success'] = 'Products elaborated: ' . ((int) $result['records'] < 0 ? '0' : $result['records']);
    } else {
        $result = ['error' => 'SIMULATION SUCCESS!'];
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result=' . print_r($result, true));
    exit(json_encode($result));
}

if ($url == 'changeDefaultCategoryProduct.php') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    $saveFlag = Tools::getIsset('val1') ? (int) Tools::getValue('val1') : 0;
    if (Tools::getIsset('val2')) {
        $sourceCategory = (int) Tools::getValue('val2');
    } else {
        exit(json_encode(['error' => 'Source id_category must be > 0']));
    }
    if (Tools::getIsset('val3')) {
        $targetCategory = (int) Tools::getValue('val3');
    } else {
        exit(json_encode(['error' => 'Target id_category must be > 0']));
    }
    if ($sourceCategory === $targetCategory) {
        exit(json_encode(['error' => 'Source and target category must be different !']));
    }

    $productReference = Tools::getIsset('val4') ? Tools::getValue('val4') : '';
    // MxbToolsDebug::writeLog('mxb_tools AJAX', $url, 'calling changeDefaultCategoryProduct() ...');

    if (!$simulate) {
        $result = MxbToolsProduct::changeDefaultCategoryProduct(
            (int) $sourceCategory,
            (int) $targetCategory,
            $productReference,
            (bool) $saveFlag,
            (int) $limit,
            (int) $id
        );
        if (empty($result['success'])) {
            $result['success'] = 'Products elaborated: ' . ((int) $result['records'] < 0 ? '0' : $result['records']);
        }
    } else {
        $result = ['error' => 'SIMULATION SUCCESS!'];
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result=' . print_r($result, true));
    exit(json_encode($result));
}

if ($url == 'removeProductsFromCategory.php') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    if (Tools::getIsset('val1')) {
        $sourceCategory = Tools::getValue('val1');
    } else {
        exit(json_encode(['error' => 'Source id_category must be > 0']));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'calling removeProductsFromCategory() ...');

    if (!$simulate) {
        $result = MxbToolsProduct::removeProductsFromCategory(
            (int) $sourceCategory,
            (int) $limit,
            (int) $id
        );
        $result['success'] = 'Products elaborated: ' . ((int) $result['records'] < 0 ? '0' : $result['records']);
    } else {
        $result = ['error' => 'SIMULATION SUCCESS!'];
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result=' . print_r($result, true));
    exit(json_encode($result));
}

if ($url == 'importCustomerCsv') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START');
    try {
        $result = MxbToolsCustomerImport::importCustomerFromCsv((int) $limit, (int) $id, (int) $tot_records);
        // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true));
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    exit(json_encode($result));
}

if ($url == 'importCartRuleCsv') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START');

    try {
        $result = MxbToolsCartRuleImport::importCartRuleFromCsv((int) $limit, (int) $id, (int) $tot_records);
        // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true));
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    exit(json_encode($result));
}

if ($url == 'importTextureCsv') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    /*
    if (Tools::getIsset('val1')) {
        $id_attribute_group = Tools::getValue('val1');
    } else {
        exit(json_encode(['error' => 'id_attribute_group must be > 0']));
    }
    */
    $id_attribute_group = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_ID_ATTRIBUTE_GROUP');

    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START');
    try {
        $result = MxbToolsTextureImport::importFromCsv(
            (int) $limit,
            (int) $id,
            (int) $tot_records,
            (int) $id_attribute_group
        );
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true));
    exit(json_encode($result));
}

if ($url == 'changeCustomerGroup.php') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    // parameter: source-group
    if (Tools::getIsset('val1')) {
        $sourceGroup = Tools::getValue('val1');
    } else {
        exit(json_encode(['error' => 'Source id_group must be > 0']));
    }
    // parameter: target-group
    if (Tools::getIsset('val2')) {
        $targetGroup = Tools::getValue('val2');
    } else {
        exit(json_encode(['error' => 'Target id_group must be > 0']));
    }
    // MxbToolsDebug::writeLog('mxb_tools AJAX', $url, 'calling changeCustomerGroup() ...');

    if (!$simulate) {
        $result = MxbToolsCustomer::changeCustomerGroup(
            (int) $sourceGroup,
            (int) $targetGroup,
            (int) $limit,
            (int) $id,
            (int) $tot_records
        );
    } else {
        $result = ['error' => 'SIMULATION SUCCESS!'];
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result=' . print_r($result, true));
    exit(json_encode($result));
}

if ($url == 'deleteOrphanImages') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    if (Tools::getIsset('val1')) {
        $curpath = Tools::getValue('val1');
    } else {
        $curpath = '1';
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START - curpath=' . $curpath, false, $logfile);
    try {
        $result = deleteOrphanImages((int) $limit, (int) $id, (int) $tot_records, $simulate, $curpath);
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true), false, $logfile);
    exit(json_encode($result));
}

if ($url == 'deleteTempImages') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }

    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START - id=' . $id, false, $logfile);
    try {
        $result = deleteTempImages((int) $limit, (int) $id, (int) $tot_records, $simulate);
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true), false, $logfile);
    exit(json_encode($result));
}

if ($url == 'deleteAllProducts') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }

    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START - id=' . $id, false, $logfile);
    try {
        $result = deleteAllProducts((int) $limit, (int) $id, (int) $tot_records, $simulate);
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true), false, $logfile);
    exit(json_encode($result));
}

if ($url == 'deleteAllAbandonedCarts') {
    if ($limit === false || $id === false || $url === false) {
        exit(json_encode(['error' => 'check params: limit, id and url']));
    }
    // parameter: days
    if (Tools::getIsset('val1')) {
        $days = Tools::getValue('val1');
    } else {
        exit(json_encode(['error' => 'Days must be > 0']));
    }
    // parameter: known users
    if (Tools::getIsset('val2')) {
        $known_users = Tools::getValue('val2');
    } else {
        $known_users = false;
    }
    // parameter: cart total
    if (Tools::getIsset('val3')) {
        $cart_total_max = (float) Tools::getValue('val3');
    } else {
        exit(json_encode(['error' => 'Cart total must be >= 0']));
    }

    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'START - id=' . $id, false, $logfile);
    try {
        $result = deleteAllAbandonedCarts(
            (int) $days,
            (bool) $known_users,
            (float) $cart_total_max,
            (int) $limit,
            (int) $id,
            (int) $tot_records,
            $simulate
        );
    } catch (Exception $e) {
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            $url,
            'error calling function: ' . $e->getMessage(),
            false,
            $logfile
        );
        exit(json_encode(['error' => 'error: ' . $e->getMessage()]));
    }
    // MxbToolsDebug::writeLog('Mxb_Tools AJAX', $url, 'result?=' . print_r($result, true), false, $logfile);
    exit(json_encode($result));
}

if ($url == 'rndkey') {
    // header('Content-type: text/plain');
    exit(json_encode(['result' => MxbToolsUtils::generateRandomString()]));
}

if ($url == 'myip') {
    // header('Content-type: text/plain');
    exit(json_encode(['result' => MxbToolsUtils::getClientIp()]));
}

if ($url == 'analyzeDisabledProducts') {
    $saveFlag = Tools::getIsset('val1') ? (int) Tools::getValue('val1') : 0;
    $fileIdx = Tools::getIsset('val2') ? (int) Tools::getValue('val2') : 0;
    $deletedRecords = Tools::getIsset('val3') ? (int) Tools::getValue('val3') : 0;
    $disabledOnlyFlag = Tools::getIsset('val4') ? Tools::getValue('val4') : '';
    $analysisResult = Tools::getIsset('val5') ? Tools::getValue('val5') : '';

    $result = MxbToolsProduct::analyzeDisabledProducts((int) $limit, (int) $id, $saveFlag, $fileIdx, $deletedRecords, $disabledOnlyFlag, $analysisResult);
    MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result=' . print_r($result, true), false, $logfile);

    exit(json_encode($result));
}

if ($url == 'analyzeMissingLinkRewrite') {
    $saveFlag = Tools::getIsset('val1') ? (int) Tools::getValue('val1') : 0;
    $fileIdx = Tools::getIsset('val2') ? (int) Tools::getValue('val2') : 0;
    $patchedRecords = Tools::getIsset('val3') ? (int) Tools::getValue('val3') : 0;
    $emptyOnlyFlag = Tools::getIsset('val4') ? Tools::getValue('val4') : '';
    $analysisResult = Tools::getIsset('val5') ? Tools::getValue('val5') : '';

    $result = MxbToolsProduct::analyzeMissingLinkRewrite((int) $limit, (int) $id, $saveFlag, $fileIdx, $patchedRecords, $emptyOnlyFlag, $analysisResult);
    MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result=' . print_r($result, true), false, $logfile);

    exit(json_encode($result));
}

if ($url == 'analyzeProductAttributeShopDb') {
    $saveFlag = Tools::getIsset('val1') ? (int) Tools::getValue('val1') : 0;
    $fileIdx = Tools::getIsset('val2') ? (int) Tools::getValue('val2') : 0;
    $deletedRecords = Tools::getIsset('val3') ? (int) Tools::getValue('val3') : 0;
    $analysisResult = Tools::getIsset('val4') ? Tools::getValue('val4') : '';
    // MxbToolsDebug::writeLog('MxbTools AJAX', $url, '_POST=' . print_r($_POST, true));

    $result = MxbToolsAdvImporterManager::analyzeProductAttributeShopDb((int) $limit, (int) $id, $saveFlag, $fileIdx, $deletedRecords, $analysisResult);
    MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result=' . print_r($result, true));
    // MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result JSON=' . print_r(json_encode($result), true));

    exit(json_encode($result));
}

if ($url == 'analyzeAdvimporterSupplierreferenceDb') {
    $saveFlag = Tools::getIsset('val1') ? (int) Tools::getValue('val1') : 0;
    $deleteCacheFile = Tools::getIsset('val2') ? (int) Tools::getValue('val2') : 0;
    $fileIdx = Tools::getIsset('val3') ? (int) Tools::getValue('val3') : 0;
    $deletedRecords = Tools::getIsset('val4') ? (int) Tools::getValue('val4') : 0;
    $analysisResult = Tools::getIsset('val5') ? Tools::getValue('val5') : '';

    $result = MxbToolsAdvImporterManager::analyzeSupplierReferenceDb((int) $limit, (int) $id, $saveFlag, $deleteCacheFile, $fileIdx, $deletedRecords, $analysisResult);
    MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result=' . print_r($result, true));

    // MxbToolsDebug::writeLog('MxbTools AJAX', $url, 'result JSON=' . print_r(json_encode($result), true));
    exit(json_encode($result));
}

// MxbToolsDebug::writeLog('mxb_tools AJAX', $url, 'nothing to do', false);
exit(json_encode(['error' => 'ajax: nothing to do for url ' . $url]));

function deleteOrphanImages($limit = 0, $id = 0, $tot_records = 0, $simulate = true, $curpath = '1')
{
    // NO DEBUG: set $simulate to false or remove !
    // $simulate = false;
    $shop_root = dirname(__FILE__) . '/../../../';
    $logfile = 'deleteOrphanImages.log';
    $errors = [];
    $success = [];
    // $id used as elaborated records count
    $cnt_records = (int) $id;
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteOrphanImages(sim=' . (int) $simulate.')',
        'id=' . $id . ', limit=' . $limit . ', curpath=' . $curpath,
        false,
        $logfile
    );
    */

    /* START TO OPERATE */
    if ($tot_records === 0) {
        $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('SELECT COUNT(1) FROM `' . _DB_PREFIX_ . 'image`');
        /*
        $res = Db::getInstance()->executeS('SELECT count(1) cnt FROM `' . _DB_PREFIX_ . 'image`');
        if (is_array($res) && isset($res[0])) {
            $tot_records = $res[0]['cnt'];
        }
        */
    }
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteOrphanImages(sim=' . (int) $simulate . ')',
        'tot_records=' . $tot_records,
        false,
        $logfile
    );
    */
    $paths = explode('/', $curpath);
    for ($i = 0; $i < 6; ++$i) {
        if (!isset($paths[$i])) {
            $paths[$i] = 0;
        } else {
            $paths[$i] = (int) $paths[$i]; // intval($paths[$i]);
        }
    }

    // reset some counters
    $counters = [
        'cnt_test' => 0,
        'cnt_files' => 0,
        'cnt_checked' => 0,
        'cnt_not_found' => 0,
        'cnt_found' => 0,
    ];

    for ($ii = $paths[0]; ($ii <= 9) && ($counters['cnt_test'] < $limit); ++$ii) {
        $path = $shop_root . 'img/p/' . $ii;
        delImage($path, $limit, $simulate, $counters, $logfile);
        for ($jj = $paths[1]; ($jj <= 9) && ($counters['cnt_test'] < $limit); ++$jj) {
            $path = $shop_root . 'img/p/' . $ii . '/' . $jj;
            delImage($path, $limit, $simulate, $counters, $logfile);
            for ($kk = $paths[2]; ($kk <= 9) && ($counters['cnt_test'] < $limit); ++$kk) {
                $path = $shop_root . 'img/p/' . $ii . '/' . $jj . '/' . $kk;
                delImage($path, $limit, $simulate, $counters, $logfile);
                for ($ll = $paths[3]; ($ll <= 9) && ($counters['cnt_test'] < $limit); ++$ll) {
                    $path = $shop_root . 'img/p/' . $ii . '/' . $jj . '/' . $kk . '/' . $ll;
                    delImage($path, $limit, $simulate, $counters, $logfile);
                    for ($aa = $paths[4]; ($aa <= 9) && ($counters['cnt_test'] < $limit); ++$aa) {
                        $path = $shop_root . 'img/p/' . $ii . '/' . $jj . '/' . $kk . '/' . $ll . '/' . $aa;
                        delImage($path, $limit, $simulate, $counters, $logfile);
                        for ($bb = $paths[5]; ($bb <= 9) && ($counters['cnt_test'] < $limit); ++$bb) {
                            $path = $shop_root . 'img/p/' . $ii . '/' . $jj . '/' . $kk . '/' . $ll . '/' . $aa . '/' . $bb;
                            delImage($path, $limit, $simulate, $counters, $logfile);
                        }
                        $paths[5] = 0;
                    }
                    $paths[4] = 0;
                }
                $paths[3] = 0;
            }
            $paths[2] = 0;
        }
        $paths[1] = 0;
    }
    $curpath = str_replace($shop_root . 'img/p/', '', $path);

    $cnt_records += $counters['cnt_not_found'];
    $ret = [
        'records' => $counters['cnt_files'],
        'tot_records' => $tot_records,
        'id' => $cnt_records,
        'val1' => $curpath,
    ];

    $success[] = 'Orphan images deleted: ' . $cnt_records . ' - current folder: ' . $curpath;
    if ($counters['cnt_files'] == 0 || strpos($curpath, '9/9/9/9/9') !== false) {
        // END OF AJAX LOOP
        $ret['records'] = 0;
        $success[] = 'Loop end';
    }
    if (count($errors)) {
        $ret['errors'] = implode('|', $errors);
    }
    if (count($success)) {
        $ret['success'] = implode('|', $success);
    }
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteOrphanImages(sim=' . (int) $simulate.')',
        'File summary - counters=' . print_r($counters, true) . ', ret=' . print_r($ret, true),
        false,
        $logfile
    );
    */

    return $ret;
}

function deleteTempImages($limit = 0, $id = 0, $tot_records = 0, $simulate = true)
{
    // NO DEBUG: set $simulate to false or remove !
    // $simulate = false;
    $shop_root = dirname(__FILE__) . '/../../../';
    // $logfile = 'deleteTempImages.log';
    $errors = [];
    $success = [];

    // $id used as elaborated records count
    $cnt_records = (int) $id;

    /* START TO OPERATE */
    $cnt_files = 0;
    $target_folder_name = $shop_root . 'img/tmp/';
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteTempImages(sim=' . (int) $simulate.')',
        'id=' . $id . ', limit=' . $limit . ', path=' . $target_folder_name,
        false,
        $logfile
    );
    */
    $dir = new DirectoryIterator($target_folder_name);

    if ($tot_records === 0) {
        $tot_records = iterator_count($dir);
    }

    foreach ($dir as $fileinfo) {
        if (!$fileinfo->isDir()) {
            $name = $fileinfo->getFilename();
            // $outStr = $target_folder_name.$name . ' - ' . $fileinfo->getSize() . ' bytes : ';

            if (strpos($name, 'fileType') !== false || strpos($name, 'index.php') !== false || $name == 'cms') {
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools AJAX',
                    'deleteTempImages(sim=' . (int) $simulate.')',
                    'file ' . $outStr . ' is protected',
                    false,
                    $logfile
                );
                */
                continue;
            }
            if (!$simulate) {
                unlink($target_folder_name . $name);
                /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools AJAX',
                        'deleteTempImages(sim=' . (int) $simulate.')',
                        'file ' . $outStr . ' DELETED',
                        false,
                        $logfile
                    );
                */
            }
            /*
            else {
                MxbToolsDebug::writeLog(
                    'Mxb_Tools AJAX',
                    'deleteTempImages(sim=' . (int) $simulate . ')',
                    'file ' . $outStr . ' simulate delete',
                    false,
                    $logfile
                );
            }
            */
            // var_dump($fileinfo->getFilename());
            if ($cnt_files >= $limit) {
                break;
            }
            ++$cnt_files;
        }
    }

    $cnt_records += $cnt_files;
    $ret = [
        'records' => $cnt_files,
        'tot_records' => $tot_records,
        'id' => $cnt_records,
    ];

    $success[] = 'Temporary images deleted: ' . $cnt_files . ' in folder: ' . $target_folder_name;
    if ($cnt_files == 0 || $cnt_files < $limit) {
        // END OF AJAX LOOP
        $ret['records'] = 0;
        $success[] = 'Loop end';
    }
    if (count($errors)) {
        $ret['errors'] = implode('|', $errors);
    }
    if (count($success)) {
        $ret['success'] = implode('|', $success);
    }
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteTempImages(sim=' . (int) $simulate.')',
        'File summary - ret=' . print_r($ret, true),
        false,
        $logfile
    );
    */
    return $ret;
}

function deleteAllProducts($limit = 0, $id = 0, $tot_records = 0, $simulate = true)
{
    // NO DEBUG: set $simulate to false or remove !
    // $simulate = true;
    // $logfile = 'deleteAllProducts.log';
    $errors = [];
    $success = [];

    // $id used as elaborated records count
    $cnt_records = (int) $id;
    // $tot_records = $limit;

    /* START TO OPERATE */

    // reset some counters
    $counters = [
        'cnt_elaborated' => 0,
        'cnt_deleted' => 0,
        'cnt_error' => 0,
    ];
    if ($tot_records === 0) {
        $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('SELECT COUNT(1) FROM `' . _DB_PREFIX_ . 'product`');
    }

    $res = Db::getInstance()->executeS('
        SELECT `id_product`
        FROM `' . _DB_PREFIX_ . 'product`
        ORDER BY `id_product` DESC
        LIMIT ' . $limit);
    // logText('Start loop', 'Starting to delete products...');
    if ($res) {
        foreach ($res as $row) {
            // logText('Deleting product ID', $row['id_product']);
            $p = new Product($row['id_product']);
            if (!$simulate) {
                if (!$p->delete()) {
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools AJAX',
                        'deleteAllProducts(sim=' . (int) $simulate.')',
                        'ERROR deleting this product ! ID=' . $row['id_product'],
                        false,
                        $logfile
                    );
                    */
                    ++$counters['cnt_error'];
                } else {
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools AJAX',
                        'deleteAllProducts(sim=' . (int) $simulate.')',
                        'Product DELETED - ID=' . $row['id_product'],
                        false,
                        $logfile
                    );
                    */
                    ++$counters['cnt_deleted'];
                }
            } else {
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools AJAX',
                    'deleteAllProducts(sim=' . (int) $simulate.')',
                    'Product SIMULATE DELETED - ID=' . $row['id_product'],
                    false,
                    $logfile
                );
                */
                ++$counters['cnt_deleted'];
            }
            ++$counters['cnt_elaborated'];
            flush();
        }
    }
    $cnt_records += $counters['cnt_elaborated'];
    $ret = [
        'records' => $counters['cnt_elaborated'],
        'tot_records' => $tot_records,
        'id' => $cnt_records,
    ];

    if ($counters['cnt_elaborated'] == 0 || $counters['cnt_elaborated'] < $limit) {
        // END OF AJAX LOOP
        $ret['records'] = 0;
        $success[] = 'Loop end';
    } else {
        $success[] = ($simulate ? 'Simulation - ' : '')
            . 'Total products elaborated: ' . $cnt_records
            . ', deleted in this session: ' . $counters['cnt_deleted'];
    }
    if (count($errors)) {
        $ret['errors'] = implode('|', $errors);
    }
    if (count($success)) {
        $ret['success'] = implode('|', $success);
    }
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteAllProducts(sim=' . (int) $simulate.')',
        'File summary - ret=' . print_r($ret, true),
        false,
        $logfile
    );
    */
    return $ret;
}

function deleteAllAbandonedCarts(
    $days = 365,
    $known_users = false,
    $cart_total_max = 0,
    $limit = 0,
    $id = 0,
    $tot_records = 0,
    $simulate = true
) {
    // NO DEBUG: set $simulate to false or remove !
    // $simulate = true;
    // $logfile = 'deleteAllAbandonedCarts.log';
    $errors = [];
    $success = [];

    // $id used as elaborated records count
    $cnt_records = (int) $id;
    // $tot_records = $limit;

    /* START TO OPERATE */
    // reset some counters
    $counters = [
        'cnt_elaborated' => 0,
        'cnt_deleted' => 0,
        'cnt_error' => 0,
    ];
    $sql_where = 'WHERE DATE(a.`date_add`) < DATE_SUB(NOW(), INTERVAL ' . (int) $days . ' DAY)'
        . (!(bool) $known_users ? ' AND a.id_customer=0' : '')
        . ' AND o.id_cart IS NULL';

    if ($tot_records === 0) {
        $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
            SELECT COUNT(1)
            FROM `' . _DB_PREFIX_ . 'cart` a
            LEFT JOIN ' . _DB_PREFIX_ . 'orders o ON (o.id_cart = a.id_cart)
            ' . $sql_where);
    }

    $res = Db::getInstance()->executeS('
        SELECT a.`id_cart`, a.`date_add`
        FROM `' . _DB_PREFIX_ . 'cart` a
        LEFT JOIN ' . _DB_PREFIX_ . 'orders o ON (o.id_cart = a.id_cart)
        ' . $sql_where . '
        ORDER BY a.`id_cart` DESC');
    // LIMIT ' . $limit);
    // logText('Start loop', 'Starting to delete products...');
    if ($res) {
        foreach ($res as $row) {
            // logText('Deleting product ID', $row['id_product']);
            $p = new Cart($row['id_cart']);
            $total = $p->getOrderTotal();
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools AJAX',
                'deleteAllAbandonedCarts(sim=' . (int) $simulate.')',
                'total=' . $total
                . ', cart_total_max=' . $cart_total_max,
                false,
                $logfile
            );
            */
            if ((float) $total <= (float) $cart_total_max) {
                if (!$simulate) {
                    if (!$p->delete()) {
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools AJAX',
                            'deleteAllProducts(sim=' . (int) $simulate.')',
                            'ERROR deleting this product ! ID=' . $row['id_product'],
                            false,
                            $logfile
                        );
                        */
                        ++$counters['cnt_error'];
                    } else {
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools AJAX',
                            'deleteAllProducts(sim=' . (int) $simulate.')',
                            'Product DELETED - ID=' . $row['id_product'],
                            false,
                            $logfile
                        );
                        */
                        ++$counters['cnt_deleted'];
                    }
                } else {
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools AJAX',
                        'deleteAllProducts(sim=' . (int) $simulate.')',
                        'Product SIMULATE DELETED - ID=' . $row['id_product'],
                        false,
                        $logfile
                    );
                    */
                    ++$counters['cnt_deleted'];
                }
            } else {
                ++$counters['cnt_error'];
            }

            /*
            if ($counters['cnt_elaborated'] + $cnt_records >= $tot_records) {
                break;
            }
            */
            ++$counters['cnt_elaborated'];
            if ($counters['cnt_deleted'] >= $limit) {
                break;
            }
            flush();
        }
    }
    $cnt_records += $counters['cnt_elaborated'];
    $ret = [
        'records' => $counters['cnt_elaborated'],
        'tot_records' => $tot_records,
        'id' => $cnt_records,
    ];
    if ($counters['cnt_elaborated'] > 0) {
        $success[] = ($simulate ? 'Simulation - ' : '')
            . 'Total carts elaborated: ' . $cnt_records
            . ', deleted in this session: ' . $counters['cnt_deleted']
            . ', not deleted: ' . $counters['cnt_error'];
    }
    if ($counters['cnt_elaborated'] == 0 || $counters['cnt_elaborated'] < $limit || $cnt_records >= $tot_records) {
        // END OF AJAX LOOP
        $ret['records'] = 0;
        $success[] = ($simulate ? 'Simulation loop end' : 'Loop end');
    }

    // if simulate: end at the first call
    if ($simulate) {
        $ret['records'] = 0;
    }

    if (count($errors)) {
        $ret['errors'] = implode('|', $errors);
    }
    if (count($success)) {
        $ret['success'] = implode('|', $success);
    }
    /*
    MxbToolsDebug::writeLog(
        'Mxb_Tools AJAX',
        'deleteAllProducts(sim=' . (int) $simulate.')',
        'File summary - ret=' . print_r($ret, true),
        false,
        $logfile
    );
    */

    return $ret;
}

function delImage($imageDir, $limit, $simulate, &$counters, $logfile = null)
{
    if ($handle = @opendir($imageDir)) { // @ is written to avoid warning message and is handled in else condition
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools AJAX',
            'delImage(sim=' . (int) $simulate.')',
            'Img folder=' . $imageDir,
            false,
            $logfile
        );
        */
        while ($counters['cnt_test'] < $limit && false !== ($entry = readdir($handle))) {
            if ($entry != '.' && $entry != '..' && !is_dir($imageDir . '/' . $entry)) {
                ++$counters['cnt_files'];
                $pi = explode('-', $entry);
                if ($pi[0] > 0 && !empty($pi[1])) {
                    ++$counters['cnt_checked'];
                    if (!checkExistsDb($pi[0], $simulate, $logfile)) {
                        ++$counters['cnt_not_found'];
                        ++$counters['cnt_test'];
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools AJAX',
                            'delImage(sim=' . (int) $simulate . ')',
                            'REMOVE ' . $imageDir . '/' . $entry,
                            false,
                            $logfile
                        );
                        */
                        if (!$simulate) {
                            unlink($imageDir . '/' . $entry);
                        }
                    } else {
                        ++$counters['cnt_found'];
                        ++$counters['cnt_test'];
                    }
                }
            }
            // $cnt_test = $cnt_not_found;
        }
        closedir($handle);
        ++$counters['cnt_test']; // count also the folder check
    }
}

function checkExistsDb($id_image, $simulate, $logfile = null)
{
    // check in image table
    $res = Db::getInstance()->executeS('
        SELECT id_image, id_product
        FROM `' . _DB_PREFIX_ . 'image`
        WHERE id_image = ' . (int) $id_image);
    $image_exists = (is_array($res) && isset($res[0]));
    // check in product table
    if ($image_exists) {
        $id_product = $res[0]['id_product'];
        $res = Db::getInstance()->executeS('
            SELECT id_product
            FROM `' . _DB_PREFIX_ . 'product`
            WHERE id_product = ' . (int) $id_product);
        $product_exists = (is_array($res) && isset($res[0]));

        // delete from image database orphan product
        if (!$product_exists) {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools AJAX',
                'checkExistsDb(sim=' . (int) $simulate . ')',
                'orphan product ' . $id_product . ': delete img ' . $id_image
                    . ' from image/image_lang/image_shop tables ... ',
                false,
                $logfile
            );
            */
            if (!$simulate) {
                Db::getInstance()->execute('DELETE FROM `' . _DB_PREFIX_ . 'image` where id_image = ' . (int) $id_image);
                Db::getInstance()->execute('DELETE FROM `' . _DB_PREFIX_ . 'image_lang` where id_image = ' . (int) $id_image);
                Db::getInstance()->execute('DELETE FROM `' . _DB_PREFIX_ . 'image_shop` where id_image = ' . (int) $id_image);
            } else {
                MxbToolsDebug::writeLog(
                    'Mxb_Tools AJAX',
                    'checkExistsDb(sim=' . (int) $simulate . ')',
                    'orphan product ' . $id_product . ': delete img ' . $id_image
                        . ' from image/image_lang/image_shop tables ... ',
                    false,
                    $logfile
                );
            }
            return false;
        }
        return true;
    }
    return false;
}
