<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
// use PrestaShop\PrestaShop\Core\Cart\AmountImmutable;

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsAdvImporterManager
{
    const AI_CACHE_FILE = _PS_MODULE_DIR_ . 'advancedimporter/classes/tmp/suppierreference.php';

    /**
     * mxb_tools tools : analyze advanced importer supplier reference database and remove records that ave no more corrispondence in prestashop entities
     *  i.e. a deleted product (and related combinations, specificPrices, tag...) must be removed from adv impoeter sr database
     * called via ajax.php
     */
    public static function analyzeSupplierReferenceDb(
        $limit = 0,
        $current_record = 0,
        $save_flag = 0,
        $delete_cache_flag = 0,
        $file_idx = 0,
        $deleted_records = 0,
        $analysis_result_string = ''
    ) {
        // $elaborated_records = 0;
        $tot_records = 0;
        $max_file_size = 16; // MB
        $target_folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        $target_report_file = $target_folder . '/' . MxbToolsUtils::normalizeFilename(str_replace('www.', '', strtolower($_SERVER['SERVER_NAME']))) . '-AI_compare_srdb_with_psdb_';
        $current_record = (int) $current_record;
        $content = '';
        $csv_header = 'object_type;supplier_reference;db_id;db_id_object;reference deleted from aidb;comment;' . PHP_EOL;

        $analysis_result = (!empty($analysis_result_string) ? json_decode($analysis_result_string, true) : []);
        // MxbToolsDebug::writeLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result_string=' . $analysis_result_string);
        // MxbToolsDebug::writeLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result=' . print_r($analysis_result, true));

        if (!$analysis_result) {
            $analysis_result = [];
        }

        if ($current_record === 0) {
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'advancedimporter_supplierreference
                WHERE 1';
            $tot_records = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            if ((int) $tot_records === 0) {
                return [
                    'error' => 'advancedimporter_supplierreference is empty',
                    'records' => 0,
                    'tot_records' => 0,
                    'id' => 0,
                ];
            }

            $content = $csv_header;
            if (file_exists($target_report_file . $file_idx . '.csv')) {
                unlink($target_report_file . $file_idx . '.csv');
            } else {
                // check folder
                if (!file_exists($target_folder)) {
                    mkdir($target_folder, 0777, true);
                }
            }
        }

        $sql_limit = ((int) $limit > 0 ? 'LIMIT ' . $limit . ' OFFSET ' . ($current_record - $deleted_records) : '');
        $sql = '
            SELECT id_advancedimporter_supplierreference as id, id_object, object_type, supplier_reference
            FROM ' . _DB_PREFIX_ . 'advancedimporter_supplierreference
            ORDER BY object_type, supplier_reference, id
            ' . $sql_limit;

        $rows = Db::getInstance()->executeS($sql, true, false);
        if (!count($rows)) {
            if ($current_record > 0) {
                // END OF ANALYSIS: SEND RESULTS
                // create links to csv report(s)
                $reportLinks = 'Click to download reports:'
                    . '|Summary: <a href="' . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '" target="_blank">'
                    . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '</a>';
                for ($i = 1; $i <= $file_idx; ++$i) {
                    $reportLinks .= '|' . $i . ') <a href="' . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '" target="_blank">'
                        . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '</a>';
                }

                // create table of totals
                $header = '<table style="width:100%;"><tr>';
                $cols = '';
                $cnt = 0;
                $tot = [];
                $summary_csv = '';
                $header_csv = '';
                foreach ($analysis_result as $object_type => $data) {
                    if ($cnt === 0) {
                        $header .= '<th>Object</th>';
                        $header_csv .= 'Entity;';
                    }
                    $cols .= '<tr><td>' . $object_type . '</td>';
                    $summary_csv .= $object_type . ';';
                    foreach ($data as $what => $val) {
                        if ($cnt === 0) {
                            $header .= '<th>' . $what . '</th>';
                            $header_csv .= $what . ';';
                        }
                        $cols .= '<td>' . $val . '</td>';
                        $summary_csv .= $val . ';';
                        if (!isset($tot[$what])) {
                            $tot[$what] = 0;
                        }
                        $tot[$what] += (int) $val;
                    }
                    $cols .= '</tr>';
                    $summary_csv .= PHP_EOL;
                    // $success .= '|';
                    ++$cnt;
                }
                $header .= '</tr>';
                $header_csv .= PHP_EOL;

                // totals row
                $cols .= '<tr><th>Totals</th>';
                $summary_csv .= 'Totals;';
                foreach ($tot as $what => $val) {
                    $cols .= '<th>' . $val . '</th>';
                    $summary_csv .= $val . ';';
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $summary_csv = $header_csv . $summary_csv;
                $success = '<b>FINAL RESULTS:</b>|'
                    . 'Total records deleted: ' . $deleted_records . '|'
                    . $header . $cols . '</table>';

                // save final report summary
                if (file_exists($target_report_file . '-summary.csv')) {
                    unlink($target_report_file . '-summary.csv');
                }

                // write to file
                $myfile = fopen($target_report_file . '-summary.csv', 'a');
                // fwrite($myfile, print_r($products, true));
                fwrite($myfile, $summary_csv);
                fclose($myfile);

                return [
                    'clear_log' => 1,
                    'success' => $success . '|' . $reportLinks,
                    'records' => 0,
                    'tot_records' => $tot_records,
                    'id' => $current_record,
                ];
            }

            return [
                'error' => 'advancedimporter_supplierreference is empty',
                'records' => 0,
                'tot_records' => $tot_records,
                'id' => $current_record,
            ];
        }
        $records = count($rows);
        $current_record += $records;

        // convert rows in convenient format for analysis
        foreach ($rows as $row) {
            if (!isset($sr[$row['object_type']])) {
                $sr[$row['object_type']] = [];
            }
            if (!isset($sr[$row['object_type']][$row['supplier_reference']])) {
                $sr[$row['object_type']][$row['supplier_reference']] = [
                    'id' => $row['id'],
                    'id_object' => $row['id_object'],
                    'object_type' => $row['object_type'],
                    'supplier_reference' => $row['supplier_reference'],
                ];
            }
        }
        $result = MxbToolsAdvImporterManager::compareSrDatabases($sr, $save_flag);
        $content .= $result['content'];
        $deleted_records += $result['deleted'];

        if ((bool) $delete_cache_flag && $deleted_records > 0) {
            MxbToolsAdvImporterManager::deleteCacheFile(true, 'old');
        }

        // check products file size
        if (file_exists($target_report_file . $file_idx . '.csv')) {
            $fileSize = filesize($target_report_file . $file_idx . '.csv') / (1024 * 1024);
            $sizeExceed = $fileSize > (float) $max_file_size;
            if ($sizeExceed) {
                // increment file index
                $file_idx = (int) $file_idx + 1;

                // add headers to csv content
                $content = $csv_header . $content;

                // delete old report
                if (file_exists($target_report_file . $file_idx . '.csv')) {
                    unlink($target_report_file . $file_idx . '.csv');
                }
            }
        }

        // write to file
        $myfile = fopen($target_report_file . $file_idx . '.csv', 'a');
        // fwrite($myfile, print_r($products, true));
        fwrite($myfile, $content);
        fclose($myfile);

        // $success = 'records elaborated: ' . $current_record . '|';
        $header = 'records elaborated: ' . $current_record . '|'
            . '<table style="width:100%;"><tr>';
        $cols = '';
        $cnt = 0;
        // update analysis result totals
        foreach ($result['analyzed'] as $object_type => $data) {
            // if ($cnt === 0) {
            //     $header .= '<th>Object</th>';
            // }
            // $cols .= '<tr><td>' . $object_type . '</td>';
            // $success .= '- ' . $object_type . ':';
            foreach ($data as $what => $val) {
                // if ($cnt === 0) {
                //     $header .= '<th>' . $what . '</th>';
                // }
                // $cols .= '<td>' . $val . '</td>';
                // $success .= ' ' . $what . ': ' . $val;

                // update analysis totals
                if (!isset($analysis_result[$object_type])) {
                    $analysis_result[$object_type] = [];
                }
                if (!isset($analysis_result[$object_type][$what])) {
                    $analysis_result[$object_type][$what] = 0;
                }
                $analysis_result[$object_type][$what] += (int) $val;
            }
            // $cols .= '</tr>';
            // $success .= '|';
            // $cnt++;
        }

        // write result table with updated sum of values of every previous loop (totals)
        foreach ($analysis_result as $object_type => $data) {
            if ($cnt === 0) {
                $header .= '<th>Object</th>';
            }
            $cols .= '<tr><td>' . $object_type . '</td>';
            // $success .= '- ' . $object_type . ':';
            foreach ($data as $what => $val) {
                if ($cnt === 0) {
                    $header .= '<th>' . $what . '</th>';
                }
                $cols .= '<td>' . $val . '</td>';
            }
            $cols .= '</tr>';
            // $success .= '|';
            ++$cnt;
        }

        $header .= '</tr>';
        $success = $header . $cols . '</table>';

        // remember: to stop ajax transition we must return 'records' => 0
        return [
            'clear_log' => 1,
            'success' => $success,
            'records' => $records,
            'tot_records' => $tot_records,
            'id' => $current_record,
            'val1' => $save_flag,
            'val2' => $delete_cache_flag,
            'val3' => $file_idx,
            'val4' => $deleted_records,
            'val5' => json_encode($analysis_result),
        ];
    }

    /**
     * mxb_tools tools : analyze product_attribute_shop and set default_on to NULL if save_flag is true
     * called via ajax.php
     */
    public static function analyzeProductAttributeShopDb(
        $limit = 0,
        $current_record = 0,
        $save_flag = 0,
        $file_idx = 0,
        $updated_records = 0,
        $analysis_result_string = ''
    ) {
        // $elaborated_records = 0;
        $tot_records = 0;
        $max_file_size = 16; // MB
        $target_folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        $target_report_file = $target_folder . '/' . MxbToolsUtils::normalizeFilename(str_replace('www.', '', strtolower($_SERVER['SERVER_NAME'])))
            . '-product_attribute_shop_analysis_';
        $current_record = (int) $current_record;
        $content = '';
        $csv_header = 'id_product;id_product_attribute;id_shop;default_on;product_attribute;product_attribute_image;shop default_on set to NULL' . PHP_EOL;

        $analysis_result = (!empty($analysis_result_string) ? json_decode($analysis_result_string, true) : []);
        // MxbToolsDebug::writeLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result_string=' . $analysis_result_string);
        // MxbToolsDebug::writeLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result=' . print_r($analysis_result, true));

        if (!$analysis_result) {
            $analysis_result = [];
        }

        if ($current_record === 0) {
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'product_attribute_shop
                WHERE 1';
            $tot_records = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            if ((int) $tot_records === 0) {
                return [
                    'error' => 'product_attribute_shop is empty',
                    'records' => 0,
                    'tot_records' => 0,
                    'id' => 0,
                ];
            }

            $content = $csv_header;
            if (file_exists($target_report_file . $file_idx . '.csv')) {
                unlink($target_report_file . $file_idx . '.csv');
            } else {
                // check folder
                if (!file_exists($target_folder)) {
                    mkdir($target_folder, 0777, true);
                }
            }
        }

        $sql_limit = ((int) $limit > 0 ? 'LIMIT ' . $limit . ' OFFSET ' . $current_record : '');
        $sql = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'product_attribute_shop
            ORDER BY id_product, id_product_attribute, id_shop
            ' . $sql_limit;

        $rows = Db::getInstance()->executeS($sql, true, false);
        if (!count($rows)) {
            if ($current_record > 0) {
                // END OF ANALYSIS: SEND RESULTS
                // create links to csv report(s)
                $reportLinks = 'Click to download reports:'
                    . '|Summary: <a href="' . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '" target="_blank">'
                    . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '</a>';

                for ($i = 1; $i <= $file_idx; ++$i) {
                    $reportLinks .= '|' . $i . ') <a href="' . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '" target="_blank">'
                        . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '</a>';
                }

                // create table of totals
                $header = '<table style="width:100%;"><tr>';
                $cols = '';
                $cnt = 0;
                $tot = [];
                $summary_csv = '';
                $header_csv = '';
                $object_type = 'product_attribute_shop';
                // foreach ($analysis_result as $object_type => $data) {
                // if ($cnt === 0) {
                $header .= '<th>Object</th>';
                $header_csv .= 'Entity;';
                // }
                $cols .= '<tr><td>' . $object_type . '</td>';
                $summary_csv .= $object_type . ';';
                foreach ($analysis_result as $what => $val) {
                    if ($cnt === 0) {
                        $header .= '<th>' . $what . '</th>';
                        $header_csv .= $what . ';';
                    }
                    $cols .= '<td>' . $val . '</td>';
                    $summary_csv .= $val . ';';
                    if (!isset($tot[$what])) {
                        $tot[$what] = 0;
                    }
                    $tot[$what] += (int) $val;
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                // $success .= '|';
                // $cnt ++;
                // }
                $header .= '</tr>';
                $header_csv .= PHP_EOL;

                // totals row
                $cols .= '<tr><th>Totals</th>';
                $summary_csv .= 'Totals;';
                foreach ($tot as $what => $val) {
                    $cols .= '<th>' . $val . '</th>';
                    $summary_csv .= $val . ';';
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $summary_csv = $header_csv . $summary_csv;
                $success = '<b>FINAL RESULTS:</b>|'
                    . 'Total records deleted: ' . $updated_records . '|'
                    . $header . $cols . '</table>';

                // save final report summary
                if (file_exists($target_report_file . '-summary.csv')) {
                    unlink($target_report_file . '-summary.csv');
                }

                // write to file
                $myfile = fopen($target_report_file . '-summary.csv', 'a');
                // fwrite($myfile, print_r($products, true));
                fwrite($myfile, $summary_csv);
                fclose($myfile);

                return [
                    'clear_log' => 1,
                    'success' => $success . '|' . $reportLinks,
                    'records' => 0,
                    'tot_records' => $tot_records,
                    'id' => $current_record,
                ];
            }

            return [
                'error' => 'product_attribute_shop is empty',
                'records' => 0,
                'tot_records' => $tot_records,
                'id' => $current_record,
            ];
        }
        $records = count($rows);
        $current_record += $records;

        $result = MxbToolsAdvImporterManager::comparePaDatabases($rows, $save_flag);
        $content .= $result['content'];
        $updated_records += $result['updated'];

        // check csv file size
        if (file_exists($target_report_file . $file_idx . '.csv')) {
            $fileSize = filesize($target_report_file . $file_idx . '.csv') / (1024 * 1024);
            $sizeExceed = $fileSize > (float) $max_file_size;
            if ($sizeExceed) {
                // increment file index
                $file_idx = (int) $file_idx + 1;

                // add headers to csv content
                $content = $csv_header . $content;

                // delete old report
                if (file_exists($target_report_file . $file_idx . '.csv')) {
                    unlink($target_report_file . $file_idx . '.csv');
                }
            }
        }

        // write to file
        $myfile = fopen($target_report_file . $file_idx . '.csv', 'a');
        // fwrite($myfile, print_r($products, true));
        fwrite($myfile, $content);
        fclose($myfile);

        // $success = 'records elaborated: ' . $current_record . '|';
        $header = 'records elaborated: ' . $current_record . '|'
            . '<table style="width:100%;"><tr>';
        $cols = '';
        $object_type = 'product_attribute_shop';
        $header .= '<th>Object</th>';
        $cols .= '<tr><td>' . $object_type . '</td>';
        // update analysis result totals
        foreach ($result['analyzed'] as $what => $val) {
            // $header .= '<th>' . $what . '</th>';
            // write values of this loop
            // $cols .= '<td>' . $val . '</td>';

            // update analysis totals
            if (!isset($analysis_result[$what])) {
                $analysis_result[$what] = 0;
            }
            $analysis_result[$what] += (int) $val;
        }

        // write result table with updated sum of values of every previous loop (totals)
        foreach ($analysis_result as $what => $val) {
            $header .= '<th>' . $what . '</th>';
            $cols .= '<td>' . $analysis_result[$what] . '</td>';
        }
        $header .= '</tr>';
        $cols .= '</tr>';
        $success = $header . $cols . '</table>';

        // remember: to stop ajax transition we must return 'records' => 0
        return [
            'clear_log' => 1,
            'success' => $success,
            'records' => $records,
            'tot_records' => $tot_records,
            'id' => $current_record,
            'val1' => $save_flag,
            'val2' => $file_idx,
            'val3' => $updated_records,
            'val4' => json_encode($analysis_result),
        ];
    }

    public static function compareSrDatabases($sr, $save_flag = false)
    {
        $content = '';
        $fun = 'compareSrDatabases';
        $analyzed = [];

        // debug / test
        // $save_flag = false;
        $deleted_records = 0;

        // compare sr with real database
        foreach ($sr as $object_type => $object_supplier_references) {
            $entity_error = false;
            if ($object_type == 'Specificprice') {
                $object_type = 'SpecificPrice';
            }

            if (empty($analyzed[$object_type])) {
                $analyzed[$object_type] = [
                    'error' => 0,
                    'total' => 0,
                    'deleted' => 0,
                    'missing' => 0,
                    'correct' => 0,
                ];
            }

            foreach ($object_supplier_references as $supplier_reference => $sr_data) {
                ++$analyzed[$object_type]['total'];
                try {
                    $entity = new $object_type($sr_data['id_object']);
                } catch (Exception $e) {
                    /*
                    logString('Entity ' . $object_type . ' (sr=' . $supplier_reference . ', id=' . $sr_data['id_object'] . ') creation error - Caught exception: '.  $e->getMessage(), $fun, $DBG['deb_color_error'], $DBG['deb_text_size'], 0);
                    logString('DB: object_type ' . $object_type
                        . ' > supplier_reference ' . $supplier_reference
                        . ' > id ' . $sr_data['id']
                        . ' > id_object ' . $sr_data['id_object']
                        . ': creation error - Caught exception: '.  $e->getMessage(),
                        $fun, $DBG['deb_color_error'], $DBG['deb_text_size'], 0);
                    */
                    ++$analyzed[$object_type]['error'];

                    $content .= $object_type . ';'
                        . MxbToolsUtils::parseStringForCSV($supplier_reference) . ';'
                        . $sr_data['id'] . ';'
                        . $sr_data['id_object'] . ';'
                        . 'no;'
                        . 'entity creation error' . PHP_EOL;
                    // die ('Caught exception: ' . $e->getMessage());
                    $entity_error = true;
                    break;
                }

                if (!Validate::isLoadedObject($entity)) {
                    if ((bool) $save_flag) {
                        Db::getInstance()->execute(
                            'DELETE FROM `' . _DB_PREFIX_ . 'advancedimporter_supplierreference`
                            WHERE `id_advancedimporter_supplierreference` = ' . (int) $sr_data['id']
                        );
                        ++$analyzed[$object_type]['deleted'];
                        $deleted = 'Yes';
                        ++$deleted_records;
                    } else {
                        $deleted = 'yes (simulated)';
                    }
                    ++$analyzed[$object_type]['missing'];
                    /*
                    logString('DB: object_type ' . $object_type
                        . ' > supplier_reference ' . $supplier_reference
                        . ' > id ' . $sr_data['id']
                        . ' > id_object ' . $sr_data['id_object']
                        . ': entity validation error > ai_db entity deleted: ' . $deleted,
                        $fun, $DBG['deb_color_alert'], $DBG['deb_text_size'], 0);
                    */
                    $content .= $object_type . ';'
                        . MxbToolsUtils::parseStringForCSV($supplier_reference) . ';'
                        . $sr_data['id'] . ';'
                        . $sr_data['id_object'] . ';'
                        . $deleted . ';'
                        . 'validation error' . PHP_EOL;
                    continue;
                }

                if ($entity->id > 0) {
                    /*
                    logString('DB: object_type ' . $object_type
                        . ' > supplier_reference ' . $supplier_reference
                        . ' > id ' . $sr_data['id']
                        . ' > id_object ' . $sr_data['id_object']
                        . ' : entity exists in ps_db',
                        $fun, $DBG['deb_color_loop']);
                    */
                    ++$analyzed[$object_type]['correct'];
                    $content .= $object_type . ';'
                        . MxbToolsUtils::parseStringForCSV($supplier_reference) . ';'
                        . $sr_data['id'] . ';'
                        . $sr_data['id_object'] . ';'
                        . 'no;'
                        . 'OK' . PHP_EOL;
                } else {
                    // this case does not exists because if entity is missing, validation aboge returns error.
                    // so this "else" should be deleted ...
                    /*
                    logString('DB: object_type ' . $object_type
                        . ' > supplier_reference ' . $supplier_reference
                        . ' > id ' . $sr_data['id']
                        . ' > id_object ' . $sr_data['id_object']
                        . ' : entity exists in ps_db',
                        $fun, $DBG['deb_color_alert']);
                    */
                    ++$analyzed[$object_type]['missing'];
                    $content .= $object_type . ';'
                        . MxbToolsUtils::parseStringForCSV($supplier_reference) . ';'
                        . $sr_data['id'] . ';'
                        . $sr_data['id_object'] . ';'
                        . 'no;'
                        . 'NOT FOUND' . PHP_EOL;
                }
                // unset($entity);
            } // foreach ($object_supplier_references as $supplier_reference => $sr_data) {
        } // foreach ($sr as $object_type => $object_supplier_references)

        return [
            'content' => $content,
            'analyzed' => $analyzed,
            'deleted' => $deleted_records,
        ];
    }

    public static function comparePaDatabases($sr, $save_flag = false)
    {
        $content = '';
        $fun = 'comparePaDatabases';
        $analyzed = [
            'error' => 0,
            'total' => 0,
            'updated' => 0,
            'missing' => 0,
            'images' => 0,
            'correct' => 0,
        ];
        // debug / test
        // $save_flag = false;

        // compare product_attribute_shop with product_attribute
        foreach ($sr as $row) {
            ++$analyzed['total'];
            $sql = '
                SELECT *
                FROM ' . _DB_PREFIX_ . 'product_attribute
                WHERE id_product=' . (int) $row['id_product'] . '
                AND id_product_attribute=' . (int) $row['id_product_attribute'];

            $rows = Db::getInstance()->executeS($sql, true, false);
            if (count($rows)) {
                // product attribute found: record is OK !
                ++$analyzed['correct'];
                /*
                logString('SHOP DB: product id=' . $row['id_product'] . ', product_attribute id=' . $row['id_product_attribute']
                    . ' > OK!',
                    $fun, $DBG['deb_color_array'], $DBG['deb_text_size'], 0);
                */
                continue;
            }
            ++$analyzed['missing'];
            $noteImg = '';
            // check also image db$sql = '
            $sql = '
                SELECT *
                FROM ' . _DB_PREFIX_ . 'product_attribute_image
                WHERE id_product_attribute=' . (int) $row['id_product_attribute'];
            $rowsImg = Db::getInstance()->executeS($sql, true, false);
            $imgIsPresent = (bool) count($rowsImg);
            if ($imgIsPresent) {
                ++$analyzed['images'];
            }

            // set "default_on" to null for orphan product_attribute
            $edited = false;
            if ($row['default_on'] == 1 && $save_flag) {
                $sql = '
                    UPDATE ' . _DB_PREFIX_ . 'product_attribute_shop
                    SET default_on = NULL
                    WHERE id_product=' . (int) $row['id_product'] . '
                    AND id_product_attribute=' . (int) $row['id_product_attribute'];
                Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                $edited = true;
                ++$analyzed['updated'];
            }

            /*
            logString('SHOP DB: product id=' . $row['id_product'] . ', product_attribute id=' . $row['id_product_attribute']
                . ' > NOT FOUND in product_attribute db'
                . ($imgIsPresent ? '> orphan product_attribute_image should be deleted too' : '')
                . ($edited ? ' > shop default_on WAS SET to NULL' : ($row['default_on'] == 1 ? ' > shop default_on should set to null' : '')),
                $fun, $DBG['deb_color_error'], $DBG['deb_text_size'], 0);
            */

            // CSV header: 'id_product;id_product_attribute;id_shop;default_on;product_attribute;product_attribute_image;shop default_on set to NULL' . PHP_EOL;
            $content .= $row['id_product'] . ';'
                . $row['id_product_attribute'] . ';'
                . $row['id_shop'] . ';'
                . ($row['default_on'] == 1 ? $row['default_on'] : 'NULL') . ';'
                . 'NOT FOUND;'
                . ($imgIsPresent ? 'FOUND' : 'Not found') . ';'
                . ($edited ? 'Yes' : ($row['default_on'] == 1 ? 'yes (simulate)' : 'No')) . PHP_EOL;
        } // foreach ($sr as $object_type => $object_supplier_references)

        return [
            'content' => $content,
            'analyzed' => $analyzed,
            'updated' => $analyzed['updated'],
        ];
    }

    public static function deleteCacheFile($backup = true, $backup_ext = 'old')
    {
        $cacheFile = self::AI_CACHE_FILE;
        if (file_exists($cacheFile)) {
            if (file_exists($cacheFile . '.old')) {
                unlink($cacheFile . '.old');
            }
            // date('Y-m-d-His')
            copy($cacheFile, $cacheFile . '.' . $backup_ext);
            unlink($cacheFile);

            return true;
        }

        return false;
    }
}
