<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

// use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsCartRule extends ObjectModel
{
    /**
     * move customer of a group to a new group
     *
     *  $check_existing:
     *    check for other customer voucher
     *    $existing_rules = CartRule::getCustomerCartRules(
     *      $id_lang,
     *      $id_customer,
     *      true,  // $active = false,
     *      false, // $includeGeneric = true,
     *      true,  // $inStock = false,
     *      null,  // Cart $cart = null,
     *      false, // $free_shipping_only = false,
     *      false  // $highlight_only = false
     *  );
     *
     *    0: don't check
     *    1+: check for other voucher that must be:
     *      1: active, inStock, includeGeneric
     *      2: inStock (can be non active)
     *      3: active (can be not in stock)
     *
     * @params:
     * cart_rule_name: name of cart rule
     *     String: 1 language (an array will be created)
     *     Array for any language
     *     if empty: name will be cart_rule_code
     * check_includeGeneric: bool
     *     if true, voucher will be created only if the customer
     *     has no other "generic" vouchers (automatic, without
     *     a target user and with or without a code to insert)
     */
    public function createCartRule(
        $cart_rule_code = null, // used as prefix if random_code is true
        $random_code = false,   // bool: true=generate a random code with "cart_rule_code" as prefix
        $code_len = 12,         // length of code (max 254 chars)
        $quantity_to_create = 1,// quantity of rules to create
        $quantity = 1,          // quantity of single cart rule
        $quantity_per_user = 1, // quantity of single cart rule per user
        $priority = 1,          // priority of the coupon (1 is better)
        $cart_rule_name = null, // string/array
        $cart_rule_description = '', // string
        $id_customer = 0,       // restrict to one customer
        $group_ids = null,      // array of groups for group restrinction / string with ID separated by comma
        $category_ids = null,   // array of categories for category restrinction / string with ID separated by comma
        $partial_use = 0,       // can cart rule be used partially ?
        $highlight = 0,         // can cart rule be listed in user cart ?
        $free_shipping = 0,     // 1 = free
        $reduction_percent = 0, // your percentage or 0.00;
        $reduction_amount = 0,  // your_amount or 0.00;
        $reduction_tax = 0,     // 1 = tax included;
        $date_from = null,      // start date / time = date('Y-m-d H:i:s');
        $date_to = null,        // end date / time = date('Y-m-d H:i:s');
        $overwrite = false,     // if true, existing voucher code will be overwritten
        $create_csv = 0,        // 1 = create new / 2 = append
        $check_active = false,  // if true voucher will be created only if the customer has no other "active" vouchers
        $check_inStock = false, // if true voucher will be created only if the customer has no other "inStock" vouchers
        $check_includeGeneric = false // if true voucher will be created only if cust. has no other "generic" vouchers
    ) {
        // $force_debug = false;
        $check_existing = $check_active || $check_inStock || $check_includeGeneric;
        $errors = [];
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCartRule::createCartRules()',
            'cart_rule_code='.$cart_rule_code
              .PHP_EOL.' random_code='.$random_code
              .PHP_EOL.' code_len='.$code_len
              .PHP_EOL.' quantity_to_create='.$quantity_to_create
              .PHP_EOL.' quantity='.$quantity
              .PHP_EOL.' quantity_per_user='.$quantity_per_user
              .PHP_EOL.' priority='.$priority
              .PHP_EOL.' cart_rule_name='.print_r($cart_rule_name, true)
              .PHP_EOL.' cart_rule_description='.$cart_rule_description
              .PHP_EOL.' id_customer='.$id_customer
              .PHP_EOL.' group_ids='.$group_ids
              .PHP_EOL.' category_ids='.$category_ids
              .PHP_EOL.' partial_use='.$partial_use
              .PHP_EOL.' highlight='.$highlight
              .PHP_EOL.' free_shipping='.$free_shipping
              .PHP_EOL.' reduction_percent='.$reduction_percent
              .PHP_EOL.' reduction_amount='.$reduction_amount
              .PHP_EOL.' reduction_tax='.$reduction_tax
              .PHP_EOL.' date_from='.$date_from
              .PHP_EOL.' date_to='.$date_to
              .PHP_EOL.' check_existing='.(int) $check_existing
              .PHP_EOL.' overwrite='.(int) $overwrite
              .PHP_EOL.' create_csv='.(int) $create_csv
        );
        */

        /*
        // apply restrictions: must add infos into db tables
        product_restriction = 1
            cart_rule_product_rule_group:
                id_product_rule_group:      id of this table
                id_cart_rule:               id of current cart_rule
                quantity:                   quantity of products that must be in cart to apply the rule

            cart_rule_product_rule:
                id_product_rule:            id of this table
                id_product_rule_group:      id of cart_rule_product_rule_group table, above
                type:                       string: "categories"

            cart_rule_product_rule_value:   * 1 record per each product/category ...
                id_product_rule:            id of table above
                id_item:                    id of category / product ...

            group_restriction = 1
            cart_rule_group:            * 1 record per each group ...
                id_cart_rule:           id of current cart_rule
                id_group:               id of group
        */

        // $gcodes = MxbToolsCodes::getCodes();
        /*
            $last_id_customer = $id_customer;
            $inserted_records = 0;
            $tot_records = 0;
        */

        $module = Module::getInstanceByName('mxb_tools');
        $id_lang = (int) Context::getContext()->language->id;
        $inserted_rules = [];

        if ($cart_rule_code === null) {
            $cart_rule_code = '';
        }
        if ($code_len > 50) { // max is 254
            $code_len = 50;
        }
        if ($code_len <= Tools::strlen($cart_rule_code)) {
            $code_len = Tools::strlen($cart_rule_code) + 1;
        }
        if (!$random_code && Tools::strlen($cart_rule_code) > 0) {
            // check duplicated code
            $sql = '
                SELECT code, id_cart_rule
                FROM ' . _DB_PREFIX_ . 'cart_rule '
                . ($overwrite ? 'WHERE code = "' . $cart_rule_code . '"' : 'WHERE code LIKE "' . $cart_rule_code . '%"')
                . ' ORDER BY code DESC LIMIT 1';

            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            if (count($result)) {
                if ($overwrite) {
                    // delete the voucher and substitute with the new one
                    $cart_rule = new CartRule($result['id_cart_rule']);
                    $cart_rule->delete();
                } else {
                    // generate a new code
                    $cart_rule_code = $result['code'] . '0';
                }
            }
        }

        // check and convert $group_ids array
        if (is_null($group_ids) || empty($group_ids)) {
            $group_ids = false;
        } elseif ((is_string($group_ids) && Tools::strlen($group_ids)) || (is_int($group_ids) && (int) $group_ids > 0)) {
            // convert into array
            $group_ids = MxbToolsUtils::idStringToArray((string) $group_ids);
        } elseif (is_array($group_ids) && !count($group_ids)) {
            $group_ids = false;
        }

        // check and convert $category_ids array
        if (is_null($category_ids) || empty($category_ids)) {
            $category_ids = false;
        } elseif (is_string($category_ids) && Tools::strlen($category_ids)) {
            // convert into array
            $category_ids = MxbToolsUtils::idStringToArray($category_ids);
        } elseif (is_array($category_ids) && !count($category_ids)) {
            $category_ids = false;
        }

        // check date from / to
        if (is_null($date_from) || is_numeric($date_from) || empty($date_from)) {
            $date_from = date('Y-m-d H:i:s');
        }
        if (is_null($date_to) || is_numeric($date_to) || empty($date_to)) {
            $month = (is_numeric($date_to) ? $date_to : 3);
            $date_to = MxbToolsUtils::addMonthToDate(null, $month, 'string');
            // $date_to = new DateTime('now');
            // $date_to->modify('+' . $month . ' month'); // or you can use '-90 day' for deduct
            // $date_to = $date_to->format('Y-m-d H:i:s');
        }

        if (is_null($cart_rule_description)) {
            $cart_rule_description = '';
        }

        if (!is_null($cart_rule_name) && !is_array($cart_rule_name)) {
            $cart_rule_name = MxbToolsUtils::createMultiLangField($cart_rule_name);
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCartRule::createCartRules()',
            'cart_rule_code=' . $cart_rule_code
                . ', quantity_to_create=' . $quantity_to_create
                . ', id_customer=' . $id_customer
        );
        */
        $valid = true;
        // check if id_customer contains an ID or an email address
        // and check fi customer exists
        if (!is_numeric($id_customer)) {
            if (is_string($id_customer) && Tools::strlen($id_customer) > 0) {
                $sql = '
                    SELECT id_customer
                    FROM ' . _DB_PREFIX_ . 'customer
                    WHERE LOWER(email) = "' . Tools::strtolower($id_customer) . '"';

                $id = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
                if ($id) {
                    $id_customer = $id;
                } else {
                    $valid = false;
                    $errors[] = 'customer ' . $id_customer . ' does not exists';
                }
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCartRule::createCartRules()',
                    'id_customer=' . $id_customer . ',
                    valid='.(int) $valid
                );
                */
            }
        }

        // check customer for existing vouchers
        // if (CartRule::haveCartRuleToday($id_customer)) {
        if ((bool) $check_existing && $id_customer) {
            $existing_rules = CartRule::getCustomerCartRules(
                $id_lang,
                $id_customer,
                $check_active, // $active = false,
                $check_includeGeneric, // $includeGeneric = true,
                $check_inStock, // $inStock = false,
                null, // Cart $cart = null,
                false, // $free_shipping_only = false,
                false // $highlight_only = false
            );
            if (count($existing_rules)) {
                // check id customer not 0
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCartRule::createCartRules()',
                    'id_customer=' . $id_customer . ', existing_rules=' . print_r($existing_rules, true)
                );
                */
                foreach ($existing_rules as $rl) {
                    if ((int) $rl['id_customer'] === (int) $id_customer) {
                        $valid = false;
                        $errors[] = 'cart_rule for customer ' . $id_customer . ' already exists';
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsCartRule::createCartRules()',
                            'id_customer=' . $id_customer . ' has already a voucher.'
                        );
                        */

                        break;
                    }
                }
                // $valid = false;
            }
        } // if ((bool) $check_existing && $id_customer)

        $inserted_records = 0;
        if ($valid) {
            for ($i = 0; $i < $quantity_to_create; ++$i) {
                $code = $cart_rule_code;
                if ($random_code) {
                    // use $cart_rule_code as suffix
                    $length = $code_len - Tools::strlen($cart_rule_code);

                    // generate code into a loop to ensure an unique code
                    // using $cnt as safe exit for max loop
                    $result = [0];
                    $cnt = 0;
                    while (count($result) && $cnt < 100) {
                        ++$cnt;
                        $code = $cart_rule_code . MxbToolsUtils::generateRandomString($length, true);
                        $sql = '
                            SELECT code
                            FROM ' . _DB_PREFIX_ . 'cart_rule
                            WHERE code LIKE "' . $code . '%"
                            ORDER BY code DESC
                            LIMIT 1';

                        // if empty, exit loop
                        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
                    }
                } elseif ($i > 0) {
                    $code .= (string) $i;
                }
                if (is_null($cart_rule_name)) {
                    $name = MxbToolsUtils::createMultiLangField($code);
                } else {
                    $name = is_array($cart_rule_name)
                        ? $cart_rule_name
                        : MxbToolsUtils::createMultiLangField($cart_rule_name);
                }
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCartRule::createCartRules()',
                    '#' . $i . ': cart_rule_code=' . $code . ', date_from=' . $date_from . ', date_to=' . $date_to
                );
                */
                $cart_rule = new CartRule();
                // just change the value by what you want:
                $cart_rule->name = $name; // should be an array of every language names with the id_lang as keys;
                $cart_rule->id_customer = $id_customer;
                $cart_rule->date_from = $date_from;
                $cart_rule->date_to = $date_to;
                $cart_rule->description = $cart_rule_description;
                $cart_rule->quantity = $quantity;
                $cart_rule->quantity_per_user = $quantity_per_user;
                $cart_rule->priority = $priority;
                $cart_rule->partial_use = $partial_use;
                $cart_rule->code = $code;
                $cart_rule->minimum_amount = 0;
                $cart_rule->minimum_amount_tax = 0;
                $cart_rule->minimum_amount_currency = 1;
                $cart_rule->minimum_amount_shipping = 0;
                $cart_rule->country_restriction = 0;
                $cart_rule->carrier_restriction = 0;
                $cart_rule->group_restriction = ($group_ids !== false);
                $cart_rule->cart_rule_restriction = 0;
                $cart_rule->product_restriction = ($category_ids !== false);
                $cart_rule->shop_restriction = 0;
                $cart_rule->free_shipping = $free_shipping;
                $cart_rule->reduction_percent = $reduction_percent;
                $cart_rule->reduction_amount = $reduction_amount;
                $cart_rule->reduction_tax = $reduction_tax;
                $cart_rule->reduction_currency = 1;
                $cart_rule->reduction_product = 0;
                $cart_rule->gift_product = 0;
                $cart_rule->gift_product_attribute = 0;
                $cart_rule->highlight = $highlight;
                $cart_rule->active = 1;
                $cart_rule->date_add = $date_from;
                $cart_rule->date_upd = $date_to;

                $cart_rule->add();
                $id_cart_rule = $cart_rule->id;

                // add group restrictons
                if ($group_ids) {
                    /*
                    group_restriction
                        cart_rule_group:      * 1 record per each group ...
                        id_cart_rule:      id of current cart_rule
                        id_group:        id of group
                    */

                    foreach ($group_ids as $group_id) {
                        $sql = '
                            INSERT INTO ' . _DB_PREFIX_ . 'cart_rule_group (id_cart_rule, id_group)
                            VALUES (' . (int) $id_cart_rule . ', ' . (int) $group_id . ')';
                        Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    }
                }

                // add category restrictons
                if ($category_ids) {
                    /*
                      product_restriction
                        cart_rule_product_rule_group:
                          id_product_rule_group:    id of this table
                          id_cart_rule:        id of current cart_rule
                          quantity:        quantity of products that must be in cart to apply the rule

                        cart_rule_product_rule:
                          id_product_rule:      id of this table
                          id_product_rule_group:    id of cart_rule_product_rule_group table, above
                          type:          string: "categories"

                        cart_rule_product_rule_value:  * 1 record per each product/category ...
                          id_product_rule:      id of table above
                          id_item:        id of category / product ...
                    */
                    $sql = '
                        INSERT INTO ' . _DB_PREFIX_ . 'cart_rule_product_rule_group (id_cart_rule, quantity)
                        VALUES (' . (int) $id_cart_rule . ', 1)';
                    Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    $id_product_rule_group = Db::getInstance()->Insert_ID();

                    $sql = '
                        INSERT INTO ' . _DB_PREFIX_ . 'cart_rule_product_rule (id_product_rule_group, type)
                        VALUES (' . (int) $id_product_rule_group . ', "categories")';
                    Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    $id_product_rule = Db::getInstance()->Insert_ID();

                    foreach ($category_ids as $category_id) {
                        $sql = '
                            INSERT INTO ' . _DB_PREFIX_ . 'cart_rule_product_rule_value (id_product_rule, id_item)
                            VALUES (' . (int) $id_product_rule . ', ' . (int) $category_id . ')';
                        Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    }
                }

                // add to cart
                // $this->context->cart->addCartRule($id_cart_rule);
                /*
                $inserted_rules[] = array('id_cart_rule' => $id_cart_rule,
                  'code' => $code,
                  'name' => $cart_rule->name,
                  'description' => $$cart_rule->description,
                  'id_customer' => $cart_rule->id_customer,
                  'date_from' => $cart_rule->date_from,
                  'date_to' => $cart_rule->date_to,
                  'quantity' => $cart_rule->quantity,
                  'quantity_per_user' => $cart_rule->quantity_per_user,
                  'priority' => $cart_rule->priority,
                  'partial_use' => (int) $cart_rule->partial_use,
                  'minimum_amount' => $cart_rule->minimum_amount,
                  'minimum_amount_tax' => $cart_rule->minimum_amount_tax,
                  'minimum_amount_currency' => $cart_rule->minimum_amount_currency,
                  'minimum_amount_shipping' => $minimum_amount_shipping,
                  'country_restriction' => $country_restriction,
                  'cart_rule_restriction' => $cart_rule_restriction,
                  'product_restriction' => $product_restriction,
                  'shop_restriction' => $shop_restriction,
                  'free_shipping' => $free_shipping,
                  'reduction_percent' => $reduction_percent,
                  'reduction_amount' => $reduction_amount,
                  'reduction_tax' => $reduction_tax,
                  'reduction_currency' => $reduction_currency,
                  'reduction_product' => $reduction_product,
                  'gift_product' => $gift_product,
                  'gift_product_attribute' => $gift_product_attribute,
                  'highlight' => $highlight,
                  'active' => $active,
                  'date_add' => $date_add,
                  'date_upd' => $date_upd,
                );
                */

                $inserted_rules[] = json_decode(json_encode($cart_rule), true);
                $last = count($inserted_rules) - 1;
                $inserted_rules[$last]['id_cart_rule'] = $id_cart_rule;
                // $inserted_rules[$last]['name'] = $id_cart_rule;
                $inserted_rules[$last]['group_ids'] = MxbToolsUtils::idArrayToString($group_ids);
                $inserted_rules[$last]['category_ids'] = MxbToolsUtils::idArrayToString($category_ids);
                ++$inserted_records;
            } // for
        } // valid = true

        // now write csv file
        $records = '';
        foreach ($inserted_rules as $rule) {
            $records .= $rule['id_cart_rule'] . ';'
                . $rule['code'] . ';'
                . $rule['name'][$id_lang] . ';'
                . $rule['description'] . ';'
                . $rule['reduction_percent'] . ';'
                . $rule['reduction_amount'] . ';'
                . $rule['reduction_tax'] . ';'
                . $rule['date_from'] . ';'
                . $rule['date_to'] . ';'
                . $rule['id_customer'] . ';'
                . $rule['group_ids'] . ';'
                . $rule['category_ids'] . ';'
                . $rule['quantity'] . ';'
                . $rule['quantity_per_user'] . ';'
                . $rule['priority'] . ';'
                . $rule['partial_use'] . ';'
                . $rule['highlight'] . ';'
                . $rule['free_shipping'] . ';'
                . $rule['minimum_amount'] . ';'
                . $rule['minimum_amount_tax'] . ';'
                . $rule['minimum_amount_currency'] . ';'
                . $rule['minimum_amount_shipping'] . ';'
                . $rule['country_restriction'] . ';'
                . $rule['carrier_restriction'] . ';'
                . $rule['group_restriction'] . ';'
                . $rule['cart_rule_restriction'] . ';'
                . $rule['product_restriction'] . ';'
                . $rule['shop_restriction'] . ';'
                . $rule['reduction_currency'] . ';'
                . $rule['reduction_product'] . ';'
                . $rule['reduction_exclude_special'] . ';'
                . $rule['gift_product'] . ';'
                . $rule['gift_product_attribute'] . ';'
                . $rule['highlight'] . ';'
                . $rule['active'] . ';'
                . $rule['date_add'] . ';'
                . $rule['date_upd'] . ';'
                . PHP_EOL;
        }
        if ((int) $create_csv === 2) { // append: no headers
            $csv_header = '';
        } else {
            $csv_header = $module->l('ID voucher') . ';'
                . $module->l('code') . ';'
                . $module->l('name') . ';'
                . $module->l('description') . ';'
                . $module->l('reduction_percent') . ';'
                . $module->l('reduction_amount') . ';'
                . $module->l('reduction_tax') . ';'
                . $module->l('date_from') . ';'
                . $module->l('date_to') . ';'
                . $module->l('ID customer') . ';'
                . $module->l('groups') . ';'
                . $module->l('categories') . ';'
                . $module->l('quantity') . ';'
                . $module->l('quantity_per_user') . ';'
                . $module->l('priority') . ';'
                . $module->l('partial_use') . ';'
                . $module->l('highlight') . ';'
                . $module->l('free_shipping') . ';'
                . $module->l('minimum_amount') . ';'
                . $module->l('minimum_amount_tax') . ';'
                . $module->l('minimum_amount_currency') . ';'
                . $module->l('minimum_amount_shipping') . ';'
                . $module->l('country_restriction') . ';'
                . $module->l('carrier_restriction') . ';'
                . $module->l('group_restriction') . ';'
                . $module->l('cart_rule_restriction') . ';'
                . $module->l('product_restriction') . ';'
                . $module->l('shop_restriction') . ';'
                . $module->l('reduction_currency') . ';'
                . $module->l('reduction_product') . ';'
                . $module->l('reduction_exclude_special') . ';'
                . $module->l('gift_product') . ';'
                . $module->l('gift_product_attribute') . ';'
                . $module->l('highlight') . ';'
                . $module->l('Is active') . ';'
                . $module->l('date_add') . ';'
                . $module->l('date_upd') . ';'
                . PHP_EOL;
        }
        $records = $csv_header . $records;
        $return_values = ['records' => $inserted_records, 'rules' => $inserted_rules];
        $return_values['filename'] = MxbToolsUtils::normalizeFilename(
            str_replace(
                '/',
                '-',
                Configuration::get('PS_SHOP_NAME')
                    . '-Voucher-generated-'
                    . MxbToolsUtils::getCurrentDateTime(false, '-', '', '')
            )
        ) . '.csv';

        // create file: only once I got quantity columns
        // create folder and CSV file and write part of headings
        $folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        $file_xml = $folder . '/' . $return_values['filename'];
        if ((bool) $create_csv) {
            $myfile = fopen($file_xml, (int) $create_csv === 1 ? 'wb' : 'ab') or exit('Unable to open file!');
            fwrite($myfile, ((int) $create_csv === 1 ? '\xEF\xBB\xBF' : '') . $records);
            fclose($myfile);
        }
        $return_values['download_url'] = str_replace('http:', 'https:', $baseUrl)
            . '/modules/mxb_tools/reports/'
            . $return_values['filename'];
        $return_values['download_link'] = '<a href="'
            . $return_values['download_url']
            . '" target="_new">'
            . $return_values['filename'] . '</a>';
        if (count($errors)) {
            $return_values['errors'] = $errors;
        }

        return $return_values;
    }
}
