<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
/*
 * Class class MxbToolsCodes --> from 'GadgetcardCodes'
 * NOTE: some code is not usefull, it is from 'Gadgetcard' and should be removed
 *   (everything about PRINT DATA is not usefull for mxb_tools)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsCodes
{
    public static function getCodes()
    {
        return json_decode(json_encode((object) MxbToolsCodes::getCodesArray()), false);
    }

    public static function getCodesArray()
    {
        // global static flags. this array is passed in smarty via Mxb_Tools values,
        // so is easy to check flags in tpl if I put it here
        // some codes (prints / siae / decoration) are from gadgetcard.
        return [
            'decoration_suffix' => 'DeCo',
            'print_option_suffix' => 'Prnt',
            'print_setup_suffix' => 'Pset',
            'siae_suffix' => 'SIAE',
            'ltm_charge_suffix' => 'LTM',
            'main_category_name' => 'Print Data',
            'decoration_category_name' => 'Print Data: prices',
            'print_option_category_name' => 'Print Data: data',
            'additional_cost_category_name' => 'Additional cost',
            'print_option_customization' => 'PRINT_OPTION',
            'print_option_position' => 'PRINT_POSITION',
            'print_option_image' => 'PRINT_IMAGE',
            'print_option_message' => 'PRINT_MESSAGE',
            'dress_size_customization_quantity' => 'DRESS_SIZE_QUANTITY',
            'type_product' => 0,
            'product' => 'Product',
            'type_print_option' => 1,
            'print_option' => 'Print',
            'feature_siae' => 'SIAE',
            'feature_siae_gb' => 'SIAE_GB',
            'feature_print_setup' => 'PRINT_SETUP',
            'feature_ltm_charge' => 'LTM_CHARGE',
            'additional_cost_total' => 'ADD_COST_TOTAL',
            'order_total' => 'ORDER_TOTAL',
            'product_total' => 'PRODUCT_TOTAL',
            'shipping_cost' => 'SHIPPING_COST',
            //  'filters_array' => GadgetcardCodes::getSearchFiltersArray(),
        ];
    }

    public static function getDressSizeAttributeArray()
    {
        $MXB_TOOLS_DRESS_SIZE_LIST = Configuration::getGlobalValue('MXB_TOOLS_DRESS_SIZE_LIST');
        // get size list array
        if ($MXB_TOOLS_DRESS_SIZE_LIST != '' && $MXB_TOOLS_DRESS_SIZE_LIST != '0') {
            // normalize list
            $sizeArray = MxbToolsUtils::idStringToArray($MXB_TOOLS_DRESS_SIZE_LIST);
        } else {
            $sizeArray = [
                '3XS',
                'XXXS',
                'XXS',
                'XS',
                'S',
                'M',
                'L',
                'XL',
                'XXL',
                'XXXL',
                '3XL',
                '4XL',
                '5XL',
                '4XLP',
                '5XLP',
                '116',
                '128',
                '104',
                '140',
                '152',
                '164',
            ];
        }

        return $sizeArray;
    }

    public static function getSearchFiltersArray()
    {
        return [
            'sale' => [
                'word' => 'gcfiltersale',
                'enabled' => false,
            ],
        ];
    }

    /**
     * explode print option / decoration reference to get an array with defined values
     */
    public static function getReference($reference)
    {
        $gcodes = MxbToolsCodes::getCodes();

        $product_reference = explode('.', $reference);
        for ($i = 0; $i < 4; ++$i) {
            if (!isset($product_reference[$i])) {
                $product_reference[$i] = '';
            }
        }

        if ($product_reference[0] === $gcodes->print_option_suffix) {
            return json_decode(json_encode((object) [
                'suffix' => $product_reference[0],
                'product_reference' => $product_reference[1],
                'product_attribute_reference' => $product_reference[2],
                'decoration_code' => $product_reference[3],
            ]), false);
        } else {
            return json_decode(json_encode((object) [
                'suffix' => $product_reference[0],
                'decoration_name' => $product_reference[1],
                'number_of_colors' => $product_reference[2],
                'logo_size_cm2' => $product_reference[3],
              ]), false);
        }
    }

    /**
     * create print option / decoration reference
     */
    public static function setReference($suffix, $product_reference, $product_attribute_reference, $decoration_code)
    {
        return $suffix . '.' . $product_reference . '.' . $product_attribute_reference . '.' . $decoration_code;
    }

    /**
     * returns a string for sql query to include only products in query
     * string starts after 'WHERE' command
     *
     * $table_alias (undefined, null or string) is the alias for the table, escluding the field name (reference)
     *
     * call it this way: 'WHERE' . sqlFilterProductsOnly('prod') . 'AND .....'
     */
    public static function sqlFilterProductsOnly($table_alias = null)
    {
        $gcodes = MxbToolsCodes::getCodes();
        $field = '`reference`';
        if ($table_alias !== null) {
            $field = $table_alias . '.' . $field;
        }
        return ' ' . $field . ' NOT LIKE \'' . $gcodes->decoration_suffix . '.%\'
            AND ' . $field . ' NOT LIKE \'' . $gcodes->print_option_suffix . '.%\'
            AND ' . $field . ' NOT LIKE \'' . $gcodes->print_setup_suffix . '.%\'
            AND ' . $field . ' NOT LIKE \'' . $gcodes->siae_suffix . '.%\' ';
    }
}
