<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

// use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsCustomer extends ObjectModel
{
    /**
     * move customer of a group to a new group
     */
    public function changeCustomerGroup(
        $id_group_source = 0,
        $id_group_target = 0,
        $limit = 0,
        $id_customer = 0,
        $tot_records = 0
    ) {
        $limit = (int) $limit;
        $last_id_customer = $id_customer;
        $inserted_records = 0;
        $success = [];
        $errors = [];
        try {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsCustomer::changeCustomerGroup',
                'id_group_source=' . $id_group_source
                    . ', id_group_target=' . $id_group_target
                    . ', limit=' . $limit
                    . ', id_customer=' . $id_customer
            );
            */
            if ($id_group_source === 0 || $id_group_target === 0) {
                return ['records' => -1, 'tot_records' => 0, 'id' => 0];
            }

            // count interested customers just once (first ajax calls: both tot_records and id_customer are 0 !)
            if ($tot_records === 0) {
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'customer
                    WHERE id_default_group = ' . $id_group_source;
                // MxbToolsDebug::writeLog('Mxb_Tools', 'MxbToolsCustomer::changeCustomerGroup', 'sql=' . $sql);
                $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
                $success[] = 'Found ' . $tot_records . ' customers with default group = ' . $id_group_source;
            }

            // load customer id list for missing customers (order by id_customer, load id_customer > last_id_customer
            $sql = '
                SELECT id_customer
                FROM ' . _DB_PREFIX_ . 'customer
                WHERE id_default_group = ' . $id_group_source
                . ($id_customer > 0 ? ' AND id_customer > ' . $id_customer : '') . '
                ORDER BY id_customer ASC';
            $customers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

            // MxbToolsDebug::writeLog('Mxb_Tools', 'MxbToolsCustomer::changeCustomerGroup', 'customers collected.');
            $inserted_records = 0;

            if ($customers && count($customers)) {
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCustomer::changeCustomerGroup',
                    'customers collected=' . count($customers)
                );
                */
                $success[] = count($customers) . ' customers to finish ...';

                foreach ($customers as $customer) {
                    set_time_limit(300);
                    $cust = new Customer($customer['id_customer']);

                    $cust->id_default_group = (int) $id_group_target;
                    if ($cust->update()) {
                        $groupsToAdd = [];
                        array_push($groupsToAdd, $cust->id_default_group);
                        $cust->updateGroup($groupsToAdd);
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsCustomer::changeCustomerGroup',
                            'id_customer: ' . $customer['id_customer'] . ' update to group ' . $id_group_target . ' OK'
                        );
                        */
                        $success[] = 'Customer id=' . $cust->id . ' moved from  group '
                            . $id_group_source . ' to group ' . $id_group_target;
                        ++$inserted_records;
                    }

                    $last_id_customer = (int) $customer['id_customer'];
                    if ($limit > 0 && $inserted_records >= $limit) {
                        break;
                    }
                } // for
                $success[] = 'Users elaborated: ' . $inserted_records;
                if ($limit === 0 || $inserted_records < $limit) {
                    $inserted_records = -1;
                }
            } else {
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCustomer::changeCustomerGroup',
                    'NO CUSTOMERS FOR GROUP SOURCE !!'
                );
                */
                $inserted_records = -1; // no customers
                if ($tot_records === 0) {
                    $success[] = 'Found 0 customers with default group = ' . $id_group_source . ': NOTHING DONE.';
                }
            }
            $ret = ['records' => $inserted_records, 'tot_records' => $tot_records, 'id' => $last_id_customer];

            if (count($errors)) {
                $ret['errors'] = implode('|', $errors);
            }
            if (count($success)) {
                $ret['success'] = implode('|', $success);
            }
            return $ret;
        } catch (Exception $e) {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsCustomer::changeCustomerGroup',
                'Exception =' . $e->getMessage()
            );
            */
            return [
                'records' => $inserted_records,
                'tot_records' => $tot_records,
                'id' => $last_id_customer,
                'error' => $e->getMessage(),
            ];
        }
    }
}
