<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

if (file_exists(_PS_MODULE_DIR_ . 'mxbmemberprogram/classes/MxbmpCustomerAdditionalData.php')) {
    include_once _PS_MODULE_DIR_ . 'mxbmemberprogram/classes/MxbmpCustomerAdditionalData.php';
}

class MxbToolsCustomerImport // extends ObjectModel
{
    /**
     * Customer import: fields list for new customer
     * used in importCustomerFromCsv()
     * fields name are copied from 'customer' table and mxbmp_customer additional_data table
     *  (MxbmpCustomerAdditionalData::$definition)
     */
    public static $definitions = [
        'field_key' => [
            'active',
            'gender',
            'email',
            'password',
            'birthday',
            'lastname',
            'firstname',
            'newsletter',
            'optin',
            'date_add',
            'groups',
            'id_default_group',
            'privacy_check',
            'customer_code',
            'company',
            'vat_number',
            'address1',
            'city',
            'postcode',
            'state',
            'country',
            'pec',
            'sdi',
            'phone',
            'dni',
        ],
        'target_folder' => _PS_MODULE_DIR_ . 'mxb_tools' . DIRECTORY_SEPARATOR . 'csv' . DIRECTORY_SEPARATOR,
        'target_filename' => 'mxb_customers_import.csv',
        'sample_filename' => 'mxb_customers_import_sample.csv',
        'csv_delimeter' => ';',
    ];

    /*
    public static $customer_csv_field_key = array(
        'active',
        'gender',
        'email',
        'password',
        'birthday',
        'lastname',
        'firstname',
        'newsletter',
        'optin',
        'date_add',
        'groups',
        'id_default_group',
        'privacy_check',
        'customer_code',
        'company',
        'vat_number',
        'address1',
        'city',
        'postcode',
        'state',
        'country',
        'pec',
        'sdi',
        'phone',
    );
    public static $customer_csv_target_folder = _PS_MODULE_DIR_ . 'mxbmemberprogram'
        .DIRECTORY_SEPARATOR.'csv'.DIRECTORY_SEPARATOR;
    public static $customer_csv_target_filename = 'mxb_customers_import.csv';
    public static $customer_csv_sample_filename = 'mxb_customers_import_sample.csv';
    public static $csv_delimeter = ';';
    */

    public static function importCustomerFromCsv($limit = 0, $id = 0, $tot_records = 0)
    {
        // NO DEBUG: set $simulate to false !
        $simulate = false;
        $errors = [];
        $success = [];
        $limit = 0;

        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            $getCustomerAdditionalData = 'MxbmpCustomerAdditionalData::getCustomerAdditionalData';
            $addCustomerAdditionalData = 'MxbmpCustomerAdditionalData::addCustomerAdditionalData';
        } else {
            $getCustomerAdditionalData = null;
            $addCustomerAdditionalData = null;
        }

        $csv = MxbToolsUtils::csvToArray(
            self::$definitions['target_folder'] . self::$definitions['target_filename'],
            self::$definitions['csv_delimeter']
        );
        if (!$csv) {
            return [
                'records' => -1,
                'tot_records' => 0,
                'id' => 0,
                'errors' => MxbToolsCustomerImport::addImportLog('-1', 'CSV FILE', 'error converting CSV file.'),
            ];
        }
        // don't accept single record
        // if ($limit === 1)
        //  $limit = 2;

        // if ($limit > 0 && $id === 0) {
        if ((int) $tot_records === 0) {
            $tot_records = count($csv); // Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
            // } else {
            //    $tot_records = 0;
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCustomerImport::importCustomerFromCsv',
            'tot_records=' . $tot_records
        );
        */
        $next_id = (int) $id + 1;
        $inserted_records = 0;

        // import main loop
        foreach ($csv as $rowkey => $row) {
            set_time_limit(300);

            // skip to current position
            if ((int) $rowkey < (int) $id) {
                continue;
            }
            $added = false;
            $next_id = (int) $rowkey + 1;

            /*
              CSV file must have every field in the right position because values will be take by its field position:
                0 Active (0/1)
                1 Titles ID (Mr = 1, Ms = 2, else 0)
                2 Email *
                3 Password *
                4 Birthday (yyyy-mm-dd)
                5 Last Name *
                6 First Name *
                7 Newsletter (0/1)
                8 Opt-in (0/1)
                9 Registration date (yyyy-mm-dd)
               10 Groups (x,y,z...)
               11 Default group ID
               12 Privacy checked
               13 Customer code
               14 Company
               15 VAT
               16 Address
               17 City
               18 CAP
               19 State / Province
               20 Country
               21 Pec
               22 Sdi
               23 Phone
            */

            $i = 0;
            $customerData = [];
            foreach ($row as $c) {
                if (isset(self::$definitions['field_key'][$i])) {
                    $customerData[self::$definitions['field_key'][$i]] = $c;
                }
                ++$i;
            }
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsCustomerImport::importCustomerFromCsv',
                'LOOP ' . $rowkey . ' ARRAY customerData=' . print_r($customerData,true)
            );
            */
            // look for existing customer
            if ($customerData['email'] == '') {
                $errors[] = MxbToolsCustomerImport::addImportLog($rowkey, '', 'missing email !!!');

                // skip this customer
                continue;
            }

            // set as [novalue] to tell the front JS to hide name and ask to for compile the field
            if ($customerData['firstname'] == '') {
                $customerData['firstname'] = 'novalue';
            }
            if ($customerData['lastname'] == '') {
                $customerData['lastname'] = 'novalue';
            }
            if ($customerData['company'] == '') {
                $customerData['company'] = 'novalue';
            }
            $customer_email = $customerData['email'];
            $sql = '
                SELECT id_customer
                FROM ' . _DB_PREFIX_ . 'customer
                WHERE LOWER(email)="' . Tools::strtolower($customer_email) . '"';
            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            $id_customer = !$result ? 0 : (int) $result[0]['id_customer'];

            // create new customer
            if ($id_customer === 0) {
                $customer = new Customer();
                $customer->lastname = ucwords($customerData['lastname']);
                $customer->firstname = ucwords($customerData['firstname']);
                $pwd = $customerData['password'];
                $customer->passwd = md5(pSQL(_COOKIE_KEY_ . $pwd));
                $customer->email = $customerData['email'];
                $customer->company = $customerData['company'];
                $customer->birthday = $customerData['birthday'];
                $customer->id_gender = (int) $customerData['gender'];
                /*
                $customer->birthday = (empty($_POST['years'])
                    ? ''
                    : (int) $_POST['years'].'-'.(int) $_POST['months'].'-'.(int) $_POST['days']);
                if (!Validate::isBirthDate($customer->birthday))
                    exit('Invalid date of birth.');
                */
                $customer->is_guest = 0;
                $customer->active = (int) $customerData['active'];
                $customer->newsletter = (int) $customerData['newsletter'];
                $customer->id_default_group = (int) $customerData['id_default_group'];
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCustomerImport::importCustomerFromCsv',
                    'Being to add customer ' . $customer->email . ' (simulate=' . ($simulate ? 'TRUE' : 'FALSE') . ')'
                );
                */
                if (!$simulate) {
                    if ($customer->add()) {
                        // get customer id
                        $id_customer = $customer->id;
                        $added = true;
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsCustomerImport::importCustomerFromCsv',
                            'ADD CUSTOMER ' . $customer->email . ' SUCCESS: ID='.$customer->id
                        );
                        */

                        // add cuistomer to default group
                        // TODO: add to other groups in "groups" field ...
                        $groupsToAdd = [];
                        array_push($groupsToAdd, $customer->id_default_group);
                        $customer->addGroups($groupsToAdd);

                        // set "reset password request" for new customers
                        // $customer->stampResetPasswordToken();
                        // $customer->update();
                        $success[] = MxbToolsCustomerImport::addImportLog($rowkey, $customerData['email'], 'ADDED.');
                    } else {
                        $errors[] = MxbToolsCustomerImport::addImportLog(
                            $rowkey,
                            $customerData['email'],
                            'error adding customer.'
                        );
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsCustomerImport::importCustomerFromCsv',
                            'ADD CUSTOMER ' . $customer->email . ' ERROR!!'
                        );
                        */
                    }

                    // add address if mxbmemberprogram is not installed
                    if (!$getCustomerAdditionalData && $id_customer > 0 && !$simulate) {
                        // get country id
                        $id_country = self::getIdCountry($customerData['country']);
                        // get state id
                        $id_state = self::getIdState($customerData['state'], $id_country);
                        if ($id_country > 0 && $id_state > 0) {
                            $customerData['id_country'] = $id_country;
                            $customerData['id_state'] = $id_state;
                            $id_address = self::saveNewAddress($customerData, $id_customer, true);
                            if (!$id_address) {
                                $errors[] = MxbToolsCustomerImport::addImportLog(
                                    $rowkey,
                                    $customerData['email'],
                                    'address was not added because data is incomplete'
                                );
                            }
                        } else {
                            $errors[] = MxbToolsCustomerImport::addImportLog(
                                $rowkey,
                                $customerData['email'],
                                'address not added because country ' . $customerData['country']
                                    . ' (id=' . $id_country . ') or state ' . $customerData['state']
                                    . ' (id=' . $id_state . ') not found'
                            );
                        }
                    }
                } else {
                    $added = true; // debug simulate
                }
            } else {
                // $id_customer > 0 !
                $customer = new Customer($id_customer);

                // change customer group
                if ((int) $customer->id_default_group !== (int) $customerData['id_default_group']) {
                    $customer->id_default_group = (int) $customerData['id_default_group'];
                    if (!$simulate) {
                        if ($customer->update()) {
                            $groupsToAdd = [];
                            array_push($groupsToAdd, $customer->id_default_group);
                            $customer->updateGroup($groupsToAdd);
                            // $updated = true;
                            $success[] = MxbToolsCustomerImport::addImportLog(
                                $rowkey,
                                $customerData['email'],
                                'UPDATED WITH NEW GROUP ' . $customer->id_default_group
                            );
                        } else {
                            $errors[] = MxbToolsCustomerImport::addImportLog(
                                $rowkey,
                                $customerData['email'],
                                'error updating customer.'
                            );
                        }
                    }
                } else {
                    $errors[] = MxbToolsCustomerImport::addImportLog(
                        $rowkey,
                        $customerData['email'],
                        'customer already exists with the same group.'
                    );
                }
                // continue with the next customer elseif we have not really added or updated this customer.
                $added = true;
            } // else ($id_customer === 0)

            // insert additional data
            if ($getCustomerAdditionalData && $id_customer > 0) {
                // check if additional data already exist
                $result = $getCustomerAdditionalData(0, $id_customer);
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCustomerImport::importCustomerFromCsv',
                    'ADD ADDITIONAL DATA:: check if additional data present, ARRAY result=' . print_r($result, true)
                );
                */

                // add additional data
                if (!$result) {
                    // get country id
                    $id_country = self::getIdCountry($customerData['country']);
                    if ($id_country === 0) {
                        $id_country = 1;
                        $errors[] = MxbToolsCustomerImport::addImportLog(
                            $rowkey,
                            $customerData['email'],
                            'country ' . $customerData['country'] . ' not found.'
                        );
                    }

                    // get state id
                    $id_state = self::getIdState($customerData['state'], $id_country);
                    if ($id_state === 0) {
                        $id_state = 1;
                        $errors[] = MxbToolsCustomerImport::addImportLog(
                            $rowkey,
                            $customerData['email'],
                            'state ' . $customerData['state'] . ' not found.'
                        );
                    }

                    $id_address = 0;
                    $addData = [
                        'id_mxbmp_customer_additional_data' => 0,
                        'customer_code' => $customerData['customer_code'],
                        'company' => addslashes($customerData['company']),
                        'id_customer' => $id_customer,
                        'id_country' => $id_country,
                        'id_state' => $id_state,
                        'id_address' => $id_address,
                        'vat_number' => $customerData['vat_number'],
                        'pec' => $customerData['pec'],
                        'sdi' => $customerData['sdi'],
                        'address1' => addslashes($customerData['address1']),
                        'address2' => '',
                        'postcode' => $customerData['postcode'],
                        'city' => addslashes(ucwords($customerData['city'])),
                        'phone' => $customerData['phone'],
                        'privacy_check' => $customerData['privacy_check'],
                    ];

                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools',
                        'MxbToolsCustomerImport::importCustomerFromCsv',
                        'ADD ADDITIONAL DATA (simulate=' . ($simulate ? 'TRUE' : 'FALSE')
                            . ') :: ARRAY addData=' . print_r($addData, true)
                    );
                    */
                    if (!$simulate) {
                        $result = $addCustomerAdditionalData($addData);
                    }
                    $added = true;
                } else {
                    continue;
                }
            }
            if ($added && $rowkey < count($csv) - 1) {
                ++$inserted_records;
            }
            if ($limit > 0 && $inserted_records >= $limit) {
                break;
            }
        } // foreach ($csv as $rowkey=>$row)

        $ret = [
            'records' => $inserted_records,
            'tot_records' => $tot_records,
            'id' => $next_id,
        ];
        if (count($errors)) {
            $ret['errors'] = implode('|', $errors);
        }
        if (count($success)) {
            $ret['success'] = implode('|', $success);
        }

        return $ret;
    }

    public static function addImportLog($rowkey = 0, $customer = '', $message = '')
    {
        return 'row ' . $rowkey . ' [' . $customer . '] ' . $message;
    }

    public static function uploadCsvFile()
    {
        $source_fileinput = 'mxb_tools-upload-csv-customer';
        /*
        MxbToolsDebug::writeLog(
            'mxb_tools',
            'MxbToolsCustomerImport::uploadCsvFile()',
            'mxb_tools-upload-csv-customer-name='.Tools::getValue('mxb_tools-upload-csv-customer-name')
        );
        */
        if (isset($_FILES[$source_fileinput])
            && isset($_FILES[$source_fileinput]['tmp_name'])
            && !empty($_FILES[$source_fileinput]['tmp_name'])
        ) {
            if (!move_uploaded_file(
                $_FILES[$source_fileinput]['tmp_name'],
                self::$definitions['target_folder'] . self::$definitions['target_filename']
            )) {
                return false;
            }

            return true;
        }

        return false;
    }

    public static function getIdCountry($name)
    {
        // help with italian language for country names ... not finished, not so important...
        $name = Tools::strtolower($name);
        $name_eng = $name;
        switch ($name) {
            case 'italia':
                $name_eng = 'italy';
                break;
            case 'spagna':
                $name_eng = 'spain';
                break;
            case 'francia':
                $name_eng = 'france';
                break;
            case 'germania':
                $name_eng = 'germany';
                break;
            case 'grecia':
                $name_eng = 'greece';
                break;
            case 'inghilterra':
                $name_eng = 'united kingdom';
                break;
        }

        $sql = '
            SELECT `id_country`
            FROM `' . _DB_PREFIX_ . 'country_lang`
            WHERE LOWER(`name`) = "' . $name . '" OR LOWER(`name`) = "' . $name_eng . '"';
        $result = Db::getInstance()->getRow($sql);
        if ($result) {
            return (int) $result['id_country'];
        }

        return 0;
    }

    public static function getIdState($name, $id_country)
    {
        $id_state = 0;
        if ($name !== '') {
            $name = Tools::strtolower($name);
            $sql = '
                SELECT `id_state`
                FROM `' . _DB_PREFIX_ . 'state`
                WHERE LOWER(`name`) = "' . $name . '" OR LOWER(`iso_code`) = "' . $name . '"
                AND id_country = ' . $id_country;
            $result = Db::getInstance()->getRow($sql);
            if ($result) {
                $id_state = (int) $result['id_state'];
            }
        }

        return $id_state;
    }

    /**
     * add new customer address
     *
     * return 0|id_address
     */
    public static function saveNewAddress($customer_data = null, $id_customer = 0, $check = true, $id_address = 0)
    {
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCustomerImport::saveNewAddress',
            'ADD ADDRESS for customer UD=' . $id_customer . ' : ' . print_r($customer_data, true),
            true
        );

        $module = Module::getInstanceByName('mxb_tools');
        // check if address data is fully compiled
        // first and last name are not important because if empty, it will be recovered from the customer data
        if (empty($customer_data['address1'])
            || empty($customer_data['postcode'])
            || empty($customer_data['city'])
        ) {
            return 0;
        }
        if (isset($customer_data['dni'])) {
            if (!Validate::isDniLite($customer_data['dni'])) {
                $customer_data['dni'] = '';
            }
        }
        $customer_address_id = 0;
        if ($id_address > 0) {
            // check address exists
            if (Address::addressExists($id_address)) {
                $customer_address_id = $id_address;
            }
        } else {
            // check if customer has alread an address
            $customer_address_id = $check ? Address::getFirstCustomerAddressId($id_customer) : 0;
        }
        if (!$customer_address_id || (int) $id_address === (int) $customer_address_id) {
            // $context = Context::getContext();
            // $translator = $context->getTranslator();

            // get customer data needed to fill new address
            if (!isset($customer_data['lastname'])) {
                $sql = '
                    SELECT firstname, lastname
                    FROM ' . _DB_PREFIX_ . 'customer
                    WHERE id_customer=' . $id_customer;
                $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
                if (!$result) {
                    // no customer ???!??
                    return 0;
                }
                $customer_data['firstname'] = $result[0]['firstname'];
                $customer_data['lastname'] = $result[0]['lastname'];
            }
            $customer_data['id_customer'] = $id_customer;

            // create a new address with additional data
            if (!$customer_address_id) {
                $new_address = new Address();
            } else {
                $new_address = new Address($customer_address_id);
            }

            $new_address->firstname = $customer_data['firstname'];
            $new_address->lastname = $customer_data['lastname'];
            $new_address->alias = $module->l('My main address');
            $new_address->id_customer = (int) $customer_data['id_customer'];
            $new_address->id_country = $customer_data['id_country'];
            $new_address->id_state = $customer_data['id_state'];
            $new_address->company = $customer_data['company'];
            $new_address->address1 = $customer_data['address1'];
            $new_address->address2 = isset($customer_data['address2']) ? $customer_data['address2'] : '';
            $new_address->postcode = $customer_data['postcode'];
            $new_address->city = $customer_data['city'];
            $new_address->phone = $customer_data['phone'];
            $new_address->vat_number = isset($customer_data['vat_number']) ? $customer_data['vat_number'] : '';
            $new_address->dni = isset($customer_data['dni']) ? $customer_data['dni'] : '';

            /*
            // MXB address table modifications: custom fields
            $new_address->pec = $customer_data['pec'];
            $new_address->sdi = $customer_data['sdi'];
            $new_address->customer_code = $customer_data['customer_code'];
            */

            if (!$customer_address_id) {
                $new_address->add();
            } else {
                $new_address->update();
            }
            /*
            // check if cart exists and add the new id_address
            if (!is_null($context->cart->id)) {
                $sql = 'UPDATE ' . _DB_PREFIX_ . 'cart
                    SET id_address_delivery = ' . $new_address->id . ',
                        id_address_invoice = ' . $new_address->id . '
                    WHERE id_cart = ' . $context->cart->id;
                Db::getInstance()->execute($sql);
            }
            */

            return $new_address->id;
        }

        return $customer_address_id;
    }
}
