<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsDebug
{
    public static function writeLog(
        $classname,
        $function,
        $message,
        $bypass_settings = false,
        $filename = null,
        $max_backtrace = null
    ) {
        if (Configuration::getGlobalValue('MXB_TOOLS_DEBUG') != true && $bypass_settings == false) {
            return;
        }
        if (!file_exists(_PS_MODULE_DIR_ . 'mxb_tools/log')) {
            mkdir(_PS_MODULE_DIR_ . 'mxb_tools/log', 0777, true);
        }
        if ($filename === null) {
            $filename = 'debug.log';
        }

        $caller = null;
        /*
        if (!$max_backtrace) {
            $maxbt = (int) Configuration::getGlobalValue('GADGETCARD_DEBUG_MAX_BACKTRACE');
            $max_backtrace = $maxbt;
        }
        */
        if ($max_backtrace && (int) $max_backtrace > 0) {
            // get caller function
            // debug_backtrace(int $options = DEBUG_BACKTRACE_PROVIDE_OBJECT, int $limit = 0): array
            $dbt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, $max_backtrace);
            // $caller = isset($dbt[1]['function']) ? $dbt[1]['function'] : null;
            // $caller = [];
            $caller = [];
            // foreach ($dbt as $k=>$f) {
            for ($k = count($dbt) - 1; $k > ($function ? 1 : 0); --$k) {
                if ($k === 0) {
                    continue;
                }
                /*
                $caller[] = [
                    'function' => $f['function'] ?? null,
                    'line' => $f['line'] ?? null,
                    'class' => $f['class'] ?? null,
                ];
                */

                $caller[] = ($dbt[$k]['class'] ?? null)
                    . '->' . ($dbt[$k]['function'] ?? null)
                    . '[' . ($dbt[$k]['line'] ?? null) . ']';
            }
            $caller = implode(' ==> ', $caller);
        }

        $logger = new FileLogger(0);
        $logger->setFilename(_PS_MODULE_DIR_ . 'mxb_tools/log/' . $filename);
        // $e = new Exception;
        // $logger->logDebug($classname . ' ' . $function . ': ' . $message);
        $logger->logDebug(
            $classname
            . ($function ? '->' . $function : '')
            . ($caller ? ' [' . (string) $caller . ']' : '')
            . ': ' . $message
        );
    }

    public static function writeConsole($classname, $function, $message)
    {
        echo '<script>console.log(\'' . $classname . ' ' . $function . ': ' . $message . '\');</script>';
    }

    public static function getAllPost() // $string_format = true)
    {
        // $ret = $string_format ? '' : [];
        $ret = '';
        if (count($_POST)) {
            $ret .= ' ::: _POST ::: ' . print_r($_POST, true);
            /*
                foreach ($_POST as $param_name => $param_val) {
                    if ($string_format) {
                        $ret .= $param_name . '=' . $param_val . ',';
                    } else {
                        $ret[$param_name] = $param_val;
                    }
                }
            */
        }
        if (count($_GET)) {
            $ret .= ' ::: _GET ::: ' . print_r($_GET, true);

            /*
                foreach ($_GET as $param_name => $param_val) {
                    if ($string_format) {
                      $ret .= $param_name.toString() . '=' . $param_val.toString() . ',';
                    } else {
                      $ret[$param_name.toString()] = $param_val.toString();
                    }
                }
            */
        }
        return $ret;
    }

    public static function getAllPostValues()
    {
        // $ret = '';
        // foreach ($_POST as $key => $value) {
        //    $ret .= htmlspecialchars($key) . ' = '.htmlspecialchars($value) . PHP_EOL;
        // }
        $ret = PHP_EOL . '_GET=' . print_r($_GET, true) . PHP_EOL . '_POST=' . print_r($_POST, true);

        return $ret;
    }

    public static function printBool($value = false)
    {
        return $value ? 'TRUE' : 'FALSE';
    }
}
