<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;

// require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
    include_once _PS_MODULE_DIR_ . 'mxbmemberprogram/classes/MxbmpUtils.php';
    include_once _PS_MODULE_DIR_ . 'mxbmemberprogram/classes/MxbmpCustomerAdditionalData.php';
} elseif (MxbToolsUtils::isModuleEnabled('gadgetcard')) {
    include_once _PS_MODULE_DIR_ . 'gadgetcard/classes/GadgetcardUtils.php';
} elseif (MxbToolsUtils::isModuleEnabled('mxb_docnum')) {
    include_once _PS_MODULE_DIR_ . 'mxb_docnum/classes/MxbDocnumUtils.php';
}

class MxbToolsExport
{
    /*
     * EXPORT ORDERED PRODUCT LIST
     *
     * ORDER ARRAY:
      [id_order] => 127
      [reference] => TAQJJNMQP
      [id_shop_group] => 1
      [id_shop] => 1
      [id_carrier] => 2
      [id_lang] => 1
      [id_customer] => 5
      [id_cart] => 209
      [id_currency] => 1
      [id_address_delivery] => 8
      [id_address_invoice] => 8
      [current_state] => 10
      [secure_key] => 85d4bc16935413f4af493e0e51516b0d
      [payment] => Bonifico bancario
      [conversion_rate] => 1.000000
      [module] => ps_wirepayment
      [recyclable] => 0
      [gift] => 0
      [gift_message] =>
      [mobile_theme] => 0
      [shipping_number] =>
      [total_discounts] => 0.000000
      [total_discounts_tax_incl] => 0.000000
      [total_discounts_tax_excl] => 0.000000
      [total_paid] => 832.410000
      [total_paid_tax_incl] => 832.410000
      [total_paid_tax_excl] => 682.300000
      [total_paid_real] => 0.000000
      [total_products] => 679.800000
      [total_products_wt] => 829.360000
      [total_shipping] => 3.050000
      [total_shipping_tax_incl] => 3.050000
      [total_shipping_tax_excl] => 2.500000
      [carrier_tax_rate] => 22.000
      [total_wrapping] => 0.000000
      [total_wrapping_tax_incl] => 0.000000
      [total_wrapping_tax_excl] => 0.000000
      [round_mode] => 2
      [round_type] => 2
      [invoice_number] => 0
      [delivery_number] => 0
      [invoice_date] => 0000-00-00 00:00:00
      [delivery_date] => 0000-00-00 00:00:00
      [valid] => 0
      [date_add] => 2020-02-26 16:41:22
      [date_upd] => 2020-02-26 16:41:23

    ORDER_DETAIL ARRAY:
      [id_order_detail] => 235
            [id_order] => 114
            [id_order_invoice] => 0
            [id_warehouse] => 0
            [id_shop] => 1
            [product_id] => 12809
            [product_attribute_id] => 30712
            [id_customization] => 0
            [product_name] => Borraccia in alluminio Oregon con moschettone - Color : verde mela
            [product_quantity] => 2
            [product_quantity_in_stock] => 2
            [product_quantity_refunded] => 0
            [product_quantity_return] => 0
            [product_quantity_reinjected] => 0
            [product_price] => 1.318500
            [reduction_percent] => 0.00
            [reduction_amount] => 0.000000
            [reduction_amount_tax_incl] => 0.000000
            [reduction_amount_tax_excl] => 0.000000
            [group_reduction] => 55.00
            [product_quantity_discount] => 0.000000
            [product_ean13] => 8713159038610
            [product_isbn] =>
            [product_upc] =>
            [product_reference] => 10000200
            [product_supplier_reference] =>
            [product_weight] => 86.000000
            [id_tax_rules_group] => 1
            [tax_computation_method] => 0
            [tax_name] =>
            [tax_rate] => 0.000
            [ecotax] => 0.000000
            [ecotax_tax_rate] => 0.000
            [discount_quantity_applied] => 0
            [download_hash] =>
            [download_nb] => 0
            [download_deadline] => 0000-00-00 00:00:00
            [total_price_tax_incl] => 3.220000
            [total_price_tax_excl] => 2.640000
            [unit_price_tax_incl] => 1.608570
            [unit_price_tax_excl] => 1.318500
            [total_shipping_price_tax_incl] => 0.000000
            [total_shipping_price_tax_excl] => 0.000000
            [purchase_supplier_price] => 1.050000
            [original_product_price] => 1.318500
            [original_wholesale_price] => 1.050000

     *
     * Danea XML documentation: https://www.danea.it/software/easyfatt/xml/documenti/
     * TAG <DocumentType>:
     *  codice corrispondente al tipo di documento; se questo tag viene omesso,
     *      il documento sar&#65533; considerato un ordine cliente.
     *   Elenco dei tipi ammessi:
     *     A: Avviso di parcella
     *     B: Vendita al banco
     *     C: Ordine cliente (tipo predefinito)
     *     D: Documento di trasporto
     *     E: Ordine fornitore
     *     F: Fattura accompagnatoria
     *     G: Rapporto d'intervento
     *     I: Fattura
     *     J: Fattura d'acconto
     *     L: Fattura pro-forma
     *     M: Autofattura
     *     N: Nota d'accredito
     *     O: Nota d'addebito
     *     P: Parcella
     *     Q: Preventivo
     *     R: Ricevuta fiscale
     *     S: Preventivo fornitore
     *     H: Arrivi merce fornitore

            https://www.imprenditoriamo.it/site/xml-caratteri-non-ammessi/
     */
    public static function exportOrders($data = [])
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        // $force_debug = false;
        $module = Module::getInstanceByName('mxb_tools');
        $folder = null;
        // set numbering class
        $generateCustomReference = null;
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            $generateCustomReference = 'MxbmpUtils::generateCustomReference';
        } elseif (MxbToolsUtils::isModuleEnabled('gadgetcard')) {
            $generateCustomReference = 'GadgetcardUtils::generateCustomReference';
        } elseif (MxbToolsUtils::isModuleEnabled('mxb_docnum')) {
            $generateCustomReference = 'MxbDocnumUtils::generateCustomReference';
        }

        $id_lang = (int) Context::getContext()->language->id;
        $return_values = [
            'values' => [],
        ];

        $allowed_output_types = [
          'orders',
          'group_totals',
          'ordered_products',
        ];
        $allowed_output_formats = [
          'xml',
          'csv',
        ];
        /*
        MxbToolsDebug::writeLog(
            'mxb_tools',
            'MxbToolsExport::exportOrders()',
            'data=' . print_r($data, true),
            true
        );
        */

        /* check values */
        if (!isset($data['output_format'])) {
            $data['output_format'] = $allowed_output_formats[0];
        } elseif (!in_array($data['output_format'], $allowed_output_formats)) {
            $data['output_format'] = $allowed_output_formats[0];
        }
        if (!isset($data['output_type'])) {
            $data['output_type'] = $allowed_output_types[0];
        } elseif (!in_array($data['output_type'], $allowed_output_types)) {
            $data['output_type'] = $allowed_output_types[0];
        }
        $group_totals_request = false;
        if ($data['output_type'] == 'group_totals') {
            $group_totals_request = true;
            $data['output_type'] = 'orders';
        }
        if (!isset($data['size_tag'])) {
            $data['size_tag'] = '';
        }
        if (!isset($data['color_tag'])) {
            $data['color_tag'] = '';
        }
        if (!isset($data['date_from'])) {
            $data['date_from'] = MxbToolsUtils::getDateString('2020-01-01');
        }
        if (!isset($data['date_to'])) {
            $data['date_to'] = MxbToolsUtils::getDateString('2030-01-01');
        }
        // force international/database date format
        $data['date_from'] = MxbToolsUtils::getDateString($data['date_from']);
        $data['date_to'] = MxbToolsUtils::getDateString($data['date_to']);
        if (!isset($data['force_order_numeric'])) {
            $data['force_order_numeric'] = false;
        }
        if (!isset($data['export_invoice'])) {
            $data['export_invoice'] = false;
        }
        if (!isset($data['export_delivery_slip'])) {
            $data['export_delivery_slip'] = false;
        }
        if (!isset($data['export_product_discount'])) {
            $data['export_product_discount'] = false;
        }
        if (!isset($data['order_prefix_from'])) {
            $data['order_prefix_from'] = '';
        }
        if (!isset($data['order_prefix_to'])) {
            $data['order_prefix_to'] = '';
        }
        if (!isset($data['numbering'])) {
            $data['numbering'] = '';
        }
        if (!isset($data['customer_prefix'])) {
            $data['customer_prefix'] = '';
        }

        // if (!isset($data['export_customer']) )  $data['export_customer'] = true;
        $data['id_from'] = isset($data['id_from']) ? (int) $data['id_from'] : 0;
        $data['id_to'] = isset($data['id_to']) ? (int) $data['id_to'] : 0;
        $data['id_order_state'] = isset($data['id_order_state']) ? (int) $data['id_order_state'] : 0;
        $data['id_order_state_next'] = isset($data['id_order_state_next']) ? (int) $data['id_order_state_next'] : 0;
        $data['id_order_state_list'] = isset($data['id_order_state_list']) ? (string) $data['id_order_state_list'] : '';
        if ($data['id_order_state_list'] == '0') {
            $data['id_order_state_list'] = '';
        }
        $data['id_order_state_list_not'] = isset($data['id_order_state_list_not'])
            ? (string) $data['id_order_state_list_not']
            : '';
        if ($data['id_order_state_list_not'] == '0') {
            $data['id_order_state_list_not'] = '';
        }

        // if no order state will be used to filter orders, no new state will be set !
        if ($data['id_order_state'] == 0 || $data['id_order_state_next'] == 0 || $data['output_type'] != 'orders') {
            $data['id_order_state'] = 0;
            $data['id_order_state_next'] = 0;
        }

        // check if status exists
        if ($data['id_order_state_next'] > 0) {
            $sql = 'SELECT *
                FROM ' . _DB_PREFIX_ . 'order_state
                WHERE id_order_state=' . $data['id_order_state_next'];
            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            if (!$result) {
                $data['id_order_state_next'] = 0;
            }
        }

        // prepare filters
        if ($data['id_order_state_next'] === 0) {
            // no status-id conversion: get status list to consider for export
            $idStatusArray = (Tools::strlen($data['id_order_state_list']) > 0
                ? MxbToolsUtils::idStringToArray($data['id_order_state_list'])
                : $idStatusArray = []);
            // normalize id_order_state_list
            $data['id_order_state_list'] = MxbToolsUtils::idArrayToString($idStatusArray);

            // get status list to NOT consider for export
            $idStatusNotArray = (Tools::strlen($data['id_order_state_list_not']) > 0
                ? MxbToolsUtils::idStringToArray($data['id_order_state_list_not'])
                : $idStatusNotArray = []);
            // normalize id_order_state_list_not
            $data['id_order_state_list_not'] = MxbToolsUtils::idArrayToString($idStatusNotArray);
        } else {
            // status-id conversion: do not consider status filters, but correct them to save it in a correct format
            $idStatusArray = [];
            $idStatusNotArray = [];
            // normalize id list
            $data['id_order_state_list'] = MxbToolsUtils::normalizeIdString($data['id_order_state_list']);
            $data['id_order_state_list_not'] = MxbToolsUtils::normalizeIdString($data['id_order_state_list_not']);
        }

        // invert IDs if needed
        if ($data['id_from'] < 0) {
            $data['id_from'] = $data['id_from'] * (-1);
        }
        if ($data['id_to'] < 0) {
            $data['id_to'] = $data['id_to'] * (-1);
        }
        if ($data['id_from'] > $data['id_to']) {
            $tmp = $data['id_from'];
            $data['id_from'] = $data['id_to'];
            $data['id_to'] = $tmp;
        }

        $data['force_order_numeric'] = (bool) ((int) $data['force_order_numeric']);

        // save correct values in config, if needed
        $return_values['data'] = $data;
        /*
        MxbToolsDebug::writeLog(
            'mxb_tools',
            'MxbToolsExport::exportOrders()',
            'data=' . print_r($data, true),
            $force_debug
        );
        */
        // set common sql usefull to update same records with new order state, if needed
        $sql_common = '
            WHERE date_add >= STR_TO_DATE("' . $data['date_from'] . '", "%Y-%m-%d")
            AND date_add <= STR_TO_DATE("' . $data['date_to'] . '", "%Y-%m-%d")';

        if ($data['id_from'] + $data['id_to'] > 0) {
            $sql_common .= ' AND id_order >= ' . $data['id_from'] . '
                AND id_order <= ' . $data['id_to'];
        }
        if ($data['id_order_state'] > 0) {
            $sql_common .= ' AND current_state = ' . $data['id_order_state'];
        }

        // filter order_status
        if (count($idStatusArray)) {
            $sql_common .= ' AND (';
            foreach ($idStatusArray as $skey => $idStatus) {
                $sql_common .= ((int) $skey > 0 ? ' OR' : '') . ' current_state = ' . (int) $idStatus;
            }
            $sql_common .= ')';
        }

        // filter order_status
        if (count($idStatusNotArray)) {
            foreach ($idStatusNotArray as $skey => $idStatus) {
                $sql_common .= ' AND current_state <> ' . (int) $idStatus;
            }
        }

        $sql = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'orders '
            . $sql_common
            . ' ORDER BY id_order ASC';
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsExport::exportOrders()',
            'sql=' . $sql,
            true
        );
        */
        $orders = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$orders) {
            $return_values['errors'] = [];
            $return_values['errors'][] = 'no orders'; // 'No orders available for the date range';
            return $return_values;
        }

        // if ($data['output_type'] == 'orders') {
        //  $xml_string = '<?xml version="1.0" encoding="UTF-8" ? >' . PHP_EOL
        //          . '<EasyfattDocuments AppVersion="2" Creator="Danea Soft" CreatorUrl="www.danea.it">' . PHP_EOL
        /*
        . '
          <Company>
              <Name>Eino Grafica e Stampa srl</Name>
              <Address>Via Crissolo, 12</Address>
              <Postcode>10138</Postcode>
              <City>Torino</City>
              <Province>TO</Province>
              <Country>Italia</Country>
              <FiscalCode>01303760282</FiscalCode>
              <VatCode>01303760282</VatCode>
              <Tel>011-2568745</Tel>
              <Fax>011-2548793</Fax>
              <Email>arredufficiosrl@arredufficio.it</Email>
              <HomePage>www.arredufficiosrl.it</HomePage>
            </Company>'
        */
        // . '<Documents>';
        //  }

        if ($data['output_format'] == 'xml') {
            $orderDocumentXML = '';
        } else {
            $orderDocumentXML = [];
        }

        // orders_by_group
        $CustomerGroupTotals = [];
        $products = [];
        $order_data = [];
        $invoices = [];

        foreach ($orders as $order) {
            $order_reference = MxbToolsUtils::strReplacePrefix(
                $order['reference'],
                $data['order_prefix_from'],
                $data['order_prefix_to']
            );
            /*
            MxbToolsDebug::writeLog('Mxb_Tools',
                'MxbToolsExport::exportOrders()',
                'order ref=' . $order['reference'] . ', new ref=' . $order_reference . ',
                order='.print_r($order, true), $force_debug
            );
            */
            if ((bool) $data['force_order_numeric']) {
                $order_reference = MxbToolsUtils::extractNumber($order_reference);
            }

            // get customer
            $customer = new Customer((int) $order['id_customer']);
            if (!Validate::isLoadedObject($customer)) {
                // SET EMPTY CUSTOMER TO WRITE XML, BUT don't save it
                $customer = new Customer();
            }

            if ($data['output_type'] == 'orders') {
                // prepare export string for XML "orders"
                // get invoice_address
                $invoice_address = new Address((int) $order['id_address_invoice']);
                if (!Validate::isLoadedObject($invoice_address)) {
                    // SET EMPTY Address TO WRITE XML, BUT don't save it
                    $invoice_address = new Address();
                }

                // get taxes
                $taxes = TaxRulesGroup::getAssociatedTaxRatesByIdCountry($invoice_address->id_country);
                $delivery_address = false;
                $deliveryAddressXml = '';
                $customerEInvoiceDestCode = $data['output_format'] == 'xml'
                    ? ''
                    : ';';
                $customerPec = $data['output_format'] == 'xml'
                    ? ''
                    : ';';
                $customerFiscalCode = Tools::strlen($invoice_address->dni) > 0
                    ? Tools::strtoupper($invoice_address->dni)
                    : $invoice_address->vat_number;

                // orders_by_group
                $id_group = $customer->id_default_group;

                if ($data['output_format'] == 'csv') {
                    $group = new Group($id_group);
                    $CustomerGroup = $group->name[$id_lang];

                    // set totals
                    if (!isset($CustomerGroupTotals[$id_group])) {
                        $CustomerGroupTotals[$id_group] = [
                            'group_name' => $CustomerGroup,
                            'order_quantity' => 0,
                            'total_paid_tax_excl' => 0,
                            'total_discounts_tax_excl' => 0,
                            'total_paid_real' => 0,
                            'total_products' => 0,
                            'total_shipping_tax_excl' => 0,
                            'total_wrapping_tax_excl' => 0,
                            'total_discount_products' => 0,
                            'total_discount_shipping' => 0,
                        ];
                    }

                    $CustomerGroupTotals[$id_group]['total_paid_tax_excl'] += (float) $order['total_paid_tax_excl'];
                    ++$CustomerGroupTotals[$id_group]['order_quantity'];
                    $CustomerGroupTotals[$id_group]['total_discounts_tax_excl'] += (float) $order['total_discounts_tax_excl'];
                    $CustomerGroupTotals[$id_group]['total_paid_real'] += (float) $order['total_paid_real'];
                    $CustomerGroupTotals[$id_group]['total_products'] += (float) $order['total_products'];
                    $CustomerGroupTotals[$id_group]['total_shipping_tax_excl'] += (float) $order['total_shipping_tax_excl'];
                    $CustomerGroupTotals[$id_group]['total_wrapping_tax_excl'] += (float) $order['total_wrapping_tax_excl'];

                    if ((float) $order['total_discounts_tax_excl'] > (float) $order['total_products']) {
                        $CustomerGroupTotals[$id_group]['total_discount_shipping'] += ((float) $order['total_discounts_tax_excl'] - (float) $order['total_products']);
                        $CustomerGroupTotals[$id_group]['total_discount_products'] += (float) $order['total_products'];
                    } else {
                        $CustomerGroupTotals[$id_group]['total_discount_products'] += (float) $order['total_discounts_tax_excl'];
                    }

                    // get order vouchers
                    $vouchers = [];

                    // VOUCHERS: cart rules used for each order
                    $sql = '
                        SELECT
                            ocr.id_order_cart_rule,
                            ocr.id_cart_rule as ocr_id_cart_rule,
                            ocr.id_order as ocr_id_order,
                            cr.*
                        FROM ' . _DB_PREFIX_ . 'order_cart_rule ocr
                        LEFT JOIN ' . _DB_PREFIX_ . 'cart_rule cr ON (cr.id_cart_rule = ocr.id_cart_rule)
                        WHERE ocr.id_order = ' . $order['id_order'] . '
                        ORDER BY ocr.id_cart_rule ASC';
                    $order_vouchers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

                    $sql = 'SELECT cr.*
                        FROM `' . _DB_PREFIX_ . 'cart_rule` cr
                        WHERE id_customer = ' . (int) $order['id_customer'] . '
                        ORDER BY `quantity` ASC';
                    $available_vouchers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
                    if ($order_vouchers) {
                        foreach ($order_vouchers as $voucher) {
                            $vouchers[] = [
                                'id_cart_rule' => $voucher['id_cart_rule'],
                                'used' => 1,
                                'code' => $voucher['code'],
                                'active' => $voucher['active'],
                                'quantity' => 0, // set qty to 0 because this voucher was already used
                                'reduction_amount' => $voucher['reduction_amount'],
                                'description' => $voucher['description'],
                                'date_to' => $voucher['date_to'],
                            ];
                        }
                    }
                    if ($available_vouchers) {
                        foreach ($available_vouchers as $voucher) {
                            $vouchers[] = [
                                'id_cart_rule' => $voucher['id_cart_rule'],
                                'used' => 0,
                                'code' => $voucher['code'],
                                'active' => $voucher['active'],
                                'quantity' => $voucher['quantity'],
                                'reduction_amount' => $voucher['reduction_amount'],
                                'description' => $voucher['description'],
                                'date_to' => $voucher['date_to'],
                            ];
                        }
                    }
                } else {
                    $CustomerGroup = '';
                    $vouchers = null;
                }

                if ((int) $order['id_address_invoice'] !== (int) $order['id_address_delivery']) {
                    // get invoice_address
                    $delivery_address = new Address((int) $order['id_address_delivery']);
                    if (!Validate::isLoadedObject($delivery_address)) {
                        $delivery_address = false;
                    }
                }

                $export_address = ($delivery_address !== false ? $delivery_address : $invoice_address);
                if ($data['output_format'] == 'xml') {
                    $deliveryAddressXml =
                        '<DeliveryName>' . $export_address->company . '</DeliveryName>' . PHP_EOL
                        . '<DeliveryAddress>' . $export_address->address1 . (!empty($export_address->address2)
                            ? ' ' . $export_address->address2
                            : '')
                        . '</DeliveryAddress>' . PHP_EOL
                        . '<DeliveryPostcode>' . $export_address->postcode . '</DeliveryPostcode>' . PHP_EOL
                        . '<DeliveryCity>' . $export_address->city . '</DeliveryCity>' . PHP_EOL
                        . '<DeliveryProvince>' . ((int) $export_address->id_state > 0
                            ? MxbToolsExport::getStateIsoCodeById($export_address->id_state)
                            : '')
                            . '</DeliveryProvince>' . PHP_EOL
                        . '<DeliveryCountry>' . $export_address->country . '</DeliveryCountry>' . PHP_EOL;
                } else {
                    $deliveryAddressXml =
                        MxbToolsUtils::parseStringForCSV($export_address->company) . ';'
                        . MxbToolsUtils::parseStringForCSV($export_address->address1)
                        . (!empty($export_address->address2)
                            ? ' ' . MxbToolsUtils::parseStringForCSV($export_address->address2)
                            : '')
                        . ' - ' . MxbToolsUtils::parseStringForCSV($export_address->postcode)
                        . ' ' . MxbToolsUtils::parseStringForCSV($export_address->city)
                        . ' (' . ((int) $export_address->id_state > 0
                            ? MxbToolsUtils::parseStringForCSV(
                                MxbToolsExport::getStateIsoCodeById($export_address->id_state)
                            )
                            : '')
                        . ');'
                        . MxbToolsUtils::parseStringForCSV($export_address->country) . ';';
                }
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsExport::exportOrders()',
                    'invoice_address='.print_r($invoice_address, true),
                    $force_debug
                );
                */
                if (isset($invoice_address->pec)) {
                    if (Tools::strlen($invoice_address->pec) > 5) {
                        $customerEInvoiceDestCode = $data['output_format'] == 'xml'
                            ? '<CustomerEInvoiceDestCode>' . $invoice_address->pec
                                . '</CustomerEInvoiceDestCode>' . PHP_EOL
                            : ';';
                        $customerPec = $data['output_format'] == 'xml'
                            ? '<CustomerPec>' . $invoice_address->pec . '</CustomerPec>' . PHP_EOL
                            : MxbToolsUtils::parseStringForCSV($invoice_address->pec) . ';';
                    }
                }
                if (isset($invoice_address->sdi)) {
                    // overwrite with sdi if present
                    if (Tools::strlen($invoice_address->sdi) > 3) {
                        $customerEInvoiceDestCode = $data['output_format'] == 'xml'
                            ? '<CustomerEInvoiceDestCode>' . $invoice_address->sdi
                                . '</CustomerEInvoiceDestCode>' . PHP_EOL
                            : MxbToolsUtils::parseStringForCSV($invoice_address->sdi) . ';';
                    }
                }

                // danea params: https://www.danea.it/software/easyfatt/ecommerce/integrazione/ricezione-ordini/
                $xml_shipping_cost = $data['output_format'] == 'xml'
                    ? '<CostDescription>' . $module->l('Shipping cost') . '</CostDescription>' . PHP_EOL
                        // . '<CostAmount>' . $order['total_shipping_tax_excl'] . '</CostAmount>' . PHP_EOL
                        . '<CostAmount>'
                        . MxbToolsUtils::xmlPriceFormat((float) $order['total_shipping_tax_excl'])
                        . '</CostAmount>' . PHP_EOL
                        . '<CostVatCode>' . (int) $order['carrier_tax_rate'] . '</CostVatCode>' . PHP_EOL
                    : MxbToolsUtils::csvPriceFormat((float) $order['total_shipping_tax_excl']) . ';'
                        . (int) $order['carrier_tax_rate'] . ';';

                $customerInfoXML = $data['output_format'] == 'xml'
                    ? '<CustomerCode>' . $data['customer_prefix'] . $customer->id . '</CustomerCode>' . PHP_EOL
                        . '<CustomerName>' . $invoice_address->company . '</CustomerName>' . PHP_EOL
                        . '<CustomerAddress>' . $invoice_address->address1 . (!empty($invoice_address->address2)
                            ? ' ' . $invoice_address->address2
                            : '')
                        . '</CustomerAddress>' . PHP_EOL
                        . '<CustomerPostcode>' . $invoice_address->postcode . '</CustomerPostcode>' . PHP_EOL
                        . '<CustomerCity>' . $invoice_address->city . '</CustomerCity>' . PHP_EOL
                        . '<CustomerProvince>' . ((int) $invoice_address->id_state > 0
                            ? MxbToolsExport::getStateIsoCodeById($invoice_address->id_state)
                            : '')
                        . '</CustomerProvince>' . PHP_EOL
                        . '<CustomerCountry>' . $invoice_address->country . '</CustomerCountry>' . PHP_EOL
                        . '<CustomerVatCode>' . $invoice_address->vat_number . '</CustomerVatCode>' . PHP_EOL
                        . '<CustomerFiscalCode>' . $customerFiscalCode . '</CustomerFiscalCode>' . PHP_EOL
                        . '<CustomerTel>' . $invoice_address->phone . '</CustomerTel>' . PHP_EOL
                        . '<CustomerEmail>' . $customer->email . '</CustomerEmail>' . PHP_EOL
                    : $CustomerGroup . ';'
                        . MxbToolsUtils::parseStringForCSV($invoice_address->company) . ';'
                        . MxbToolsUtils::parseStringForCSV($invoice_address->address1)
                        . (!empty($invoice_address->address2)
                            ? ' ' . MxbToolsUtils::parseStringForCSV($invoice_address->address2)
                            : '')
                        . ' - ' . MxbToolsUtils::parseStringForCSV($invoice_address->postcode)
                        . ' ' . MxbToolsUtils::parseStringForCSV($invoice_address->city)
                        . ' (' . ((int) $invoice_address->id_state > 0
                            ? MxbToolsUtils::parseStringForCSV(
                                MxbToolsExport::getStateIsoCodeById($invoice_address->id_state)
                            ) : '')
                        . ');'
                        . MxbToolsUtils::parseStringForCSV($invoice_address->country) . ';'
                        . MxbToolsUtils::parseStringForCSV($invoice_address->vat_number) . ';'
                        . MxbToolsUtils::parseStringForCSV($customerFiscalCode) . ';'
                        . MxbToolsUtils::parseStringForCSV($invoice_address->phone) . ';'
                        . MxbToolsUtils::parseStringForCSV($customer->email) . ';';

                $paymentTypeXML = $data['output_format'] == 'xml'
                    ? '<PaymentName>' . $order['payment'] . '</PaymentName>' . PHP_EOL
                        . '<PaymentBank></PaymentBank>' . PHP_EOL
                    : MxbToolsUtils::parseStringForCSV($order['payment']) . ';';
            } elseif ($data['output_type'] == 'ordered_products') {
                $id_order = $order['id_order'];
                $order_data[$id_order] = [
                    'reference' => MxbToolsUtils::strReplacePrefix(
                        $order['reference'],
                        $data['order_prefix_from'],
                        $data['order_prefix_to']
                    ),
                    'customer_email' => $customer->email,
                    'order_total_tax_excl' => (float) $order['total_products']
                        + (float) $order['total_shipping_tax_excl']
                        + (float) $order['total_wrapping_tax_excl'],
                    'total_paid_tax_excl' => (float) $order['total_paid_tax_excl'],
                    'total_discounts_tax_excl' => (float) $order['total_discounts_tax_excl'],
                ];
            }

            $order_details = OrderDetail::getList($order['id_order']);
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsExport::exportOrders()',
                'order_detail='.print_r($order_details, true),
                true
            );
            */
            $productsXML = $data['output_format'] == 'xml' ? '' : [];

            foreach ($order_details as $order_detail) {
                $idx = (string) $order_detail['product_id'] . '-' . (string) $order_detail['product_attribute_id'];

                if ($data['output_type'] == 'orders') {
                    // get product taxes
                    $tax_rate = isset($taxes[(int) $order_detail['id_tax_rules_group']])
                        ? (string) $taxes[(int) $order_detail['id_tax_rules_group']]
                        : '0';

                    // prepare export string for XML "orders"
                    $rowXML = $data['output_format'] == 'xml'
                        ? '<Row>' . PHP_EOL
                            . '<Code>' . $order_detail['product_reference'] . '</Code>' . PHP_EOL
                            . '<Description>' . $order_detail['product_name'] . '</Description>' . PHP_EOL
                            . '<Qty>' . $order_detail['product_quantity'] . '</Qty>' . PHP_EOL
                            . '<Um>pz</Um>' . PHP_EOL
                            . '<Price>'
                            . MxbToolsUtils::xmlPriceFormat((float) $order_detail['unit_price_tax_excl'])
                            . '</Price>' . PHP_EOL
                            . ($data['export_product_discount']
                                ? '<Discounts>' . $order_detail['group_reduction'] . '%</Discounts>' . PHP_EOL
                                : '<Discounts>0%</Discounts>')  // da vedere xchè manca product discount
                            . '<VatCode Perc="' . (int) $tax_rate
                            . '" Class="Imponibile" Description="Aliquota ' . (int) $tax_rate . '%">'
                            . (int) $tax_rate . '</VatCode>' . PHP_EOL
                            . '<Notes></Notes>' . PHP_EOL
                            . '</Row>' . PHP_EOL
                        : $order_detail['product_reference'] . ';'
                            . MxbToolsUtils::parseStringForCSV($order_detail['product_name']) . ';'
                            . $order_detail['product_quantity'] . ';'
                            . 'pz;'
                            . MxbToolsUtils::csvPriceFormat((float) $order_detail['unit_price_tax_excl']) . ';'
                            . (int) $order_detail['group_reduction'] . ';' // da vedere xchè manca product discount
                            . (int) $tax_rate . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $order_detail['total_price_tax_excl']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $order_detail['total_price_tax_incl']) . ';';

                    if ($data['output_format'] == 'xml') {
                        $productsXML .= $rowXML;
                    } else {
                        $productsXML[] = $rowXML;
                    }

                    if ((int) $order_detail['id_order_invoice'] > 0) {
                        if (!isset($invoices[(int) $order_detail['id_order_invoice']])) {
                            $invoices[(int) $order_detail['id_order_invoice']] = [];
                            $invoices[(int) $order_detail['id_order_invoice']]['products'] = '';
                        }
                        $invoices[(int) $order_detail['id_order_invoice']]['products'] .= $rowXML;
                        $invoices[(int) $order_detail['id_order_invoice']]['order_reference'] = $order_reference;
                        $invoices[(int) $order_detail['id_order_invoice']]['order_date'] =
                            MxbToolsUtils::removeTimeFromDatetime($order['date_add']);
                        $invoices[(int) $order_detail['id_order_invoice']]['id_shop'] = $order['id_shop'];
                        $invoices[(int) $order_detail['id_order_invoice']]['customerInfoXML'] = $customerInfoXML;
                        $invoices[(int) $order_detail['id_order_invoice']]['customerPec'] = $customerPec;
                        $invoices[(int) $order_detail['id_order_invoice']]['customerEInvoiceDestCode'] = $customerEInvoiceDestCode;
                        $invoices[(int) $order_detail['id_order_invoice']]['deliveryAddressXml'] = $deliveryAddressXml;
                    }
                } elseif ($data['output_type'] == 'ordered_products') {
                    // prepare array for csv "ordered products"

                    if (isset($products[$idx])) {
                        $products[$idx]['quantity'] += (int) $order_detail['product_quantity'];
                        $products[$idx]['total_price_tax_excl'] +=
                            (float) $order_detail['unit_price_tax_excl'] * (int) $order_detail['product_quantity'];
                        $products[$idx]['total_price_tax_incl'] +=
                            (float) $order_detail['unit_price_tax_incl'] * (int) $order_detail['product_quantity'];

                        // set quantity per order
                        if (isset($products[$idx]['order_quantity'][$order['id_order']])) {
                            $products[$idx]['order_quantity'][$order['id_order']] +=
                                (int) $order_detail['product_quantity'];
                        } else {
                            $products[$idx]['order_quantity'][$order['id_order']] =
                                (int) $order_detail['product_quantity'];
                        }
                        // save product price for every order to consider a price change
                        $products[$idx]['order_product_price'][$order['id_order']]['price_tax_excl'] = (float) $order_detail['unit_price_tax_excl'];
                        $products[$idx]['order_product_price'][$order['id_order']]['price_tax_incl'] = (float) $order_detail['unit_price_tax_incl'];
                        if ($products[$idx]['price_min_tax_excl'] > (float) $order_detail['unit_price_tax_excl']) {
                            $products[$idx]['price_min_tax_excl'] = (float) $order_detail['unit_price_tax_excl'];
                        }
                        if ($products[$idx]['price_max_tax_excl'] < (float) $order_detail['unit_price_tax_excl']) {
                            $products[$idx]['price_max_tax_excl'] = (float) $order_detail['unit_price_tax_excl'];
                        }
                    } else {
                        // try to split name from color and size tags
                        // name format: product name - Size-tag : SIZE- Color-tag : COLOR
                        // sample: Giacca imbottita - Taglia : M- Colore : Blu navy
                        $name = $order_detail['product_name'];
                        $size = null;
                        $color = null;
                        $final_name = null;

                        // sample resulting array:
                        //  [0] => "Giacca imbottita "
                        //  [1] => "Taglia : M"
                        //  [2] => "Colore : Blu navy"

                        // > 1.7.8.x sample resulting array:
                        // string = "Felpa zip intera (Taglia: M - Colore: Nero)"
                        //  [0] => "Giacca imbottita "
                        //  [1] => "(Taglia: M"
                        //  [2] => "Colore: Blu navy)"

                        if (strpos($name, '(' . $data['size_tag']) !== false) {
                            $t1 = explode('(' . $data['size_tag'], $name);
                            $t2 = explode('- ', $data['size_tag'] . $t1[1]);
                            $nameArray = [$t1[0]];
                            foreach ($t2 as $t) {
                                $nameArray[] = $t;
                            }
                        } elseif (strpos($name, '(' . $data['color_tag']) !== false) {
                            $t1 = explode('(' . $data['color_tag'], $name);
                            $t2 = explode('- ', $data['color_tag'] . $t1[1]);
                            $nameArray = [$t1[0]];
                            foreach ($t2 as $t) {
                                $nameArray[] = $t;
                            }
                        } else {
                            $nameArray = explode('- ', $name);
                        }

                        $final_name = '';
                        foreach ($nameArray as $checkString) {
                            $checkString = str_replace(' :', ':', $checkString);
                            if (strpos($checkString, $data['size_tag']) !== false) {
                                // $size = $checkString;
                                // $size = explode(' : ', $checkString)[1];
                                $t = explode(':', $checkString);
                                if (isset($t[1])) {
                                    $size = trim($t[1]);
                                } else {
                                    $size = trim($t[0]);
                                }
                                $size = str_replace(')', '', $size);
                            } elseif (strpos($checkString, $data['color_tag']) !== false) {
                                // $color = $checkString;
                                // $color = explode(' : ', $checkString)[1];
                                $t = explode(':', $checkString);
                                if (isset($t[1])) {
                                    $color = trim($t[1]);
                                } else {
                                    $color = trim($t[0]);
                                }
                                $color = str_replace(')', '', $color);
                            } else {
                                if (!$final_name) {
                                    $final_name = $checkString;  // this is the name of the product (array[0])
                                } else {
                                    // insert separator and then another unknown attribute
                                    $final_name .= '- ' . $checkString;
                                }
                            }
                        }

                        // get category
                        $record_categories = '-';
                        $categories = Product::getProductCategoriesFull($order_detail['product_id']);
                        if (count($categories)) {
                            $arr = [];
                            foreach ($categories as $category) {
                                if ((int) $category['id_category'] !== 2) {
                                    $arr[] = $category['name'];
                                }
                            }
                            $record_categories = implode(', ', $arr);
                        }

                        $products[$idx] = [
                            'category' => $record_categories,
                            'full_name' => $order_detail['product_name'],
                            'name' => $final_name,
                            'color' => $color !== null ? $color : '',
                            'size' => $size !== null ? $size : '',
                            'quantity' => (int) $order_detail['product_quantity'],
                            'reference' => $order_detail['product_reference'],
                            'id_customization' => (int) $order_detail['id_customization'],
                            'product_ean13' => (int) $order_detail['product_ean13'],
                            'price_tax_excl' => (float) $order_detail['unit_price_tax_excl'],
                            'price_tax_incl' => (float) $order_detail['unit_price_tax_incl'],
                            'price_min_tax_excl' => (float) $order_detail['unit_price_tax_excl'],
                            'price_max_tax_excl' => (float) $order_detail['unit_price_tax_excl'],
                            'total_price_tax_excl' => (float) $order_detail['total_price_tax_excl'],
                            'total_price_tax_incl' => (float) $order_detail['total_price_tax_incl'],
                            'purchase_supplier_price' => (float) $order_detail['purchase_supplier_price'],
                            'original_product_price' => (float) $order_detail['original_product_price'],
                            'original_wholesale_price' => (float) $order_detail['original_wholesale_price'],
                            'order_quantity' => [$order['id_order'] => (int) $order_detail['product_quantity']],
                            'order_product_price' => [
                                $order['id_order'] => [
                                    'price_tax_excl' => (float) $order_detail['unit_price_tax_excl'],
                                    'price_tax_incl' => (float) $order_detail['unit_price_tax_incl'],
                                ],
                            ],
                        ];
                    }
                }
            } // foreach order_detail

            if ($data['output_type'] == 'orders') {
                if ((float) $order['total_discounts_tax_excl'] > 0) {
                    // discount: prepare export string for XML 'orders'
                    if ($data['output_format'] == 'xml') {
                        $productsXML .= '<Row>' . PHP_EOL
                            . '<Code>' . $module->l('DIS') . '</Code>' . PHP_EOL
                            . '<Description>' . $module->l('Your discount') . '</Description>' . PHP_EOL
                            . '<Qty>1</Qty>' . PHP_EOL
                            . '<Um>pz</Um>' . PHP_EOL
                            . '<Price>-' . MxbToolsUtils::xmlPriceFormat((float) $order['total_discounts_tax_excl'])
                            . '</Price>' . PHP_EOL
                            // . ($data['export_product_discount'] ? '<Discounts>0%</Discounts>' . PHP_EOL : '')
                            . '<Discounts>0%</Discounts>'
                            . '<VatCode Perc="22" Class="Imponibile" Description="Aliquota 22%">22</VatCode>' . PHP_EOL
                            . '<Notes></Notes>' . PHP_EOL
                            . '</Row>' . PHP_EOL;
                    } else {
                        $tmp = $module->l('DIS') . ';'
                            . $module->l('Order discount') . ';'
                            . '1;'
                            . ';'
                            . '0;'
                            . '0;'
                            . '22;'
                            . '-' . MxbToolsUtils::csvPriceFormat((float) $order['total_discounts_tax_excl']) . ';'
                            . '-' . MxbToolsUtils::csvPriceFormat((float) $order['total_discounts_tax_excl'] * 1.22) . ';';

                        // get used vouchers
                        if ($vouchers) {
                            $total_discounts_tax_excl = (float) $order['total_discounts_tax_excl'];
                            foreach ($vouchers as $voucher) {
                                $reduction_amount = (float) $voucher['reduction_amount'];
                                $tmp .= ((bool) $voucher['used'] ? '[used]' : '[avail]') . ' '
                                    . MxbToolsExport::getVoucherCsvDescription($voucher, $reduction_amount) . ';'
                                    . MxbToolsUtils::csvPriceFormat($reduction_amount) . ';';
                                if ((bool) $voucher['used'] && $total_discounts_tax_excl > 0) {
                                    $remain = round($reduction_amount - $total_discounts_tax_excl, 3);
                                    $spent = $reduction_amount - $remain;
                                    if ($spent < 0) {
                                        $spent = 0;
                                    } elseif ($spent > $reduction_amount) {
                                        $spent = $reduction_amount;
                                    }
                                    $miss = ($remain < 0 ? abs($remain) : 0);
                                    if ($remain < 0) {
                                        $remain = 0;
                                    }
                                    $tmp .= MxbToolsUtils::csvPriceFormat($spent) . ';'
                                        . MxbToolsUtils::csvPriceFormat($remain) . ';'
                                        . MxbToolsUtils::csvPriceFormat($miss) . ';';
                                    $total_discounts_tax_excl =
                                        round($total_discounts_tax_excl - $reduction_amount, 3);
                                } else {
                                    $tmp .= ';;;';
                                }
                            }
                            // $tmp = rtrim($tmp, ", ") . ';';
                        }
                        $productsXML[] = $tmp;
                    }
                }
                /*
                elseif ($data['output_type'] == 'csv') {
                    // zero discount columns for css version
                    $productsXML[] = $module->l('DIS') . ';'
                            . $module->l('Order discount') . ';'
                            . '0;'
                            . 'pz' . ';'
                            . '0;'
                            . '0%;'
                            . '22;';
                }
                */

                // close products list tag
                // $productsXML .= '</Rows>' . PHP_EOL;

                // prepare export string for XML "orders"
                // NOTA: DA VEDERE COME FARE A CAPIRE SE E' STATO PAGATO !!!!
                $paymentsXML = '<Payments>' . PHP_EOL
                    . '<Payment>' . PHP_EOL
                    . '<Advance>false</Advance>' . PHP_EOL
                    . '<Date>' . MxbToolsUtils::removeTimeFromDatetime($order['invoice_date']) . '</Date>' . PHP_EOL
                    . '<Amount>' . MxbToolsUtils::xmlPriceFormat((float) $order['total_paid_tax_incl'])
                    . '</Amount>' . PHP_EOL
                    . '<Paid>' . ((int) $order['valid'] === 0 ? 'false' : 'true') . '</Paid>' . PHP_EOL
                    . '</Payment>' . PHP_EOL
                    . '</Payments>' . PHP_EOL;

                // ORDER
                if ($data['output_format'] == 'xml') {
                    $orderDocumentXML .= '<Document>' . PHP_EOL
                        . '<DocumentType>C</DocumentType>' . PHP_EOL
                        . $customerInfoXML
                        . $customerPec
                        . $customerEInvoiceDestCode
                        . $deliveryAddressXml
                        // date format: 2010-03-23
                        . '<Date>' . MxbToolsUtils::removeTimeFromDatetime($order['date_add']) . '</Date>' . PHP_EOL
                        . '<Number>' . $order_reference . '</Number>' . PHP_EOL
                        . (!empty($data['numbering'])
                            ? '<Numbering>' . $data['numbering'] . '</Numbering>' . PHP_EOL
                            : '')
                        . '<Total>' . MxbToolsUtils::xmlPriceFormat((float) $order['total_paid_tax_excl'])
                        . '</Total>' . PHP_EOL
                        . $paymentTypeXML
                        . '<InternalComment></InternalComment>' . PHP_EOL
                        . '<CustomField2></CustomField2>' . PHP_EOL
                        . '<SalesAgent></SalesAgent>' . PHP_EOL
                        . $xml_shipping_cost
                        . '<Rows>' . PHP_EOL
                        . $productsXML
                        . '</Rows>' . PHP_EOL
                        . $paymentsXML
                        . '</Document>' . PHP_EOL;
                } else { // CSV
                    $orderStatus = new OrderState((int) $order['current_state']);
                    if (!Validate::isLoadedObject($orderStatus)) {
                        $orderStateName = ';';
                    } else {
                        $orderStateName = $orderStatus->name[$id_lang] . ';';
                    }

                    $orderRowCSV = MxbToolsUtils::removeTimeFromDatetime($order['date_add']) . ';' // format: 2010-03-23
                        . $order_reference . (!empty($data['numbering']) ? $data['numbering'] : '') . ';'
                        . $orderStateName
                        . MxbToolsUtils::csvPriceFormat(
                            (float) $order['total_products']
                                + (float) $order['total_shipping_tax_excl']
                                + (float) $order['total_wrapping_tax_excl']
                        )
                        . ';'
                        . MxbToolsUtils::csvPriceFormat((float) $order['total_paid_tax_excl']) . ';'
                        . MxbToolsUtils::csvPriceFormat((float) $order['total_paid_tax_incl']) . ';'
                        . $xml_shipping_cost
                        . $paymentTypeXML
                        . $customerInfoXML
                        . $customerPec
                        . $customerEInvoiceDestCode
                        . $deliveryAddressXml;

                    // create a row for every product in order
                    foreach ($productsXML as $prodCSV) {
                        $orderDocumentXML[] = $orderRowCSV . $prodCSV;
                    }
                }
            }

            // change orders state if needed
            // https://www.prestashop.com/forums/topic/221403-changing-order-status-vith-php/
            if ($data['output_format'] == 'xml' && $data['id_order_state_next'] > 0) {
                // $objOrder = new Order(1); //order with id=1
                $history = new OrderHistory();
                $history->id_order = (int) $order['id_order'];
                $history->changeIdOrderState($data['id_order_state_next'], (int) $order['id_order']); // order status=3
                $history->add();
            }
        } // foreach order

        if ($data['output_type'] == 'orders') {
            // DELIVERY SLIP
            $deliverySlipDocumentXML = '';

            // INVOICE
            $invoiceDocumentXML = '';
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsExport::exportOrders()',
                'export_invoice='.(int) $data['export_invoice']
                . ', export_delivery_slip='.(int) $data['export_delivery_slip']
                . ', invoices='.print_r($invoices, true),
                true
            );
            */
            if (((bool) $data['export_invoice'] || (bool) $data['export_delivery_slip']) && count($invoices)) {
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsExport::exportOrders()',
                    'export invoice/del.slip: invoices count='.count($invoices),
                    $force_debug
                );
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsExport::exportOrders()',
                    'invoices='.print_r($invoices, true),
                    true
                );
                */
                foreach ($invoices as $key => $productsRowXML) {
                    $orderInvoice = new OrderInvoice($key);
                    $invoice_reference = $orderInvoice->getInvoiceNumberFormatted($id_lang);
                    $invoice_reference_full = $invoice_reference;

                    // change prefix
                    $invoice_reference = MxbToolsUtils::strReplacePrefix(
                        $invoice_reference,
                        $data['order_prefix_from'],
                        $data['order_prefix_to']
                    );

                    // force reference to numeric value
                    if ((bool) $data['force_order_numeric']) {
                        $invoice_reference = MxbToolsUtils::extractNumber($invoice_reference);
                    }
                    $invoiceShippingCost =
                        '<CostDescription>' . $module->l('Shipping cost') . '</CostDescription>' . PHP_EOL
                        . '<CostAmount>' . (float) $orderInvoice->total_shipping_tax_excl . '</CostAmount>' . PHP_EOL
                        . '<CostVatCode>' . (int) $order['carrier_tax_rate'] . '</CostVatCode>' . PHP_EOL;

                    $invoicePaymentsXML =
                        '<Payments>' . PHP_EOL
                        . '<Payment>' . PHP_EOL
                        . '<Advance>false</Advance>' . PHP_EOL
                        . '<Date>' . MxbToolsUtils::removeTimeFromDatetime($orderInvoice->date_add)
                        . '</Date>' . PHP_EOL
                        . '<Amount>' . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_paid_tax_incl)
                        . '</Amount>' . PHP_EOL
                        . '<Paid>' . ((float) $orderInvoice->getTotalPaid() > 0 ? 'true' : 'false') . '</Paid>' . PHP_EOL
                        . '</Payment>' . PHP_EOL
                        . '</Payments>' . PHP_EOL;

                    $invoiceDiscount = '';
                    if ((float) $orderInvoice->total_discount_tax_excl > 0) {
                        // discount: prepare export string for XML "orders"
                        $invoiceDiscount =
                            '<Row>' . PHP_EOL
                            . '<Code>' . $module->l('DIS') . '</Code>' . PHP_EOL
                            . '<Description>' . $module->l('Your discount') . '</Description>' . PHP_EOL
                            . '<Qty>1</Qty>' . PHP_EOL
                            . '<Um>pz</Um>' . PHP_EOL
                            . '<Price>-' . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_discount_tax_excl)
                            . '</Price>' . PHP_EOL
                            . '<Discounts>0%</Discounts>' . PHP_EOL  // da vedere perchè manca il product discount...
                            . '<VatCode Perc="22" Class="Imponibile" Description="Aliquota 22%">22</VatCode>' . PHP_EOL
                            . '<Notes></Notes>' . PHP_EOL
                            . '</Row>' . PHP_EOL;
                    }

                    $delivery_slip_reference = '';
                    $delivery_slip_reference_full = '';
                    if ((int) $orderInvoice->delivery_number > 0) {
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsExport::exportOrders()',
                            'generateCustomReference='.$generateCustomReference,
                            $force_debug
                        );
                        */
                        if ($generateCustomReference) {
                            $delivery_slip_reference = $generateCustomReference(
                                'delivery-slip',
                                (int) $orderInvoice->delivery_number
                            );
                        } else {
                            $delivery_slip_reference = Configuration::get(
                                'PS_DELIVERY_PREFIX',
                                $id_lang,
                                null,
                                $productsRowXML['id_shop']
                            )
                            . $orderInvoice->delivery_number . '.pdf';
                        }
                        $delivery_slip_reference_full = $delivery_slip_reference;

                        // change prefix
                        $delivery_slip_reference = MxbToolsUtils::strReplacePrefix(
                            $delivery_slip_reference,
                            $data['order_prefix_from'],
                            $data['order_prefix_to']
                        );

                        // force reference to number
                        if ((bool) $data['force_order_numeric']) {
                            $delivery_slip_reference = MxbToolsUtils::extractNumber($delivery_slip_reference);
                        }

                        if ((bool) $data['export_delivery_slip']) {
                            $deliverySlipDocumentXML .= $data['output_format'] == 'xml'
                                ? '<Document>' . PHP_EOL
                                    . '<DocumentType>D</DocumentType>' . PHP_EOL
                                    . $productsRowXML['customerInfoXML']
                                    . $productsRowXML['customerPec']
                                    . $productsRowXML['customerEInvoiceDestCode']
                                    . $productsRowXML['deliveryAddressXml']
                                    // . $customerInfoXML
                                    // . $customerPec
                                    // . $customerEInvoiceDestCode
                                    // . $deliveryAddressXml
                                    . '<Date>'
                                    // format: 2010-03-23:
                                    . MxbToolsUtils::removeTimeFromDatetime($orderInvoice->delivery_date)
                                    . '</Date>' . PHP_EOL
                                    . '<Number>' . $delivery_slip_reference . '</Number>' . PHP_EOL
                                    . (!empty($data['numbering'])
                                        ? '<Numbering>' . $data['numbering'] . '</Numbering>' . PHP_EOL
                                        : '')
                                    . '<TotalWithoutTax>'
                                    . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_paid_tax_excl)
                                    . '</TotalWithoutTax>' . PHP_EOL
                                    . '<Total>'
                                    . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_paid_tax_incl)
                                    . '</Total>' . PHP_EOL
                                    . '<PricesIncludeVat>false</PricesIncludeVat>'
                                    . $paymentTypeXML
                                    // . '<InternalComment></InternalComment>' . PHP_EOL
                                    // . '<CustomField2></CustomField2>' . PHP_EOL
                                    // . '<SalesAgent></SalesAgent>' . PHP_EOL
                                    . $invoiceShippingCost
                                    // order / delivery slip reference
                                    . '<Rows>' . PHP_EOL
                                    . '<Row>' . PHP_EOL
                                    . '<Code/>' . PHP_EOL
                                    . '<Description>' . $module->l('Ref.') . ' ' . $module->l('order')
                                    . ' ' . $productsRowXML['order_reference'] . ' ' . $module->l('of')
                                    . ' ' . $productsRowXML['order_date']
                                    . (!empty($invoice_reference_full)
                                        ? ' / ' . $module->l('inv') . ' ' . $invoice_reference_full
                                            . ' ' . $module->l('of') . ' '
                                            . MxbToolsUtils::removeTimeFromDatetime($orderInvoice->date_add)
                                        : '')
                                    . '</Description>' . PHP_EOL // DDT 49 del 18/2/09:</Description>' . PHP_EOL
                                    . '<Qty/>' . PHP_EOL
                                    . '<Um/>' . PHP_EOL
                                    . '<Price/>' . PHP_EOL
                                    . '<Discounts/>' . PHP_EOL
                                    . '<VatCode/>' . PHP_EOL
                                    . '<Total/>' . PHP_EOL
                                    . '<Stock>false</Stock>' . PHP_EOL
                                    . '</Row>' . PHP_EOL
                                    . $productsRowXML['products']
                                    . $invoiceDiscount  // as product row
                                    . '</Rows>' . PHP_EOL
                                    . $invoicePaymentsXML
                                    . '</Document>' . PHP_EOL
                                : '';
                            /*
                                unused
                                : $delivery_slip_reference
                                  . (!empty($data['numbering']) ? $data['numbering'] : '')
                                  . ';';
                            */
                        }
                    }

                    if ((bool) $data['export_invoice']) {
                        $invoiceDocumentXML .= $data['output_format'] == 'xml'
                            ? '<Document>' . PHP_EOL
                                . '<DocumentType>I</DocumentType>' . PHP_EOL
                                . $productsRowXML['customerInfoXML']
                                . $productsRowXML['customerPec']
                                . $productsRowXML['customerEInvoiceDestCode']
                                . $productsRowXML['deliveryAddressXml']
                                // . $customerInfoXML
                                // . $customerPec
                                // . $customerEInvoiceDestCode
                                // . $deliveryAddressXml
                                . '<Date>' . MxbToolsUtils::removeTimeFromDatetime($orderInvoice->date_add)
                                . '</Date>' . PHP_EOL // format: 2010-03-23
                                . '<Number>' . $invoice_reference . '</Number>' . PHP_EOL
                                . (!empty($data['numbering'])
                                    ? '<Numbering>' . $data['numbering'] . '</Numbering>' . PHP_EOL
                                    : '')
                                . '<TotalWithoutTax>'
                                . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_paid_tax_excl)
                                . '</TotalWithoutTax>' . PHP_EOL
                                . '<Total>'
                                . MxbToolsUtils::xmlPriceFormat((float) $orderInvoice->total_paid_tax_incl)
                                . '</Total>' . PHP_EOL
                                . '<PricesIncludeVat>false</PricesIncludeVat>'
                                . $paymentTypeXML
                                // . '<InternalComment></InternalComment>' . PHP_EOL
                                // . '<CustomField2></CustomField2>' . PHP_EOL
                                // . '<SalesAgent></SalesAgent>' . PHP_EOL
                                . $invoiceShippingCost
                                // order / delivery slip reference
                                . '<Rows>' . PHP_EOL
                                . '<Row>' . PHP_EOL
                                . '<Code/>' . PHP_EOL
                                . '<Description>' . $module->l('Ref.') . ' ' . $module->l('order')
                                . ' ' . $productsRowXML['order_reference'] . ' ' . $module->l('of')
                                . ' ' . $productsRowXML['order_date']
                                . (!empty($delivery_slip_reference_full)
                                    ? ' / ' . $module->l('DS') . ' ' . $delivery_slip_reference_full
                                        . ' ' . $module->l('of')
                                        . ' ' . MxbToolsUtils::removeTimeFromDatetime($orderInvoice->delivery_date)
                                    : '')
                                . '</Description>' . PHP_EOL // DDT 49 del 18/2/09:</Description>' . PHP_EOL
                                . '<Qty/>' . PHP_EOL
                                . '<Um/>' . PHP_EOL
                                . '<Price/>' . PHP_EOL
                                . '<Discounts/>' . PHP_EOL
                                . '<VatCode/>' . PHP_EOL
                                . '<Total/>' . PHP_EOL
                                . '<Stock>false</Stock>' . PHP_EOL
                                . '</Row>' . PHP_EOL
                                . $productsRowXML['products']
                                . $invoiceDiscount  // as product row
                                . '</Rows>' . PHP_EOL
                                . $invoicePaymentsXML
                                . '</Document>' . PHP_EOL
                              : '';
                        /*
                            unused
                              : $invoice_reference
                                . (!empty($data['numbering']) ? $data['numbering'] : '')
                                . ';';
                        */
                    }
                } // foreach invoices
            }

            if ($data['output_format'] == 'xml') {
                // prepare export string for XML "orders"
                $xml_string = '<?xml version="1.0" encoding="UTF-8"?>' . PHP_EOL
                    . '<EasyfattDocuments AppVersion="2" Creator="Danea Soft" CreatorUrl="www.danea.it">' . PHP_EOL
                    . '<Documents>'
                    . $orderDocumentXML
                    . $invoiceDocumentXML
                    . $deliverySlipDocumentXML
                    . '</Documents>' . PHP_EOL
                    . '</EasyfattDocuments>';
            } else {
                // set headings
                $xml_string = $module->l('Date') . ';'
                    . $module->l('Reference') . ';'
                    . $module->l('Order state') . ';'
                    . $module->l('Total order tax excl') . ';'
                    . $module->l('Total paid tax excl') . ';'
                    . $module->l('Total paid tax incl') . ';'
                    . $module->l('Shipping tax excl') . ';'
                    . $module->l('Shipping tax %') . ';'
                    . $module->l('Payment') . ';'
                    . $module->l('User group') . ';'
                    . $module->l('Customer company') . ';'
                    . $module->l('Address') . ';'
                    . $module->l('Country') . ';'
                    . $module->l('VAT number') . ';'
                    . $module->l('Siret') . ';'
                    . $module->l('Phone') . ';'
                    . $module->l('E-mail') . ';'
                    . $module->l('Pec') . ';'
                    . $module->l('Sdi') . ';'
                    . $module->l('Delivery to') . ';'
                    . $module->l('Address') . ';'
                    . $module->l('Country') . ';'
                    . $module->l('Product reference') . ';'
                    . $module->l('Product name') . ';'
                    . $module->l('Quantity') . ';'
                    . $module->l('Unit') . ';'
                    . $module->l('Price tax excl') . ';'
                    . $module->l('Discount %') . ';'
                    . $module->l('Tax %') . ';'
                    . $module->l('Total tax excl') . ';'
                    . $module->l('Total tax incl') . ';'
                    . $module->l('Vouchers') . ' 1;'
                    . $module->l('amount') . ';'
                    . $module->l('Amount spent') . ';'
                    . $module->l('Remaining amount') . ';'
                    . $module->l('Missing amount') . ';'
                    . $module->l('Vouchers') . ' 2;'
                    . $module->l('amount') . ';'
                    . $module->l('Amount spent') . ';'
                    . $module->l('Remaining amount') . ';'
                    . $module->l('Missing amount') . ';'
                    . $module->l('Vouchers') . ' 3;'
                    . $module->l('amount') . ';'
                    . $module->l('Amount spent') . ';'
                    . $module->l('Remaining amount') . ';'
                    . $module->l('Missing amount') . ';'
                    // . ($invoiceDocumentXML != '' ? $module->l('Invoice') . ';' : '')
                    // . ($deliverySlipDocumentXML != '' ? $module->l('Delivery slip') . ';' : '')
                    . PHP_EOL;

                foreach ($orderDocumentXML as $ordCSV) {
                    $xml_string .= $ordCSV
                        . $invoiceDocumentXML
                        . $deliverySlipDocumentXML
                        . PHP_EOL;
                }
            }
        }

        if ($data['output_type'] == 'ordered_products') {
            // $number_of_orders = count($orders);
            $order_quantity_heading = '';
            // $order_details_heading = '';
            $heading_done = false;
            $records = '';
            $real_cnt = 0;
            foreach ($products as $product) {
                $order_quantity_cols = '';
                // $order_details_cols = '';
                // search for quantity per order columns
                foreach ($orders as $order) {
                    // set some usefull order infos
                    $order_details_heading = '[' . $order_data[$order['id_order']]['customer_email']
                        . ' - tot:'
                        . MxbToolsUtils::csvPriceFormat((float) $order_data[$order['id_order']]['order_total_tax_excl'])
                        . ', paid:'
                        . MxbToolsUtils::csvPriceFormat((float) $order_data[$order['id_order']]['total_paid_tax_excl'])
                        . ', disc:'
                        . MxbToolsUtils::csvPriceFormat(
                            (float) $order_data[$order['id_order']]['total_discounts_tax_excl']
                        )
                        . ']';

                    if (!$heading_done) {
                        $order_ref = MxbToolsUtils::strReplacePrefix(
                            $order['reference'],
                            $data['order_prefix_from'],
                            $data['order_prefix_to']
                        );
                        if ((bool) $data['force_order_numeric']) {
                            $order_ref = MxbToolsUtils::extractNumber($order_ref);
                        }
                        $order_quantity_heading .= $order['id_order']
                            . ') ' . $order_ref
                            . ' ' . $order_details_heading . ';';
                    }
                    if (isset($product['order_quantity'][$order['id_order']])) {
                        $order_quantity_cols .= $product['order_quantity'][$order['id_order']] . ';';
                    } else {
                        $order_quantity_cols .= ';';
                    }
                }

                $heading_done = true; // first time we have set headers for quantity columns

                ++$real_cnt;
                $records .=
                    $product['reference'] . ';'
                    . $product['full_name'] . ';'
                    . $product['category'] . ';'
                    . $product['name'] . ';'
                    . $product['color'] . ';'
                    . $product['size'] . ';'
                    . $product['quantity'] . ';'
                    // . MxbToolsUtils::csvPriceFormat((float) $product['price_tax_excl']) . ';'
                    . MxbToolsUtils::csvPriceFormat((float) $product['price_min_tax_excl']) . ';'
                    . MxbToolsUtils::csvPriceFormat((float) $product['price_max_tax_excl']) . ';'
                    . MxbToolsUtils::csvPriceFormat(round((float) $product['total_price_tax_excl'] / $product['quantity'], 2)) . ';'
                    . MxbToolsUtils::csvPriceFormat((float) $product['total_price_tax_excl']) . ';'
                    . $order_quantity_cols
                    . PHP_EOL;
            }
        }

        // create file: only once I got quantity columns
        if ($folder === null) {
            // create folder and CSV file and write part of headings
            $folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
            if (!file_exists($folder)) {
                mkdir($folder, 0777, true);
            }
        }

        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        $shop_name = MxbToolsExport::getShopName();
        if ($data['output_type'] == 'ordered_products') {
            // set filename
            if (!isset($data['filename'])) {
                $return_values['filename'] = MxbToolsUtils::normalizeFilename(
                    str_replace(
                        '/',
                        '-',
                        $shop_name
                        . '_' . $module->l('ordered products list')
                        . '_[' . $module->l('from') . '_'
                        . $data['date_from']
                        . '_' . $module->l('to') . '_'
                        . $data['date_to'] . ']'
                        . (($data['id_from'] + $data['id_to']) > 0
                            ? '[' . $module->l('order id')
                                . '_' . $module->l('from') . '_' . (string) $data['id_from']
                                . '_' . $module->l('to') . '_' . (string) $data['id_to'] . ']'
                            : '')
                    )
                ) . '.csv';
            } else {
                $return_values['filename'] = MxbToolsUtils::normalizeFilename($data['filename']);
            }
            $file_csv = $folder . '/' . $return_values['filename'];
            // write CSV file
            $myfile = fopen($file_csv, 'wb') or exit('Unable to open file!');
            // utf8 blob for JS: "\uFEFF"
            // utf8 blob for PHP: "\xEF\xBB\xBF"
            // open file binary for UTF8: "wb"

            // write file for the first time: add heading
            $records = $module->l('Reference') . ';'
                . $module->l('Full name') . ';'
                . $module->l('Category') . ';'
                . $module->l('name') . ';'
                . $data['color_tag'] . ';'
                . $data['size_tag'] . ';'
                . $module->l('quantity') . ';'
                . $module->l('Min unit price') . ';'
                . $module->l('Max unit price') . ';'
                . $module->l('Average unit price') . ';'
                . $module->l('Total price') . ';'
                . $order_quantity_heading
                . PHP_EOL
                . $records;
            fwrite($myfile, "\xEF\xBB\xBF" . $records);
            fclose($myfile);
            $return_values['content'] = "\xEF\xBB\xBF" . $records;

            // $records = '';
            $return_values['download_url'] = $baseUrl . '/modules/mxb_tools/reports/'
                . $return_values['filename'];
            $return_values['download_link'] = '<a href="' . $return_values['download_url'] . '" target="_new">'
                . $return_values['filename'] . '</a>';
        } elseif ($data['output_type'] == 'orders') {
            if ($data['output_format'] == 'xml') {
                // write orders danea xml file
                $records = MxbToolsUtils::parseStringForXML($xml_string);
                $return_values['filename'] = 'danea_orders.xml';
            } else {
                // write orders CSV file
                $records = $xml_string;
                if (!isset($data['filename'])) {
                    $return_values['filename'] = MxbToolsUtils::normalizeFilename(
                        str_replace(
                            '/',
                            '-',
                            $shop_name
                                . '_' . $module->l('order list')
                                . '_[' . $module->l('from') . '_'
                                . $data['date_from']
                                . '_' . $module->l('to') . '_'
                                . $data['date_to'] . ']'
                                . (($data['id_from'] + $data['id_to']) > 0
                                    ? '[' . $module->l('order id')
                                        . '_' . $module->l('from') . '_' . (string) $data['id_from']
                                        . '_' . $module->l('to') . '_' . (string) $data['id_to'] . ']'
                                    : '')
                        )
                    ) . '.csv';
                } else {
                    $return_values['filename'] = MxbToolsUtils::normalizeFilename($data['filename']);
                }
            }
            $file_xml = $folder . '/' . $return_values['filename'];
            $myfile = fopen($file_xml, 'wb') or exit('Unable to open file!');
            fwrite($myfile, "\xEF\xBB\xBF" . $records);
            fclose($myfile);
            $return_values['content'] = "\xEF\xBB\xBF" . $records;
            $return_values['download_url'] = $baseUrl . '/modules/mxb_tools/reports/' . $return_values['filename'];
            $return_values['download_link'] = '<a href="' . $return_values['download_url'] . '" target="_new">'
                . $return_values['filename'] . '</a>';

            if ($data['output_format'] == 'csv') {
                // orders_by_group
                if (count($CustomerGroupTotals)) {
                    $records = $module->l('User group') . ';'
                        . $module->l('Quantity') . ';'
                        . $module->l('Total paid tax excl') . ';'
                        . $module->l('Total discount') . ';'
                        . $module->l('Total order tax excl') . ';'
                        . $module->l('Total products') . ';'
                        . $module->l('Total shipping tax excl') . ';'
                        . $module->l('Total wrapping tax excl') . ';'
                        . $module->l('Total discount on products') . ';'
                        . $module->l('Total discount on shipping') . ';'
                        . PHP_EOL;

                    foreach ($CustomerGroupTotals as $tot) {
                        $records .= $tot['group_name'] . ';'
                            . $tot['order_quantity'] . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_paid_tax_excl']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_discounts_tax_excl']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_products']
                                + (float) $tot['total_shipping_tax_excl']
                                + (float) $tot['total_wrapping_tax_excl']
                            ) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_products']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_shipping_tax_excl']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_wrapping_tax_excl']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_discount_products']) . ';'
                            . MxbToolsUtils::csvPriceFormat((float) $tot['total_discount_shipping']) . ';'
                            . PHP_EOL;
                    }

                    $return_values['other_files'] = [0 => []];
                    $return_values['other_files'][0]['filename'] = MxbToolsUtils::normalizeFilename(
                        str_replace(
                            '/',
                            '-',
                            $shop_name
                            . '_' . $module->l('orders total per customer group')
                            . '_[' . $module->l('from') . '_'
                            . $data['date_from']
                            . '_' . $module->l('to') . '_'
                            . $data['date_to'] . ']'
                            . (($data['id_from'] + $data['id_to']) > 0
                                ? '[' . $module->l('order id')
                                    . '_' . $module->l('from') . '_' . (string) $data['id_from']
                                    . '_' . $module->l('to') . '_' . (string) $data['id_to'] . ']'
                                : '')
                        )
                    ) . '.csv';
                    $file_xml = $folder . '/' . $return_values['other_files'][0]['filename'];

                    $myfile = fopen($file_xml, 'wb') or exit('Unable to open file!');
                    fwrite($myfile, "\xEF\xBB\xBF" . $records);
                    fclose($myfile);
                    $return_values['other_files'][0]['content'] = "\xEF\xBB\xBF" . $records;
                    $return_values['other_files'][0]['download_url'] = $baseUrl
                        . '/modules/mxb_tools/reports/'
                        . $return_values['other_files'][0]['filename'];
                    $return_values['other_files'][0]['download_link'] =
                        '<a href="' . $baseUrl . '/modules/mxb_tools/reports/'
                        . $return_values['other_files'][0]['filename'] . '" target="_new">'
                        . $return_values['other_files'][0]['filename'] . '</a>';

                    if ($group_totals_request) {
                        $return_values = $return_values['other_files'][0];
                    }
                }
            }
        }
        return $return_values;
    }

    public static function exportCustomers($data = [])
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        // $force_debug = false;
        $module = Module::getInstanceByName('mxb_tools');

        // set numbering class
        $getCustomerAdditionalData = null;
        if (class_exists('MxbmpCustomerAdditionalData')) {
            $getCustomerAdditionalData = 'MxbmpCustomerAdditionalData::getCustomerAdditionalData';
        }

        // $gcodes = GadgetcardCodes::getCodes();
        $id_lang = (int) Context::getContext()->language->id;
        // $priceFormatter = new PriceFormatter();
        $return_values = [
            'values' => [],
            // 'confirmations' => [],
            // 'errors' => [],
            // 'download_link' => ''
        ];

        $allowed_output_types = [
            'customers',
        ];
        $allowed_output_formats = [
            'xml',
            'csv',
        ];

        /* check values */
        if (!isset($data['output_format'])) {
            $data['output_format'] = $allowed_output_formats[0];
        } elseif (!in_array($data['output_format'], $allowed_output_formats)) {
            $data['output_format'] = $allowed_output_formats[0];
        }
        if (!isset($data['output_type'])) {
            $data['output_type'] = $allowed_output_types[0];
        } elseif (!in_array($data['output_type'], $allowed_output_types)) {
            $data['output_type'] = $allowed_output_types[0];
        }
        if (!isset($data['date_from'])) {
            $data['date_from'] = MxbToolsUtils::getDateString('2020-01-01');
        }
        if (!isset($data['date_to'])) {
            $data['date_to'] = MxbToolsUtils::getDateString('2030-01-01');
        }
        // force international/database date format
        $data['date_from'] = MxbToolsUtils::getDateString($data['date_from']);
        $data['date_to'] = MxbToolsUtils::getDateString($data['date_to']);

        $data['id_order_state_list'] = isset($data['id_order_state_list'])
            ? (string) $data['id_order_state_list']
            : '';
        $data['id_order_state_list_not'] = isset($data['id_order_state_list_not'])
            ? (string) $data['id_order_state_list_not']
            : '';

        // get status list to consider for export
        if ($data['id_order_state_list'] != '' && $data['id_order_state_list'] != '0') {
            $idStatusArray = MxbToolsUtils::idStringToArray($data['id_order_state_list']);
            // normalize id_order_state_list
            $data['id_order_state_list'] = MxbToolsUtils::idArrayToString($idStatusArray);
        } else {
            $idStatusArray = [];
        }

        // get status list to NOT consider for export
        if ($data['id_order_state_list_not'] != '' && $data['id_order_state_list_not'] != '0') {
            $idStatusNotArray = MxbToolsUtils::idStringToArray($data['id_order_state_list_not']);
            // normalize id_order_state_list_not
            $data['id_order_state_list_not'] = MxbToolsUtils::idArrayToString($idStatusNotArray);
        } else {
            $idStatusNotArray = [];
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsExport::exportCustomers()',
            'data='.print_r($data, true),
            $force_debug
        );
        */
        $sql = 'SELECT `id_customer`
            FROM `' . _DB_PREFIX_ . 'customer`
            WHERE 1
            ORDER BY `company`, `lastname` ASC';
        // WHERE `active` = 1
        $customers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$customers) {
            $return_values['errors'] = [];
            $return_values['errors'][] = 'no customers'; // 'No customers available for the date range';
            return $return_values;
        }

        $customer_orders = [];
        /*
        $customer_vouchers = array('used' => [],
                      'available' => [],
                      'disabled' => [],
        );
        */
        $customer_vouchers = MxbToolsExport::getCustomerVoucherCsvValues(); // empty array

        $records = '';
        foreach ($customers as $customerId) {
            $id_customer = (int) $customerId['id_customer'];
            $customer = new Customer($id_customer);
            if (!Validate::isLoadedObject($customer)) {
                // no customer ??? it should be.... anyway, jump to next loop
                continue;
            }
            $invoice_address = null;
            $id_address = 0;
            $privacy_check = false;
            $customer_code = '';
            // check if there is mxbmemberprogram invoice address
            if ($getCustomerAdditionalData) {
                $cad = $getCustomerAdditionalData(0, $id_customer);
                if (count($cad)) {
                    $id_address = (int) $cad[0]['id_address'];
                    $privacy_check = (bool) $cad[0]['privacy_check'];
                    $customer_code = $cad[0]['customer_code'];
                }
            }

            // get total orders
            // TODO: ADD EXCLUDE ORDER_STATE FIELD TO REMOVE STATIC EXCLUDE !!!

            // FILTERS
            // set common sql usefull to update same records with new order state, if needed
            $sql_common = '
                WHERE date_add >= STR_TO_DATE("' . $data['date_from'] . '", "%Y-%m-%d")
                AND date_add <= STR_TO_DATE("' . $data['date_to'] . '", "%Y-%m-%d")';

            // filter order_status
            if (count($idStatusArray)) {
                $sql_common .= ' AND (';
                foreach ($idStatusArray as $skey => $idStatus) {
                    $sql_common .= ((int) $skey > 0 ? ' OR' : '') . ' current_state = ' . (int) $idStatus;
                }
                $sql_common .= ')';
            }

            // filter order_status
            if (count($idStatusNotArray)) {
                foreach ($idStatusNotArray as $skey => $idStatus) {
                    $sql_common .= ' AND current_state <> ' . (int) $idStatus;
                }
            }

            $sql = '
                SELECT *
                FROM ' . _DB_PREFIX_ . 'orders '
                . $sql_common
                . ' AND id_customer = ' . $id_customer
                . ' ORDER BY id_order ASC';
            /*
            $sql = 'SELECT o.*, osl.name as order_state_name
                FROM ' . _DB_PREFIX_ . 'orders o
                LEFT JOIN ' . _DB_PREFIX_ . 'order_state os ON (o.current_state = os.id_order_state)
                LEFT JOIN ' . _DB_PREFIX_ . 'order_state_lang osl ON (osl.id_lang = ' . $id_lang . ')
                WHERE o.current_state != 6 AND o.current_state != 7 AND o.id_customer = ' . $id_customer . '
                ORDER BY id_order ASC';
            */
            // WHERE o.valid = 1 AND o.id_customer = ' . $id_customer . '
            $orders = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsExport::exportCustomers()',
                'orders (' . count($orders) . ') sql=' . $sql,
                $force_debug
            );
            */
            $order_vouchers = [];
            if ($orders) {
                $customer_orders[$id_customer] = [
                    'order_quantity' => 0,
                    'order_num' => '',
                    'total_paid_tax_excl' => 0,
                    'total_discounts_tax_excl' => 0,
                    'total_paid_real' => 0,
                    'total_products' => 0,
                    'total_shipping_tax_excl' => 0,
                    'total_wrapping_tax_excl' => 0,
                ];
                foreach ($orders as $order) {
                    ++$customer_orders[$id_customer]['order_quantity'];
                    $customer_orders[$id_customer]['order_num'] .= $order['reference']
                        . ' (' . MxbToolsUtils::removeTimeFromDatetime($order['date_add']) . '), ';
                    $customer_orders[$id_customer]['total_paid_tax_excl'] += (float) $order['total_paid_tax_excl'];
                    $customer_orders[$id_customer]['total_discounts_tax_excl'] += (float) $order['total_discounts_tax_excl'];
                    // $customer_orders[$id_customer]['total_paid_real'] += (float) $order['total_paid_real'];
                    $customer_orders[$id_customer]['total_paid_real'] += (float) $order['total_products']
                        + (float) $order['total_shipping_tax_excl']
                        + (float) $order['total_wrapping_tax_excl'];
                    $customer_orders[$id_customer]['total_products'] += (float) $order['total_products'];
                    $customer_orders[$id_customer]['total_shipping_tax_excl'] += (float) $order['total_shipping_tax_excl'];
                    $customer_orders[$id_customer]['total_wrapping_tax_excl'] += (float) $order['total_wrapping_tax_excl'];

                    // VOUCHERS: cart rules used for each order
                    $sql = '
                        SELECT ocr.id_order_cart_rule,
                            ocr.id_cart_rule as ocr_id_cart_rule,
                            ocr.id_order as ocr_id_order,
                            o.total_discounts_tax_excl,
                            o.reference,
                            o.id_order,
                            cr.*
                        FROM ' . _DB_PREFIX_ . 'order_cart_rule ocr
                        LEFT JOIN ' . _DB_PREFIX_ . 'cart_rule cr ON (cr.id_cart_rule = ocr.id_cart_rule)
                        LEFT JOIN ' . _DB_PREFIX_ . 'orders o ON (o.id_order = ocr.id_order)
                        WHERE ocr.id_order = ' . $order['id_order'] . '
                        ORDER BY ocr.id_cart_rule ASC';
                    $this_order_vouchers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

                    if ($this_order_vouchers) {
                        // set amounts
                        $total_discounts_tax_excl = null;
                        $reduction_amount = 0;
                        foreach ($this_order_vouchers as &$ov) {
                            if ($total_discounts_tax_excl === null) {
                                $total_discounts_tax_excl = (float) $ov['total_discounts_tax_excl'];
                            }
                            $reduction_amount += (float) $ov['reduction_amount'];
                            $remaining_amount = $reduction_amount - $total_discounts_tax_excl;
                            $ov['order_discount'] = $total_discounts_tax_excl;
                            $ov['remaining_amount'] = $remaining_amount;
                        }
                        $order_vouchers = array_merge($order_vouchers, $this_order_vouchers);
                    }
                }
                $customer_orders[$id_customer]['order_num'] = rtrim($customer_orders[$id_customer]['order_num'], ', ');
            }

            // VOUCHERS: customer available cart rules
            $sql = 'SELECT cr.*
                FROM `' . _DB_PREFIX_ . 'cart_rule` cr
                WHERE cr.id_customer = ' . $id_customer . '
                ORDER BY `id_cart_rule`';
            /*
                filter by active and available:
                AND date_from <= now()
                AND date_to > now()
                AND cr.quantity > 0
                AND cr.active > 0
            */
            // AND date_from <= "' . date('Y-m-d H:i:s') . '
            // AND date_to > "' . date('Y-m-d H:i:s') . '
            $available_vouchers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

            // join arrays filtered by used values
            $vouchers = [];
            if ($order_vouchers) {
                // $total_discounts_tax_excl = null;
                foreach ($order_vouchers as $voucher) {
                    // if ($total_discounts_tax_excl === null)
                    // $total_discounts_tax_excl = (float) $voucher['total_discounts_tax_excl'];
                    // $remaining_amount = (float) $voucher['reduction_amount'] - $total_discounts_tax_excl;
                    $vouchers[(int) $voucher['id_cart_rule']] = [
                        'id_cart_rule' => $voucher['id_cart_rule'],
                        'code' => $voucher['code'],
                        'active' => $voucher['active'],
                        'quantity' => 0, // set qty to 0 because this voucher was already used
                        'reduction_amount' => $voucher['reduction_amount'],
                        'description' => $voucher['description'],
                        'date_to' => $voucher['date_to'],
                        'status' => 'used',
                        'id_order' => (float) $voucher['id_order'],
                        'order_reference' => $voucher['reference'],
                        'order_discount' => $voucher['order_discount'],
                        'remaining_amount' => $voucher['remaining_amount'],
                        // 'enabled' => false,
                    ];
                    // $total_discounts_tax_excl -= (float) $voucher['reduction_amount'];
                }
            }
            if ($available_vouchers) {
                $now = date('Y-m-d H:i:s');
                foreach ($available_vouchers as $voucher) {
                    if (!isset($vouchers[(int) $voucher['id_cart_rule']])) {
                        $vouchers[(int) $voucher['id_cart_rule']] = [
                            'id_cart_rule' => $voucher['id_cart_rule'],
                            'code' => $voucher['code'],
                            'active' => $voucher['active'],
                            'quantity' => $voucher['quantity'],
                            'reduction_amount' => $voucher['reduction_amount'],
                            'description' => $voucher['description'],
                            'date_to' => $voucher['date_to'],
                            'status' => (bool) $voucher['active']
                                && (int) $voucher['quantity']
                                && $now >= $voucher['date_from']
                                && $now < $voucher['date_to']
                                ? 'available'
                                : 'disabled',
                        ];
                    }
                }
            }

            if ($vouchers) {
                /*
                $customer_vouchers[$id_customer] = array(
                      'voucher_quantity' => 0,
                      'description' => '',
                      'reduction_amount' => 0,
                );

                $customer_disabled_vouchers[$id_customer] = array(
                      'voucher_quantity' => 0,
                      'description' => '',
                      'reduction_amount' => 0,
                );
                */
                $customer_vouchers[$id_customer] = [];
                foreach ($vouchers as $key => $voucher) {
                    if (!isset($customer_vouchers[$id_customer][$voucher['status']])) {
                        $customer_vouchers[$id_customer][$voucher['status']] = null;
                    }
                    $customer_vouchers[$id_customer][$voucher['status']] = MxbToolsExport::getCustomerVoucherCsvValues(
                        $voucher,
                        $customer_vouchers[$id_customer][$voucher['status']]
                    );
                }
                foreach ($customer_vouchers[$id_customer] as &$customer_voucher) {
                    if (isset($customer_voucher['description'])) {
                        $customer_voucher['description'] = rtrim($customer_voucher['description'], ', ');
                    }
                }
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsExport::exportCustomers()',
                    'customer vouchers : ' . print_r($customer_vouchers[$id_customer], true),
                    $force_debug
                );
                */
                /*
                if (!isset($customer_vouchers[$id_customer]))
                    $customer_vouchers[$id_customer]['description'] =
                        rtrim($customer_vouchers[$id_customer]['description'], ', ');

                if (!isset($customer_disabled_vouchers[$id_customer]))
                    $customer_disabled_vouchers[$id_customer]['description'] =
                        rtrim($customer_disabled_vouchers[$id_customer]['description'], ', ');
                */
            }

            if ((int) $id_address > 0) {
                $invoice_address = new Address((int) $id_address);
            } else {// if ((int) $id_address === 0) {
                // get customer addresses from database
                $sql = '
                    SELECT *
                    FROM `' . _DB_PREFIX_ . 'address`
                    WHERE `id_customer` = ' . (int) $customer->id . '
                    AND `deleted` = 0
                    ORDER BY id_address ASC';

                $addresses = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
                if (!$addresses) {
                    // SET EMPTY Address TO WRITE XML, BUT don't save it
                    $invoice_address = new Address();
                } elseif (count($addresses) == 1) {
                    $invoice_address = new Address((int) $addresses[0]['id_address']);
                } else {
                    // seek for an address with invoice data
                    foreach ($addresses as $addr) {
                        if (!empty($addr['vat_number']) && Tools::strlen($addr['vat_number']) > 1) {
                            $invoice_address = new Address((int) $addr['id_address']);
                            break;
                        }
                    }
                    // if vat not found, get the first address in list
                    if (empty($invoice_address)) {
                        $invoice_address = new Address((int) $addresses[0]['id_address']);
                    }
                }
            }

            $pec = isset($invoice_address->pec) ? MxbToolsUtils::parseStringForCSV($invoice_address->pec) : '';
            $sdi = isset($invoice_address->sdi) ? MxbToolsUtils::parseStringForCSV($invoice_address->sdi) : '';
            $group = new Group($customer->id_default_group);
            $company = (Tools::strlen($customer->company) > 0
                ? $customer->company
                : (isset($invoice_address->company) ? $invoice_address->company : '')
            );
            $records .= $customer->id . ';'
                . $customer_code . ';'
                . MxbToolsUtils::parseStringForCSV($customer->firstname) . ';'
                . MxbToolsUtils::parseStringForCSV($customer->lastname) . ';'
                . MxbToolsUtils::parseStringForCSV($company) . ';'
                . MxbToolsUtils::parseStringForCSV($invoice_address->address1) . (!empty($invoice_address->address2)
                    ? ' ' . MxbToolsUtils::parseStringForCSV($invoice_address->address2)
                    : '')
                . ';'
                . MxbToolsUtils::parseStringForCSV($invoice_address->postcode) . ';'
                . MxbToolsUtils::parseStringForCSV($invoice_address->city) . ';'
                . ((int) $invoice_address->id_state > 0
                    ? MxbToolsUtils::parseStringForCSV(MxbToolsExport::getStateIsoCodeById($invoice_address->id_state))
                    : '')
                    . ';'
                /*
                . MxbToolsUtils::parseStringForCSV($invoice_address->address1) . (!empty($invoice_address->address2)
                    ? ' ' . MxbToolsUtils::parseStringForCSV($invoice_address->address2)
                    : '')
                . ' - ' . MxbToolsUtils::parseStringForCSV($invoice_address->postcode)
                . ' ' . MxbToolsUtils::parseStringForCSV($invoice_address->city)
                . ' (' . ((int) $invoice_address->id_state > 0
                    ? MxbToolsUtils::parseStringForCSV(MxbToolsExport::getStateIsoCodeById($invoice_address->id_state))
                    : '')
                . ');'
                */
                . MxbToolsUtils::parseStringForCSV($invoice_address->country) . ';'
                . MxbToolsUtils::parseStringForCSV($invoice_address->vat_number) . ';'
                . MxbToolsUtils::parseStringForCSV($customer->siret) . ';'
                . MxbToolsUtils::parseStringForCSV($invoice_address->phone) . ';'
                . MxbToolsUtils::parseStringForCSV($customer->email) . ';'
                . $pec . ';'
                . $sdi . ';'
                . ((bool) $privacy_check ? 'V' : '') . ';'
                . ((bool) $customer->optin ? 'V' : '') . ';'
                . ((bool) $customer->active ? 'V' : '') . ';'
                . MxbToolsUtils::parseStringForCSV($customer->date_add) . ';'
                . $group->name[$id_lang] . ';'
                . ((bool) $customer->newsletter ? $customer->newsletter_date_add : '') . ';'
                . MxbToolsUtils::parseStringForCSV($customer->website) . ';'
                . MxbToolsUtils::parseStringForCSV($customer->birthday) . ';'
                . MxbToolsUtils::parseStringForCSV($customer->note) . ';'
                . (isset($customer_orders[$id_customer])
                    ? ($customer_orders[$id_customer]['order_quantity'] . ';'
                        . $customer_orders[$id_customer]['order_num'] . ';'
                        . MxbToolsUtils::csvPriceFormat($customer_orders[$id_customer]['total_products']) . ';'
                        . MxbToolsUtils::csvPriceFormat($customer_orders[$id_customer]['total_shipping_tax_excl'])
                        . ';'
                        . MxbToolsUtils::csvPriceFormat($customer_orders[$id_customer]['total_wrapping_tax_excl'])
                        . ';'
                        . MxbToolsUtils::csvPriceFormat($customer_orders[$id_customer]['total_paid_real']) . ';'
                        . MxbToolsUtils::csvPriceFormat($customer_orders[$id_customer]['total_paid_tax_excl']) . ';'
                        . MxbToolsUtils::csvPriceFormat(
                            $customer_orders[$id_customer]['total_discounts_tax_excl']
                        )
                        . ';')
                    : '0;;0;0;0;0;0;0;'
                );

            $voucher = [
                'used' => ';0;0;',
                'available' => ';0;0;',
                'disabled' => ';0;0;',
            ];
            if (isset($customer_vouchers[$id_customer])) {
                foreach ($customer_vouchers[$id_customer] as $key => $cus_voucher) {
                    $voucher[$key] = MxbToolsUtils::parseStringForCSV($cus_voucher['description']) . ';'
                        . $cus_voucher['voucher_quantity'] . ';'
                        . MxbToolsUtils::csvPriceFormat($cus_voucher['reduction_amount']) . ';';
                    // . MxbToolsUtils::parseStringForCSV($cus_voucher['amounts']) . ';'
                    // . $key . ': '
                }
            }

            $records .= $voucher['used']
                . $voucher['available']
                . $voucher['disabled'];
            /*
                $records .= (isset($customer_vouchers[$id_customer])
                  ? ($customer_vouchers[$id_customer]['voucher_quantity'] . ';'
                    . MxbToolsUtils::csvPriceFormat($customer_vouchers[$id_customer]['reduction_amount']) . ';'
                    // . MxbToolsUtils::parseStringForCSV($customer_vouchers[$id_customer]['amounts']) . ';'
                    . MxbToolsUtils::parseStringForCSV($customer_vouchers[$id_customer]['description']) . ';'
                    )
                  :  '0;0;;');
            */
            $records .= PHP_EOL;
        } // foreach $customers

        $records = $module->l('ID customer') . ';'
            . $module->l('Customer code') . ';'
            . $module->l('Firstname') . ';'
            . $module->l('Lastname') . ';'
            . $module->l('Customer company') . ';'
            . $module->l('Address') . ';'
            . $module->l('Postcode') . ';'
            . $module->l('City') . ';'
            . $module->l('State') . ';'
            . $module->l('Country') . ';'
            . $module->l('VAT number') . ';'
            . $module->l('Siret') . ';'
            . $module->l('Phone') . ';'
            . $module->l('E-mail') . ';'
            . $module->l('Pec') . ';'
            . $module->l('Sdi') . ';'
            . $module->l('Privacy check') . ';'
            . $module->l('Partner special offer') . ';'
            . $module->l('Is active') . ';'
            . $module->l('Since') . ';'
            . $module->l('User group') . ';'
            . $module->l('Newsletter') . ';'
            . $module->l('Website') . ';'
            . $module->l('Birthday') . ';'
            . $module->l('Note') . ';'
            . $module->l('Orders') . ' ' . $module->l('Quantity') . ';'
            . $module->l('Orders') . ' ' . $module->l('Reference') . '(date);'
            . $module->l('Total products') . ';'
            . $module->l('Total shipping tax excl') . ';'
            . $module->l('Total wrapping tax excl') . ';'
            . $module->l('Total order tax excl') . ';'
            . $module->l('Total paid tax excl') . ';'
            . $module->l('Total discount') . ';'
            . $module->l('Vouchers') . ' ' . $module->l('used')
            . ': [id] ' . $module->l('code')
            . ' (' . $module->l('status') . ' ' . $module->l('quantity')
            . ': ' . $module->l('amount')
            . ': ' . $module->l('description')
            . ' - ' . $module->l('date_to') . ');'
            . $module->l('quantity') . ';'
            . $module->l('amount') . ';'
            . $module->l('Vouchers') . ' ' . $module->l('available')
            . ': [id] ' . $module->l('code') . ' (' . $module->l('status')
            . ' ' . $module->l('quantity') . ': ' . $module->l('amount')
            . ': ' . $module->l('description') . ' - ' . $module->l('date_to') . ');'
            . $module->l('quantity') . ';'
            . $module->l('amount') . ';'
            . $module->l('Vouchers') . ' ' . $module->l('disabled')
            . ': [id] ' . $module->l('code') . ' (' . $module->l('status')
            . ' ' . $module->l('quantity') . ': ' . $module->l('amount')
            . ': ' . $module->l('description')
            . ' - ' . $module->l('date_to') . ');'
            . $module->l('quantity') . ';'
            . $module->l('amount') . ';'
            . PHP_EOL
            . $records;

        $shop_name = MxbToolsExport::getShopName();
        $return_values['filename'] = MxbToolsUtils::normalizeFilename(
            str_replace(
                '/',
                '-',
                $shop_name
                    . '_' . $module->l('customer list')
                    . '_' . MxbToolsUtils::getCurrentDateTime(false, '-', '', '')
            )
        ) . '.csv';

        // create file: only once I got quantity columns
        // create folder and CSV file and write part of headings
        $folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        $file_xml = $folder . '/' . $return_values['filename'];
        $myfile = fopen($file_xml, 'wb') or exit('Unable to open file!');
        fwrite($myfile, "\xEF\xBB\xBF" . $records);
        fclose($myfile);

        $return_values['download_url'] = $baseUrl . '/modules/mxb_tools/reports/' . $return_values['filename'];
        $return_values['download_link'] = '<a href="' . $return_values['download_url']
            . '" target="_new">' . $return_values['filename'] . '</a>';

        return $return_values;
    }

    public static function getCustomerVoucherCsvValues($voucher = null, $arr = null)
    {
        if (!$arr) {
            $arr = [
                'voucher_quantity' => 0,
                'description' => '',
                'reduction_amount' => 0,
            ];
            if (!$voucher) {
                // empty array was requested
                return $arr;
            }
        }
        ++$arr['voucher_quantity'];
        $arr['description'] .= '(' . MxbToolsExport::getVoucherCsvDescription($voucher);
        if (isset($voucher['order_discount'])) {
            $arr['description'] .= ' [order ref:' . $voucher['order_reference']
                . ', disc:' . MxbToolsUtils::csvPriceFormat($voucher['order_discount']);
            if ((float) $voucher['remaining_amount'] !== (float) 0) {
                $arr['description'] .= ', '
                    . ($voucher['remaining_amount'] > 0
                        ? 'surplus:'
                        : 'miss:-')
                    . MxbToolsUtils::csvPriceFormat($voucher['remaining_amount']);
            }
            $arr['description'] .= ']';
        }

        // if ($voucher['quantity'] > 0) {
        $arr['reduction_amount'] += (float) $voucher['reduction_amount'];
        //  $arr['description'] .= ' - ' . $voucher['date_to'];
        // }
        $arr['description'] .= '), ';
        return $arr;
    }

    public static function getVoucherCsvDescription($voucher, $reduction_amount = null)
    {
        if ($reduction_amount === null) {
            $reduction_amount = $voucher['reduction_amount'];
        }
        $descr = '[' . $voucher['id_cart_rule'] . '] '
            . (Tools::strlen($voucher['code'])
                ? $voucher['code']
                : 'nocode')
            . ' - '
            . ((bool) $voucher['active']
                ? 'active'
                : 'disabled')
            . ', qty:' . (int) $voucher['quantity']
            . ', value:' . MxbToolsUtils::csvPriceFormat($reduction_amount)
            . ', end:' . $voucher['date_to'] . ' - ' . $voucher['description'];
        return $descr;
    }

    public static function exportVouchers($data = [])
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        // $force_debug = false;
        $module = Module::getInstanceByName('mxb_tools');

        // $gcodes = GadgetcardCodes::getCodes();
        $id_lang = (int) Context::getContext()->language->id;
        // $priceFormatter = new PriceFormatter();
        $return_values = [
            'values' => [],
            // 'confirmations' => [],
            // 'errors' => [],
            // 'download_link' => ''
        ];

        $allowed_output_types = [
            'vouchers',
        ];
        $allowed_output_formats = [
            'xml',
            'csv',
        ];

        /* check values */
        if (!isset($data['output_format'])) {
            $data['output_format'] = $allowed_output_formats[0];
        } elseif (!in_array($data['output_format'], $allowed_output_formats)) {
            $data['output_format'] = $allowed_output_formats[0];
        }
        if (!isset($data['output_type'])) {
            $data['output_type'] = $allowed_output_types[0];
        } elseif (!in_array($data['output_type'], $allowed_output_types)) {
            $data['output_type'] = $allowed_output_types[0];
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsExport::exportVouchers()',
            'data='. print_r($data, true),
            $force_debug
        );
        */
        $sql = 'SELECT cr.*
            FROM `' . _DB_PREFIX_ . 'cart_rule` cr
            ORDER BY `id_cart_rule`';
        // LEFT_JOIN `' . _DB_PREFIX_ . 'customer` c ON (cr.id_customer = c.id_customer)
        $vouchers = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!$vouchers) {
            $return_values['errors'] = [];
            $return_values['errors'][] = 'no vouchers'; // 'No customers available for the date range';
            return $return_values;
        }

        $records = '';
        foreach ($vouchers as $voucher) {
            $id_customer = (int) $voucher['id_customer'];
            $customer = null;
            $group = null;
            if ($id_customer > 0) {
                $customer = new Customer($id_customer);
                if (!Validate::isLoadedObject($customer)) {
                    // no customer ??? it should be.... anyway, set it to null
                    $customer = null;
                } else {
                    $group = new Group($customer->id_default_group);
                }
            }

            $records .= $voucher['id_cart_rule'] . ';'
                . $voucher['id_customer'] . ';'
                . ($customer ? MxbToolsUtils::parseStringForCSV($customer->email) : '') . ';'
                . ($customer ? MxbToolsUtils::parseStringForCSV($customer->company) : '') . ';'
                . ($group ? $group->name[$id_lang] : '') . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['date_add']) . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['date_upd']) . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['description']) . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['code']) . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['date_from']) . ';'
                . MxbToolsUtils::parseStringForCSV($voucher['date_to']) . ';'
                . $voucher['quantity'] . ';'
                . $voucher['quantity_per_user'] . ';'
                . $voucher['priority'] . ';'
                . $voucher['partial_use'] . ';'
                . $voucher['minimum_amount'] . ';'
                . $voucher['minimum_amount_tax'] . ';'
                . $voucher['minimum_amount_currency'] . ';'
                . $voucher['minimum_amount_shipping'] . ';'
                . $voucher['country_restriction'] . ';'
                . $voucher['carrier_restriction'] . ';'
                . $voucher['group_restriction'] . ';'
                . $voucher['cart_rule_restriction'] . ';'
                . $voucher['product_restriction'] . ';'
                . $voucher['shop_restriction'] . ';'
                . $voucher['free_shipping'] . ';'
                . $voucher['reduction_percent'] . ';'
                . $voucher['reduction_amount'] . ';'
                . $voucher['reduction_tax'] . ';'
                . $voucher['reduction_currency'] . ';'
                . $voucher['reduction_product'] . ';'
                . $voucher['reduction_exclude_special'] . ';'
                . $voucher['gift_product'] . ';'
                . $voucher['gift_product_attribute'] . ';'
                . $voucher['highlight'] . ';'
                . $voucher['active'] . ';'
                . PHP_EOL;
            /*
                id_cart_rule
                id_customer
                $customer->email
                customer->company
                group->name
                date_from
                date_to
                description
                quantity
                quantity_per_user
                priority
                partial_use
                code
                minimum_amount
                minimum_amount_tax
                minimum_amount_currency
                minimum_amount_shipping
                country_restriction
                carrier_restriction
                group_restriction
                cart_rule_restriction
                product_restriction
                shop_restriction
                free_shipping
                reduction_percent
                reduction_amount
                reduction_tax
                reduction_currency
                reduction_product
                reduction_exclude_special
                gift_product
                gift_product_attribute
                highlight
                active
                date_add
                date_upd
            */
        } // foreach $vouchers

        $records = $module->l('ID voucher') . ';'
            . $module->l('ID customer') . ';'
            . $module->l('E-mail') . ';'
            . $module->l('Customer company') . ';'
            . $module->l('User group') . ';'
            . $module->l('date_add') . ';'
            . $module->l('date_upd') . ';'
            . $module->l('description') . ';'
            . $module->l('code') . ';'
            . $module->l('date_from') . ';'
            . $module->l('date_to') . ';'
            . $module->l('quantity') . ';'
            . $module->l('quantity_per_user') . ';'
            . $module->l('priority') . ';'
            . $module->l('partial_use') . ';'
            . $module->l('minimum_amount') . ';'
            . $module->l('minimum_amount_tax') . ';'
            . $module->l('minimum_amount_currency') . ';'
            . $module->l('minimum_amount_shipping') . ';'
            . $module->l('country_restriction') . ';'
            . $module->l('carrier_restriction') . ';'
            . $module->l('group_restriction') . ';'
            . $module->l('cart_rule_restriction') . ';'
            . $module->l('product_restriction') . ';'
            . $module->l('shop_restriction') . ';'
            . $module->l('free_shipping') . ';'
            . $module->l('reduction_percent') . ';'
            . $module->l('reduction_amount') . ';'
            . $module->l('reduction_tax') . ';'
            . $module->l('reduction_currency') . ';'
            . $module->l('reduction_product') . ';'
            . $module->l('reduction_exclude_special') . ';'
            . $module->l('gift_product') . ';'
            . $module->l('gift_product_attribute') . ';'
            . $module->l('highlight') . ';'
            . $module->l('Is active') . ';'
            . PHP_EOL
            . $records;

        $shop_name = MxbToolsExport::getShopName();
        $return_values['filename'] = MxbToolsUtils::normalizeFilename(
            str_replace(
                '/',
                '-',
                $shop_name
                    . '_' . $module->l('voucher list')
                    . '_' . MxbToolsUtils::getCurrentDateTime(false, '-', '', '')
            )
        ) . '.csv';

        // create file: only once I got quantity columns
        // create folder and CSV file and write part of headings
        $folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
        $file_xml = $folder . '/' . $return_values['filename'];
        $myfile = fopen($file_xml, 'wb') or exit('Unable to open file!');
        fwrite($myfile, "\xEF\xBB\xBF" . $records);
        fclose($myfile);

        $return_values['download_url'] = $baseUrl . '/modules/mxb_tools/reports/' . $return_values['filename'];
        $return_values['download_link'] = '<a href="' . $return_values['download_url']
            . '" target="_new">' . $return_values['filename'] . '</a>';

        return $return_values;
    }

    /**
     * Get a state iso code with its ID.
     *
     * @param int $idState Country ID
     *
     * @return string State iso code
     */
    public static function getStateIsoCodeById($idState)
    {
        if (!$idState) {
            return false;
        }
        // $cacheId = 'MxbToolsExport::getStateIsoCodeById_' . (int) $idState;
        // if (!Cache::isStored($cacheId)) {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
            SELECT `iso_code`
            FROM `' . _DB_PREFIX_ . 'state`
            WHERE `id_state` = ' . (int) $idState);
        // Cache::store($cacheId, $result);

        return $result;
        // }
        // return Cache::retrieve($cacheId);
    }

    /**
     * EXPORT PRODUCT LIST
     */
    public static function exportProducts($data = [])
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        // $force_debug = true;

        $folder = null;
        $module = Module::getInstanceByName('mxb_tools');
        $context = Context::getContext();
        $id_lang = (int) $context->language->id;
        $id_currency = null;
        $id_country = null;
        $return_values = [
            'success' => [],
            'errors' => [],
        ];

        $allowed_output_formats = [
          'xml',
          'csv',
        ];

        if (!isset($data['output_format'])) {
            $data['output_format'] = $allowed_output_formats[0];
        } elseif (!in_array($data['output_format'], $allowed_output_formats)) {
            $data['output_format'] = $allowed_output_formats[0];
        }

        $data['export_last_record'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_last_record', 1);
        $data['id_customer'] = MxbToolsUtils::setArrayValueIfNew($data, 'id_customer', 1);
        $data['quantity_columns_min'] = MxbToolsUtils::setArrayValueIfNew($data, 'quantity_columns_min', 4);
        $data['products_per_loop'] = MxbToolsUtils::setArrayValueIfNew($data, 'products_per_loop', 500);
        $data['export_zero_price'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_zero_price', true);
        $data['export_category_id'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_category_id', true);
        $data['export_flag_show_category'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_show_category',
            true
        );
        $data['export_flag_stock_quantity'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_stock_quantity',
            true
        );
        $data['export_flag_product_price'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_product_price',
            true
        );
        $data['export_flag_wholesale_price'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_wholesale_price',
            true
        );
        $data['export_flag_gross_price'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_gross_price',
            true
        );
        $data['export_flag_description'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_description',
            true
        );
        $data['export_flag_description_short'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_flag_description_short',
            true
        );
        $data['export_flag_tags'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_flag_tags', true);
        $data['export_flag_price_charge'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_flag_price_charge', true);
        $data['export_flag_discount'] = MxbToolsUtils::setArrayValueIfNew($data, 'export_flag_discount', true);
        $data['export_material_text'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_material_text',
            Configuration::getGlobalValue('MXB_TOOLS_MATERIAL_PRODUCT_TAG', $id_lang)
        );
        $data['export_segment_text'] = MxbToolsUtils::setArrayValueIfNew(
            $data,
            'export_segment_text',
            Configuration::getGlobalValue('MXB_TOOLS_SEGMENT_PRODUCT_TAG', $id_lang)
        );

        $id_customer = $data['id_customer'];
        $start_record = $data['export_last_record'];
        $return_values['data'] = $data;

        // get default group id for passed customer
        $row = Db::getInstance()->getRow('
            SELECT c.id_default_group, c.id_customer, c.id_shop, gl.name
            FROM ' . _DB_PREFIX_ . 'customer c
            LEFT JOIN ' . _DB_PREFIX_ . 'group_lang gl ON (gl.id_group = c.id_default_group AND gl.id_lang = '
            . $id_lang . ')
            WHERE c.`id_customer` = ' . (int) $id_customer);

        if (isset($row['id_customer'])) {
            $id_group = $row['id_default_group'];
            $group_name = $row['name'];
            $id_shop = $row['id_shop'];
        } else {
            $id_group = 0;
            $group_name = 'none';
            $id_shop = null;
        }
        // get category name
        if ($data['export_category_id'] > 0) {
            $cat = new Category($data['export_category_id'], $id_lang);
            $category_name = $cat->name;
        } else {
            $category_name = $module->l('all categories');
        }

        // set output filename
        $shop_name = MxbToolsExport::getShopName();
        $filename = MxbToolsUtils::normalizeFilename(
            str_replace(
                '/',
                '-',
                $shop_name
                . '_' . $module->l('product list')
                . '_' . $group_name
                . '_' . $category_name
                . '_[' . $module->l('prices')
                . (!$data['export_flag_product_price']
                    ? '-no'
                    : (($data['export_flag_wholesale_price'] ? '-' . $module->l('wholesale') : '')
                        . ($data['export_flag_gross_price'] ? '-' . $module->l('gross') : '')))
                . ']'
                . '_' . MxbToolsUtils::getCurrentDateTime(false, '-', '', '')
            )
        ) . '.csv';
        // $quantity_columns_min = 4;

        // main query on product table to get products
        $sql = '
            SELECT p.id_product,
                pa.`id_product_attribute`,
                p.reference AS product_reference,
                pl.id_product,
                pl.name,
                m.name AS brand,
                pl.description,
                pl.description_short,
                pa.`reference` AS product_attribute_reference,
                sa.`quantity` AS stock_quantity
            FROM ' . _DB_PREFIX_ . 'product p
            LEFT JOIN ' . _DB_PREFIX_ . 'product_lang pl ON (pl.id_product = p.id_product AND pl.id_lang = '
            . $id_lang . ')
            LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute` pa ON pa.`id_product` = p.`id_product`
            LEFT JOIN `' . _DB_PREFIX_ . 'stock_available` sa ON (pa.id_product_attribute = sa.id_product_attribute)
            LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer` m ON (p.id_manufacturer = m.id_manufacturer)
            ' . ((int) $data['export_category_id'] > 0
                ? 'LEFT JOIN `' . _DB_PREFIX_ . 'category_product` pc ON pc.`id_product` = p.`id_product` '
                : '') . '
            WHERE ' . MxbToolsCodes::sqlFilterProductsOnly('p') . '
            ' . ((int) $data['export_category_id'] > 0
                ? 'AND pc.`id_category` = ' . (int) $data['export_category_id']
                : '') . '
            AND p.active = 1
            ORDER BY p.reference ASC, pa.reference ASC';
        $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (isset($products[0])) {
            // set support variabiles
            $price_columns_count = 0;
            $records = '';
            $cnt = 0;           // products counter
            $partial_cnt = 0;   // products counter for current session
            $real_cnt = 0;      // exported products counter: need to decrement when a product is not exported
            $tot_records = count($products);

            // main product loop
            foreach ($products as &$product) {
                // set server timeout to 1 sec for each cycle
                set_time_limit(300);
                /*
                MxbToolsDebug::writeLog(
                    $this->name,
                    $this->class_name . '::exportProductCSV()',
                    'start_record=' . $start_record
                        . ', cnt=' . $cnt
                        . ', real_cnt=' . $real_cnt
                        . ', product_reference=' . $product['product_reference'],
                    $force_debug
                );
                */
                ++$cnt;

                // check if this record is already done by previous session
                if ($product['product_reference'] == '') {
                    continue;
                }

                // no other reason to discard this product for now: increment other counters
                ++$real_cnt; // increment real exported product counter

                // product already exported ?
                if ($cnt < $start_record) {
                    continue;
                }

                $id_product = (int) $product['id_product'];
                $id_product_attribute = (int) $product['id_product_attribute'];

                //  set records: category names
                if ($data['export_flag_show_category']) {
                    $categories = Product::getProductCategoriesFull($id_product);
                    if (count($categories)) {
                        $arr = [];
                        foreach ($categories as $category) {
                            if ((int) $category['id_category'] !== 2) {
                                $arr[] = $category['name'];
                            }
                        }
                        $record_categories = implode(', ', $arr) . ';';
                    } else {
                        $record_categories = '-;';
                    }
                } else {
                    $record_categories = null;
                }

                // get combination data
                $combination = new Combination($id_product_attribute);
                $combinations = $combination->getAttributesName($id_lang);

                //  create $color_id_attribute_array to check color attributes later ...
                //    loop $combinations for id_attribute:
                //    1. attribute_group ->   look for is_color_group=1 (group_type=color)
                //              get id_attribute_group
                //    2. attribute -> check for id_attribute_group and id_attribute

                // get id_attribute_group for colors and a list of related id_attribute
                $sql = '
                    SELECT a.id_attribute
                    FROM ' . _DB_PREFIX_ . 'attribute_group ag
                    LEFT JOIN `' . _DB_PREFIX_ . 'attribute` a ON ag.`id_attribute_group` = a.`id_attribute_group`
                    WHERE ag.is_color_group = 1
                    AND ag.group_type="color"';
                $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

                // put all color id_attributes into array
                $color_id_attribute_array = [];
                if (is_array($result) && count($result)) {
                    foreach ($result as $attr) {
                        $color_id_attribute_array[] = $attr['id_attribute'];
                    }
                }

                // get product "mark segment" value
                $segment = MxbToolsProduct::getProductFeatureValue(
                    $data['export_segment_text'],
                    0,
                    $product,
                    $id_lang
                );
                if (is_null($segment)) {
                    $segment = '';
                }

                // get product "material" value
                $material = MxbToolsProduct::getProductFeatureValue(
                    $data['export_material_text'],
                    0,
                    $product,
                    $id_lang
                );
                if (is_null($material)) {
                    $material = '';
                }

                // get for_sale flag
                $for_sale = MxbToolsProduct::isSaleProduct($id_product, $id_product_attribute) ? 'sale' : '';
                $column_heading = [];

                // get quantity discount
                $specific_prices = MxbToolsPrice::getQuantityDiscountPrices(
                    $id_product,
                    $id_product_attribute,
                    null,
                    $id_group,
                    $id_shop,
                    $id_currency,
                    $id_country,
                    $id_customer,
                    $context
                );
                /*
                MxbToolsDebug::writeLog(
                    'mxb_tools',
                    'MxbToolsExport::exportProducts()',
                    'specific_prices=' . print_r($specific_prices, true),
                    true
                );
                */
                // no zero price export: exit if no specific prices or price = 0
                if (!$data['export_zero_price']) {
                    if (!isset($specific_prices[0])
                        || MxbToolsProduct::isHiddenProduct(
                            $id_product,
                            $id_product_attribute,
                            true,
                            $id_customer
                        ) <= 0
                    ) {
                        // decrement real exported product counter
                        --$real_cnt;

                        continue;
                    }
                }
                ++$partial_cnt;

                // get group and special discounts for this product and user group
                $discounts = MxbToolsPrice::getProductDiscountsCollection(
                    $id_product,
                    $id_group,
                    $id_customer,
                    $id_lang
                );

                if (isset($specific_prices[0])) {
                    // save the number of "quantity" columns if not already done
                    $specific_prices_columns = count($specific_prices);
                    if ($price_columns_count == 0) {
                        /*
                        $price_columns_count = $specific_prices_columns < $data['quantity_columns_min']
                            ? $data['quantity_columns_min']
                            : $specific_prices_columns;
                        */
                        // force price_columns_count to specific_prices_columns
                        // to have always the same number of columns
                        $price_columns_count = $data['quantity_columns_min'];

                        $column_heading = [
                            'quantity' => Tools::ucfirst($module->l('qty')) . ';',
                            'neutral_price' => '',
                            'gross_neutral_price' => '',
                            'wholesale_neutral_price' => '',
                            'price_charge' => '',
                            'discount' => '',
                            // 'total_discount' => '',
                        ];

                        for ($i = 0; $i < $price_columns_count; ++$i) {
                            if ($i > 0) {
                                $column_heading['quantity'] .= Tools::ucfirst($module->l('qty'))
                                    . ' col.' . ($i + 1) . ';';
                            }
                            $column_heading['neutral_price'] .= Tools::ucfirst($module->l('price'))
                                . ' col.' . ($i + 1) . ';';
                            $column_heading['gross_neutral_price'] .= Tools::ucfirst($module->l('public price'))
                                . ' col.' . ($i + 1) . ';';
                            $column_heading['wholesale_neutral_price'] .= Tools::ucfirst($module->l('wholesale price'))
                                . ' col.' . ($i + 1) . ';';
                            $column_heading['price_charge'] .= Tools::ucfirst($module->l('charge'))
                                . '% col.' . ($i + 1) . ';';
                            $column_heading['discount'] .= Tools::ucfirst($module->l('discount'))
                                . '% col.' . ($i + 1) . ';';
                        }
                    }
                }
                // add total prices to specific_prices array
                foreach ($specific_prices as &$specific_price) {
                    $specific_price['totals'] = MxbToolsPrice::getProductPrices(
                        $id_product,
                        $id_product_attribute,
                        (int) $specific_price['from_quantity'],
                        $id_customer
                    );
                }
                /*
                MxbToolsDebug::writeLog(
                    'mxb_tools',
                    'MxbToolsExport::exportProducts()',
                    'specific_prices=' . print_r($specific_prices, true),
                    $force_debug
                );
                */
                unset($specific_price);

                // set records: prepare attributes name and colors lists
                $attr_array = [];
                $col_array = [];
                $size_array = [];
                $last = '';
                foreach ($combinations as $comb) {
                    if ($last != $comb['name']) {  // avoid duplicates
                        if (in_array($comb['id_attribute'], $color_id_attribute_array)) {
                            $col_array[] = $comb['name'];
                        } elseif (in_array($comb['name'], MxbToolsCodes::getDressSizeAttributeArray())
                            || MxbToolsProduct::getAttributeUsbDimension($comb['name']) > 0) {
                            $size_array[] = $comb['name'];
                        } else {
                            $attr_array[] = $comb['name'];
                        }
                        $last = $comb['name'];
                    }
                }

                //  set record: category names
                if ($record_categories !== null) {
                    $records .= $record_categories;
                }

                // set record: product data
                $records .= $product['product_reference'] . ';' . $product['product_attribute_reference'] . ';'
                    . MxbToolsUtils::parseStringForCSV($product['name']) . ';'
                    . (count($col_array) ? implode(', ', $col_array) : '') . ';'// colors
                    . (count($size_array) ? implode(', ', $size_array) : '') . ';'// size
                    . (count($attr_array)
                        ? MxbToolsUtils::parseStringForCSV(implode(', ', $attr_array))
                        : '') . ';' // attributes
                    . MxbToolsUtils::parseStringForCSV($material) . ';'
                    . MxbToolsUtils::parseStringForCSV($product['brand']) . ';'
                    . '\'' . $combination->ean13 . ';'
                    . MxbToolsUtils::parseStringForCSV($segment) . ';'
                    . $for_sale . ';'
                    . ($data['export_flag_stock_quantity'] ? $product['stock_quantity'] . ';' : '')
                    . ($data['export_flag_description_short']
                        ? MxbToolsUtils::parseStringForCSV($product['description_short']) . ';'
                        : '')
                    . ($data['export_flag_description']
                        ? MxbToolsUtils::parseStringForCSV($product['description']) . ';'
                        : '');

                // get product tags
                if ($data['export_flag_tags']) {
                    $tags = Tag::getProductTags($product['id_product']);
                    if (is_array($tags) && !empty($tags)) {
                        // I should find tags for current id_lang.
                        // If not, I'll get the first language tags I find into the array (should be the main language)
                        $idx = $id_lang;
                        if (!isset($tags[$idx])) {
                            foreach ($tags as $key => $tag) {
                                $idx = $key;

                                break;
                            }
                        }
                        // get all product tags
                        foreach ($tags[$idx] as $tag) {
                            $records .= MxbToolsUtils::parseStringForCSV($tag) . ', ';
                        }
                        // remove last comma and close tag column
                        $records = rtrim($records, ', ') . ';';
                    } else {
                        // no tag: close empty column
                        $records .= ';';
                    }
                }

                // set record: quantity and prices
                $column_values = [
                    'quantity' => '',
                    'neutral_price' => '',
                    'gross_neutral_price' => '',
                    'wholesale_neutral_price' => '',
                    'price_charge' => '',
                    'discount' => '',
                ];

                // specific prices
                foreach ($specific_prices as $spcnt => $specific_price) {
                    if ((int) $spcnt >= (int) $price_columns_count) {
                        break;
                    }
                    // set record: quantity columns
                    $column_values['quantity'] .= $specific_price['from_quantity'] . ';';
                    // set record: neutral product price
                    $column_values['neutral_price']
                        .= MxbToolsUtils::csvPriceFormat((float) $specific_price['totals']['real_price'])
                        . ';';
                    // set record: wholesale prices
                    $column_values['wholesale_neutral_price']
                        .= MxbToolsUtils::csvPriceFormat((float) $specific_price['totals']['real_wholesale_price'])
                        . ';';
                    // set record: gross prices
                    $column_values['gross_neutral_price']
                        .= MxbToolsUtils::csvPriceFormat((float) $specific_price['totals']['real_full_price'])
                        . ';';
                    // charge
                    $column_values['price_charge'] .= MxbToolsUtils::csvPriceFormat(MxbToolsUtils::getChargeFromPrice(
                        (float) $specific_price['totals']['real_full_price'],
                        (float) $specific_price['totals']['real_wholesale_price']
                    )) . ';';
                    // discount
                    $column_values['discount'] .= MxbToolsUtils::csvPriceFormat(MxbToolsUtils::getDiscountFromPrice(
                        (float) $specific_price['totals']['real_full_price'],
                        (float) $specific_price['totals']['real_price']
                    )) . ';';
                }

                // adjust columns:
                if (isset($specific_prices_columns)) {
                    if ($specific_prices_columns < $price_columns_count) {
                        for ($i = $specific_prices_columns + 1; $i <= $price_columns_count; ++$i) {
                            $column_values['quantity'] .= '-;';
                            $column_values['neutral_price'] .= '-;';
                            $column_values['wholesale_neutral_price'] .= '-;';
                            $column_values['gross_neutral_price'] .= '-;';
                            $column_values['price_charge'] .= '-;';
                            $column_values['discount'] .= '-;';
                        }
                    }
                } else {
                    // if no specific_prices yet, specific_prices_columns was not set and no specific prices will be set
                    // if no specific_prices yet, I set the default quantity columns with no prices
                    for ($i = 0; $i < $data['quantity_columns_min']; ++$i) {
                        $column_values['quantity'] .= '-;';
                        $column_values['neutral_price'] .= '-;';
                        $column_values['wholesale_neutral_price'] .= '-;';
                        $column_values['gross_neutral_price'] .= '-;';
                        $column_values['price_charge'] .= '-;';
                        $column_values['discount'] .= '-;';
                    }
                }

                $records .= $column_values['quantity']
                    . ($data['export_flag_wholesale_price'] && $data['export_flag_product_price']
                        ? $column_values['wholesale_neutral_price']
                        : '')
                    . ($data['export_flag_product_price'] ? $column_values['neutral_price'] : '')
                    . ($data['export_flag_gross_price'] && $data['export_flag_product_price']
                        ? $column_values['gross_neutral_price']
                        : '')
                    . ($data['export_flag_price_charge']
                        ? $column_values['price_charge']
                        : '')
                    . ($data['export_flag_discount']
                        ? $column_values['discount']
                        : '')
                    . PHP_EOL;

                unset($column_values);
                unset($specific_prices);
                unset($discounts);

                // create file: only once I got quantity columns
                if (!isset($folder)) {
                    // create folder and CSV file and write part of headings
                    $folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
                    if (!file_exists($folder)) {
                        mkdir($folder, 0777, true);
                    }
                    $file_csv = $folder . '/' . $filename;
                    if ($start_record == 1) {
                        $myfile = fopen($file_csv, 'wb') or exit('Unable to open file!');
                        // utf8 blob for JS: '\uFEFF'
                        // utf8 blob for PHP: "\xEF\xBB\xBF"
                        // open file binary for UTF8: 'wb'

                        $records = ($data['export_flag_show_category'] ? $module->l('category') . ';' : '')
                            . Tools::ucfirst($module->l('model')) . ';'
                            . Tools::ucfirst($module->l('reference')) . ';'
                            . Tools::ucfirst($module->l('name')) . ';'
                            . Tools::ucfirst($module->l('color')) . ';'
                            . Tools::ucfirst($module->l('size')) . ';'
                            . Tools::ucfirst($module->l('other')) . ';'
                            . Tools::ucfirst($module->l('material')) . ';'
                            . Tools::ucfirst($module->l('brand')) . ';'
                            . Tools::ucfirst($module->l('ean 13')) . ';'
                            . Tools::ucfirst($module->l('segment')) . ';'
                            . Tools::ucfirst($module->l('for sale')) . ';'
                            . ($data['export_flag_stock_quantity']
                                ? Tools::ucfirst($module->l('stock qty')) . ';'
                                : '')
                            . ($data['export_flag_description_short']
                                ? Tools::ucfirst($module->l('short description')) . ';'
                                : '')
                            . ($data['export_flag_description'] ? Tools::ucfirst($module->l('description')) . ';' : '')
                            . ($data['export_flag_tags'] ? Tools::ucfirst($module->l('tags')) . ';' : '')
                            . $column_heading['quantity']
                            . ($data['export_flag_wholesale_price'] && $data['export_flag_product_price']
                                ? $column_heading['wholesale_neutral_price']
                                : '')
                            . ($data['export_flag_product_price'] ? $column_heading['neutral_price'] : '')
                            . ($data['export_flag_gross_price'] && $data['export_flag_product_price']
                                ? $column_heading['gross_neutral_price']
                                : '')
                            . ($data['export_flag_price_charge']
                                ? $column_heading['price_charge']
                                : '')
                            . ($data['export_flag_discount']
                                ? $column_heading['discount']
                                : '')
                            . PHP_EOL
                            . $records;
                        fwrite($myfile, "\xEF\xBB\xBF" . $records);
                    } else { // if ($partial_cnt == 500) {
                        $myfile = fopen($file_csv, 'ab') or exit('Unable to open file!');
                        fwrite($myfile, $records);
                    }
                    $records = '';
                    fclose($myfile);
                } else { // if ($partial_cnt == 500) {
                    $myfile = fopen($file_csv, 'ab') or exit('Unable to open file!');
                    fwrite($myfile, $records);
                    $records = '';
                    fclose($myfile);
                    // $partial_cnt = 0;
                }

                if ($partial_cnt >= $data['products_per_loop']) {
                    // if a product has more print options than the maximum $products_per_loop,
                    // counter must be incremented by one,
                    // or set to start_record + 1, to avoid infinite page reload loop !!
                    if ((int) $cnt <= (int) $start_record) {
                        $cnt = (int) $start_record + 1;
                    }

                    $return_values['errors'][] = $module->l('Please wait untill the end of the export loop');
                    $return_values['success'][] = '... '
                        . $module->l('exported products') . ': ' . $real_cnt . '/' . $tot_records . '. ';
                    $return_values['export_last_record'] = $cnt;
                    $return_values['export_tot_records'] = $tot_records;
                    return $return_values;
                }
            } // main product loop

            $baseUrl = _PS_BASE_URL_ . __PS_BASE_URI__;
            $return_values['success'][] = 'CSV ' . $module->l('file is ready for download! Click here')
                . ': <a href="' . $baseUrl . '/modules/mxb_tools/reports/' . $filename . '" target="_new">'
                . $filename . '</a> (tot products exported: ' . $real_cnt . ')';
        } else {
            // no products found
            $return_values['errors'][] = $module->l(
                'No products available for given parameters. Please check customer and category IDs'
            );
        }
        return $return_values;
    }

    public static function getShopName($for_filename = true)
    {
        $shop_name = Configuration::get('PS_SHOP_NAME');
        if ($for_filename && Tools::strlen($shop_name) > 20) {
            $shop_name = Tools::substr($shop_name, 0, 20);
        }
        return $shop_name;
    }
}
