<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
/*
 * https://devwl.pl/disable-all-modules-in-prestashop-using-sql/
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsMenu
{
    public static function getCommonModuleList($context = null, $activeOnly = true, $fullData = false)
    {
        $commonModules = [
            'ps_contactinfo',
            'ps_emailalerts',
            'ps_shoppingcart',
            'ps_mainmenu',
            'ps_banner',
            'ps_imageslider',
            'ps_searchbar',
            'ps_facetedsearch',
            'ps_categorytree',
            'ps_emailsubscription',
            'paypal',
            'ps_checkpayment',
            'ps_wirepayment',
            'blockreassurance',
        ];
        // sort($commonModules);
        $search = [];
        foreach ($commonModules as $cm) {
            $search[] = 'name="' . $cm . '"';
        }

        // collect mxb modules menu
        $sql = '
            SELECT *
            FROM `' . _DB_PREFIX_ . 'module`
            WHERE ' . implode(' OR ', $search) . '
            AND `active`=1
            ORDER BY `name` ASC';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$result || !isset($result[0])) {
            return null;
        }
        if ($context === null) {
            $context = Context::getContext();
        }
        $modules = [];
        $odd = true;
        foreach ($result as $mod) {
            if ($activeOnly) {
                if (!MxbToolsUtils::isModuleEnabled($mod['name'])) {
                    continue;
                }
            }
            $modules[$mod['name']] = [
                'id_module' => $mod['id_module'],
                'name' => $mod['name'],
                'version' => $mod['version'],
                'link' => $context->link->getAdminLink('AdminModules') . '&configure=' . $mod['name'],
                'odd' => (int) $odd,
                'enabled' => (int) ($activeOnly ? true : MxbToolsUtils::isModuleEnabled($mod['name'])),
            ];
            $odd = !$odd;
            if ($fullData) {
                $moduleData = self::getModuleFullData($mod);
                if ($moduleData) {
                    $modules[$mod['name']]['full_data'] = $moduleData;
                }
                /*
                $module = Module::getInstanceByName($mod['name']);
                if ($module) {
                    $modules[$mod['name']]['full_data'] = [];
                    $modules[$mod['name']]['full_data']['module_name'] = $mod['name'];
                    $modules[$mod['name']]['full_data']['name'] = $module->displayName;
                    $modules[$mod['name']]['full_data']['description'] = $module->description;
                    $modules[$mod['name']]['full_data']['url'] = '';
                    $modules[$mod['name']]['full_data']['thumbnail'] = '';
                }
                */
            }
        }

        // sort by module name
        uasort($modules, 'self::sortModuleByName');

        return $modules;
    }

    public static function getMxbModuleList($context = null, $activeOnly = true, $fullData = false)
    {
        // collect mxb modules menu
        $sql = '
            SELECT *
            FROM `' . _DB_PREFIX_ . 'module`
            WHERE `name` LIKE "mxb_%"
            AND `active`=1
            ORDER BY `name` ASC';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (!$result || !isset($result[0])) {
            return null;
        }
        if ($context === null) {
            $context = Context::getContext();
        }
        $modules = [];
        $odd = true;
        foreach ($result as $mod) {
            if ($activeOnly) {
                if (!MxbToolsUtils::isModuleEnabled($mod['name'])) {
                    continue;
                }
            }
            $modules[$mod['name']] = [
                'id_module' => $mod['id_module'],
                'name' => $mod['name'],
                'version' => $mod['version'],
                'link' => $context->link->getAdminLink('AdminModules') . '&configure=' . $mod['name'],
                'odd' => (int) $odd,
                'enabled' => (int) ($activeOnly ? true : MxbToolsUtils::isModuleEnabled($mod['name'])),
            ];
            $odd = !$odd;
            if ($fullData) {
                $moduleData = self::getModuleFullData($mod);
                if ($moduleData) {
                    $modules[$mod['name']]['full_data'] = $moduleData;
                }
                /*
                $module = Module::getInstanceByName($mod['name']);
                if ($module) {
                    $modules[$mod['name']]['full_data'] = [];
                    $modules[$mod['name']]['full_data']['module_name'] = $mod['name'];
                    $modules[$mod['name']]['full_data']['name'] = $module->displayName;
                    $modules[$mod['name']]['full_data']['description'] = $module->description;
                    $modules[$mod['name']]['full_data']['url'] = isset($module->mxb_url) ? $module->mxb_url : '';
                    $modules[$mod['name']]['full_data']['thumbnail'] = '';
                }
                */
            }
        }
        
        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleList', 'modules=' . print_r($modules, true));

        // sort by module name
        uasort($modules, 'self::sortModuleByName');

        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleList', 'after sortModuleByName: modules=' . print_r($modules, true));

        return $modules;
    }

    public static function getModuleFullData($module_data = null)
    {
        if (!$module_data) {
            return null;
        }
        $module_name = $module_data['name'];
        $id_module = $module_data['id_module'];
        /*
        if (!$id_module) {
            $id_module = Db::getInstance()->getValue('
                SELECT `id_module`
                FROM `' . _DB_PREFIX_ . 'module`
                WHERE `name` = "' . $module_name . '"
            ');
        }
        */
        $module = Module::getInstanceByName($module_name);
        /*
        MxbToolsDebug::writeLog(
            'MxbToolsMenu',
            'getModuleFullData()',
            'module=' . print_r($module, true)
        );
        */

        if ($module) {
            $thumbnailUrl = isset($module->thumbnailUrl)
                ? $module->thumbnailUrl
                : MxbToolsUtils::path2url(MxbToolsUtils::getModulePath($module_name)) . '/logo.png';

            return [
                'name' => $module_name,
                'id' => $id_module,
                'display_name' => $module->displayName,
                'description' => $module->description,
                'version' => $module->version,
                'url' => isset($module->documentationUrl) ? $module->documentationUrl : '',
                'thumbnail' => $thumbnailUrl,
                'enabled' => (int) MxbToolsUtils::isModuleEnabled($module_name),
                'groups' => MxbToolsMenu::getModuleGroupsByName($module_name),
                'hooks' => MxbToolsMenu::getModuleHooks($id_module),
            ];
        }
        return null;
    }

    public static function getMxbToolsMenu($context = null)
    {
        $module = Module::getInstanceByName('mxb_tools');
        //    $module->l(string)

        if ($context === null) {
            $context = Context::getContext();
        }

        $menu = [
            [
                'name' => $module->l('Configuration'),
                'link' => $context->link->getAdminLink('AdminMxbToolsConfiguration') . '&m=0',
            ],
            [
                'name' => $module->l('Tools'),
                'link' => $context->link->getAdminLink('AdminMxbToolsTools') . '&m=0',
            ],
            [
                'name' => $module->l('Calculators'),
                'link' => $context->link->getAdminLink('AdminMxbToolsCalculators') . '&m=0',
            ],
            [
                'name' => $module->l('Import'),
                'link' => $context->link->getAdminLink('AdminMxbToolsImport') . '&m=0',
            ],
            [
                'name' => $module->l('Export'),
                'link' => $context->link->getAdminLink('AdminMxbToolsExport') . '&m=0',
            ],
        ];

        $modules = self::getMxbModuleList($context);
        if (is_array($modules) && count($modules)) {
            $menuext = [
                'name' => $module->l('Modules'),
                'link' => 'mxb_modules_menu',
                'module' => [],
            ];
            foreach ($modules as $mod) {
                $menuext['submenu'][] = [
                    'name' => $module->l($mod['name']),
                    'link' => $mod['link'],
                ];
            }
            $menu[] = $menuext;
        }

        $modules = self::getCommonModuleList($context);
        if (count($modules)) {
            $menuext = [
                'name' => $module->l('Common modules'),
                'link' => 'common_modules_menu',
                'module' => [],
            ];
            foreach ($modules as $mod) {
                $menuext['submenu'][] = [
                    'name' => $module->l($mod['name']),
                    'link' => $mod['link'],
                ];
            }
            $menu[] = $menuext;
        }

        return $menu;
    }

    public static function getMxbModuleDownloadList($context = null, $notInstalledOnly = true)
    {
        // get json from mxbinformatica.it url ...
        $list = MxbToolsUtils::getJsonFileContent('https://www.mxbinformatica.it/ps_modules/mxb_module_list.json', true);
        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleDownloadList', 'notInstalledOnly=' . (int) $notInstalledOnly);
        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleDownloadList', 'ONLINE list=' . print_r($list, true));

        /*
        $list = [
            [
                'module_name' => 'mxb_tools',
                'name' => 'Mxb Tools',
                'link' => [
                    'it' => 'https://www.mxbinformatica.it/it/prestashop-mxb-tools/',
                    'en' => 'https://www.mxbinformatica.it/en/prestashop-mxb-tools/',
                ],
                'thumbnail' => 'https://www.mxbinformatica.it/public/ps_modules/thumbs/mxb_tools.jpg',
                'description' => [
                    'en' => 'module description',
                    'it' => 'descrizione modulo',
                ],
            ],
            [
                'module_name' => 'mxb_customblocks_one',
                'name' => 'Mxb Custom Blocks',
                'link' => [
                    'it' => 'https://www.mxbinformatica.it/it/prestashop-mxb-customblocks/',
                    'en' => 'https://www.mxbinformatica.it/en/prestashop-mxb-customblocks/',
                ],
                'thumbnail' => 'https://www.mxbinformatica.it/public/ps_modules/thumbs/mxb_customblocks.jpg',
                'description' => [
                    'en' => 'module description',
                    'it' => 'descrizione modulo',
                ],
            ],
        ];
        */
        if (!$list) {
            return null;
        }

        if ($context === null) {
            $context = Context::getContext();
        }

        $iso_code = strtolower($context->language->iso_code);
        if (strpos($iso_code, 'it') !== false) {
            $iso_code = 'it';
        } else {
            $iso_code = 'en';
        }

        // $installed = ($notInstalledOnly ? self::getMxbModuleList($context, false) : []);
        $installed_modules = self::getMxbModuleList($context, false);
        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleDownloadList', 'installed_modules=' . print_r($installed_modules, true));

        $modules = [];
        foreach ($list as $mod) {
            // if ($notInstalledOnly && !empty($installed_modules[$mod['module_name']])) {
            //     continue;
            // }
            $installed = false;
            if (!empty($installed_modules)) {
                if (!empty($installed_modules[$mod['name']])) {
                    // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleDownloadList', 'module ' . $mod['name'] . ' IS INSTALLED');
                    $installed = true;
                } else {
                    // check if there is a clone installed using the root name
                    if (!empty($mod['root_name'])) {
                        foreach ($installed_modules as $key => $val) {
                            if (strpos($key, $mod['root_name']) !== false) {
                                $installed = true;
                                break;
                            }
                        }
                    }
                }
            }
            if ($installed && $notInstalledOnly) {
                continue;
            }
            $modules[] = [
                'name' => $mod['name'],
                'display_name' => $mod['display_name'][$iso_code],
                'description' => $mod['description'][$iso_code],
                'version' => $mod['version'],
                'url' => $mod['url'][$iso_code],
                'thumbnail' => $mod['thumbnail'],
                'installed' => $installed,
            ];
        }
        
        // MxbToolsDebug::writeLog('mxb_tools', 'getMxbModuleDownloadList', 'modules=' . print_r($modules, true));

        return count($modules) ? $modules : null;
    }

    public static function getModuleGroupsByName($module_name = null)
    {
        $result = Db::getInstance()->executeS('
            SELECT mg.`id_group`, gl.`name` as name, m.`id_module`, m.`name` as module_name FROM `' . _DB_PREFIX_ . 'module` m
            LEFT JOIN `' . _DB_PREFIX_ . 'module_group` mg ON (m.`id_module` = mg.`id_module`
                AND `id_shop` = ' . (int) Context::getContext()->shop->id . ')
            LEFT JOIN `' . _DB_PREFIX_ . 'group` g ON (g.`id_group` = mg.`id_group`)
            LEFT JOIN `' . _DB_PREFIX_ . 'group_lang` gl ON (gl.`id_group` = g.`id_group`
                AND `id_lang` = ' . (int) Context::getContext()->language->id . ')
            WHERE m.`name` = "' . $module_name . '"
        ');
        if (!$result) {
            return [];
        }
        return $result;
    }

    public static function getModuleHooks($id_module)
    {
        $result = Db::getInstance()->executeS('
            SELECT h.`name`, h.`title`, h.`description`
            FROM `' . _DB_PREFIX_ . 'hook_module` hm
            LEFT JOIN `' . _DB_PREFIX_ . 'hook` h ON (h.`id_hook` = hm.`id_hook`)
            WHERE hm.`id_module` = ' . (int) $id_module . '
            AND `id_shop` = ' . (int) Context::getContext()->shop->id . '
            ORDER BY h.`name` ASC
        ');
        if (!$result) {
            return [];
        }

        return $result;
    }

    public static function sortModuleByName($a, $b)
    {
        if (isset($a['full_data'])) {
            return strcmp(strtolower($a['full_data']['display_name']), strtolower($b['full_data']['display_name']));
        }
        return false;
    }
}
