<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
// use PrestaShop\PrestaShop\Adapter\ServiceLocator;
// use PrestaShop\PrestaShop\Adapter\Presenter\Cart\CartPresenter;

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsPrice
{
    const COST_ONLY = 1;
    const COST_AND_TOTAL = 2;
    /**
     * requires discount in float format (20% -> 0.2)
     */
    public static function calculatePriceReduction($price, $discount)
    {
        return (float) $price - ((float) $price * $discount);
    }

    /**
     * reduction -> in % or float format
     * incidence -> in % format
     */
    public static function calculateReductionIncidence($reduction, $incidence)
    {
        if ($incidence === 100) {
            return $reduction;
        } elseif ($incidence === 0) {
            return 0;
        }

        return (float) $reduction - ((float) $reduction * (100 - (float) $incidence) / 100);
    }

    public static function calculateProductPriceReduction($price, $discount)
    {
        // if I have negative discount, that will be considered a charge !
        if ($discount !== 0) {
            return (float) $price - ((float) $price * $discount);
        } else {
            return (float) $price;
        }
    }

    public static function getProductFeatureValue($id_product, $feature_name, $id_lang)
    {
        return MxbToolsProduct::getProductFeatureValue($feature_name, $id_product, null, $id_lang);
    }

    public static function getProductTaxCalculator(
        $id_product = null,
        $id_address = null,
        $cur_cart = null,
        $context = null
    ) {
        if (!$context) {
            $context = Context::getContext();
        }
        if (!$cur_cart) {
            $cur_cart = $context->cart;
        }

        // no cart, no address, no tax calculator
        if (!$cur_cart || $cur_cart === null) {
            return null;
        }
        // LogDebug::writeLog('Gadgetcard', 'getProductTaxCalculator', 'CONTEXT: '.print_r($context, true));
        /*
            // without a cart, I cannot get address and then taxes: exit
            if (!Validate::isLoadedObject($cur_cart)) {
                LogDebug::writeLog('Gadgetcard', 'getProductTaxCalculator', 'CUR_CART NOT Validated: EXIT !!!');
                // return false;
            }
        */

        if (!$id_address) {
            $id_address = $cur_cart->{Configuration::get('PS_TAX_ADDRESS_TYPE')};
        }
        // $cart_id = $cur_cart->id;

        // retrieve address informations
        $address = Address::initialize($id_address, true);
        // $id_country = (int) $address->id_country;
        // $id_state = (int) $address->id_state;
        // $zipcode = $address->postcode;
        // LogDebug::writeLog('Gadgetcard', 'getProductTaxCalculator', 'TAX: id_cart='.$cart_id);

        if ($id_product !== null && (int) $id_product > 0) {
            $tax_manager = TaxManagerFactory::getManager(
                $address,
                Product::getIdTaxRulesGroupByIdProduct((int) $id_product, $context)
            );
        } else {
            $tax_manager = TaxManagerFactory::getManager(
                $address,
                (int) Configuration::get('PS_GIFT_WRAPPING_TAX_RULES_GROUP')
            );
        }

        return $tax_manager->getTaxCalculator();
    }

    /*
        getProductPrices : returns product prices in variuos format, with and without reductions
        Product::getPriceStatic(...) will be used to retrive the correct product price without group reduction
        public static function getPriceStatic(
            $id_product,
            $usetax = true,
            $id_product_attribute = null,
            $decimals = 6,
            $divisor = null,
            $only_reduc = false,
            $usereduc = true,
            $quantity = 1,
            $force_associated_tax = false,
            $id_customer = null,
            $id_cart = null,
            $id_address = null,
            &$specific_price_output = null,
            $with_ecotax = true,
            $use_group_reduction = true,
            Context $context = null,
            $use_customer_price = true,
            $id_customization = null
    */
    /**
     * sample functions: classes/product.php -> priceCalculations, getPriceStatic etc
     */
    public static function getProductPrices($id_product, $id_product_attribute, $quantity = 1, $id_customer = null)
    {
        // $prices: collect all real (not formatted) prices to format and put in smarty
        // taxes are added automatically by the function getPriceStatic if $use_tax is true
        $prices = [];
        $specific_price_output = null;
        $context = Context::getContext();
        if ($id_customer === null) {
            $id_customer = (int) $context->cookie->id_customer;
        }

        // $id_lang = Context::getContext()->language->id;
        if ((int) $id_customer > 0) {
            $taxCalculationMethod = Product::getTaxCalculationMethod($id_customer);
        } else {
            $taxCalculationMethod = PS_TAX_EXC;
        }

        // NOT duplicated values:
        // . $prices['full_price'] is the maximum price without quantity reduction;
        // . $prices['real_full_price'] is the gross price with quantity reduction

        // full price without ANY discount: get the product price without group reduction, for quantity = 1
        $prices['full_price'] = Product::getPriceStatic(
            (int) $id_product,
            $taxCalculationMethod != PS_TAX_EXC, // $usetax = true,
            $id_product_attribute,  // $id_product_attribute = null,
            6,      // ($taxCalculationMethod == PS_TAX_EXC ? 2 : 6),  // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            false,  // $usereduc = true,
            1,      // $quantity = 1,
            false,  // $force_associated_tax = false,
            null,   // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            true,   // $with_ecotax = true,
            false   // use_group_reduction: no
            // null, //Context $context = null,
            // true, // $use_customer_price = true,
            // null  // $id_customization = null
        );

        // get also wholesale_price
        $prices['real_wholesale_price'] = Product::getPriceStatic(
            (int) $id_product,
            false,  // $usetax = true,
            $id_product_attribute,  // $id_product_attribute = null,
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            false,  // $usereduc = true,
            1,      // $quantity = 1,
            false,  // $force_associated_tax = false,
            null,   // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            false,  // $with_ecotax = true,
            false,  // use_group_reduction: no
            null,   // Context $context = null,
            true,   // $use_customer_price = true,
            null    // $id_customization = null
        );

        // full price without ANY discount but quantity discount:
        // get the product price without group reduction, for quantity wanted
        $prices['real_full_price'] = Product::getPriceStatic(
            (int) $id_product,
            $taxCalculationMethod != PS_TAX_EXC,  // $usetax = true,
            $id_product_attribute,  // $id_product_attribute = null,
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            true,   // $usereduc = true,
            $quantity,  // $quantity = 1,
            false,  // $force_associated_tax = false,
            null,   // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            true,   // $with_ecotax = true,
            false   // use_group_reduction: no
            // null, // Context $context = null,
            // true, // $use_customer_price = true,
            // null  // $id_customization = null
        );

        // with gadgetcard product.php override,
        // price with group_reduction includes the special discount option
        $prices['real_price'] = Product::getPriceStatic(
            (int) $id_product,
            $taxCalculationMethod != PS_TAX_EXC,
            $id_product_attribute,  // ($taxCalculationMethod == PS_TAX_EXC ? 2 : 6),
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            true,   // $usereduc = true,
            $quantity,  // $quantity = 1,
            false,  // $force_associated_tax = false,
            $id_customer,   // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            true,   // $with_ecotax = true,
            true    // use_group_reduction: yes
            // null, //Context $context = null,
            // true, // $use_customer_price = true,
            // null  // $id_customization = null
        );
        // $prices['real_full_price'] = $prices['full_price'];

        // temporary set total price (print + product) as product price
        $prices['real_total_price'] = $prices['real_price'];
        $prices['real_full_total_price'] = $prices['real_full_price'];
        $prices['real_wholesale_total_price'] = $prices['real_wholesale_price'];
        // temporary set print price to zero: in case there is no print option linked to this product
        // $prices['real_print_price'] = 0;

        // totale costo relativo alla quantita selezionata
        $prices['quantity_wanted_real_total_price'] = (int) $quantity * (float) $prices['real_price'];
        $prices['quantity_wanted_real_price'] = (int) $quantity * (float) $prices['real_price'];

        return $prices;
    }

    /**
     * get specific price with group reduction
     * I want both price(float) and formatted price
     * Also I need to join print prices for the same quantities columns
     */
    public static function getQuantityDiscountPrices(
        $id_product,
        $id_product_attribute,
        // $discounts = null,
        $quantity_discounts = null,
        $id_group = null,
        $id_shop = null,
        $id_currency = null,
        $id_country = null,
        $id_customer = null,
        $context = null
    ) {
        $specific_price_output = null;
        $priceFormatter = new PriceFormatter();
        if (is_null($context)) {
            $context = Context::getContext();
        }
        if (is_null($id_customer)) {
            $id_customer = (int) $context->cookie->id_customer;
        }

        $taxCalculationMethod = Product::getTaxCalculationMethod($id_customer);
        // do I have to add tax ?
        if ($taxCalculationMethod != PS_TAX_EXC) {
            // get tax calculator
            $product_tax_calculator = MxbToolsPrice::getProductTaxCalculator($id_product, null, null, $context);

            // in fo tax calculator, force 'price tax excluded'
            if ($product_tax_calculator === null) {
                $taxCalculationMethod = PS_TAX_EXC;
            }
        }
        if (is_null($id_group)) {
            $id_group = (int) Group::getCurrent()->id;
        }

        // if no quantity discounts passed, try to get it
        if (is_null($quantity_discounts)) {
            if (is_null($id_shop)) {
                $id_shop = $context->shop->id;
            }
            if (is_null($id_currency)) {
                $id_currency = (int) $context->cookie->id_currency;
            }
            if (is_null($id_country)) {
                $id_country = (int) Tools::getCountry();
                /*
                $id_country = $id_customer
                    ? (int) Customer::getCurrentCountry($id_customer)
                    : (int) Tools::getCountry();
                */
            }
            $quantity_discounts = MxbToolsPrice::getQuantityDiscountsCharged(
                $id_product,
                $id_shop,
                $id_currency,
                $id_country,
                $id_group,
                $id_product_attribute,
                false,
                (int) $id_customer
            );
        }
        $quantity_discount_with_group_reduction = [];
        // $group = null; // Price::calculatePrintPriceReduction will set this value at the first calculation
        /*
        $quantity_discounts = SpecificPrice::getQuantityDiscounts(
            $id_product,
            $id_shop,
            $id_currency,
            $id_country,
            $id_group,
            $id_product_attribute,
            false,
            (int) $id_customer
        );
        */
        // get product price for quantity = 1 to pass it to print price charge calculation function
        $specificPrices = [];
        $specificPrices['from_quantity'] = 1;
        $specificPrices['real_full_price'] = Product::getPriceStatic(
            (int) $id_product,
            false,    // $usetax = true,
            $id_product_attribute,  // $id_product_attribute = null,
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            true,   // $usereduc = true,
            1,      // $quantity = 1,
            false,  // $force_associated_tax = false,
            $id_customer, // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            false,  // $with_ecotax = true,
            false,  // use_group_reduction: no
            null,   // Context $context = null,
            true,   // $use_customer_price = true,
            null    // $id_customization = null
        );

        $specificPrices['real_price'] = Product::getPriceStatic(
            (int) $id_product,
            false,  // $usetax = true,
            $id_product_attribute, // $id_product_attribute = null,
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            false,  // $usereduc = true,
            1,      // $quantity = 1,
            false,  // $force_associated_tax = false,
            $id_customer, // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            true,   // $with_ecotax = true,
            true,   // use_group_reduction: no
            null,   // Context $context = null,
            true,   // $use_customer_price = true,
            null    // $id_customization = null
        );

        $specificPrices['real_wholesale_price'] = $specificPrices['real_full_price']; // TODO: get wholesale price
        // do I have to add tax ?
        if ($taxCalculationMethod != PS_TAX_EXC && $product_tax_calculator != null) {
            try {
                $specificPrices['real_full_price'] =
                    $product_tax_calculator->addTaxes($specificPrices['real_full_price']);
                $specificPrices['real_price'] = $product_tax_calculator->addTaxes($specificPrices['real_price']);
            } catch (Exception $e) {
            }
        }

        // format prices
        $specificPrices['full_price'] = $priceFormatter->format((float) $specificPrices['real_full_price']);
        $specificPrices['price'] = $priceFormatter->format($specificPrices['real_price']);
        $specificPrices['wholesale_price'] = $priceFormatter->format($specificPrices['real_wholesale_price']);
        // this is the specific prices new array with all the collected informations
        $quantity_discount_with_group_reduction[] = $specificPrices;

        foreach ($quantity_discounts as $quantity_discount) {
            $specificPrices = [];
            $specificPrices['from_quantity'] = $quantity_discount['from_quantity'];
            $specificPrices['real_full_price'] = $quantity_discount['price'];
            $specificPrices['real_wholesale_price'] = $quantity_discount['wholesale_price'];
            $specificPrices['real_price'] = Product::getPriceStatic(
                (int) $id_product,
                false,    // $usetax = true,
                $id_product_attribute,  // $id_product_attribute = null,
                6,      // $decimals = 6,
                null,   // $divisor = null,
                false,  // $only_reduc = false,
                true,   // $usereduc = true,
                (int) $quantity_discount['from_quantity'],  // $quantity = 1,
                false,  // $force_associated_tax = false,
                $id_customer, // $id_customer = null,
                null,   // $id_cart = null,
                null,   // $id_address = null,
                $specific_price_output, // &$specific_price_output = null,
                true,   // $with_ecotax = true,
                true,   // use_group_reduction: no
                null,   // Context $context = null,
                true,   // $use_customer_price = true,
                null
            );

            // do I have to add tax ?
            if ($taxCalculationMethod != PS_TAX_EXC && $product_tax_calculator != null) {
                try {
                    $specificPrices['real_full_price'] = $product_tax_calculator->addTaxes(
                        $specificPrices['real_full_price']
                    );
                    $specificPrices['real_price'] = $product_tax_calculator->addTaxes(
                        $specificPrices['real_price']
                    );
                } catch (Exception $e) {
                }
            }

            // format prices
            $specificPrices['full_price'] = $priceFormatter->format((float) $specificPrices['real_full_price']);
            $specificPrices['price'] = $priceFormatter->format($specificPrices['real_price']);
            $specificPrices['wholesale_price'] = $priceFormatter->format($specificPrices['real_wholesale_price']);

            // this is the specific prices new array with all the collected informations
            $quantity_discount_with_group_reduction[] = $specificPrices;
        } // foreach quantity_discount

        return $quantity_discount_with_group_reduction;
    }

    /**
     * create product discount collection for product and print discounts
     *
     * $specific_price_discount_float -> specific_price_discounts from rules (category discounts...) in float format
     */
    public static function getProductDiscountsCollection(
        $id_product = null,
        $id_group = null,
        $id_customer = null,
        $id_lang = null,
        $specific_price_discount_float = 0
    ) {
        if (is_null($id_product)) {
            return null;
        }
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }
        if (is_null($id_group)) {
            $id_group = (int) Group::getCurrent()->id;
        }
        if (is_null($id_customer)) {
            $id_customer = (int) Context::getContext()->cookie->id_customer;
        }
        // prepare special discount variabile and try to get group special_discount_perc
        // to know if the user can use the special discount instead the group discount
        // $special_discount = -1;
        // $special_discount = 0;
        // $special_discount_name = '';
        // $special_discount_incidence = 0;
        // $group_exclusive = 0;
        // $method = 0;
        $total_charge = 0;
        $total_discount = 0;

        // group reduction
        $group_reduction = GroupReduction::getValueForProduct((int) $id_product, (int) $id_group);
        if ($group_reduction === false) {
            $group_reduction = Group::getReduction((int) $id_customer) / 100;
        }
        if ($group_reduction === false || $group_reduction == 0) {
            $group_reduction = (($reduc = Group::getReductionByIdGroup($id_group)) != 0) ? ($reduc / 100) : 0;
            // $group_reduction = 0;
        }

        // NOTE: group_reduction is in float 0-1 format (value% / 100)
        // prepare the total discount variabile
        if ($group_reduction > 0) {
            $total_discount = $group_reduction;
        } else {
            $total_charge = $group_reduction;
        }
        // the real group discount is stored here. ** $group_discount will be set to 0 if there is a special discount
        $real_group_reduction = $group_reduction;

        // manage discounts: select the highest, charges ...
        $total_product_discount = 0;
        // $total_print_option_discount = 0;
        $total_product_charge = 0;
        // $total_print_option_charge = 0;

        if ($specific_price_discount_float > 0) {
            // for product: use group_discount in any case if group_discount is < 0 (charge)
            if ($group_reduction < 0) {
                // $total_product_discount = $group_reduction;
                $total_product_charge = $group_reduction;
                $total_product_discount += $specific_price_discount_float;
            } else {  // use the highest discount
                $total_product_discount = $total_discount > $specific_price_discount_float
                    ? $total_discount
                    : $specific_price_discount_float;
                $total_product_charge = $total_charge;
            }
        } else {
            $total_product_discount = (float) $total_discount;
            $total_product_charge = (float) $total_charge;
        }

        if ($total_charge < 0) {
            $total_charge *= -1;
        }
        if ($total_product_charge < 0) {
            $total_product_charge *= -1;
        }
        /*
        $total_print_option_discount = $print_options_apply_group_discount ? $total_discount : 0;
        $total_print_option_charge = $print_options_apply_group_discount ? $total_charge : 0;
        */

        $discounts = [];
        $discounts['group_reduction'] = $group_reduction;
        $discounts['real_group_reduction'] = $real_group_reduction;
        // $discounts['special_discount'] = $special_discount;
        $discounts['total_discount'] = $total_discount;   // group + special
        $discounts['total_charge'] = $total_charge;       // group + special
        $discounts['total_product_discount'] = $total_product_discount; // including specific_price_discount_float case
        $discounts['total_product_charge'] = $total_product_charge; // including specific_price_discount_float case
        // $discounts['total_print_option_discount'] = $total_print_option_discount; // same of total_discount, for now
        // $discounts['total_print_option_charge'] = $total_print_option_charge; // same of total_charge, for now
        // $discounts['special_discount_name'] = $special_discount_name;
        // $discounts['special_discount_incidence'] = $special_discount_incidence;
        // $discounts['group_exclusive'] = $group_exclusive;
        return json_decode(json_encode((object) $discounts), false);
    }

    public static function getQuantityDiscounts(
        $id_product,
        $id_shop,
        $id_currency,
        $id_country,
        $id_group,
        $id_product_attribute,
        $all_combinations = false,
        $id_customer = 0
    ) {
        $quantity_discounts = SpecificPrice::getQuantityDiscounts(
            $id_product,
            $id_shop,
            $id_currency,
            $id_country,
            $id_group,
            $id_product_attribute,
            $all_combinations,
            $id_customer
        );
        // $wholesale_price_charged = 0;
        $specific_price_output = null;

        // TODO: get wholesale_price
        $wholesale_price = Product::getPriceStatic(
            (int) $id_product,
            false,  // $usetax = true,
            $id_product_attribute, // $id_product_attribute = null,
            6,      // $decimals = 6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            false,  // $usereduc = true,
            1,      // $quantity = 1,
            false,  // $force_associated_tax = false,
            null,   // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            false,  // $with_ecotax = true,
            false,  // use_group_reduction: no
            null,   // Context $context = null,
            true,   // $use_customer_price = true,
            null    // $id_customization = null
        );

        // check for fixed price
        foreach ($quantity_discounts as &$discount) {
            $discount['wholesale_price'] = $wholesale_price;
        }

        return $quantity_discounts;
    }

    /*
     * SpecificPrice::getQuantityDiscounts()
     * call it from here to add price charge, if any
     */
    public static function getQuantityDiscountsCharged(
        $id_product,
        $id_shop,
        $id_currency,
        $id_country,
        $id_group,
        $id_product_attribute = null,
        $all_combinations = false,
        $id_customer = 0
    ) {
        $quantity_discounts = MxbToolsPrice::getQuantityDiscounts(
            $id_product,
            $id_shop,
            $id_currency,
            $id_country,
            $id_group,
            $id_product_attribute,
            $all_combinations,
            $id_customer
        );
        $specific_price_output = null;
        // add price charge and column discount
        foreach ($quantity_discounts as &$discount) {
            $discount['price'] = Product::getPriceStatic(
                (int) $id_product,
                false,  // $usetax = true,
                $id_product_attribute,  // $id_product_attribute = null,
                6,      // $decimals = 6,
                null,   // $divisor = null,
                false,  // $only_reduc = false,
                false,  // $usereduc = true,
                (int) $discount['from_quantity'], // $quantity = 1,
                false,  // $force_associated_tax = false,
                $id_customer, // $id_customer = null,
                null,   // $id_cart = null,
                null,   // $id_address = null,
                $specific_price_output, // &$specific_price_output = null,
                true,   // $with_ecotax = true,
                false,  // use_group_reduction: no
                null,   // Context $context = null,
                true,   // $use_customer_price = true,
                null    // $id_customization = null
            );
        }
        return $quantity_discounts;
    }
}
