<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

/** Class class MxbToolsProduct */
class MxbToolsProduct extends ObjectModel
{
    /**
     * get product data
     * in convenient format for mxb_tools queries
     */
    public static function getProduct($id_lang = null, $id_product = null)
    {
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }

        $sql = '
            SELECT a.id_product, a.reference AS product_reference, al.id_product, al.name
            FROM ' . _DB_PREFIX_ . 'product a
            LEFT JOIN ' . _DB_PREFIX_ . 'product_lang al ON (al.id_product = a.id_product)
            WHERE al.id_lang = ' . (int) $id_lang;
        if (!is_null($id_product) && $id_product > 0) {
            $sql .= ' AND a.id_product = ' . (int) $id_product;
        }
        $sql .= ' ORDER BY al.name ASC';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        // cast to objects and create "caption" value
        foreach ($result as &$ds) {
            // $ds['caption'] = $ds['name'] . ' - ' . $ds['number_of_colors'] . ' colors';
            $ds = json_decode(json_encode((object) $ds), false);
            // $ds->name = $ds->name . ' - ' . $ds->number_of_colors . ' colors';
            // $ds['name'] = $ds['name'] . ' ' . $ds['number_of_colors'];
        }
        return $result;
    }

    /**
     * get product attributes by id_product (all attributes of a product)
     * or by attribute reference (one specific attribute)
     * in convenient format for queries
     */
    public static function getProductAttributes(
        $id_lang = null,
        $id_product = null,
        $reference = null,
        $obj_format = true,
        $attribute_filter_array = null,
        $on_sale = false,
        $getImages = false
    ) {
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }
        if (!is_null($id_product)) {
            $where = 'WHERE pa.id_product = ' . (int) $id_product;
        } elseif (!is_null($reference)) {
            $where = 'WHERE pa.reference = \'' . $reference . '\'';
        }
        if ($on_sale) {
            $where .= ' AND pa.location <> ""';
        }

        if (!is_null($attribute_filter_array) && is_array($attribute_filter_array)) {
            // $size_array = array('3XS','XXXS','XXS','XS','S','M','L','XL','XXL','XXXL','3XL');
            $where .= ' AND (';
            foreach ($attribute_filter_array as $key => $attr) {
                $where .= ($key > 0 ? ' OR ' : ' ') . ' al.`group_name` = \'' . $attr . '\'';
            }
            $where .= ')';
        }

        $sql = '
            SELECT pa.*,
                al.id_attribute,
                pa.id_product_attribute,
                pa.`reference` AS product_attribute_reference,
                sa.quantity AS stock_quantity,
                GROUP_CONCAT(ag.`id_attribute_group` SEPARATOR \'|\') AS id_attribute_group_list,
                agl.`name` AS group_name,
                al.`name` AS attribute_name,
                att.`id_attribute`,
                att.`position`,
                CONCAT(pa.`reference`, \': \', GROUP_CONCAT(al.`name` SEPARATOR \', \')) AS caption,
                GROUP_CONCAT(al.`name` SEPARATOR \'|\') AS attribute_name_list,
                GROUP_CONCAT(al.`id_attribute` SEPARATOR \'|\') AS attribute_id_list,
                pl.`link_rewrite`,
                pl.`name` as product_name
            FROM ' . _DB_PREFIX_ . 'product_attribute pa
            LEFT JOIN `' . _DB_PREFIX_ . 'product` p ON p.`id_product` = pa.`id_product`
            LEFT JOIN `' . _DB_PREFIX_
            . 'product_lang` pl ON (p.`id_product` = pl.`id_product` AND pl.`id_lang` = ' . (int) $id_lang . ')
            LEFT JOIN `'
            . _DB_PREFIX_
            . 'product_attribute_combination` pac ON pac.`id_product_attribute` = pa.`id_product_attribute`
            LEFT JOIN `'
            . _DB_PREFIX_ . 'attribute` att ON att.`id_attribute` = pac.`id_attribute`
            LEFT JOIN `'
            . _DB_PREFIX_ . 'attribute_group` ag ON ag.`id_attribute_group` = att.`id_attribute_group`
            LEFT JOIN `'
            . _DB_PREFIX_ . 'attribute_lang` al ON (att.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '
            . (int) $id_lang . ')
            LEFT JOIN `'
            . _DB_PREFIX_
            . 'attribute_group_lang` agl ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '
            . (int) $id_lang . ')
            LEFT JOIN `'
            . _DB_PREFIX_ . 'stock_available` sa ON (pa.id_product_attribute = sa.id_product_attribute)
            ' . $where . '
            AND al.id_lang = ' . (int) $id_lang . '
            GROUP BY pa.`id_product_attribute`
            ORDER BY pa.`id_product_attribute`, caption';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if ((bool) $getImages && count($result)) {
            $ids = [];
            foreach ($result as $product_attribute) {
                $ids[] = (int) $product_attribute['id_product_attribute'];
            }
            $imgs = Db::getInstance()->executeS('
                SELECT pai.`id_image`, pai.`id_product_attribute`, il.`legend`
                FROM `' . _DB_PREFIX_ . 'product_attribute_image` pai
                LEFT JOIN `' . _DB_PREFIX_
                . 'image_lang` il ON (il.`id_image` = pai.`id_image` AND il.`id_lang` = '
                . (int) $id_lang . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'image` i ON (i.`id_image` = pai.`id_image`)
                WHERE pai.`id_product_attribute` IN (' . implode(', ', $ids) . ') AND il.`id_lang` = '
                . (int) $id_lang . '
                ORDER by pai.`id_product_attribute`, i.`position`');
            $link = Context::getContext()->link;
            if ($imgs) {
                foreach ($result as &$product_attribute) {
                    $product_attribute['images'] = [];
                    foreach ($imgs as &$img) {
                        if ($img['id_product_attribute'] == $product_attribute['id_product_attribute']) {
                            $img['link_small'] = $link->getImageLink(
                                !empty($product_attribute['link_rewrite'])
                                    ? $product_attribute['link_rewrite']
                                    : $product_attribute['product_name'],
                                (int) $img['id_image'],
                                ImageType::getFormattedName('small')
                            ); // 'small_default');
                            $img['link_medium'] = $link->getImageLink(
                                !empty($product_attribute['link_rewrite'])
                                    ? $product_attribute['link_rewrite']
                                    : $product_attribute['product_name'],
                                (int) $img['id_image'],
                                ImageType::getFormattedName('medium')
                            ); // 'medium_default');
                            $img['link_large'] = $link->getImageLink(
                                !empty($product_attribute['link_rewrite'])
                                    ? $product_attribute['link_rewrite']
                                    : $product_attribute['product_name'],
                                (int) $img['id_image'],
                                ImageType::getFormattedName('large')
                            ); // 'large_default');
                            $product_attribute['images'][] = $img;
                        } elseif (count($product_attribute['images'])) {
                            break;
                        }
                    }
                }
            }
        }

        if ($obj_format) {
            // cast array to object (and create "caption" value if needed)
            foreach ($result as &$ds) {
                $ds = json_decode(json_encode((object) $ds), false);
            }
        }
        return $result;
    }

    /**
     * get product attributes using product reference
     * in convenient format for mxb_tools queries
     */
    public static function getProductAttributeByReference($id_lang = null, $reference = null, $obj_format = true)
    {
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }

        $sql = '
            SELECT
                pa.*,
                al.name,
                al.id_attribute,
                pa.id_product_attribute,
                pa.`reference` AS product_attribute_reference,
                sa.quantity AS stock_quantity,
                ag.`id_attribute_group`,
                agl.`name` AS group_name,
                al.`name` AS attribute_name,
                at.`id_attribute`,
                at.`position`,
                CONCAT(pa.`reference`, \': \', GROUP_CONCAT(al.`name` SEPARATOR \', \')) AS caption,
                GROUP_CONCAT(al.`name` SEPARATOR \'|\') AS attribute_name_list,
                GROUP_CONCAT(al.`id_attribute` SEPARATOR \'|\') AS attribute_id_list
            FROM ' . _DB_PREFIX_ . 'product_attribute pa
            LEFT JOIN `' . _DB_PREFIX_
            . 'product_attribute_combination` pac ON pac.`id_product_attribute` = pa.`id_product_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute` at ON at.`id_attribute` = pac.`id_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group` ag ON ag.`id_attribute_group` = at.`id_attribute_group`
            LEFT JOIN `' . _DB_PREFIX_
            . 'attribute_lang` al ON (at.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '
            . (int) $id_lang . ')
            LEFT JOIN `' . _DB_PREFIX_
            . 'attribute_group_lang` agl ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '
            . (int) $id_lang . ')
            LEFT JOIN `' . _DB_PREFIX_ . 'stock_available` sa ON (pa.id_product_attribute = sa.id_product_attribute)
            WHERE pa.reference = \'' . $reference . '\'
            AND al.id_lang = ' . (int) $id_lang . '
            GROUP BY pa.`id_product_attribute`
            ORDER BY pa.`id_product_attribute`, caption';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (isset($result[0])) {
            return $obj_format ? json_decode(json_encode((object) $result[0]), false) : $result[0];
        }
        return false;
    }

    /**
     * get product attributes by id_product
     */
    public static function getAllProductAttributes($id_lang = null, $id_product = null)
    {
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }

        $product = new Product((int) $id_product);
        if (!Validate::isLoadedObject($product)) {
            return false;
        }
        return $product->getAttributesGroups($id_lang);
    }

    /**
     * check if a product must be hidden
     * check product price with discount and for major quantity (should be the lower product price)
     *
     * @return:
     *   price = 0     -> product has final price < 0.01
     *   price = -1   -> product is an additional cost !
     *   price = float  -> lowest product price
     */
    public static function isHiddenProduct(
        $id_product,
        $id_product_attribute,
        $lowest_price = false,
        $id_customer = null
    ) {
        $specific_price_output = null;
        $quantity = ($lowest_price ? 9999999 : 1);
        $price = Product::getPriceStatic(
            (int) $id_product,
            false,  // $usetax = true,
            $id_product_attribute,  // $id_product_attribute = null,
            6,
            null,   // $divisor = null,
            false,  // $only_reduc = false,
            true,   // $usereduc = true,
            $quantity,  // $quantity = 1,
            false,  // $force_associated_tax = false,
            $id_customer, // $id_customer = null,
            null,   // $id_cart = null,
            null,   // $id_address = null,
            $specific_price_output, // &$specific_price_output = null,
            false,  // $with_ecotax = true,
            true    // use_group_reduction: no
        );

        // product not available for buy or show in lists
        if ((float) $price < 0.01) {
            $price = 0;
        }
        if ((float) $price === 0) {
            return 0;
        }
        return $price;
    }

    /**
     * get feature value with $name for a specific product
     * use $product if not null, or get product by id_product
     */
    public static function getProductFeatureValue(
        $feature_name = 'mark segment',
        $id_product = 0,
        $product = null,
        $id_lang = null
    ) {
        if ($id_product === 0 && !is_null($product)) {
            if (is_array($product) && isset($product['id_product'])) {
                $id_product = $product['id_product'];
            } elseif (is_object($product) && property_exists($product, 'id_product')) {
                $id_product = $product->id_product;
            } elseif (is_object($product) && property_exists($product, 'id')) {
                $id_product = $product->id;
            } else {
                return null;
            }
        }
        if ($id_product > 0) {
            return MxbToolsProduct::getProductFeatureValueByIdProduct($feature_name, $id_product, $id_lang);
        }
        /*
            if (is_null($id_lang))
              $id_lang = Context::getContext()->language->id;
            if (is_null($product))
              $product = new Product($id_product);

             $features = $myproduct->getFrontFeatures($id_lang);
             foreach ($features as $key => $feature) {
              // if a feature with name = $ws_tag string, return its value
              if (strtolower($feature_name) == strtolower($feature['name']))
                return $feature['value'];
            }
        */
        return null;
    }

    public static function getProductFeatureValueByIdProduct(
        $feature_name = 'mark segment',
        $id_product = 0,
        $id_lang = null
    ) {
        if ($id_product === 0) {
            return null;
        }
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }

        $sql = '
            SELECT
                fl.id_feature as id_feature,
                fp.id_feature_value as id_feature_value,
                fp.id_product as id_product
            FROM ' . _DB_PREFIX_ . 'feature_lang fl
            LEFT JOIN ' . _DB_PREFIX_ . 'feature_product fp ON (fp.id_feature = fl.id_feature AND fp.id_product = '
            . (int) $id_product . ')
            WHERE fl.name = \'' . $feature_name . '\'
            AND fl.id_lang = ' . (int) $id_lang;
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (!isset($result[0])) {
            return null;
        }

        // $id_feature = (int) $result[0]['id_feature'];
        $id_feature_value = (int) $result[0]['id_feature_value'];
        if ($id_feature_value === 0) {
            return null;
        }

        $sql = '
            SELECT fvl.value as value
            FROM ' . _DB_PREFIX_ . 'feature_value_lang fvl
            WHERE fvl.id_feature_value = ' . $id_feature_value . '
            AND fvl.id_lang = ' . (int) $id_lang;
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        if (isset($result[0])) {
            return $result[0]['value'];
        }
        return null;
    }

    /**
     * get information from product_attribute -> location = "C" (as imported from PF feeds)
     */
    public static function isSaleProduct($id_product = 0, $id_product_attribute = 0)
    {
        // TODO !!!! Keep debug call to justify function parameters ...
        MxbToolsDebug::writeLog(
            'mxb_tools',
            'MxbToolsProduct::isSaleProduct',
            'id_product: ' . $id_product
                . ', id_product_attribute: ' . $id_product_attribute
                . ' :: function not yet developed!'
        );
        return false;
    }

    /**
     * scan attribute array to find "GB quantity" for USB
     */
    public static function getAttributeUsbDimension($attribute_name)
    {
        // remove last space and set lower case
        $name = Tools::strtolower(trim($attribute_name));
        $unit = 0;
        $pos = strpos($name, 'gb');
        if ($pos !== false) {
            $unit = 1;
        } else {
            $pos = strpos($name, 'tb');
            if ($pos !== false) {
                $unit = 1000;
            }
        }

        if ($unit > 0) {
            $dim = trim(Tools::substr($name, 0, $pos));
            $arr = explode(' ', $dim);
            $dim = (int) end($arr);
            if ($dim > 0) {
                $gb_quantity = (int) ($dim * $unit);

                return $gb_quantity;
            }
        }

        /*
            if ($pos !== false) {
              if ($unit === 0)
                $unit = 1000;
              $dim = trim(substr($name, 0, $pos));
              $arr = explode(' ', $dim);
              $dim = intval(end($arr));
              if ($dim > 0) {
                $gb_quantity = (int) ($dim * $unit);
                 return $gb_quantity;
              }
            }
        */
        return 0;
    }

    public static function getUsbDimensionString($gb_quantity)
    {
        // return 'USB ' . $gb_quantity . ' GB';
        return ' ' . $gb_quantity . ' GB';
    }

    /**
     * copyed from Product->getCombinationImages()
     */
    /*
    public static function getCombinationImages(
        $id_product,
        $id_product_attribute = 0,
        $id_lang = null,
        $link_rewrite = null
    ) {
        // Not yet used !!!! Keep debug call to justify function parameters: link_rewrite
        MxbToolsDebug::writeLog(
            'mxb_tools',
            'MxbToolsProduct::getCombinationImages',
            'id_product: ' . $id_product
                . ', id_product_attribute: ' . $id_product_attribute
                . ', link_rewrite: ' . $link_rewrite
                . ' :: function not yet developed!'
        );

        if (!Combination::isFeatureActive()) {
            return false;
        }
        $product_attributes = Db::getInstance()->executeS('
            SELECT `id_product_attribute`
            FROM `' . _DB_PREFIX_ . 'product_attribute`
            WHERE `id_product` = ' . (int) $id_product
            . ((int) $id_product_attribute > 0 ? ' AND id_product_attribute = ' . $id_product_attribute : ''));

        if (!$product_attributes) {
            return false;
        }
        if (is_null($id_lang)) {
            $id_lang = Context::getContext()->language->id;
        }
        $ids = [];
        foreach ($product_attributes as $product_attribute) {
            $ids[] = (int) $product_attribute['id_product_attribute'];
        }

        $result = Db::getInstance()->executeS('
            SELECT pai.`id_image`, pai.`id_product_attribute`, il.`legend`
            FROM `' . _DB_PREFIX_ . 'product_attribute_image` pai
            LEFT JOIN `' . _DB_PREFIX_ . 'image_lang` il ON (il.`id_image` = pai.`id_image`)
            LEFT JOIN `' . _DB_PREFIX_ . 'image` i ON (i.`id_image` = pai.`id_image`)
            WHERE pai.`id_product_attribute` IN (' . implode(', ', $ids) . ') AND il.`id_lang` = '
            . (int) $id_lang . ' ORDER by i.`position`');
        if (!$result) {
            return false;
        }

        $images = [];
        foreach ($result as $row) {
            $images[$row['id_product_attribute']][] = $row;
        }
        return $images;
    }
    */

    /**
     * insert products of a category to a new category
     */
    public function addCategoryProductsToCategory(
        $id_category_source = 0,
        $id_category_target = 0,
        $limit = 0,
        $id_product = 0
    ) {
        // $gcodes = MxbToolsCodes::getCodes();
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsProduct::addCategoryProductsToCategory',
            'id_category_source=' . $id_category_source
                . ', id_category_target=' . $id_category_target
                . ', limit=' . $limit
                . ', id_product=' . $id_product
        );
        */
        if ($id_category_source === 0 || $id_category_target === 0) {
            return ['records' => -1, 'tot_records' => 0, 'id' => 0];
        }

        if ($limit > 0 && $id_product === 0) {
            // $category = new Category($id_category_source);
            // $products = $category->getProducts($this->context->language->id);
            /*
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product p
                    LEFT_JOIN ' . _DB_PREFIX_ . 'category_product cp ON (p.id_product = cp.id_product)
                    WHERE ' . MxbToolsCodes::sqlFilterProductsOnly('p'). '
                  AND cp.id_category = ' . $id_category_source;
            */
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'category_product
                WHERE id_category = ' . $id_category_source;
            $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        } else {
            $tot_records = 0;
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsProduct::addCategoryProductsToCategory',
            'tot_records=' . $tot_records
        );
        */
        $sql = '
            SELECT id_product
            FROM ' . _DB_PREFIX_ . 'category_product
            WHERE id_category = ' . $id_category_source .
            ($id_product > 0 ? ' AND id_product > ' . $id_product : '') . '
            ORDER BY id_product ASC'; // . ($limit > 0 ? ' LIMIT ' . $limit : '');
        $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        $last_id_product = $id_product;
        $inserted_records = 0;

        if ($products) {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsProduct::addCategoryProductsToCategory',
                'products collected=' . count($products)
            );
            */
            // get all languages ids
            // $languages = Language::getLanguages();
            /*
            $sql = '
                SELECT id_lang
                FROM ' . _DB_PREFIX_ . 'lang
                ORDER BY id_lang';
                $languages = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            */

            foreach ($products as $product) {
                set_time_limit(300);
                // check if product is already into this category
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'category_product
                    WHERE id_category = ' . $id_category_target . '
                    AND id_product = ' . (int) $product['id_product'];
                $already_present = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql) > 0;
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsProduct::addCategoryProductsToCategory',
                    'product ' . $product['id_product'] . ': already_present=' . $already_present
                );
                */
                $last_id_product = (int) $product['id_product'];
                if (!$already_present) {
                    // insert product into new category
                    $sql = '
                        INSERT INTO ' . _DB_PREFIX_ . 'category_product (id_product, id_category)
                        VALUES (' . (int) $product['id_product'] . ', ' . (int) $id_category_target . ')';
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools',
                        'MxbToolsProduct::addCategoryProductsToCategory',
                        'product ' . $product['id_product'] . ': linkin to new ctegory... sql: ' . $sql
                    );
                    */
                    Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools',
                        'MxbToolsProduct::addCategoryProductsToCategory',
                        'product ' . $product['id_product'] . ': linked !'
                    );
                    */
                    ++$inserted_records;
                    if ($limit > 0 && $inserted_records >= ($limit - 1)) {
                        break;
                    }
                }
            }
        } else {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsProduct::addCategoryProductsToCategory',
                'NO PRODUCTS FOR CATEGORY SOURCE !!'
            );
            */
            $inserted_records = -1; // no products
        }

        return ['records' => $inserted_records, 'tot_records' => $tot_records, 'id' => $last_id_product];
    }

    /**
     * change product default category
     */
    public function changeDefaultCategoryProduct(
        $id_category_source = 0,
        $id_category_target = 0,
        $product_reference = 0,
        $saveFlag = false,
        $limit = 0,
        $id_product = 0
    ) {
        $success = [];
        $id_lang = (int) Context::getContext()->language->id;
        $tot_records = 0;

        if ((int) $id_category_source === 0 || (int) $id_category_source === (int) $id_category_target) {
            return ['records' => -1, 'tot_records' => 0, 'id' => 0];
        }

        $where = '';
        // $joinCP = '';
        if (!empty($product_reference)) {
            $where .= ' AND p.reference LIKE "' . $product_reference . '" ';
        }
        if ((int) $id_category_target > 0) {
            // if ($category->id_parent != 0 && !$category->is_root_category) {
            // $where .= ' AND c.id_parent > 0 AND c.is_root_category = 0 '
            $where .= ' AND c.id_category = ' . (int) $id_category_target;
        }

        if ((int) $limit > 0 && (int) $id_product === 0) {
            $sql = '
                SELECT p.id_product
                FROM ' . _DB_PREFIX_ . 'product p
                INNER JOIN ' . _DB_PREFIX_ . 'category_product cp ON (cp.id_product = p.id_product)
                INNER JOIN ' . _DB_PREFIX_ . 'category c ON (c.id_category = cp.id_category AND c.id_parent > 0 AND c.is_root_category = 0)
                WHERE p.id_category_default = ' . $id_category_source
                . $where . '
                GROUP BY p.id_product
                ORDER BY p.id_product';
            // $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsProduct::changeDefaultCategoryProduct',
                'result=' . print_r($result, true)
                . ', sql=' . $sql
            );
            */
            $tot_records = (is_array($result)
                ? count($result)
                : 0);
        } else {
            $tot_records = 0;
        }

        $sql = '
            SELECT p.id_product, c.id_category, p.reference, cl.name as category_name
            FROM ' . _DB_PREFIX_ . 'product p
            INNER JOIN ' . _DB_PREFIX_ . 'category_product cp ON (cp.id_product = p.id_product)
            INNER JOIN ' . _DB_PREFIX_ . 'category c ON (c.id_category = cp.id_category AND c.id_parent > 0 AND c.is_root_category = 0)
            INNER JOIN ' . _DB_PREFIX_ . 'category_lang cl ON (c.id_category = cl.id_category AND cl.id_lang=' . $id_lang . ')
            WHERE p.id_category_default = ' . $id_category_source
            . $where
            . ($id_product > 0 ? ' AND p.id_product > ' . $id_product : '') . '
            ORDER BY p.id_product ASC, c.level_depth DESC';
        $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        $last_id_product = $id_product;
        $inserted_records = 0;

        if ($products) {
            /*
            if ($limit > 0 && $id_product === 0) {
                $tot_records = count($products);
            }
            */
            $last_id_product = 0;
            foreach ($products as $key => $product) {
                set_time_limit(300);

                if ((int) $product['id_product'] === (int) $last_id_product) {
                    continue;
                }
                $last_id_product = (int) $product['id_product'];

                // set this category as product default category
                try {
                    $productObj = new Product($last_id_product);
                } catch (Exception $e) {
                    // MxbToolsDebug::writeLog('Mxb_Tools', 'MxbToolsProduct::changeDefaultCategoryProduct', 'exception for product ' . $last_id_product);
                    continue;
                }
                if (!Validate::isLoadedObject($productObj)) {
                    // MxbToolsDebug::writeLog('Mxb_Tools', 'MxbToolsProduct::changeDefaultCategoryProduct', 'not validated product ' . $last_id_product);
                    continue;
                }
                $productObj->id_category_default = $product['id_category'];
                if ($saveFlag) {
                    $productObj->save();
                }
                $success[] = ($saveFlag ? '' : '[simulation] ') . 'product ' . $product['reference'] . ' (' . $product['id_product'] . '): default category changed to ' . $product['category_name'];
                // MxbToolsDebug::writeLog('Mxb_Tools', 'MxbToolsProduct::changeDefaultCategoryProduct', end($success));
                ++$inserted_records;
                if ($limit > 0 && $inserted_records >= ($limit - 1)) {
                    break;
                }
            }
        } else {
            $inserted_records = -1; // no products
        }
        $ret = [
            'records' => $inserted_records,
            'tot_records' => $tot_records,
            'id' => $last_id_product,
            'success' => implode('|', $success),
        ];

        return $ret;
    }

    /**
     * remove products from a category
     */
    public function removeProductsFromCategory($id_category = 0, $limit = 0, $id_product = 0)
    {
        // $gcodes = MxbToolsCodes::getCodes();
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsProduct::removeProductsFromCategory',
            'id_category=' . $id_category
            . ', limit=' . $limit
            . ', id_product=' . $id_product
        );
        */
        if ($id_category === 0) {
            return ['records' => -1, 'tot_records' => 0, 'id' => 0];
        }
        if ($limit > 0 && $id_product === 0) {
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'category_product
                WHERE id_category = ' . $id_category;
            $tot_records = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        } else {
            $tot_records = 0;
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsProduct::removeProductsFromCategory',
            'tot_records=' . $tot_records
        );
        */
        $sql = '
            SELECT id_product
            FROM ' . _DB_PREFIX_ . 'category_product
            WHERE id_category = ' . $id_category .
            ($id_product > 0 ? ' AND id_product > ' . $id_product : '') . '
            ORDER BY id_product ASC'; // . ($limit > 0 ? ' LIMIT ' . $limit : '');
        $products = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsProduct::removeProductsFromCategory',
            'products collected.'
        );
        */
        $last_id_product = $id_product;
        $inserted_records = 0;

        if ($products) {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsProduct::removeProductsFromCategory',
                'products collected=' . count($products)
            );
            */
            // get all languages ids
            // $languages = Language::getLanguages();
            /*
                $sql = '
                    SELECT id_lang
                    FROM ' . _DB_PREFIX_ . 'lang
                    ORDER BY id_lang';
                $languages = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
            */

            foreach ($products as $product) {
                set_time_limit(300);
                // check if product is into another category for a safe removal
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'category_product
                    WHERE id_category <> ' . $id_category . '
                    AND id_product = ' . (int) $product['id_product'];
                $already_present = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql) > 0;
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsProduct::removeProductsFromCategory',
                    'product ' . $product['id_product'] . ': already_present (1/0)? ' . (int) $already_present
                );
                */
                $last_id_product = (int) $product['id_product'];
                if ($already_present) {
                    // insert product into new category
                    $sql = '
                        DELETE FROM ' . _DB_PREFIX_ . 'category_product
                        WHERE id_product=' . (int) $product['id_product'] . '
                        AND id_category=' . (int) $id_category;
                    Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);
                    /*
                    MxbToolsDebug::writeLog(
                        'Mxb_Tools',
                        'MxbToolsProduct::removeProductsFromCategory',
                        'product ' . $product['id_product'] . ': deleted !'
                    );
                    */
                    ++$inserted_records;
                    if ($limit > 0 && $inserted_records >= ($limit - 1)) {
                        break;
                    }
                }
            }
        } else {
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsProduct::removeProductsFromCategory',
                'NO PRODUCTS FOR CATEGORY !!'
            );
            */
            $inserted_records = -1; // no products
        }
        return ['records' => $inserted_records, 'tot_records' => $tot_records, 'id' => $last_id_product];
    }

    public static function getRecursiveCategoriesList(
        $idLang = null,
        $nestedCategories = null,
        $curLevel = 0,
        $levelPrefix = '--'
    ) {
        $result = [];
        if ($idLang === null) {
            $idLang = (int) Context::getContext()->language->id;
        }
        if ($nestedCategories === null) {
            /*
            public static function getNestedCategories(
                $idRootCategory = null,
                $idLang = false,
                $active = true,
                $groups = null,
                $useShopRestriction = true,
                $sqlFilter = '',
                $orderBy = '',
                $limit = '')
            */
            $nestedCategories = Category::getNestedCategories(
                null,
                $idLang,
                true,
                null,
                true,
                '',
                ' ORDER BY c.`level_depth` ASC, `name` ASC',
                ''
            );
        }

        foreach ($nestedCategories as $category) {
            $result[] = [
                'id' => $category['id_category'],
                'name' => $category['name'],
                'nested_name' => ($curLevel === 0
                    ? ''
                    : str_repeat($levelPrefix, $curLevel)) . ' ' . $category['name'],
                'depth' => $curLevel,
            ];
            if (isset($category['children'])) {
                $resultNested = MxbToolsProduct::getRecursiveCategoriesList(
                    $idLang,
                    $category['children'],
                    $curLevel + 1,
                    $levelPrefix
                );
                $result = array_merge($result, $resultNested);
            }
        }

        return $result;
    }

    public static function getAttributeGroupList($idLang = null)
    {
        if ($idLang === null) {
            $idLang = (int) Context::getContext()->language->id;
        }
        $result = Db::getInstance()->executeS('
            SELECT DISTINCT agl.`name` as name, ag.`id_attribute_group` as id
            FROM `' . _DB_PREFIX_ . 'attribute_group` ag
            ' . Shop::addSqlAssociation('attribute_group', 'ag') . '
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group_lang` agl
                ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND `id_lang` = ' . $idLang . ')
            WHERE ag.is_color_group = 1
            ORDER BY `name` ASC
        ');

        return $result;
    }

    public static function getGroupList($idLang = null, $id_shop = false)
    {
        if ($idLang === null) {
            $idLang = (int) Context::getContext()->language->id;
        }
        $shop_criteria = '';
        if ($id_shop) {
            $shop_criteria = Shop::addSqlAssociation('group', 'g');
        }

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT DISTINCT g.`id_group` as id, g.`reduction`, g.`price_display_method`, g.`show_prices`, gl.`name`
            FROM `' . _DB_PREFIX_ . 'group` g
            LEFT JOIN `' . _DB_PREFIX_ . 'group_lang` AS gl ON (g.`id_group` = gl.`id_group` AND gl.`id_lang` = '
            . (int) $idLang . ')
            ' . $shop_criteria . '
            ORDER BY g.`id_group` ASC
        ');
        return $result;
    }

    public static function getOrderStateList($idLang = null)
    {
        if ($idLang === null) {
            $idLang = (int) Context::getContext()->language->id;
        }
        $result = OrderState::getOrderStates($idLang);
        foreach ($result as &$state) {
            $state['id'] = $state['id_order_state'];
        }
        return $result;
    }

    /**
     * Mxb tools : delete disabled products and relater print options
     * called via ajax.php
     */
    public static function analyzeDisabledProducts(
        $limit = 0,
        $current_record = 0,
        $save_flag = 0,
        $file_idx = 0,
        $deleted_records = 0,
        $disabled_only = 0,
        $analysis_result_string = ''
    ) {
        // $elaborated_records = 0;
        $tot_records = 0;
        $max_file_size = 16; // MB
        $target_folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        $target_report_file = $target_folder . '/' . MxbToolsUtils::normalizeFilename(str_replace('www.', '', strtolower($_SERVER['SERVER_NAME'])))
            . '-' . ($disabled_only ? 'disabled' : '') . '_products_analysis';
        $current_record = (int) $current_record;
        $content = '';
        $csv_header = 'id_product;reference;enabled;combinations;specific prices;prints;deleted;comment' . PHP_EOL;

        $analysis_result = (!empty($analysis_result_string) ? json_decode($analysis_result_string, true) : []);
        // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'analysis_result_string=' . $analysis_result_string);
        // MxbToolsDebug::WriteLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result=' . print_r($analysis_result, true));

        if (!$analysis_result) {
            $analysis_result = [];
        }

        // if Gadgetcard module is installed and enabled, we must discart "print options" from query
        if (MxbToolsUtils::isModuleInstalled('gadgetcard')) {
            require_once _PS_MODULE_DIR_ . 'gadgetcard/classes/GadgetcardCodes.php';
            $gadgetcardPrints = true;
        } else {
            $gadgetcardPrints = false;
        }
        $advancedImporter = (bool) MxbToolsUtils::isModuleInstalled('advancedimporter');

        if ($current_record === 0) {
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'product
                WHERE ' . ($gadgetcardPrints ? GadgetcardCodes::sqlFilter_productsOnly() : '1')
                . ($disabled_only ? ' AND active=0' : '');
            $tot_records = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            if ((int) $tot_records === 0) {
                return [
                    'error' => 'no' . ($disabled_only ? ' disabled' : '') . ' product exists in product database',
                    'records' => 0,
                    'tot_records' => 0,
                    'id' => 0,
                ];
            }

            $content = $csv_header;
            if (file_exists($target_report_file . $file_idx . '.csv')) {
                // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'delete CSV file ' . $target_report_file . $file_idx . '.csv');
                unlink($target_report_file . $file_idx . '.csv');
            } else {
                // check folder
                if (!file_exists($target_folder)) {
                    mkdir($target_folder, 0777, true);
                }
            }
        }

        /*
        // force limit to 1 if disabled products will be deleted because deletion takes long time
        if ($save_flag) {
            $limit = 1;
        }
        */
        $sql_limit = ((int) $limit > 0 ? 'LIMIT ' . $limit . ' OFFSET ' . ($current_record - $deleted_records) : '');
        $sql = '
            SELECT id_product, reference, active
            FROM ' . _DB_PREFIX_ . 'product
            WHERE ' . ($gadgetcardPrints ? GadgetcardCodes::sqlFilter_productsOnly() : '1')
            . ($disabled_only ? ' AND active=0' : '') . '
            ORDER BY id_product
            ' . $sql_limit;

        $rows = Db::getInstance()->executeS($sql, true, false);
        if (!count($rows)) {
            if ($current_record > 0) {
                // END OF ANALYSIS: SEND RESULTS
                // create links to csv report(s)
                $reportLinks = 'Click to download reports:'
                    . '|Summary: <a href="' . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '" target="_blank">'
                    . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '</a>';
                for ($i = 1; $i <= $file_idx; ++$i) {
                    $reportLinks .= '|' . $i . ') <a href="' . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '" target="_blank">'
                        . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '</a>';
                }

                // create table of totals
                $header = '<table style="width:100%;"><tr>';
                $cols = '';
                $cnt = 0;
                $tot = [];
                $summary_csv = '';
                $header_csv = '';
                $object_type = 'product';
                $header .= '<th>Object</th>';
                $header_csv .= 'Entity;';
                $cols .= '<tr><td>' . $object_type . '</td>';
                $summary_csv .= $object_type . ';';
                foreach ($analysis_result as $what => $val) {
                    if ($cnt === 0) {
                        $header .= '<th>' . $what . '</th>';
                        $header_csv .= $what . ';';
                    }
                    $cols .= '<td>' . $val . '</td>';
                    $summary_csv .= $val . ';';
                    if (!isset($tot[$what])) {
                        $tot[$what] = 0;
                    }
                    $tot[$what] += (int) $val;
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $header .= '</tr>';
                $header_csv .= PHP_EOL;

                // totals row
                $cols .= '<tr><th>Totals</th>';
                $summary_csv .= 'Totals;';
                foreach ($tot as $what => $val) {
                    $cols .= '<th>' . $val . '</th>';
                    $summary_csv .= $val . ';';
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $summary_csv = $header_csv . $summary_csv;
                $success = '<b>FINAL RESULTS:</b>|'
                    . 'Total records deleted: ' . $deleted_records . '|'
                    . $header . $cols . '</table>';

                // save final report summary
                if (file_exists($target_report_file . '-summary.csv')) {
                    unlink($target_report_file . '-summary.csv');
                }

                // write to file
                $myfile = fopen($target_report_file . '-summary.csv', 'a');
                // fwrite($myfile, print_r($products, true));
                fwrite($myfile, $summary_csv);
                fclose($myfile);

                return [
                    'clear_log' => 1,
                    'success' => $success . '|' . $reportLinks,
                    'records' => 0,
                    'tot_records' => $tot_records,
                    'id' => $current_record,
                ];
            }

            return [
                'error' => 'no' . ($disabled_only ? ' disabled' : '') . ' product exists in product database',
                'records' => 0,
                'tot_records' => $tot_records,
                'id' => $current_record,
            ];
        }
        // $records = count($rows);
        // $current_record += $records;

        $result = MxbToolsProduct::analyzeDeleteProductsArray($rows, $save_flag, $gadgetcardPrints, $advancedImporter);
        $content .= $result['content'];
        $deleted_records += $result['deleted'];

        // when a product is deleted, analyzeDeleteProductsArray() stops execution and returns the total row analyzed (and deleted)
        // to refresh client interface, because deletion could take time and is better to update user before start to delete another record.
        $records = $result['total'];
        $current_record += $records;

        // check csv file size
        if (file_exists($target_report_file . $file_idx . '.csv')) {
            $fileSize = filesize($target_report_file . $file_idx . '.csv') / (1024 * 1024);
            $sizeExceed = $fileSize > (float) $max_file_size;
            if ($sizeExceed) {
                // increment file index
                $file_idx = (int) $file_idx + 1;

                // add headers to csv content
                $content = $csv_header . $content;

                // delete old report
                if (file_exists($target_report_file . $file_idx . '.csv')) {
                    // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'delete CSV file ' . $target_report_file . $file_idx . '.csv');
                    unlink($target_report_file . $file_idx . '.csv');
                }
            }
        }

        // write to file
        // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'write content to CSV file: ' . $content);
        $myfile = fopen($target_report_file . $file_idx . '.csv', 'a');
        // fwrite($myfile, print_r($products, true));
        fwrite($myfile, $content);
        fclose($myfile);

        // $success = 'records elaborated: ' . $current_record . '|';
        $header = 'records elaborated: ' . $current_record . '|'
            . '<table style="width:100%;"><tr>';
        $cols = '';
        $object_type = 'product';
        $header .= '<th>Object</th>';
        $cols .= '<tr><td>' . $object_type . '</td>';
        // update analysis result totals
        foreach ($result['analyzed'] as $what => $val) {
            // $header .= '<th>' . $what . '</th>';
            // write values of this loop
            // $cols .= '<td>' . $val . '</td>';

            // update analysis totals
            if (!isset($analysis_result[$what])) {
                $analysis_result[$what] = 0;
            }
            $analysis_result[$what] += (int) $val;
        }

        // write result table with updated sum of values of every previous loop (totals)
        foreach ($analysis_result as $what => $val) {
            $header .= '<th>' . $what . '</th>';
            $cols .= '<td>' . $analysis_result[$what] . '</td>';
        }

        $header .= '</tr>';
        $cols .= '</tr>';
        $success = $header . $cols . '</table>';

        // remember: to stop ajax transition we must return 'records' => 0
        return [
            'clear_log' => 1,
            'success' => $success,
            'records' => $records,
            'tot_records' => $tot_records,
            'id' => $current_record,
            'val1' => $save_flag,
            'val2' => $file_idx,
            'val3' => $deleted_records,
            'val4' => $disabled_only,
            'val5' => json_encode($analysis_result),
        ];
    }

    public static function analyzeDeleteProductsArray($sr, $save_flag = false, $gadgetcardPrints = false, $advancedImporter = false)
    {
        $content = '';
        $fun = 'analyzeDeleteProductsArray';
        $deleted_records = 0;
        $analyzed = [
            'error' => 0,
            'total' => 0,
            'disabled' => 0,
            'combinations' => 0,
            'combination-shop' => 0,
            'specific-prices' => 0,
            'images' => 0,
            'features' => 0,
            // 'prints' => 0,
            // // 'print-prices' => 0,
            // 'print-positions' => 0,
            // 'print-position-shop' => 0,
            // 'print-images' => 0,
            // 'importer-references' => 0,
            'total-records' => 0,
            'deleted' => 0,
        ];

        if ($gadgetcardPrints) {
            $analyzed['prints'] = 0;
            // $analyzed['print-prices'] = 0;
            $analyzed['print-positions'] = 0;
            $analyzed['print-position-shop'] = 0;
            $analyzed['print-images'] = 0;
        }
        if ($advancedImporter) {
            $analyzed['importer-references'] = 0;
        }

        // for debug / test
        // $save_flag = false;

        // count products, collect infos and delete
        foreach ($sr as $row) {
            ++$analyzed['total'];
            MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'analyze product ' . $row['id_product'] . ', ref=' . $row['reference']);

            // recover combination quantity
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'product_attribute
                WHERE id_product=' . (int) $row['id_product'];
            $combinationQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            // recover specific prices quantity
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'specific_price
                WHERE id_product=' . (int) $row['id_product'];
            $specificPriceQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            // recover product images quantity
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'image
                WHERE id_product=' . (int) $row['id_product'];
            $imageQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            // recover attribute_shop quantity
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'product_attribute_shop
                WHERE id_product=' . (int) $row['id_product'];
            $attributeShopQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            // recover feature quantity
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'feature_product
                WHERE id_product=' . (int) $row['id_product'];
            $featureQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            // GADGETCARD PRINTS
            if ($gadgetcardPrints) {
                // recover print quantity
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product
                    WHERE (' . GadgetcardCodes::sqlFilter_printOptionsOnly(null, $row['reference']) . '
                    OR ' . GadgetcardCodes::sqlFilter_disabledPrintOptionsOnly(null, $row['reference']) . ')';
                $printQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

                // recover print related combination quantity
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product p
                    LEFT JOIN ' . _DB_PREFIX_ . 'product_attribute pa
                        ON (pa.id_product = p.id_product)
                    WHERE (' . GadgetcardCodes::sqlFilter_printOptionsOnly('p', $row['reference']) . '
                    OR ' . GadgetcardCodes::sqlFilter_disabledPrintOptionsOnly('p', $row['reference']) . ')';
                $printCombinationQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

                // recover print related specific_price quantity
                /* NO: prices are related to decorations (DeCo.print-code ...) and not to the product print Prnt.reference....
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product p
                    LEFT JOIN ' . _DB_PREFIX_ . 'specific_price pa
                        ON (pa.id_product = p.id_product)
                    WHERE (' . GadgetcardCodes::sqlFilter_printOptionsOnly('p', $row['reference']) . '
                    OR ' . GadgetcardCodes::sqlFilter_disabledPrintOptionsOnly('p', $row['reference']) . ')';
                $printSpecificPriceQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
                */

                // recover print related images quantity
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product p
                    LEFT JOIN ' . _DB_PREFIX_ . 'image pa
                        ON (pa.id_product = p.id_product)
                    WHERE (' . GadgetcardCodes::sqlFilter_printOptionsOnly('p', $row['reference']) . '
                    OR ' . GadgetcardCodes::sqlFilter_disabledPrintOptionsOnly('p', $row['reference']) . ')';
                $printImageQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

                // recover print related images quantity
                $sql = '
                    SELECT count(*)
                    FROM ' . _DB_PREFIX_ . 'product p
                    LEFT JOIN ' . _DB_PREFIX_ . 'product_attribute_shop pa
                        ON (pa.id_product = p.id_product)
                    WHERE (' . GadgetcardCodes::sqlFilter_printOptionsOnly('p', $row['reference']) . '
                    OR ' . GadgetcardCodes::sqlFilter_disabledPrintOptionsOnly('p', $row['reference']) . ')';
                $printAttributeShopQty = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
            } else {
                // no gadgetcard prints: reset qty values
                $printQty = 0;
                $printCombinationQty = 0;
                $printImageQty = 0;
                $printAttributeShopQty = 0;
            }

            /*
            contare anche:
                product_attribute_shop
                product_attribute_image
                customization
                combinazioni, prezzi, shop e image di ogni stampa
                adv importer supplier reference (somma di prodotti, combinazioni, immagini, stampe con relative combinazioni etc)
                numero totale record database (somma di tutto)
            */

            // update totals
            $analyzed['combinations'] += $combinationQty;
            $analyzed['specific-prices'] += $specificPriceQty;
            $analyzed['combination-shop'] += $attributeShopQty;
            $analyzed['images'] += $imageQty;
            $analyzed['features'] += $featureQty;
            if ($gadgetcardPrints) {
                $analyzed['prints'] += $printQty;
                // $analyzed['print-prices'] += $printSpecificPriceQty;
                $analyzed['print-positions'] += $printCombinationQty;
                $analyzed['print-images'] += $printImageQty;
                $analyzed['print-position-shop'] += $printAttributeShopQty;
            }
            $importerReferencesQty =
                1 // $analyzed['total']
                + $combinationQty
                + $specificPriceQty
                + $imageQty
                + $featureQty
                + $printQty
                + $printCombinationQty
                // + $printSpecificPriceQty
                + $printImageQty;

            if ($advancedImporter) {
                $analyzed['importer-references'] += $importerReferencesQty;
            }

            $analyzed['total-records'] +=
                $importerReferencesQty
                + $attributeShopQty
                + $printAttributeShopQty;

            // delete product
            $deleted = false;
            $errorString = null;
            if ((int) $row['active'] === 0) {
                MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'product NOT active');
                ++$analyzed['disabled'];
                try {
                    $product = new Product($row['id_product']);
                } catch (Exception $e) {
                    ++$analyzed['error'];
                    // $csv_header = 'id_product;reference;enabled;combinations;specific prices;prints;deleted;comment'
                    $content .= $row['id_product'] . ';'
                        . GadgetcardUtils::parseStringForCSV($row['reference']) . ';'
                        . $row['active'] . ';'
                        . $combinationQty . ';'
                        . $specificPriceQty . ';'
                        . $printQty . ';'
                        . 'no;'
                        . 'entity creation error' . PHP_EOL;
                    // die ('Caught exception: ' . $e->getMessage());
                    $entity_error = true;

                    MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'ENTITY CREATION ERROR FOR PRODUCT ' . $row['id_product']);
                    break;
                }
                if (Validate::isLoadedObject($product)) {
                    MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'product validated - save_flag=' . $save_flag);
                    if ((bool) $save_flag) {
                        $startTime = microtime(true);
                        $res = $product->delete();
                        if ($res) {
                            ++$analyzed['deleted'];
                            $deleted = true;
                            ++$deleted_records;
                            $errorString = 'successfully deleted in ' . (microtime(true) - $startTime) . ' sec';
                        } else {
                            ++$analyzed['error'];
                            $errorString = 'error deleting product';
                        }
                        // exit loop after the first product deletion to refresh client side interface_exists
                        MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'product successfully deleted: ' . (int) $res);
                        break;
                    }
                } else {
                    ++$analyzed['error'];
                    $errorString = 'error validating product';
                    MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'ENTITY VALIDATION ERROR FOR PRODUCT ' . $row['id_product']);
                }
            }

            /*
            logString('SHOP DB: product id=' . $row['id_product'] . ', product_attribute id=' . $row['id_product_attribute']
                . ' > NOT FOUND in product_attribute db'
                . ($imgIsPresent ? '> orphan product_attribute_image should be deleted too' : '')
                . ($edited ? ' > shop default_on WAS SET to NULL' : ($row['default_on'] == 1 ? ' > shop default_on should set to null' : '')),
                $fun, $DBG['deb_color_error'], $DBG['deb_text_size'], 0);
            */

            // $csv_header = 'id_product;reference;enabled;combinations;specific prices;prints;deleted;comment'
            $content .= $row['id_product'] . ';'
                . MxbToolsUtils::parseStringForCSV($row['reference']) . ';'
                . (int) $row['active'] . ';'
                . $combinationQty . ';'
                . $specificPriceQty . ';'
                . $printQty . ';'
                . ($deleted ? ($save_flag ? 'yes' : 'yes (simulated)') : 'no') . ';'
                . ($errorString ? $errorString : '') . PHP_EOL;

            MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDeleteProductsArray', 'deleted=' . ($deleted ? ($save_flag ? 'yes' : 'yes (simulated)') : 'no') . ', errorString=' . $errorString);
        } // foreach ($sr as $object_type => $object_supplier_references)

        return [
            'content' => $content,
            'analyzed' => $analyzed,
            'total' => $analyzed['total'],
            'deleted' => $analyzed['deleted'],
        ];
    }

    /**
     * Mxb tools : analyze products to dind missing link_rewrite, names and references
     * link_rewrite can be filled if requested, using the name
     * called via ajax.php
     */
    public static function analyzeMissingLinkRewrite(
        $limit = 0,
        $current_record = 0,
        $save_flag = 0,
        $file_idx = 0,
        $patched_records = 0,
        $empty_only = 0,
        $analysis_result_string = ''
    ) {
        $context = $context = Context::getContext();
        $id_lang = $context->language->id;
        // $link = $context->link;
        // $elaborated_records = 0;
        $tot_records = 0;
        $max_file_size = 16; // MB
        $target_folder = _PS_MODULE_DIR_ . 'mxb_tools/reports';
        $target_report_file = $target_folder . '/' . MxbToolsUtils::normalizeFilename(str_replace('www.', '', strtolower($_SERVER['SERVER_NAME'])))
            . '-' . ($empty_only ? 'empty' : '')
            . '_link_rewrite_analysis';
        $current_record = (int) $current_record;
        $content = '';
        $csv_header = 'entity;id;active;reference;name;link rewrite;link rewrite patched;comment' . PHP_EOL;

        $analysis_result = (!empty($analysis_result_string) ? json_decode($analysis_result_string, true) : []);
        // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'analysis_result_string=' . $analysis_result_string);
        // MxbToolsDebug::WriteLog('MxbToolsAdvImporterManager', 'analyzeSupplierReferenceDb', 'analysis_result=' . print_r($analysis_result, true));

        if (!$analysis_result) {
            $analysis_result = [];
        }

        // if Gadgetcard module is installed and enabled, we must discart "print options" from query
        if (MxbToolsUtils::isModuleInstalled('gadgetcard')) {
            require_once _PS_MODULE_DIR_ . 'gadgetcard/classes/GadgetcardCodes.php';
            $gadgetcardPrints = true;
        } else {
            $gadgetcardPrints = false;
        }
        $advancedImporter = (bool) MxbToolsUtils::isModuleInstalled('advancedimporter');

        if ($current_record === 0) {
            $sql = '
                SELECT count(*)
                FROM ' . _DB_PREFIX_ . 'product p
                INNER JOIN ' . _DB_PREFIX_ . 'product_lang pl ON (p.id_product = pl.id_product and pl.id_lang = ' . $id_lang . ')
                WHERE ' . ($gadgetcardPrints ? GadgetcardCodes::sqlFilter_productsOnly('p') : '1')
                . ($empty_only ? ' AND pl.link_rewrite = ""' : '');
            $tot_records = (int) Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

            if ((int) $tot_records === 0) {
                return [
                    'error' => 'no' . ($empty_only ? ' empty link_rewrite' : '') . ' product exists in product database',
                    'records' => 0,
                    'tot_records' => 0,
                    'id' => 0,
                ];
            }

            $content = $csv_header;
            if (file_exists($target_report_file . $file_idx . '.csv')) {
                // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'delete CSV file ' . $target_report_file . $file_idx . '.csv');
                unlink($target_report_file . $file_idx . '.csv');
            } else {
                // check folder
                if (!file_exists($target_folder)) {
                    mkdir($target_folder, 0777, true);
                }
            }
        }

        /*
        // force limit to 1 if disabled products will be deleted because deletion takes long time
        if ($save_flag) {
            $limit = 1;
        }
        */
        $sql_limit = ((int) $limit > 0 ? 'LIMIT ' . $limit . ' OFFSET ' . ($current_record - $patched_records) : '');
        $sql = '
            SELECT p.id_product as id_product, pl.link_rewrite, p.reference, p.active
            FROM ' . _DB_PREFIX_ . 'product p
            INNER JOIN ' . _DB_PREFIX_ . 'product_lang pl ON (p.id_product = pl.id_product and pl.id_lang = ' . $id_lang . ')
            WHERE ' . ($gadgetcardPrints ? GadgetcardCodes::sqlFilter_productsOnly('p') : '1')
            . ($empty_only ? ' AND pl.link_rewrite = ""' : '') . '
            ORDER BY p.id_product
            ' . $sql_limit;

        $rows = Db::getInstance()->executeS($sql, true, false);
        if (!count($rows)) {
            if ($current_record > 0) {
                // END OF ANALYSIS: SEND RESULTS
                // create links to csv report(s)
                $reportLinks = 'Click to download reports:'
                    . '|Summary: <a href="' . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '" target="_blank">'
                    . MxbToolsUtils::path2url($target_report_file . '-summary.csv') . '</a>';
                for ($i = 1; $i <= $file_idx; ++$i) {
                    $reportLinks .= '|' . $i . ') <a href="' . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '" target="_blank">'
                        . MxbToolsUtils::path2url($target_report_file . $i . '.csv') . '</a>';
                }

                // create table of totals
                $header = '<table style="width:100%;"><tr>';
                $cols = '';
                $cnt = 0;
                $tot = [];
                $summary_csv = '';
                $header_csv = '';
                $object_type = 'product';
                $header .= '<th>Object</th>';
                $header_csv .= 'Entity;';
                $cols .= '<tr><td>' . $object_type . '</td>';
                $summary_csv .= $object_type . ';';
                foreach ($analysis_result as $what => $val) {
                    if ($cnt === 0) {
                        $header .= '<th>' . $what . '</th>';
                        $header_csv .= $what . ';';
                    }
                    $cols .= '<td>' . $val . '</td>';
                    $summary_csv .= $val . ';';
                    if (!isset($tot[$what])) {
                        $tot[$what] = 0;
                    }
                    $tot[$what] += (int) $val;
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $header .= '</tr>';
                $header_csv .= PHP_EOL;

                // totals row
                $cols .= '<tr><th>Totals</th>';
                $summary_csv .= 'Totals;';
                foreach ($tot as $what => $val) {
                    $cols .= '<th>' . $val . '</th>';
                    $summary_csv .= $val . ';';
                }
                $cols .= '</tr>';
                $summary_csv .= PHP_EOL;
                $summary_csv = $header_csv . $summary_csv;
                $success = '<b>FINAL RESULTS:</b>|'
                    . 'Total records patched: ' . $patched_records . '|'
                    . $header . $cols . '</table>';

                // save final report summary
                if (file_exists($target_report_file . '-summary.csv')) {
                    unlink($target_report_file . '-summary.csv');
                }

                // write to file
                $myfile = fopen($target_report_file . '-summary.csv', 'a');
                // fwrite($myfile, print_r($products, true));
                fwrite($myfile, $summary_csv);
                fclose($myfile);

                return [
                    'clear_log' => 1,
                    'success' => $success . '|' . $reportLinks,
                    'records' => 0,
                    'tot_records' => $tot_records,
                    'id' => $current_record,
                ];
            }

            return [
                'error' => 'no' . ($empty_only ? ' disabled' : '') . ' product exists in product database',
                'records' => 0,
                'tot_records' => $tot_records,
                'id' => $current_record,
            ];
        }
        // $records = count($rows);
        // $current_record += $records;

        $result = MxbToolsProduct::analyzeMissingLinkRewriteArray($rows, $save_flag);

        $content .= $result['content'];
        $patched_records += $result['patched'];

        // when a product is deleted, analyzeDeleteProductsArray() stops execution and returns the total row analyzed (and deleted)
        // to refresh client interface, because deletion could take time and is better to update user before start to delete another record.
        $records = $result['total'];
        $current_record += $records;

        // check csv file size
        if (file_exists($target_report_file . $file_idx . '.csv')) {
            $fileSize = filesize($target_report_file . $file_idx . '.csv') / (1024 * 1024);
            $sizeExceed = $fileSize > (float) $max_file_size;
            if ($sizeExceed) {
                // increment file index
                $file_idx = (int) $file_idx + 1;

                // add headers to csv content
                $content = $csv_header . $content;

                // delete old report
                if (file_exists($target_report_file . $file_idx . '.csv')) {
                    // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'delete CSV file ' . $target_report_file . $file_idx . '.csv');
                    unlink($target_report_file . $file_idx . '.csv');
                }
            }
        }

        // write to file
        // MxbToolsDebug::WriteLog('MxbToolsProduct', 'analyzeDisabledProducts', 'write content to CSV file: ' . $content);
        $myfile = fopen($target_report_file . $file_idx . '.csv', 'a');
        // fwrite($myfile, print_r($products, true));
        fwrite($myfile, $content);
        fclose($myfile);

        // $success = 'records elaborated: ' . $current_record . '|';
        $header = 'records elaborated: ' . $current_record . '|'
            . '<table style="width:100%;"><tr>';
        $cols = '';
        $object_type = 'product';
        $header .= '<th>Object</th>';
        $cols .= '<tr><td>' . $object_type . '</td>';
        // update analysis result totals
        foreach ($result['analyzed'] as $what => $val) {
            // $header .= '<th>' . $what . '</th>';
            // write values of this loop
            // $cols .= '<td>' . $val . '</td>';

            // update analysis totals
            if (!isset($analysis_result[$what])) {
                $analysis_result[$what] = 0;
            }
            $analysis_result[$what] += (int) $val;
        }

        // write result table with updated sum of values of every previous loop (totals)
        foreach ($analysis_result as $what => $val) {
            $header .= '<th>' . $what . '</th>';
            $cols .= '<td>' . $analysis_result[$what] . '</td>';
        }

        $header .= '</tr>';
        $cols .= '</tr>';
        $success = $header . $cols . '</table>';

        // remember: to stop ajax transition we must return 'records' => 0
        return [
            'clear_log' => 1,
            'success' => $success,
            'records' => $records,
            'tot_records' => $tot_records,
            'id' => $current_record,
            'val1' => $save_flag,
            'val2' => $file_idx,
            'val3' => $patched_records,
            'val4' => $empty_only,
            'val5' => json_encode($analysis_result),
        ];
    }

    public static function analyzeMissingLinkRewriteArray($sr, $save_flag = false)
    {
        $context = $context = Context::getContext();
        $id_lang = $context->language->id;

        $content = '';
        $fun = 'analyzeMissingLinkRewriteArray';
        $deleted_records = 0;
        $analyzed = [
            'error' => 0,
            'total' => 0,
            'empty' => 0,
            'patched' => 0,
        ];

        // for debug / test
        // $save_flag = false;

        // count products, collect infos and patch it
        $cnt = 0;
        foreach ($sr as $row) {
            ++$analyzed['total'];
            MxbToolsDebug::WriteLog('MxbToolsProduct', $fun, 'analyze product ' . $row['id_product'] . ', ref=' . $row['reference']);

            $id_product = $row['id_product'];
            $product = new Product($id_product, false, $id_lang);
            $comment = '';
            $error = false;
            $patched = false;
            if (false === Validate::isLoadedObject($product)) {
                ++$analyzed['error'];
                $comment = 'error validating product';
            } elseif ((int) $product->id === 0) {
                ++$analyzed['error'];
                $comment = 'product does not exist';
            }
            if ($error) {
                $content .=
                    'product;'
                    . $id_product . ';'
                    . ';'
                    . ';'
                    . ';'
                    . ';'
                    . ';'
                    . $comment . PHP_EOL;
                continue;
            }

            $link_rewrite_patched = '';
            if (empty($product->link_rewrite)) {
                ++$analyzed['empty'];

                if (!empty($product->name)) {
                    $link_rewrite_patched = GadgetcardUtils::slugify($product->name, true);
                    if ($save_flag) {
                        $product->link_rewrite = $link_rewrite_patched;
                        $product->save();
                        ++$analyzed['patched'];
                        $patched = true;
                        $comment = 'link_rewrite successfully patched !';
                    } else {
                        $comment = 'link_rewrite should be patched ...';
                    }
                } else {
                    $comment = 'name missing: cannot patch link_rewrite !';
                }
            }

            /*$analyzed['rows'][$cnt] = [
                'entity' => 'product',
                'id' => $product->id,
                'active' => $product->active,
                'reference' => $product->reference,
                'name' => $product->name,
                'link_rewrite' => $product->link_rewrite,
                'link_rewrite patched' => $product->link_rewrite_patched,
                'comment' => '',

            ];*/

            // $csv_header = 'entity;id;active;reference;name;link rewrite;link rewrite patched;comment' . PHP_EOL;
            $content .=
                'product;'
                . $product->id . ';'
                . $product->active . ';'
                . $product->reference . ';'
                . $product->name . ';'
                . $product->link_rewrite . ';'
                . $link_rewrite_patched . ';'
                . $comment . PHP_EOL;
            ++$cnt;

            MxbToolsDebug::WriteLog('MxbToolsProduct', $fun, 'patched=' . ($patched ? ($save_flag ? 'yes' : 'yes (simulated)') : 'no') . ', comment=' . $comment);
        } // foreach ($sr as $object_type => $object_supplier_references)

        return [
            'content' => $content,
            'analyzed' => $analyzed,
            'total' => $analyzed['total'],
            'patched' => $analyzed['patched'],
        ];
    }
}
