<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsUtils
{
    public static function asyncCurl($url = '')
    {
        try {
            $ch = curl_init($url);
            curl_setopt_array(
                $ch,
                [
                    CURLOPT_HEADER => true,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_NOSIGNAL => 2, // to timeout immediately if the value is < 5000 ms
                    CURLOPT_TIMEOUT_MS => 1000, // The maximum number of mseconds to allow cURL functions to execute
                    CURLOPT_VERBOSE => 1,
                    CURLOPT_SSL_VERIFYHOST => false,
                    CURLOPT_SSL_VERIFYPEER => false,
                    // CURLOPT_HEADER => 1
                ]
            );
            $ret = true;
            $out = curl_exec($ch);
            if ($out === false) {
                $ret = false;
            }
            // else {
            //    echo 'Operation completed without any errors';
            // }
            curl_close($ch);

            return $ret;
        } catch (Exception $e) {
            return false;
        }
    }

    public static function deleteFile($filename = null)
    {
        if (!$filename) {
            return false;
        }
        try {
            if (file_exists($filename)) {
                unlink($filename);
                return true;
            }
        } catch (Exception $e) {
            // echo 'Caught exception: ',  $e->getMessage(), '\n';
        }
        return false;
    }

    public static function csvToArray($filename = '', $delimiter = ',')
    {
        if (!file_exists($filename) || !is_readable($filename)) {
            return false;
        }
        $header = null;
        $data = [];

        // Read file from beginning.
        if (($handle = fopen($filename, 'r')) !== false) {
            // remove BOM header if any
            $bom = "\xef\xbb\xbf";
            // Progress file pointer and get first 3 characters to compare to the BOM string.
            if (fgets($handle, 4) !== $bom) {
                // BOM not found - rewind pointer to start of file.
                rewind($handle);
            }

            while (($row = fgetcsv($handle, 1000, $delimiter)) !== false) {
                if (!$header) {
                    $header = $row;
                } else {
                    try {
                        $data[] = array_combine($header, $row);
                    } catch (Exception $e) {
                        // echo 'Caught exception: ',  $e->getMessage(), '\n';
                        $data[] = $row;
                    }
                }
            }
            fclose($handle);
        }
        return $data;
    }

    public static function saveCsvFile($file_csv, $content = '')
    {
        $myfile = fopen($file_csv, 'wb') or exit('Unable to open file!');
        fwrite($myfile, '\xEF\xBB\xBF' . $content);
        fclose($myfile);
    }

    public static function parseStringForCSV($str, $divider = ';', $replace_with = ',')
    {
        $ret = trim($str);
        $ret = str_replace(PHP_EOL, '', $ret);
        $ret = str_replace('\r', '', $ret);
        $ret = str_replace('\n', '', $ret);
        $ret = str_replace('\r\n', '', $ret);
        $ret = str_replace('\t', ' ', $ret);
        $ret = str_replace('&quot;', '\'', $ret);
        $ret = str_replace('"', '\'', $ret);
        $ret = str_replace($divider, $replace_with, $ret);

        return $ret;
    }

    public static function csvPriceFormat($price = null)
    {
        if ($price === null) {
            return '-';
        }
        // $ret = str_replace('.', ',', preg_replace('/[^0-9\.\,]/', '', (string) Tools::displayPrice((float) $price)));
        $ret = str_replace('.', ',', preg_replace('/[^0-9\.\,]/', '', (string) ((float) $price)));
        return $ret;
    }

    public static function xmlPriceFormat($price = null)
    {
        if ($price === null) {
            return '0';
        }
        $ret = (string) ((float) $price);
        return $ret;
    }

    public static function extractNumber($str = null)
    {
        if ($str === null) {
            return 0;
        }
        // $ret = intval(preg_replace('/[^0-9]+/', '', $str), 10);
        $ret = (int) preg_replace('/[^0-9]+/', '', $str);
        return $ret;
        // return (int) preg_replace('/\D/', '', $str);
    }

    public static function removeTimeFromDatetime($datetime = null)
    {
        if ($datetime === null) {
            return '';
        }
        if (Tools::strlen($datetime) <= 10) {
            return $datetime;
        }
        return Tools::substr($datetime, 0, 10);
    }

    /**
     * da completare: https://www.imprenditoriamo.it/site/xml-caratteri-non-ammessi/
     */
    public static function parseStringForXML($str, $convert_tags = false)
    {
        $ret = trim($str);
        $ret = str_replace('&', '&#38;', $ret);
        $ret = str_replace('“', '&#34;', $ret);
        $ret = str_replace('‘', '&#39;', $ret);
        if ($convert_tags === true) {
            $ret = str_replace('<', '&#60;', $ret);
            $ret = str_replace('>', '&#62;', $ret);
        }
        return $ret;
    }

    public static function startsWith($haystack, $needle)
    {
        $length = Tools::strlen($needle);
        return Tools::substr($haystack, 0, $length) === $needle;
    }

    public static function endsWith($haystack, $needle)
    {
        $length = Tools::strlen($needle);
        if ($length == 0) {
            return true;
        }
        return Tools::substr($haystack, -$length) === $needle;
    }

    public static function getModulePathUri()
    {
        // $uri =  Context::getContext()->link->getPageLink('',true)
        //    . 'modules/mxb_tools' . DIRECTORY_SEPARATOR;  // adds language folder to path: not ok !
        // $uri =  Tools::getHttpHost(true).__PS_BASE_URI__ . '/modules/mxb_tools/';  // first method: OK
        $uri = MxbToolsUtils::path2url(MxbToolsUtils::getModulePath());      // second method: OK
        if (Tools::substr($uri, -1) != DIRECTORY_SEPARATOR) {
            $uri .= DIRECTORY_SEPARATOR;
        }
        return $uri;
    }

    public static function getModulePath($module = 'mxb_tools')
    {
        return _PS_MODULE_DIR_ . $module . DIRECTORY_SEPARATOR;
    }

    public static function normalizeFilename($filename)
    {
        // Remove anything which isn't a word, whitespace, number
        // or any of the following caracters -_~,;[]().
        // If you don't need to handle multi-byte characters
        // you can use preg_replace rather than mb_ereg_replace
        // Thanks @&#321;ukasz Rysiak!
        $filename = mb_ereg_replace('([^\w\s\d\-_~,;\[\]\(\).])', '', $filename);
        // Remove any runs of periods (thanks falstro!)
        $filename = mb_ereg_replace('([\.]{2,})', '', $filename);

        return $filename;
    }

    public static function getChargeFromPrice($public_price = 0, $wholesale_price = 0)
    {
        // ((lordo-acq) * 100) / acq
        if ((float) $wholesale_price <= 0) {
            return 0;
        }
        return (float) (($public_price - $wholesale_price) * 100) / $wholesale_price;
    }

    public static function getDiscountFromPrice($full_price = 0, $discounted_price = 0)
    {
        // 100-((netto/lordo)*100)
        if ((float) $full_price <= 0) {
            return 0;
        }
        return (float) 100 - (($discounted_price / $full_price) * 100);
    }

    public static function getShopValuesForSmarty()
    {
        $values = MxbToolsUtils::getShopValues();
        $ret = [];
        foreach ($values as $key => $val) {
            $ret['mxb_tools_' . $key] = $val;
        }
        return $ret;
    }

    /**
     * getShopValues
     */
    public static function getShopValues(
        &$id_lang = null,
        &$id_shop = null,
        &$id_group = null,
        &$id_currency = null,
        &$id_customer = null,
        &$id_country = null,
        &$id_cart = null,
        &$taxCalculationMethod = null
    ) {
        $context = Context::getContext();
        if (is_null($id_customer)) {
            $id_customer = (int) $context->cookie->id_customer;
        }
        $values = [
            'id_lang' => is_null($id_lang) ? $context->language->id : $id_lang,
            'id_shop' => is_null($id_lang) ? $context->shop->id : $id_shop,
            'id_group' => is_null($id_group) ? (int) Group::getCurrent()->id : $id_group,
            'id_currency' => is_null($id_currency) ? $context->cookie->id_currency : $id_currency,
            'id_country' => (is_null($id_country)
                ? ($id_customer ? (int) Customer::getCurrentCountry($id_customer) : (int) Tools::getCountry())
                : $id_country),
            'id_cart' => (is_null($id_cart)
                ? (isset($context->cookie->id_cart) ? $context->cookie->id_cart : 0)
                : $id_cart),
            'taxCalculationMethod' => (is_null($taxCalculationMethod)
                ? ($id_customer ? Product::getTaxCalculationMethod($id_customer) : PS_TAX_EXC)
                : $taxCalculationMethod),
            'id_customer' => $id_customer,
            // 'highligh' => (int)GroupList::isAdminUser($id_customer),
        ];

        $id_lang = $values['id_lang'];
        $id_shop = $values['id_shop'];
        $id_group = $values['id_group'];
        $id_currency = $values['id_currency'];
        $id_country = $values['id_country'];
        $id_cart = $values['id_cart'];
        $taxCalculationMethod = $values['taxCalculationMethod'];

        return $values;
    }

    public static function getCurrentCurrency($context = null)
    {
        if ($context === null) {
            $context = Context::getContext();
        }
        $id_currency = ((int) $context->cookie->id_currency > 0
            ? (int) $context->cookie->id_currency
            : (int) Configuration::get('PS_CURRENCY_DEFAULT'));
        $currency = Currency::getCurrency($id_currency);
        return new Currency(Currency::getIdByIsoCode($currency['iso_code']));
    }

    public static function getCurrentCurrencySymbol($context = null)
    {
        $currency = MxbToolsUtils::getCurrentCurrency($context);
        return $currency->symbol;
    }

    public static function readModuleFile($filename = null)
    {
        // check if file exists and create it if necessary
        if (!$filename) {
            return null;
        }
        $filedir = MxbToolsUtils::getModulePath();
        $filename = $filedir . $filename;

        if (!file_exists($filename)) {
            return null;
        }
        if (filesize($filename) == 0) {
            return null;
        }
        $myfile = fopen($filename, 'r');
        $content = fread($myfile, filesize($filename));
        fclose($myfile);

        return $content;
    }

    public static function readCssFile(
        $filename = 'customcss/global.css',
        $filename_default = 'css/_global_default.css'
    ) {
        // check if file exists and create it if necessary
        $filedir = MxbToolsUtils::getModulePath() . 'views' . DIRECTORY_SEPARATOR;
        $filename = $filedir . $filename;
        $exists = file_exists($filename);

        
        // check folder
        $pinfo = pathinfo($filename);
        if (!is_dir($pinfo['dirname'])) {
            mkdir($pinfo['dirname']);
        }

        if (!$exists) {
            if ($filename_default === null) {
                return null;
            }

            copy($filedir . $filename_default, $filename);
        } else {
            if (filesize($filename) == 0) {
                if ($filename_default === null) {
                    return null;
                }
                copy($filedir . $filename_default, $filename);
            }
        }
        if (filesize($filename) == 0) {
            return null;
        }
        $myfile = fopen($filename, 'r');
        $content = fread($myfile, filesize($filename));
        fclose($myfile);

        return $content;
    }

    public static function saveCssFile($content, $filename = 'css/global.css')
    {
        $filedir = MxbToolsUtils::getModulePath() . 'views' . DIRECTORY_SEPARATOR;
        $filename = $filedir . $filename;

        // check folder
        $pinfo = pathinfo($filename);
        if (!is_dir($pinfo['dirname'])) {
            mkdir($pinfo['dirname']);
        }

        $myfile = fopen($filename, 'w');
        fwrite($myfile, $content);
        fclose($myfile);

        return true;
    }
    public static function saveJsFile($content, $filename = 'js/custom.js')
    {
        return self::saveCssFile($content, $filename);
    }
    /*
        // FrontController Class
          $this->context->smarty->assign(array(
            // Usefull for layout.tpl
            'mobile_device' => $this->context->getMobileDevice(),
            'link' => $link,
            'cart' => $cart,
            'currency' => $currency,
            'cookie' => $this->context->cookie,
            'page_name' => $page_name,
            'hide_left_column' => !$this->display_column_left,
            'hide_right_column' => !$this->display_column_right,
            'base_dir' => _PS_BASE_URL_.__PS_BASE_URI__,
            'base_dir_ssl' => $protocol_link.Tools::getShopDomainSsl().__PS_BASE_URI__,
            'force_ssl' => Configuration::get('PS_SSL_ENABLED') && Configuration::get('PS_SSL_ENABLED_EVERYWHERE'),
            'content_dir' => $protocol_content.Tools::getHttpHost().__PS_BASE_URI__,
            'base_uri' => $protocol_content.Tools::getHttpHost()
                .__PS_BASE_URI__.(!Configuration::get('PS_REWRITING_SETTINGS') ? 'index.php' : ''),
            'tpl_dir' => _PS_THEME_DIR_,
            'tpl_uri' => _THEME_DIR_,
            'modules_dir' => _MODULE_DIR_,
            'mail_dir' => _MAIL_DIR_,
            'lang_iso' => $this->context->language->iso_code,
            'lang_id' => (int) $this->context->language->id,
            'language_code' => $this->context->language->language_code
                ? $this->context->language->language_code
                : $this->context->language->iso_code,
            'come_from' => Tools::getHttpHost(true, true).Tools::htmlentitiesUTF8(
                str_replace(array('\'', '\\'),
                '',
                urldecode($_SERVER['REQUEST_URI']))
            ),
            'cart_qties' => (int) $cart->nbProducts(),
            'currencies' => Currency::getCurrencies(),
            'languages' => $languages,
            'meta_language' => implode(',', $meta_language),
            'priceDisplay' => Product::getTaxCalculationMethod((int) $this->context->cookie->id_customer),
            'is_logged' => (bool) $this->context->customer->isLogged(),
            'is_guest' => (bool) $this->context->customer->isGuest(),
            'add_prod_display' => (int) Configuration::get('PS_ATTRIBUTE_CATEGORY_DISPLAY'),
            'shop_name' => Configuration::get('PS_SHOP_NAME'),
            'roundMode' => (int) Configuration::get('PS_PRICE_ROUND_MODE'),
            'use_taxes' => (int) Configuration::get('PS_TAX'),
            'show_taxes' => (int) (Configuration::get('PS_TAX_DISPLAY') == 1 && (int) Configuration::get('PS_TAX')),
            'display_tax_label' => (bool) $display_tax_label,
            'vat_management' => (int) Configuration::get('VATNUMBER_MANAGEMENT'),
            'opc' => (bool)Configuration::get('PS_ORDER_PROCESS_TYPE'),
            'PS_CATALOG_MODE' => (bool)Configuration::get('PS_CATALOG_MODE')
                || (Group::isFeatureActive() && !(bool)Group::getCurrent()->show_prices),
            'b2b_enable' => (bool)Configuration::get('PS_B2B_ENABLE'),
            'request' => $link->getPaginationLink(false, false, false, true),
            'PS_STOCK_MANAGEMENT' => Configuration::get('PS_STOCK_MANAGEMENT'),
            'quick_view' => (bool)Configuration::get('PS_QUICK_VIEW'),
            'shop_phone' => Configuration::get('PS_SHOP_PHONE'),
            'compared_products' => is_array($compared_products) ? $compared_products : [],
            'comparator_max_item' => (int) Configuration::get('PS_COMPARATOR_MAX_ITEM'),
            'currencyRate' => (float) $currency->getConversationRate(),
            'currencySign' => $currency->sign,
            'currencyFormat' => $currency->format,
            'currencyBlank' => $currency->blank,
        ));
     */
    public static function getConfigValues($admin = false)
    {
        $context = Context::getContext();
        $fields_values = MxbToolsUtils::getConfigFormValues('config');

        // other usefull values
        $fields_values['debug'] = Configuration::getGlobalValue('MXB_TOOLS_DEBUG');
        $link = new Link();
        // $ajax_link = $link->getModuleLink('mxb_tools', 'ajax');
        // $product_pdf_link = $link->getModuleLink('mxb_tools', 'productPdf');
        // $empty_cart_link = $link->getModuleLink('mxb_tools', 'emptyCart');
        $fields_values['mxb_tools_module_link'] = $link->getModuleLink('mxb_tools', '');
        $fields_values['mxb_tools_ajax_link'] = $link->getModuleLink('mxb_tools', 'ajax');
        $fields_values['mxb_tools_product_pdf_link'] = $link->getModuleLink('mxb_tools', 'productPdf');
        $fields_values['mxb_tools_empty_cart_link'] = $link->getModuleLink('mxb_tools', 'emptyCart');
        if ($admin) {
            $fields_values = array_merge($fields_values, MxbToolsUtils::getCommonAdminTplValues($context));
            // $fields_values['admin_token'] = Tools::getAdminTokenLite('AdminModules');
            // $fields_values['mxb_tools_base_url'] = MxbToolsUtils::getModulePathUri();
        }

        return $fields_values;
    }

    public static function getCommonAdminTplValues($context = null)
    {
        if ($context === null) {
            $context = Context::getContext();
        }

        return [
            'languages' => Language::getLanguages(false),
            'defaultFormLanguage' => (int) $context->language->id,
            'module_dir' => MxbToolsUtils::getModulePath(),
            'admin_template_dir' => MxbToolsUtils::getModulePath() . '/views/templates/admin',
            'admin_menu_tpl' => MxbToolsUtils::getModulePath() . '/views/templates/admin/_partials/menu.tpl',
            'admin_module_list_tpl' => MxbToolsUtils::getModulePath() . '/views/templates/admin/_partials/module-list.tpl',
            'admin_token' => Tools::getAdminTokenLite('AdminModules'),
            'mxb_tools_base_url' => MxbToolsUtils::getModulePathUri(),
            // duplicated: TODO: search where "uri" was used and substitute it with "mxb_tools_base_url"
            'uri' => MxbToolsUtils::getModulePathUri(),
        ];
    }

    public static function getConfigValue($key, $default = null, $type = null, $save = false)
    {
        // dedicated management for following keys:
        if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_CSS' && !Tools::getIsset($key)) {
            $content = MxbToolsUtils::readCssFile('customcss/global.css', 'css/_global_default.css');

            return $content;
        }
        if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_JS' && !Tools::getIsset($key)) {
            $content = MxbToolsUtils::readModuleFile('views/customjs/custom.js');

            return ($content ? $content : '');
        }
        if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_FOOTER_JS' && !Tools::getIsset($key)) {
            $content = MxbToolsUtils::readModuleFile('views/customjs/custom_footer.js');

            return ($content ? $content : '');
        }

        if (Tools::getIsset($key)) {
            if ($save) {
                MxbToolsUtils::setConfigValue($key, Tools::getValue($key), $type);
            }
            $ret = MxbToolsUtils::setValueType(Tools::getValue($key), $type);

            return $ret;
        }
        $ret = Configuration::getGlobalValue($key);
        if ($default !== null && $ret === false) {
            $ret = $default;
        }
        if ($save && $default !== null) {
            MxbToolsUtils::setConfigValue($key, $ret, $type);
        }

        return MxbToolsUtils::setValueType($ret, $type);
    }

    public static function setConfigValue($key, $value, $type = null)
    {
        Configuration::updateGlobalValue($key, MxbToolsUtils::setValueType($value, $type));
    }

    public static function setValueType($value, $type = null)
    {
        if ($type == 'string') {
            return $value;
        } elseif ($type == 'int') {
            return (int) $value;
        } elseif ($type == 'bool') {
            return (int) $value;
        }
        return $value;
    }

    /**
     * Get values for the inputs.
     */
    public static function getConfigFormValues($type = 'all')
    {
        $fields_values = [];
        if ($type == 'all' || $type == 'config') {
            foreach (Mxb_Tools::configVaues() as $value) {
                if ($value['lang']) {
                    foreach (Language::getLanguages(false) as $lang) {
                        $fields_values[$value['name']][$lang['id_lang']] = Tools::getValue(
                            $value['name'] . (string) $lang['id_lang'],
                            Configuration::getGlobalValue($value['name'], (int) $lang['id_lang'])
                        );
                    }
                } else {
                    $fields_values[$value['name']] = MxbToolsUtils::getConfigValue($value['name']);
                }
            }
        }
        if ($type == 'all' || $type == 'tools') {
            foreach (Mxb_Tools::toolsVaues() as $value) {
                if ($value['lang']) {
                    foreach (Language::getLanguages(false) as $lang) {
                        $fields_values[$value['name']][$lang['id_lang']] = Tools::getValue(
                            $value['name'] . (string) $lang['id_lang'],
                            Configuration::getGlobalValue($value['name'], (int) $lang['id_lang'])
                        );
                    }
                } else {
                    $fields_values[$value['name']] = MxbToolsUtils::getConfigValue($value['name']);
                }
            }
        }
        return $fields_values;
    }

    /**
     * https://devdocs.prestashop.com/1.7/modules/concepts/controllers/admin-controllers/tabs/
     */
    public static function installTab(
        $className,
        $tabName,
        $tabParentName = false,
        $position = null,
        $update = true,
        $moduleName = 'mxb_tools'
    ) {
        // check if already exists
        $tabId = Tab::getIdFromClassName($className);

        if (!$update && (int) $tabId > 0) {
            return false;
        }

        // $tab = new Tab();
        $tab = MxbToolsUtils::getTab($className, $moduleName);
        $tab->active = 1;
        $tab->class_name = $className;
        $tab->name = [];

        foreach (Language::getLanguages(true) as $lang) {
            $tab->name[$lang['id_lang']] = $tabName;
        }

        if ($position) {
            $tab->position = $position;
        }
        if (is_int($tabParentName)) {
            $tab->id_parent = $tabParentName;
        } else {
            if ($tabParentName && $tabParentName !== 'DEFAULT' && $tabParentName !== 'DEFAULT_MTR') {
                $tab->id_parent = (int) Tab::getIdFromClassName($tabParentName);
            } else {
                if (version_compare(_PS_VERSION_, '1.7', '<')) {
                    $tab->id_parent = 0;
                } else {
                    $id_parent = (int) Tab::getIdFromClassName($tabParentName);
                    if (!$id_parent && $tabParentName == 'DEFAULT_MTR') {
                        $id_parent = (int) Tab::getIdFromClassName('DEFAULT');
                    }
                    if (!$id_parent) {
                        $id_parent = (int) Tab::getIdFromClassName('IMPROVE');
                    }
                    $tab->id_parent = $id_parent;
                }
                // $tab->id_parent = 0;
            }
        }
        $tab->module = $moduleName;
        if ((int) $tabId > 0) {
            return $tab->save();
        }

        return $tab->add();
    }

    public static function getTab($controller, $module = 'mxb_tools')
    {
        if (class_exists('PrestaShopCollection')) {
            $collection = new PrestaShopCollection('Tab');
        } else {
            $collection = new Collection('Tab');
        }
        $collection->where('class_name', '=', $controller);
        if ($module) {
            $collection->where('module', '=', $module);
        }
        $tab = $collection->getFirst();
        if (!$tab) {
            $tab = new Tab();
        }
        $tab->class_name = $controller;
        $tab->module = $module;

        return $tab;
    }

    public static function uninstallTab($className)
    {
        $tab = new Tab(Tab::getIdFromClassName($className));
        $tab->delete();
    }

    /**
     * tabs must be installed in PS_VERSION is 1.6.x - from 1.7 it should not be installed
     * because once $tabs array is defined, installation is automatic, by Prestashop
     */
    public static function checkTabDontInstall()
    {
        // success on automatic tab installation is not so clear and sure: install tab in any case
        // return version_compare(_PS_VERSION_, '1.7', '>');
        return false;
    }

    /* Not used
    public static function checkTabIdParent($className, $tabName, $tabParentName = false, $moduleName = 'mxb_tools')
    {
        // check if already exists
        $tabId = Tab::getIdFromClassName($className);

        // tab does not exists: exit
        if ((int) $tabId <= 0) {
            return false;
        }
        // no parent name: exit
        if (!$tabParentName) {
            return true;
        }
        // tab exists but check is not needed
        if (version_compare(_PS_VERSION_, '1.7', '<')) {
            return true;
        }

        // check tab parent
        $context = Context::getContext();
        $idLang = (int) $context->language->id;
        $tab = new Tab($tabId, $idLang);

        // id_parent is already set
        if ($tab->id_parent > 0) {
            return true;
        }

        // set the correct id_parent
        // get id_parent
        $tabParentId = Tab::getIdFromClassName($tabParentName);

        //parent class does not exists !
        if (!$tabParentId) {
            return false;
        }
        $tab->id_parent = $tabParentId;
        $tab->save();
        return true;
    }
    */

    /**
     * ADD COLUMN TO TABLE: THIS FUNCTION IS USED FOR MODIFY DATABASES DURING UPDATES
     * in Mxb_Tools.php / AdminMxb_ToolsConfigurationController.php
     */
    public static function addColumnToTable($table_name, $column_name, $column_type)
    {
        $sql = 'DESCRIBE ' . _DB_PREFIX_ . $table_name;
        $columns = Db::getInstance()->executeS($sql);
        $found = false;
        foreach ($columns as $col) {
            if ($col['Field'] == $column_name) {
                $found = true;
                break;
            }
        }
        if (!$found) {
            if (Db::getInstance()->execute('
                ALTER TABLE `' . _DB_PREFIX_ . $table_name . '`
                ADD `' . $column_name . '` ' . $column_type) == false) {
                return false;
            }
        }

        return true;
    }

    public static function getCurrentDate($date_separator = '-')
    {
        return date('Y' . $date_separator . 'm' . $date_separator . 'd');
    }

    public static function getCurrentDateTime(
        $seconds = true,
        $block_separator = ' ',
        $date_separator = '-',
        $time_separator = ':'
    ) {
        return date('Y' . $date_separator . 'm' . $date_separator . 'd' .
            $block_separator . 'H' . $time_separator . 'i'
            . ((bool) $seconds ? $time_separator . 's' : ''));
    }

    public static function strRemovePrefix($str, $prefix = null)
    {
        if (empty($prefix)) {
            return $str;
        }
        if (Tools::substr($str, 0, Tools::strlen($prefix)) == $prefix) {
            $str = Tools::substr($str, Tools::strlen($prefix));
        }
        return $str;
    }

    public static function strReplacePrefix($str, $prefix = null, $new_prefix = null)
    {
        if (!empty($prefix)) {
            $str = MxbToolsUtils::strRemovePrefix($str, $prefix);
        }
        if (!empty($new_prefix)) {
            $str = $new_prefix . $str;
        }
        return $str;
    }

    // check if IP is allowed to run scripts.
    // Set the IP list in mxbtools > configuration
    public static function isAllowedIP($check_ip = null)
    {
        if ($check_ip === null) {
            $check_ip = MxbToolsUtils::getClientIpAddress();
        }
        $allowed_ip = Configuration::getGlobalValue('MXB_TOOLS_SCRIPTS_ALLOWED_IP_ADDRESS');
        if ($allowed_ip === false || empty($allowed_ip)) {
            return false;
        }
        $allowed_ip = str_replace(',', ';', $allowed_ip);
        $ips = explode(';', $allowed_ip);
        foreach ($ips as $ip) {
            if (trim($ip) == $check_ip) {
                return true;
            }
        }
        return false;
    }

    // if IP Address is not allowed to run scripts: do something !
    public static function isAllowedIPOrDie($check_ip = null)
    {
        $allow = MxbToolsUtils::isAllowedIP($check_ip);
        if ($allow) {
            return true;
        }
        Tools::redirect('index.php');
        // header('Location: ../');
        exit;
    }

    /* Function to get the client IP address */
    public static function getClientIpAddress()
    {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP')) {
            $ipaddress = getenv('HTTP_CLIENT_IP');
        } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        } elseif (getenv('HTTP_X_FORWARDED')) {
            $ipaddress = getenv('HTTP_X_FORWARDED');
        } elseif (getenv('HTTP_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        } elseif (getenv('HTTP_FORWARDED')) {
            $ipaddress = getenv('HTTP_FORWARDED');
        } elseif (getenv('REMOTE_ADDR')) {
            $ipaddress = getenv('REMOTE_ADDR');
        } else {
            $ipaddress = 'UNKNOWN';
        }
        return $ipaddress;
    }

    public static function idArrayToString($id_array)
    {
        if (empty($id_array)) {
            return '';
        }
        $new_string = '';
        foreach ($id_array as $id) {
            $new_string .= (int) $id . ',';
        }

        return rtrim($new_string, ',');
    }

    public static function idStringToArray($id_string = '')
    {
        if (strpos($id_string, ';') !== false) {
            $id_string = str_replace(';', ',', $id_string);
        }

        return explode(',', str_replace(' ', '', $id_string));
    }

    public static function idArrayToSmartyString($id_array)
    {
        if (empty($id_array)) {
            return '';
        }
        $new_string = '';
        foreach ($id_array as $id) {
            $new_string .= '[' . (int) $id . ']';
        }

        return $new_string;
    }

    public static function idStringToSmartyString($id_string = '')
    {
        $arr = MxbToolsUtils::idStringToArray($id_string);

        return MxbToolsUtils::idArrayToSmartyString($arr);
    }

    public static function checkIdStringId($id = 0, $id_string = '')
    {
        if (strpos($id_string, ';') !== false) {
            str_replace(';', ',', $id_string);
        }
        if (strpos($id_string, ',') !== false) {
            $id_string = MxbToolsUtils::idStringToSmartyString($id_string);
        }

        return strpos($id_string, '[' . $id . ']') !== false;
    }

    public static function normalizeIdString($id_string = '')
    {
        $arr = MxbToolsUtils::idStringToArray($id_string);

        return MxbToolsUtils::idArrayToString($arr);
    }

    public static function generateRandomString($length = 10, $upper_case = false)
    {
        $characters = '0123456789'
            . (!$upper_case ? 'abcdefghijklmnopqrstuvwxyz' : '')
            . 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = Tools::strlen($characters);
        $randomString = '';

        for ($i = 0; $i < $length; ++$i) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }

        return $randomString;
    }

    public static function createMultiLangField($field)
    {
        $res = [];
        foreach (Language::getIDs(false) as $id_lang) {
            $res[$id_lang] = $field;
        }

        return $res;
    }

    public static function addMonthToDate($date = null, $month = 1, $format = 'date')
    {
        if (is_null($date)) {
            $date = new DateTime('now');
        }
        $date->modify('+' . $month . ' month'); // or you can use '-90 day' for deduct

        return $format == 'string' ? $date->format('Y-m-d H:i:s') : $date;
    }

    /*
        // ready to generate an AES ecryption code
        function generateRandomString(
            $length = 32,
            $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        ) {
          // $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ@-_';
          $charactersLength = Tools::strlen($characters);
          $randomString = '';
          for ($i = 0; $i < $length; ++$i) {
              $randomString .= $characters[rand(0, $charactersLength - 1)];
          }
          return $randomString;
        }
    */

    public static function getClientIp()
    {
        $ipaddress = '';
        if (getenv('HTTP_CLIENT_IP')) {
            $ipaddress = getenv('HTTP_CLIENT_IP');
        } elseif (getenv('HTTP_X_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_X_FORWARDED_FOR');
        } elseif (getenv('HTTP_X_FORWARDED')) {
            $ipaddress = getenv('HTTP_X_FORWARDED');
        } elseif (getenv('HTTP_FORWARDED_FOR')) {
            $ipaddress = getenv('HTTP_FORWARDED_FOR');
        } elseif (getenv('HTTP_FORWARDED')) {
            $ipaddress = getenv('HTTP_FORWARDED');
        } elseif (getenv('REMOTE_ADDR')) {
            $ipaddress = getenv('REMOTE_ADDR');
        } else {
            $ipaddress = 'UNKNOWN';
        }

        return $ipaddress;
    }

    public static function path2url($file, $Protocol = null)
    {
        if ($Protocol === null) {
            $Protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http') . '://';
        }
        $DOCUMENT_ROOT = $_SERVER['DOCUMENT_ROOT'];
        if (Tools::substr($DOCUMENT_ROOT, -1) != DIRECTORY_SEPARATOR) {
            $DOCUMENT_ROOT .= DIRECTORY_SEPARATOR;
        }
        $HTTP_HOST = $_SERVER['HTTP_HOST'];
        if (Tools::substr($HTTP_HOST, -1) != DIRECTORY_SEPARATOR) {
            $HTTP_HOST .= DIRECTORY_SEPARATOR;
        }

        $checkFile = str_replace('\\', '/', $file);
        $checkDOCUMENT_ROOT = str_replace('\\', '/', $DOCUMENT_ROOT);
        /*
        MxbToolsDebug::writeLog(
            'MxbToolsUtils',
            'path2url()',
            'Protocol=' . $Protocol
            . ', HTTP_HOST=' . $HTTP_HOST
            . ', DOCUMENT_ROOT=' . $DOCUMENT_ROOT
            . ', file=' . $file
        );
        */

        return $Protocol . $HTTP_HOST . str_replace($checkDOCUMENT_ROOT, '', $checkFile);
    }

    public static function getServerHttp($get_uri = true)
    {
        return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http')
            . ':// $_SERVER[HTTP_HOST]' . ($get_uri ? '$_SERVER[REQUEST_URI]' : '/');
    }

    public static function getAppRootHttp($folder = null)
    {
        $lnk = explode('?', MxbToolsUtils::getServerHttp(false));
        $http_link = (isset($lnk[0]) ? $lnk[0] : MxbToolsUtils::getServerHttp(false));
        // remove last slash
        $http_link = rtrim($http_link, '/');
        // remove folder, if any
        if ($folder !== null) {
            $http_link = rtrim($http_link, $folder);
            // remove last slash
            $http_link = rtrim($http_link, '/');
            // add folder
            $http_link .= '/' . $folder;
        }

        return $http_link;
    }

    public static function tryRenderPdf($pdf = null)
    {
        if (!$pdf) {
            return false;
        }
        try {
            $pdf->render();
        } catch (Exception $e) {
            return 'Something went wrong during the export : ' . $e->getMessage();
        }

        return true;
    }

    public static function isModuleInstalled($moduleName)
    {
        return Module::isInstalled($moduleName);
    }

    public static function isModuleEnabled($moduleName)
    {
        return Module::isEnabled($moduleName);
    }
    /*
    public static function generateImageByHex(
        $hex,
        $image_folder = null,
        $name = null,
        $image_type = 'jpeg',
        $w = 90,
        $h = 90
    ) {
        if (!$hex || Tools::strlen((string) $hex) < 5) {
            return false;
        }
        if (!$image_folder) {
            return false;
        }
        if (!$name) {
            $name = 'image';
        }
        if (!MxbToolsUtils::endsWith(Tools::strtolower($name), 'jpg')
                && !MxbToolsUtils::endsWith(Tools::strtolower($name), 'png')) {
            // $image_ext = '.jpg';
            if (!$image_type) {
                $image_type = 'jpeg';
            }
            if ($image_type === 'jpeg') {
                $image_ext = '.jpg';
            } else {
                $image_ext = '.' . $image_type;
            }
            $name = $name . $image_ext;
        }
        $hexArray = explode(',', $hex);
        // $colorspan = '';
        $colors = count($hexArray);
        // $w = 90;
        // $h = 90;
        $bw = (int) ($w / $colors);

        $my_img = imagecreatetruecolor($w, $h);
        // $background = imagecolorallocate($my_img, 0, 60, 255);
        // imagecolorallocate($my_img, 0, 60, 255);
        $x = 0;
        // imagesetthickness ( $my_img, $bw );
        foreach ($hexArray as $hx) {
            $he = ltrim($hx, '#');
            $a = hexdec(Tools::substr($he, 0, 2));
            $b = hexdec(Tools::substr($he, 2, 2));
            $c = hexdec(Tools::substr($he, 4, 2));
            $line_colour = imagecolorallocate($my_img, $a, $b, $c);
            // imageline( $my_img, $x, 0, $x+$bw, $h, $line_colour );
            // imagerectangle ( resource $image , int $x1 , int $y1 , int $x2 , int $y2 , int $color )
            imagefilledrectangle($my_img, $x, 0, $x+$bw, $h, $line_colour);
            $x += $bw;
        }
        // save image
        // if ($image_folder !== null) {
        if (!file_exists($image_folder)) {
            mkdir($image_folder, 0777, true);
        }
        $save = $image_folder . '/' . Tools::strtolower($name);
        //chmod($save, 0755);
        if ($image_type == 'png') {
            imagepng($my_img, $save, -1, null);
        } else {
            imagejpeg($my_img, $save, 75);
        }
        imagedestroy($my_img);
        return $save;
    }
    */

    public static function generateImageByHexWeb(
        $hex,
        $image_folder = null,
        $name = null,
        $image_type = 'jpeg',
        $w = 90,
        $h = 90
    ) {
        $hex = str_replace('#', '', $hex);
        $curlSession = curl_init();
        $url = 'https://www.mxbinformatica.it/tools/img/?type=' . $image_type
            . '&w=' . $w
            . '&h=' . $h
            . '&hex=' . $hex;
        curl_setopt($curlSession, CURLOPT_URL, $url);
        curl_setopt($curlSession, CURLOPT_BINARYTRANSFER, true);
        curl_setopt($curlSession, CURLOPT_RETURNTRANSFER, true);

        $image = curl_exec($curlSession);
        curl_close($curlSession);

        if (!file_exists($image_folder)) {
            mkdir($image_folder, 0777, true);
        }
        $save = $image_folder . '/' . Tools::strtolower($name);

        file_put_contents($save, $image);
    }

    /**
     * NOTE: format must be one yyyy-mm-dd or dd-mm-yyyy with - or / as separator
     */
    public static function getDateString($date, $srcFormat = null, $tgtFormat = null)
    {
        $getTime = (Tools::strlen($date) > 12);
        if (!$tgtFormat) {
            $tgtFormat = 'Y-m-d' . ($getTime ? ' H:i:s' : '');
        }
        if (!$srcFormat) {
            // check and correct format
            /*
            $split = (str_contains($date, '/') ? '/' : '-'),
            $d = explode($split, $date);
            if (count($d) >= 3) {
                // check if first number is the year YYYY
                if (strlen($d[0]) === 4) {

                }
            }
            */
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsUtils::getDateString()',
                'date=' . $date . ' - date return=' . date('Y-m-d' . ($getTime ? ' H:i:s' : ''), strtotime($date)),
                true
            );
            */
            return date('Y-m-d' . ($getTime ? ' H:i:s' : ''), strtotime($date));
        }

        return DateTime::createFromFormat($srcFormat, $date)->format($tgtFormat);
    }

    public static function setArrayValueIfNew($array, $element, $defaultValue = null)
    {
        if (!is_array($array)) {
            return false;
        }
        if (!isset($array[$element])) {
            return $defaultValue;
        }

        return $array[$element];
    }

    public static function isCategoryEmpty($id_category)
    {
        // return false because categories must be tested within its subcategories
        // return false;
        $sql = '
            SELECT COUNT(id_category)
            FROM ' . _DB_PREFIX_ . 'category
            WHERE id_parent=' . (int) $id_category . '
            AND active=1';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

        // if category have subcategories, it's not empty.
        if ((int) $result > 0) {
            // Mxb_ToolsDebug::WriteLog($this->name, 'isCategoryEmpty(' . $id_category . ')', 'id_parent: result=' . $result, true);
            return false;
        }
        /*
        $sql = '
            SELECT COUNT(id_category)
            FROM ' . _DB_PREFIX_ . 'category_product
            WHERE id_category=' . (int) $id_category;
        */
        $sql = '
            SELECT COUNT(cp.id_category)
            FROM ' . _DB_PREFIX_ . 'category_product cp,
            ' . _DB_PREFIX_ . 'product pr
            WHERE cp.id_category = ' . $id_category . '
            AND cp.id_product = pr.id_product
            AND pr.active = 1';

        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);

        // Mxb_ToolsDebug::WriteLog($this->name, 'isCategoryEmpty(' . $id_category . ')', ': result=' . $result, true);
        return (int) $result === 0;
    }

    public static function isManufacturerEmpty($id_manufacturer)
    {
        $sql = '
            SELECT COUNT(id_manufacturer)
            FROM ' . _DB_PREFIX_ . 'product
            WHERE id_manufacturer=' . (int) $id_manufacturer . '
            AND active=1';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
        // Mxb_ToolsDebug::WriteLog($this->name, 'isManufacturerEmpty()', 'result=' . $result, true);

        return (int) $result === 0;
    }

    public static function getJsonFileContent($dataFile, $associative = true)
    {
        try {
            // if (!file_exists($dataFile)) {
            //     return false;
            // }
            $string = Tools::file_get_contents($dataFile);
            // MxbToolsDebug::writeLog('mxb_tools', 'getJsonFileContent', $dataFile . ': ' . print_r($string, true));
            if ($string === false) {
                return false;
            }
            $json_a = json_decode($string, $associative);
            if ($json_a === null) {
                // MxbToolsDebug::writeLog('mxb_tools', 'getJsonFileContent', 'JSON ERROR');
                return false;
            }
        } catch (Exception $e) {
            // MxbToolsDebug::writeLog('mxb_tools', 'getJsonFileContent', 'Caught exception: ' . $e->getMessage());
            // echo 'Caught exception: ',  $e->getMessage(), '\n';
            return false;
        }

        return $json_a;
    }

    public static function setJsonFileContent($dataFile, $data)
    {
        $myfile = fopen($dataFile, 'w');
        fwrite($myfile, json_encode($data));
        fclose($myfile);
    }

    public static function urlExists($url)
    {
        $file_headers = @get_headers($url);
        if ($file_headers[0] == 'HTTP/1.1 404 Not Found') {
            return false;
        }

        return true;
    }

    /* used to convert product name into link_rewrite */
    public function slugify($text, $strict = false)
    {
        $text = html_entity_decode($text, ENT_QUOTES, 'UTF-8');
        // replace non letter or digits by -
        $text = preg_replace('~[^\\pL\d.]+~u', '-', $text);

        // trim
        $text = trim($text, '-');
        setlocale(LC_CTYPE, 'en_GB.utf8');
        // transliterate
        if (function_exists('iconv')) {
            $text = iconv('utf-8', 'us-ascii// TRANSLIT', $text);
        }

        // lowercase
        $text = strtolower($text);
        // remove unwanted characters
        $text = preg_replace('~[^-\w.]+~', '', $text);
        if (empty($text)) {
            return 'empty_$';
        }
        if ($strict) {
            $text = str_replace('.', '_', $text);
        }

        return $text;
    }
}
