<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/pdf/MxbToolsHTMLTemplatePdf.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class HTMLTemplateCoverjobPdf extends MxbToolsHTMLTemplatePdf
{
    public function __construct($order_object, $smarty)
    {
        parent::__construct($order_object, $smarty);

        $this->pdf_name = 'coverjob';
        $this->main_tpl_name = 'order';
        $this->title = MxbToolsHTMLTemplatePdf::l('Cover Job');
        // $this->hide_price = true;
        $this->tpls['product_tab'] = 'coverjob.product-tab.tpl';
        $this->tpls['addresses_tab'] = 'coverjob.addresses-tab.tpl';
        $this->tpls['payment_tab'] = null;
        $this->tpls['total_tab'] = null;
        $this->tpls['tax_tab'] = null;
        $this->tpls['shipping_tab'] = null;
        $this->tpls['add_data_tab'] = 'coverjob.add_data-tab.tpl';
        $this->available_in_your_account = false;
        $this->legal_free_text = false;

        $this->layout_cols_width = [
            'reference' => [
                'width' => 15,
            ],
            'product' => [
                'width' => 75,
            ],
            'quantity' => [
                'width' => 10,
            ],
            'tax_code' => null,
            'unit_price_tax_excl' => null,
            'total_tax_excl' => null,
        ];
    }

    /** don't delete the following 3 methods because are abstract and need to be here */
    /**
     * Returns the template's HTML content
     * @return string HTML content
     */
    public function getContent()
    {
        $module = Module::getInstanceByName('mxb_tools');

        $strings = [
            // coverjob PDF: address-tab
            'Delivery date' => $module->l('Delivery date'),
            'Event date' => $module->l('Event date'),
            'Program date' => $module->l('Program date'),
            'Job' => $module->l('Job'),
            'Job date' => $module->l('Job date'),
            'Job name' => $module->l('Job name'),
            'DDT number' => $module->l('DDT number'),
            'Invoice number' => $module->l('Invoice number'),
            'Customer' => $module->l('Customer'),
            'Order n.' => $module->l('Order n.'),

            // coverjob PDF: add-data-tab
            'Weight' => $module->l('Weight'),
            'Volume' => $module->l('Volume'),
            'Package q.ty' => $module->l('Package q.ty'),
            'Pallets' => $module->l('Pallets'),
            'Aesthetic check' => $module->l('Aesthetic check'),
            'Functionality check' => $module->l('Functionality check'),
            'Print tightness control' => $module->l('Print tightness control'),
            'Print type' => $module->l('Print type'),
            'Embroidery' => $module->l('Embroidery'),
            'Frameworks' => $module->l('Frameworks'),
            'Unpacking' => $module->l('Unpacking'),
            'Packaging' => $module->l('Packaging'),
            'Subpacking' => $module->l('Subpacking'),
            'Packing' => $module->l('Packing'),
            'Label' => $module->l('Label'),
            'Scotch tape' => $module->l('Scotch tape'),
            'Transport' => $module->l('Transport'),
            'Courier' => $module->l('Courier'),
            'Shipping times' => $module->l('Shipping times'),
        ];

        $this->smarty->assign('mxb_strings', $strings);
        /*
            $this->smarty->assign('mxb_config', array('MXB_TOOLS_COVERJOB_TYPE');
            $file_name = Configuration::getGlobalValue('MXB_TOOLS_COVERJOB_LOGO');
        */

        $this->smarty->assign('mxbtConfigValues', MxbToolsUtils::getConfigValues(true));

        return parent::getContent();
    }

    /**
     * Returns the template filename
     * @return string filename
     */
    public function getFilename()
    {
        return parent::getFilename();
    }

    /**
     * Returns the template filename when using bulk rendering
     * @return string filename
     */
    public function getBulkFilename()
    {
        return parent::getBulkFilename();
    }

    /**
     * Returns the invoice logo.
     *
     * @return string|null
     */
    protected function getLogo()
    {
        // MxbToolsDebug::writeLog('Mxb_Tools', 'HTMLTemplateCoverjobPdf::getLogo()',  'filename:' . $this->getCoverjobLogo());
        // MXB: get coverjob logo
        return $this->getCoverjobLogo();
        /*
            $ret = parent::getLogo();
            $file_name = Configuration::getGlobalValue('MXB_TOOLS_COVERJOB_LOGO');
            // MxbToolsDebug::writeLog('Mxb_Tools', 'HTMLTemplateCoverjobPdf::getLogo()',  'image name: ' . $file_name);
            if ($file_name && file_exists(_PS_IMG_DIR_ . $file_name)) {
                return $file_name;
            }
            return $ret;
        */
    }

    public function getHeader()
    {
        // get original logo to display smaller on the right
        // $logo = parent::getLogo();
        $logo = $this->getShopLogo();

        MxbToolsDebug::writeLog('Mxb_Tools', 'HTMLTemplateCoverjobPdf::getHeader()', 'filename:' . $logo);

        $width = 0;
        $height = 0;
        if (!empty($logo)) {
            list($width, $height) = getimagesize(_PS_IMG_DIR_ . $logo);
        }

        // Limit the height of the logo for the PDF render
        $maximum_height = 30;
        if ($height > $maximum_height) {
            $ratio = $maximum_height / $height;
            $height *= $ratio;
            $width *= $ratio;
        }

        $this->smarty->assign([
            'subtitle' => $this->order->getUniqReference(),
            'small_logo_path' => Tools::getShopProtocol() . Tools::getMediaServer(_PS_IMG_) . _PS_IMG_ . $logo,
            'width_small_logo' => $width,
            'height_small_logo' => $height,
        ]);
        // MxbToolsDebug::writeLog('Mxb_Tools', 'HTMLTemplateCoverjobPdf::assignCommonHeaderData()',  'subtitle: ' . $this->order->getUniqReference());

        return parent::getHeader();
    }

    protected function getTemplate($template_name)
    {
        return parent::getTemplate($template_name);
    }

    public function getShopLogo()
    {
        $id_shop = (int) $this->shop->id;
        $logo = Configuration::get('PS_LOGO', null, null, $id_shop);
        if ($logo && file_exists(_PS_IMG_DIR_ . $logo)) {
            return $logo;
        }

        return null;
    }

    public function getCoverjobLogo()
    {
        $file_name = Configuration::getGlobalValue('MXB_TOOLS_COVERJOB_LOGO');
        // MxbToolsDebug::writeLog('Mxb_Tools', 'HTMLTemplateCoverjobPdf::getLogo()',  'image name: ' . $file_name);
        if ($file_name && file_exists(_PS_IMG_DIR_ . $file_name)) {
            return $file_name;
        }

        return null;
    }
}
