<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/mxb_tools.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsProduct.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class HTMLTemplateProductPdf extends HTMLTemplate
{
    public $product_object;
    // public $order_invoice;
    public $template_dir;
    public $module;
    public $mxb_strings;

    public function __construct($product_object, $smarty)
    {
        $this->product_object = $product_object;
        $this->module = new mxb_tools();

        $this->mxb_strings = ['combinations' => $this->module->l('Variants')];
        // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'product_object='.print_r($product_object, true));
        $this->smarty = $smarty;
        $this->template_dir = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/hook/pdf/';

        // header informations
        $this->date = Tools::displayDate(date('Y-m-d H:i:s'));
        $id_lang = Context::getContext()->language->id;
        $this->title = $this->module->l('Product sheet');
        // footer informations
        $this->shop = new Shop(Context::getContext()->shop->id);
    }

    /**
     * Compute layout elements size.
     *
     * @param $params Array Layout elements
     *
     * @return array Layout elements columns size
     */
    protected function computeLayout($params)
    {
        $layout = [
            'reference' => [
                'width' => 15,
            ],
            'product' => [
                'width' => 40,
            ],
            'quantity' => [
                'width' => 8,
            ],
            'tax_code' => [
                'width' => 8,
            ],
            'unit_price_tax_excl' => [
                'width' => 0,
            ],
            'total_tax_excl' => [
                'width' => 0,
            ],
        ];

        if (isset($params['has_discount']) && $params['has_discount']) {
            $layout['before_discount'] = ['width' => 0];
            $layout['product']['width'] -= 7;
            $layout['reference']['width'] -= 3;
        }

        $total_width = 0;
        $free_columns_count = 0;
        foreach ($layout as $data) {
            if ($data['width'] === 0) {
                ++$free_columns_count;
            }
            $total_width += $data['width'];
        }
        $delta = 100 - $total_width;

        foreach ($layout as $row => $data) {
            if ($data['width'] === 0) {
                $layout[$row]['width'] = $delta / $free_columns_count;
            }
        }
        $layout['_colCount'] = count($layout);

        return $layout;
    }

    /**
     * Returns the template's HTML content
     * @return string HTML content
     */
    public function getContent()
    {
        // get product attributes
        $id_product = (int) $this->product_object['productData']['id_product'];
        $id_lang = (int) $this->product_object['productData']['id_lang'];
        $product = new Product((int) $id_product);
        if (!Validate::isLoadedObject($product)) {
            exit('The product cannot be found within your database.');
        }

        /*
         * Get attributes:
        [0] => Array
            (
                [id_attribute_group] => 151
                [is_color_group] => 1
                [group_name] => Colore
                [public_group_name] => Colore
                [id_attribute] => 2290
                [attribute_name] => nero
                [attribute_color] => #000000
                [id_product_attribute] => 93161
                [quantity] => 614
                [price] => 108.330000
                [ecotax] => 0.000000
                [weight] => 0.000000
                [default_on] =>
                [reference] => 37505992
                [unit_price_impact] => 0.000000
                [minimal_quantity] => 1
                [available_date] => 0000-00-00
                [group_type] => color
            )
        */

        // public static function getProductAttributes($id_lang = null, $id_product = null)
        // $attributes = MxbToolsProduct::getAllProductAttributes($id_lang, $id_product);
        $attributes = $product->getAttributesGroups($id_lang);
        // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'productAttributes ORIG='.print_r($attributes, true));
        if ($attributes) {
            $productAttributesArray = [];

            // collect attributes
            foreach ($attributes as $pa) {
                $name = $pa['public_group_name'];
                if (!isset($productAttributesArray[$name])) {
                    $productAttributesArray[$name] = [];
                }
                if (!isset($productAttributesArray[$name][$pa['id_attribute']])) {
                    $productAttributesArray[$name][$pa['id_attribute']] = $pa['attribute_name'];
                }
            }

            // sort values and create html
            $productAttributes = '<table class="product-attributes">';
            foreach ($productAttributesArray as $name => &$pa) {
                asort($pa);
                $productAttributes .= '<tr><td>' . $name . ': </td><td>' . implode(', ', $pa) . '</td></tr>';
            }
            $productAttributes .= '</table>';

            $this->product_object['productData']['productAttributes'] = $productAttributes;
            // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'productAttributes='.$this->product_object['productData']['productAttributes']);
        }

        // manipulate content
        if (isset($this->product_object['productData']['discountTable']) && $this->product_object['productData']['discountTable']) {
            $this->product_object['productData']['discountTable'] = $this->prepareHtmlContent($this->product_object['productData']['discountTable']);
            // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'discountTable='.$this->product_object['productData']['discountTable']);
        }

        if (isset($this->product_object['productData']['printTable']) && $this->product_object['productData']['printTable']) {
            $this->product_object['productData']['printTable'] = $this->prepareHtmlContent($this->product_object['productData']['printTable']);
            // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'printTable='.$this->product_object['productData']['printTable']);
        }

        if (isset($this->product_object['productData']['featuresTable']) && $this->product_object['productData']['featuresTable']) {
            $content = $this->product_object['productData']['featuresTable'];
            $content = $this->prepareHtmlContent($content);
            $content = str_replace('h6', 'h1', $content);
            // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'featuresTable='.$content);
            $this->product_object['productData']['featuresTable'] = $content;
        }

        if (isset($this->product_object['productData']['printDetailsTable']) && $this->product_object['productData']['printDetailsTable']) {
            $content = $this->product_object['productData']['printDetailsTable'];
            $content = $this->prepareHtmlContent($content);
            $content = $this->adjustUL_toHorizontal($content, 'product-images');
            $content = str_replace('h6', 'h1', $content);

            // add class to dimensions table
            // $content = str_replace('<table>', '<table class="print-dimensions">', $content);

            // adjust thumbs dimensions
            $w = (int) (32 / 3);
            $h = (int) (39 / 3);
            $content = str_replace('<img class="print-color-thumb"', '<img class="print-color-thumb" width="' . $w . '" height="' . $h . '" style="width:' . $w . 'px; height:' . $h . 'px;"', $content);
            $w = (int) (12 / 2);
            $h = (int) (33 / 2);
            $content = str_replace('<img class="print-height-thumb"', '<img class="print-height-thumb" width="' . $w . '" height="' . $h . '" style="width:' . $w . 'px; height:' . $h . 'px;"', $content);
            $w = (int) (34 / 2);
            $h = (int) (11 / 2);
            $content = str_replace('<img class="print-width-thumb"', '<img class="print-width-thumb" width="' . $w . '" height="' . $h . '" style="width:' . $w . 'px; height:' . $h . 'px;"', $content);

            // $content = str_replace('<br><span', '<span>&nbsp;</span><span', $content);
            $content = str_replace('span><br><span', 'span><span', $content);
            $content = str_replace('<span class="print-dimensions-container"', '<br><span class="print-dimensions-container"', $content);
            $content = str_replace('<span class="print-details-container"', '<br><span class="print-details-container"', $content);
            $content = str_replace('<span class="product-print-data-summary"', '<br><span class="product-print-data-summary"', $content);
            $content = str_replace('<span class="print-height-container"', '<span>&nbsp;</span><span class="print-height-container"', $content);

            $content = str_replace('<span class="print-option-name-title-tab"', '<br><span class="print-option-name-title-tab"', $content);
            $content = str_replace('<span class="print-option-value-title-tab"', '<br><span class="print-option-value-title-tab"', $content);
            $content = str_replace('<span class="print-option-name-subtitle-tab"', '<br><br><span class="print-option-name-subtitle-tab"', $content);

            // MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'printDetailsTable=' . $content);
            $this->product_object['productData']['printDetailsTable'] = $content;
        }

        /*
            if (isset($this->product_object['productData']['printTable']) && $this->product_object['productData']['printTable']) {
              $content = $this->product_object['productData']['printTable'];

              $content = $this->prepareHtmlContent($content);
              $content = str_replace("h6", "h1", $content);

              // split tables
              $printTable = [];
              $arr = explode("<table", $content);
              $printTable[] = $arr[0];
              if (count($arr)>2) {
                foreach ($arr as $k=>$a) {
                  if ($k>0)
                    $printTable[] = '<table' . $a;
                }
              }

              $this->product_object['productData']['printTable'] = $printTable;

              MxbToolsDebug::writeLog('mxb_tools', 'HTMLTemplateProductPdf', 'printTable='.print_r($printTable, true));
            }
        */
        $this->product_object['mxb_strings'] = $this->mxb_strings;

        $width = 0;
        $height = 0;
        if (!empty($this->product_object['productData']['imageUrl'])) {
            try {
                list($width, $height) = getimagesize($this->product_object['productData']['imageUrl']);
                // Limit the height of the logo for the PDF render
                $maximum_width = 300;
                if ($width > 0 && $width > $maximum_width) {
                    $ratio = $maximum_width / $width;
                    $width *= $ratio;
                    $height *= $ratio;
                }
            } catch (Exception $e) {
            }
        }
        $this->product_object['productData']['width_cover'] = $width;
        $this->product_object['productData']['height_cover'] = $height;
        $this->smarty->assign($this->product_object);

        if (Tools::getValue('debug')) {
            exit(json_encode($this->product_object));
        }

        $tpls = [
            'style_tab' => $this->smarty->fetch($this->template_dir . 'product.style-tab.tpl'),
            /*
                // 'product_image_tab' => $this->smarty->fetch($this->template_dir . 'product.image-tab.tpl'),
                // 'product_prices_tab' => $this->smarty->fetch($this->template_dir . 'product.prices-tab.tpl'),
                // 'product_discounts_tab' => $this->smarty->fetch($this->template_dir . 'product.discounts-tab.tpl'),
                'addresses_tab' => $this->smarty->fetch($this->template_dir . 'order.addresses-tab.tpl'),
                'summary_tab' => $this->smarty->fetch($this->template_dir . 'order.summary-tab.tpl'),
                'product_tab' => $this->smarty->fetch($this->template_dir . 'order.product-tab.tpl'),
                'tax_tab' => $this->getTaxTabContent(),
                'payment_tab' => $this->smarty->fetch($this->template_dir . 'order.payment-tab.tpl'),
                'note_tab' => $this->smarty->fetch($this->template_dir . 'order.note-tab.tpl'),
                'total_tab' => $this->smarty->fetch($this->template_dir . 'order.total-tab.tpl'),
                'shipping_tab' => $this->smarty->fetch($this->template_dir . 'order.shipping-tab.tpl'),
            */
        ];
        $this->smarty->assign($tpls);
        /*
            $this->smarty->assign(array(
              'order' => $this->order['order'],
            ));
        */

        return $this->smarty->fetch($this->template_dir . 'product.tpl');
    }

    protected function prepareHtmlContent($content)
    {
        $content = $this->cleanHtml($content);
        $content = $this->adjustDL($content);
        $content = $this->adjustDiv($content);

        return $content;
    }

    /*
        APPUNTI:
        $text = '[b]this will be bold[/b] but this will be not.';
        $ret = $text; // So we don't overwrite the original variable.
        $ret = preg_replace('#\[b\](.+)\[\/b\]#iUs', '<b>$1</b>', $ret);
     */
    protected function cleanHtml($content)
    {
        // remove eol
        $content = str_replace(['\r', '\n', '\t'], '', $content);
        // remove spaces between tags
        $content = preg_replace('/\>\s+\</m', '><', $content);

        return $content;
    }

    protected function adjustDiv($content)
    {
        // adjust divs and paragraphs
        $content = str_replace('<table', '<br><table', $content);
        // $content = preg_replace('#\<div(.+)\<\/div#iUs', '<span$1</span', $content);
        $content = str_replace('<div', '<span', $content);
        // $content = str_replace('</div></td>', '</span></td>', $content);
        $content = str_replace('</div>', '</span>', $content);
        $content = str_replace('<p', '<span', $content);
        $content = str_replace('</p>', '</span>', $content);
        $content = str_replace('</span><span', '</span><br><span', $content);
        $content = str_replace('</span><table', '</span><br><table', $content);
        // $content = str_replace('<table', '<div class="table-container"><table', $content);
        // $content = str_replace('</table>', '</table></div>', $content);
        $content = str_replace('<br><br>', '<br>', $content);
        $content = str_replace('</table>', '</table><br><br>', $content);

        return $content;
    }

    protected function adjustDL($content)
    {
        // adjust description lists
        $content = str_replace('<dl', '<table', $content);
        $content = str_replace('</dl', '</table', $content);
        $content = str_replace('<dt', '<tr><td', $content);
        $content = str_replace('</dt', '</td', $content);
        $content = str_replace('<dd', '<td', $content);
        $content = str_replace('</dd>', '</td></tr>', $content);

        return $content;
    }

    protected function adjustUL_toHorizontal($content, $class = null)
    {
        // adjust description lists
        if ($class !== null) {
            $content = str_replace('<ul class="' . $class . '"', '<table class="' . $class . '"><tr', $content);
        }
        $content = str_replace('<ul', '<table><tr', $content);
        $content = str_replace('</ul', '</tr></table', $content);
        $content = str_replace('<li', '<td', $content);
        $content = str_replace('</li', '</td', $content);

        return $content;
    }

    protected function adjustUL_toVertical($content, $class = null)
    {
        // adjust description lists
        if ($class !== null) {
            $content = str_replace('<ul class="' . $class . '"', '<table class="' . $class . '"', $content);
        }
        $content = str_replace('<ul', '<table', $content);
        $content = str_replace('</ul', '</table', $content);
        $content = str_replace('<li', '<tr><td', $content);
        $content = str_replace('</li>', '</td></tr>', $content);

        return $content;
    }

    /**
     * Returns the tax tab content.
     *
     * @return string Tax tab html content
     */
    /*
    public function getTaxTabContent()
    {
        $debug = Tools::getValue('debug');

        $address = new Address((int) $this->order->{Configuration::get('PS_TAX_ADDRESS_TYPE')});
        $tax_exempt = Configuration::get('VATNUMBER_MANAGEMENT')
                            && !empty($address->vat_number)
                            && $address->id_country != Configuration::get('VATNUMBER_COUNTRY');
        $carrier = new Carrier($this->order->id_carrier);

        $tax_breakdowns = $this->getTaxBreakdown();

        $data = array(
            'tax_exempt' => $tax_exempt,
            'use_one_after_another_method' => $this->order_invoice->useOneAfterAnotherTaxComputationMethod(),
            'display_tax_bases_in_breakdowns' => false, // $this->order_invoice->displayTaxBasesInProductTaxesBreakdown(),
            'product_tax_breakdown' => $this->order_invoice->getProductTaxesBreakdown($this->order),
            'shipping_tax_breakdown' => $this->order_invoice->getShippingTaxesBreakdown($this->order),
            'ecotax_tax_breakdown' => $this->order_invoice->getEcoTaxTaxesBreakdown(),
            'wrapping_tax_breakdown' => $this->order_invoice->getWrappingTaxesBreakdown(),
            'tax_breakdowns' => $tax_breakdowns,
            'order' => $debug ? null : $this->order,
            'order_invoice' => $debug ? null : $this->order_invoice,
            'carrier' => $debug ? null : $carrier,
        );

        if ($debug) {
            return $data;
        }

        $this->smarty->assign($data);

        return $this->smarty->fetch($this->template_dir . 'order.tax-tab.tpl');
    }
    */

    /**
     * Returns different tax breakdown elements.
     *
     * @return array Different tax breakdown elements
     */
    /*

    protected function getTaxBreakdown()
    {
        $breakdowns = array(
            'product_tax' => $this->order_invoice->getProductTaxesBreakdown($this->order),
            'shipping_tax' => $this->order_invoice->getShippingTaxesBreakdown($this->order),
            'ecotax_tax' => $this->order_invoice->getEcoTaxTaxesBreakdown(),
            'wrapping_tax' => $this->order_invoice->getWrappingTaxesBreakdown(),
        );

        foreach ($breakdowns as $type => $bd) {
            if (empty($bd)) {
                unset($breakdowns[$type]);
            }
        }

        if (empty($breakdowns)) {
            $breakdowns = false;
        }

        if (isset($breakdowns['product_tax'])) {
            foreach ($breakdowns['product_tax'] as &$bd) {
                $bd['total_tax_excl'] = $bd['total_price_tax_excl'];
            }
        }

        if (isset($breakdowns['ecotax_tax'])) {
            foreach ($breakdowns['ecotax_tax'] as &$bd) {
                $bd['total_tax_excl'] = $bd['ecotax_tax_excl'];
                $bd['total_amount'] = $bd['ecotax_tax_incl'] - $bd['ecotax_tax_excl'];
            }
        }

        return $breakdowns;
    }
    */

    /*
      public function getLogo()
      {
        $this->smarty->assign(array(
          'order' => $this->order,
        ));

        return $this->smarty->fetch($this->template_dir . 'logo.tpl');
      }
    */
    public function getHeader()
    {
        $this->smarty->assign(['shop_address' => $this->product_object['shop_address']]);
        $this->assignCommonHeaderData();
        $this->smarty->assign([
            'reference_description' => $this->product_object['productData']['reference_description'],
        ]);

        return $this->smarty->fetch($this->template_dir . 'product.header.tpl');
    }

    /**
     * Returns the template filename
     * @return string filename
     */
    public function getFooter()
    {
        return $this->smarty->fetch($this->template_dir . 'product-footer.tpl');
    }

    /**
     * Returns the template filename
     * @return string filename
     */
    public function getFilename()
    {
        return 'product-' . $this->product_object['productData']['attribute_reference'] . '.pdf';
    }

    /**
     * Returns the template filename when using bulk rendering
     * @return string filename
     */
    public function getBulkFilename()
    {
        return 'product-' . $this->product_object['productData']['attribute_reference'] . '.pdf';
    }
}
