<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsHTMLTemplatePdf extends HTMLTemplate
{
    public $order;
    public $order_invoice;
    public $template_dir;
    public $pdf_name;
    public $tpls;
    public $layout_cols_width;
    public $main_tpl_name;
    public $legal_free_text = true;
    // public $hide_price;

    public function __construct($order_object, $smarty)
    {
        $this->order = $order_object;
        $this->order_invoice = $this->order; // temporary set to understand collections differences
        $this->smarty = $smarty;
        $this->template_dir = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/hook/pdf/';
        $this->pdf_name = 'order';
        $this->main_tpl_name = 'order';
        // $this->hide_price = false;

        // If shop_address is null, then update it with current one.
        // But no DB save required here to avoid massive updates for bulk PDF generation case.
        // (DB: bug fixed in 1.6.1.1 with upgrade SQL script to avoid null shop_address in old orderInvoices)
        if (!isset($this->order_invoice->shop_address) || !$this->order_invoice->shop_address) {
            $this->order_invoice->shop_address = OrderInvoice::getCurrentFormattedShopAddress((int) $this->order->id_shop);
            // if (!$bulk_mode) {
            //    OrderInvoice::fixAllShopAddresses();
            // }
        }

        // header informations
        $this->date = Tools::displayDate($this->order->date_add);
        $id_lang = Context::getContext()->language->id;
        $this->title = MxbToolsHTMLTemplatePdf::l('Order');
        // footer informations
        $this->shop = new Shop(Context::getContext()->shop->id);

        /* if an element is null, it's tpl won't be loaded */
        $this->tpls = [
            'style_tab' => 'order.style-tab.tpl',
            'addresses_tab' => 'order.addresses-tab.tpl',
            'summary_tab' => 'order.summary-tab.tpl',
            'product_tab' => 'order.product-tab.tpl',
            'tax_tab' => '', // $this->getTaxTabContent(),
            'payment_tab' => 'order.payment-tab.tpl',
            'note_tab' => 'order.note-tab.tpl',
            'total_tab' => 'order.total-tab.tpl',
            'shipping_tab' => 'order.shipping-tab.tpl',
            'add_data_tab' => null,
        ];

        /* if an element is null, it's column will be hidden */
        $this->layout_cols_width = [
            'reference' => [
                'width' => 15,
            ],
            'product' => [
                'width' => 40,
            ],
            'quantity' => [
                'width' => 8,
            ],
            'tax_code' => [
                'width' => 8,
            ],
            'unit_price_tax_excl' => [
                'width' => 0,
            ],
            'total_tax_excl' => [
                'width' => 0,
            ],
        ];
    }

    /**
     * Compute layout elements size.
     *
     * @param $params Array Layout elements
     *
     * @return array Layout elements columns size
     */
    protected function computeLayout($params)
    {
        $layout = $this->layout_cols_width;
        if ($layout['unit_price_tax_excl'] === null) {
            return $layout;
        }

        if (isset($params['has_discount']) && $params['has_discount']) {
            $layout['before_discount'] = ['width' => 0];
            $layout['product']['width'] -= 7;
            $layout['reference']['width'] -= 3;
        }

        $total_width = 0;
        $free_columns_count = 0;
        foreach ($layout as $data) {
            if ($data['width'] === 0) {
                ++$free_columns_count;
            }
            $total_width += $data['width'];
        }

        $delta = 100 - $total_width;
        foreach ($layout as $row => $data) {
            if ($data['width'] === 0) {
                $layout[$row]['width'] = $delta / $free_columns_count;
            }
        }

        $layout['_colCount'] = count($layout);
        return $layout;
    }

    /**
     * Returns the template's HTML content
     * @return string HTML content
     */
    public function getContent()
    {
        $order_details = $this->order_invoice->getProducts();
        if (Module::isInstalled('gadgetcard')) {
            $module = Module::getInstanceByName('gadgetcard');
            $getCodes = 'GadgetcardCodes::getCodes';
            $getProductPrintOption = 'ProductPrintOption::getProductPrintOption';

            // set descriptions for print options
            $gcodes = $getCodes();
            $translator = Context::getContext()->getTranslator();
            $gadgetcard_order_details = [];

            foreach ($order_details as &$order_detail) {
                if (isset($order_detail['customizedDatas']) && count($order_detail['customizedDatas'])) {
                    foreach ($order_detail['customizedDatas'] as $customizationPerAddress) {
                        foreach ($customizationPerAddress as $customizationId => $customization) {
                            if (isset($customization['datas'][Product::CUSTOMIZE_TEXTFIELD]) && count($customization['datas'][Product::CUSTOMIZE_TEXTFIELD])) {
                                foreach ($customization['datas'][Product::CUSTOMIZE_TEXTFIELD] as $customization_infos) {
                                    $ref = $customization_infos['value'];
                                    // LogDebug::writeLog('Gadgetcard', 'HTMLTemplateDeliverySlip::getContent', 'ref=' . $ref);
                                    if (strlen((string) $ref)) {
                                        $references = explode('|', $ref);
                                        $descr = '';
                                        foreach ($references as $reference) {
                                            // get type of customization (setup / print option (decoration) / position )
                                            // LogDebug::writeLog('Gadgetcard', 'HTMLTemplateDeliverySlip::getContent', 'check reference=' . $reference);
                                            $printOption = $getProductPrintOption($reference);
                                            if ($printOption === null) {
                                                continue;
                                            }
                                            $descr .= $printOption['description'] . ' ';
                                            // LogDebug::writeLog('Gadgetcard', 'HTMLTemplateDeliverySlip::getContent', 'descr=' . $descr);
                                        }
                                        // $name = $printOption['suffix'] == $gcodes->decoration_suffix ? $translator->trans('Print option', [], 'Shop.Pdf')
                                        //  : $translator->trans('Print position', [], 'Shop.Pdf');
                                        $name = $printOption['suffix'] == $gcodes->decoration_suffix ? $module->l('Print option')
                                            : $module->l('Print position');
                                        $gadgetcard_order_details[$ref] = [
                                            'value' => $descr,
                                            'name' => $name,
                                        ];
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (count($gadgetcard_order_details)) {
                $this->smarty->assign('mxb_order_details', $gadgetcard_order_details);
            }
        }

        // get customer data
        $customerData = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
            SELECT *
            FROM `' . _DB_PREFIX_ . 'customer`
            WHERE `id_customer` = \'' . (int) $this->order->id_customer . '\'');
        $invoiceAddressPatternRules = json_decode(Configuration::get('PS_INVCE_INVOICE_ADDR_RULES'), true);
        $deliveryAddressPatternRules = json_decode(Configuration::get('PS_INVCE_DELIVERY_ADDR_RULES'), true);

        $invoice_address = new Address((int) $this->order->id_address_invoice);
        $country = new Country((int) $invoice_address->id_country);
        $formatted_invoice_address = AddressFormat::generateAddress($invoice_address, $invoiceAddressPatternRules, '<br />', ' ');

        $delivery_address = null;
        $formatted_delivery_address = '';
        if (isset($this->order->id_address_delivery) && $this->order->id_address_delivery) {
            $delivery_address = new Address((int) $this->order->id_address_delivery);
            $formatted_delivery_address = AddressFormat::generateAddress($delivery_address, $deliveryAddressPatternRules, '<br />', ' ');
        }

        $customer = new Customer((int) $this->order->id_customer);
        $carrier = new Carrier((int) $this->order->id_carrier);
        // $order_details = $this->order_invoice->getProducts();

        $has_discount = false;
        foreach ($order_details as $id => &$order_detail) {
            // set missing fields for the template, copied from invoice template...
            if (!isset($order_detail['ecotax_tax_excl'])) {
                $order_detail['ecotax_tax_excl'] = $order_detail['ecotax'];
            }
            if (!isset($order_detail['unit_price_tax_excl_including_ecotax'])) {
                $order_detail['unit_price_tax_excl_including_ecotax'] = $order_detail['unit_price_tax_excl'] + $order_detail['ecotax'];
            }
            if (!isset($order_detail['total_price_tax_excl_including_ecotax'])) {
                $order_detail['total_price_tax_excl_including_ecotax'] = $order_detail['total_price_tax_excl'] + ($order_detail['ecotax'] * $order_detail['product_quantity']);
            }

            // Find out if column 'price before discount' is required
            if ($order_detail['reduction_amount_tax_excl'] > 0) {
                $has_discount = true;
                $order_detail['unit_price_tax_excl_before_specific_price'] = $order_detail['unit_price_tax_excl_including_ecotax'] + $order_detail['reduction_amount_tax_excl'];
            } elseif ($order_detail['reduction_percent'] > 0) {
                $has_discount = true;
                if ($order_detail['reduction_percent'] == 100) {
                    $order_detail['unit_price_tax_excl_before_specific_price'] = 0;
                } else {
                    $order_detail['unit_price_tax_excl_before_specific_price'] = (100 * $order_detail['unit_price_tax_excl_including_ecotax']) / (100 - $order_detail['reduction_percent']);
                }
            }

            // Set tax_code
            $taxes = OrderDetail::getTaxListStatic($id);
            $tax_temp = [];
            foreach ($taxes as $tax) {
                $obj = new Tax($tax['id_tax']);
                $translator = Context::getContext()->getTranslator();
                $tax_temp[] = $translator->trans(
                    '%taxrate%%space%%',
                    [
                        '%taxrate%' => ($obj->rate + 0),
                        '%space%' => ' ',
                    ],
                    'Shop.Pdf'
                );
            }

            $order_detail['order_detail_tax'] = $taxes;
            $order_detail['order_detail_tax_label'] = implode(', ', $tax_temp);
        }
        unset(
            $tax_temp,
            $order_detail
        );
        /*
            if (Configuration::get('PS_PDF_IMG_INVOICE')) {
                foreach ($order_details as &$order_detail) {
                    if ($order_detail['image'] != null) {
                        $name = 'product_mini_' . (int) $order_detail['product_id'] . (isset($order_detail['product_attribute_id']) ? '_' . (int) $order_detail['product_attribute_id'] : '') . '.jpg';
                        $path = _PS_PROD_IMG_DIR_ . $order_detail['image']->getExistingImgPath() . '.jpg';

                        $order_detail['image_tag'] = preg_replace(
                            '/\.*' . preg_quote(__PS_BASE_URI__, '/') . '/',
                            _PS_ROOT_DIR_ . DIRECTORY_SEPARATOR,
                            ImageManager::thumbnail($path, $name, 45, 'jpg', false),
                            1
                        );

                        if (file_exists(_PS_TMP_IMG_DIR_ . $name)) {
                            $order_detail['image_size'] = getimagesize(_PS_TMP_IMG_DIR_ . $name);
                        } else {
                            $order_detail['image_size'] = false;
                        }
                    }
                }
                unset($order_detail); // don't overwrite the last order_detail later
            }
        */
        $cart_rules = $this->order->getCartRules($this->order_invoice->id);
        $free_shipping = false;
        foreach ($cart_rules as $key => $cart_rule) {
            if ($cart_rule['free_shipping']) {
                $free_shipping = true;
                /*
                 * Adjust cart rule value to remove the amount of the shipping.
                 * We're not interested in displaying the shipping discount as it is already shown as 'Free Shipping'.
                 */
                $cart_rules[$key]['value_tax_excl'] -= $this->order_invoice->total_shipping_tax_excl;
                $cart_rules[$key]['value'] -= $this->order_invoice->total_shipping_tax_incl;

                /*
                 * Don't display cart rules that are only about free shipping and don't create
                 * a discount on products.
                 */
                if ($cart_rules[$key]['value'] == 0) {
                    unset($cart_rules[$key]);
                }
            }
        }

        $product_taxes = 0;
        foreach ($this->order_invoice->getProductTaxesBreakdown($this->order) as $details) {
            $product_taxes += $details['total_amount'];
        }

        // $product_discounts_tax_excl = $this->order_invoice->total_discount_tax_excl;
        // $product_discounts_tax_incl = $this->order_invoice->total_discount_tax_incl;
        $product_discounts_tax_excl = $this->order_invoice->total_discounts_tax_excl;
        $product_discounts_tax_incl = $this->order_invoice->total_discounts_tax_incl;
        if ($free_shipping) {
            $product_discounts_tax_excl -= $this->order_invoice->total_shipping_tax_excl;
            $product_discounts_tax_incl -= $this->order_invoice->total_shipping_tax_incl;
        }
        $products_after_discounts_tax_excl = $this->order_invoice->total_products - $product_discounts_tax_excl;
        $products_after_discounts_tax_incl = $this->order_invoice->total_products_wt - $product_discounts_tax_incl;
        $shipping_tax_excl = $free_shipping ? 0 : $this->order_invoice->total_shipping_tax_excl;
        $shipping_tax_incl = $free_shipping ? 0 : $this->order_invoice->total_shipping_tax_incl;
        $shipping_taxes = $shipping_tax_incl - $shipping_tax_excl;
        $wrapping_taxes = $this->order_invoice->total_wrapping_tax_incl - $this->order_invoice->total_wrapping_tax_excl;
        $total_taxes = $this->order_invoice->total_paid_tax_incl - $this->order_invoice->total_paid_tax_excl;

        $footer = [
            'products_before_discounts_tax_excl' => $this->order_invoice->total_products,
            'product_discounts_tax_excl' => $product_discounts_tax_excl,
            'products_after_discounts_tax_excl' => $products_after_discounts_tax_excl,
            'products_before_discounts_tax_incl' => $this->order_invoice->total_products_wt,
            'product_discounts_tax_incl' => $product_discounts_tax_incl,
            'products_after_discounts_tax_incl' => $products_after_discounts_tax_incl,
            'product_taxes' => $product_taxes,
            'shipping_tax_excl' => $shipping_tax_excl,
            'shipping_taxes' => $shipping_taxes,
            'shipping_tax_incl' => $shipping_tax_incl,
            'wrapping_tax_excl' => $this->order_invoice->total_wrapping_tax_excl,
            'wrapping_taxes' => $wrapping_taxes,
            'wrapping_tax_incl' => $this->order_invoice->total_wrapping_tax_incl,
            'ecotax_taxes' => $total_taxes - $product_taxes - $wrapping_taxes - $shipping_taxes,
            'total_taxes' => $total_taxes,
            'total_paid_tax_excl' => $this->order_invoice->total_paid_tax_excl,
            'total_paid_tax_incl' => $this->order_invoice->total_paid_tax_incl,
        ];

        foreach ($footer as $key => $value) {
            $footer[$key] = Tools::ps_round($value, _PS_PRICE_COMPUTE_PRECISION_, $this->order->round_mode);
        }

        /**
         * Need the $round_mode for the tests.
         */
        $round_type = null;
        switch ($this->order->round_type) {
            case Order::ROUND_TOTAL:
                $round_type = 'total';

                break;
            case Order::ROUND_LINE:
                $round_type = 'line';

                break;
            case Order::ROUND_ITEM:
                $round_type = 'item';

                break;
            default:
                $round_type = 'line';

                break;
        }

        $display_product_images = Configuration::get('PS_PDF_IMG_INVOICE');
        $tax_excluded_display = Group::getPriceDisplayMethod($customer->id_default_group);
        $layout = $this->computeLayout(['has_discount' => $has_discount]);

        if ($this->legal_free_text) {
            $legal_free_text = Hook::exec('displayInvoiceLegalFreeText', ['order' => $this->order]);
            if (!$legal_free_text) {
                $legal_free_text = Configuration::get('PS_INVOICE_LEGAL_FREE_TEXT', (int) Context::getContext()->language->id, null, (int) $this->order->id_shop);
            }
        } else {
            $legal_free_text = null;
        }

        $data = [
            'order' => $this->order,
            'order_invoice' => $this->order_invoice,
            'order_details' => $order_details,
            'carrier' => $carrier,
            'cart_rules' => $cart_rules,
            'delivery_address' => $formatted_delivery_address,
            'invoice_address' => $formatted_invoice_address,
            'addresses' => ['invoice' => $invoice_address, 'delivery' => $delivery_address],
            'tax_excluded_display' => $tax_excluded_display,
            'display_product_images' => $display_product_images,
            'layout' => $layout,
            // 'tax_tab' => $this->getTaxTabContent(),
            'customer' => $customer,
            'footer' => $footer,
            'ps_price_compute_precision' => _PS_PRICE_COMPUTE_PRECISION_,
            'round_type' => $round_type,
            'legal_free_text' => $legal_free_text,
            'customer_data' => $customerData,
        ];

        if (Tools::getValue('debug')) {
            exit(json_encode($data));
        }
        $this->smarty->assign($data);
        /*
            $tpls = array(
                'style_tab' => $this->smarty->fetch($this->template_dir . 'order.style-tab.tpl'),
                'addresses_tab' => $this->smarty->fetch($this->template_dir . 'order.addresses-tab.tpl'),
                'summary_tab' => $this->smarty->fetch($this->template_dir . 'order.summary-tab.tpl'),
                'product_tab' => $this->smarty->fetch($this->template_dir . 'order.product-tab.tpl'),
                'tax_tab' => $this->getTaxTabContent(),
                'payment_tab' => $this->smarty->fetch($this->template_dir . 'order.payment-tab.tpl'),
                'note_tab' => $this->smarty->fetch($this->template_dir . 'order.note-tab.tpl'),
                'total_tab' => $this->smarty->fetch($this->template_dir . 'order.total-tab.tpl'),
                'shipping_tab' => $this->smarty->fetch($this->template_dir . 'order.shipping-tab.tpl'),
            );
        */
        $tpls = [];
        foreach ($this->tpls as $key => $tpl) {
            if ($tpl !== null) {
                if ($key == 'tax_tab') {
                    $tpls[$key] = $this->getTaxTabContent();
                } else {
                    $tpls[$key] = $this->smarty->fetch($this->template_dir . $tpl);
                }
            }
        }
        $this->smarty->assign($tpls);

        return $this->smarty->fetch($this->template_dir . $this->main_tpl_name . '.tpl');
    }

    /**
     * Returns the tax tab content.
     *
     * @return string Tax tab html content
     */
    public function getTaxTabContent()
    {
        $debug = Tools::getValue('debug');
        $address = new Address((int) $this->order->{Configuration::get('PS_TAX_ADDRESS_TYPE')});
        $tax_exempt = Configuration::get('VATNUMBER_MANAGEMENT')
            && !empty($address->vat_number)
            && $address->id_country != Configuration::get('VATNUMBER_COUNTRY');
        $carrier = new Carrier($this->order->id_carrier);
        $tax_breakdowns = $this->getTaxBreakdown();

        $data = [
            'tax_exempt' => $tax_exempt,
            'use_one_after_another_method' => $this->order_invoice->useOneAfterAnotherTaxComputationMethod(),
            'display_tax_bases_in_breakdowns' => false, // $this->order_invoice->displayTaxBasesInProductTaxesBreakdown(),
            'product_tax_breakdown' => $this->order_invoice->getProductTaxesBreakdown($this->order),
            'shipping_tax_breakdown' => $this->order_invoice->getShippingTaxesBreakdown($this->order),
            'ecotax_tax_breakdown' => $this->order_invoice->getEcoTaxTaxesBreakdown(),
            'wrapping_tax_breakdown' => $this->order_invoice->getWrappingTaxesBreakdown(),
            'tax_breakdowns' => $tax_breakdowns,
            'order' => $debug ? null : $this->order,
            'order_invoice' => $debug ? null : $this->order_invoice,
            'carrier' => $debug ? null : $carrier,
        ];

        if ($debug) {
            return $data;
        }

        $this->smarty->assign($data);

        return $this->smarty->fetch($this->template_dir . 'order.tax-tab.tpl');
    }

    /**
     * Returns different tax breakdown elements.
     *
     * @return array Different tax breakdown elements
     */
    protected function getTaxBreakdown()
    {
        $breakdowns = [
            'product_tax' => $this->order_invoice->getProductTaxesBreakdown($this->order),
            'shipping_tax' => $this->order_invoice->getShippingTaxesBreakdown($this->order),
            'ecotax_tax' => $this->order_invoice->getEcoTaxTaxesBreakdown(),
            'wrapping_tax' => $this->order_invoice->getWrappingTaxesBreakdown(),
        ];

        foreach ($breakdowns as $type => $bd) {
            if (empty($bd)) {
                unset($breakdowns[$type]);
            }
        }

        if (empty($breakdowns)) {
            $breakdowns = false;
        }

        if (isset($breakdowns['product_tax'])) {
            foreach ($breakdowns['product_tax'] as &$bd) {
                $bd['total_tax_excl'] = $bd['total_price_tax_excl'];
            }
        }

        if (isset($breakdowns['ecotax_tax'])) {
            foreach ($breakdowns['ecotax_tax'] as &$bd) {
                $bd['total_tax_excl'] = $bd['ecotax_tax_excl'];
                $bd['total_amount'] = $bd['ecotax_tax_incl'] - $bd['ecotax_tax_excl'];
            }
        }

        return $breakdowns;
    }

    /*
      public function getLogo()
      {
        $this->smarty->assign(array(
          'order' => $this->order,
        ));

        return $this->smarty->fetch($this->template_dir . 'logo.tpl');
      }
    */
    public function getHeader()
    {
        if ((bool) Configuration::getGlobalValue('GADGETCARD_INVOICE_ALT_ADDRESS')) {
            $this->smarty->assign(['shop_address' => str_replace('\n', '<br />', (string) Configuration::getGlobalValue('GADGETCARD_INVOICE_ALT_ADDRESS'))]);
        } else {
            $this->smarty->assign(['shop_address' => $this->order_invoice->shop_address]);
        }
        /*
            $this->assignCommonHeaderData();
                $this->smarty->assign(array('header' => Context::getContext()->getTranslator()->trans('Order', [], 'Shop.Pdf')));
                return $this->smarty->fetch($this->template_dir . 'header.tpl');
            $this->smarty->assign(array(
              'order' => $this->order,
            ));
            return $this->smarty->fetch($this->template_dir . 'header.tpl');
        */

        return parent::getHeader();
    }

    /**
     * Returns the template filename
     *
     * @return string filename
     */

    /*
        public function getFooter()
        {
            return $this->smarty->fetch($this->template_dir . 'footer.tpl');
        }
    */

    /**
     * Returns the template filename
     *
     * @return string filename
     */
    public function getFilename()
    {
        return $this->pdf_name . '-' . $this->order->reference . '.pdf';
    }

    /**
     * Returns the template filename when using bulk rendering
     *
     * @return string filename
     */
    public function getBulkFilename()
    {
        return $this->pdf_name . '-' . $this->order->reference . '.pdf';
    }

    /**
     * Returns the invoice logo.
     *
     * @return string|null
     */
    protected function getLogo()
    {
        /*
            $id_shop = (int) $this->shop->id;
            $invoiceLogo = Configuration::get('PS_LOGO_INVOICE', null, null, $id_shop);
            if ($invoiceLogo && file_exists(_PS_IMG_DIR_ . $invoiceLogo)) {
                return $invoiceLogo;
            }
            $logo = Configuration::get('PS_LOGO', null, null, $id_shop);
            if ($logo && file_exists(_PS_IMG_DIR_ . $logo)) {
                return $logo;
            }
            return null;
        */
        return parent::getLogo();
    }

    /**
     * Assign common header data to smarty variables.
     */
    public function assignCommonHeaderData()
    {
        return parent::assignCommonHeaderData();
    }

    /**
     * If the template is not present in the theme directory, it will return the default template
     * in _PS_PDF_DIR_ directory.
     *
     * @param $template_name
     *
     * @return string
     */
    protected function getTemplate($template_name)
    {
        // MXB: check if the template exists in MXB_TOOLS
        $template = false;
        $default_template = $this->template_dir . $template_name . '.tpl';
        // $overridden_template = _PS_ALL_THEMES_DIR_ . $this->shop->theme->getName() . DIRECTORY_SEPARATOR . 'pdf' . DIRECTORY_SEPARATOR . $template_name . '.tpl';
        if (file_exists($default_template)) {
            $template = $default_template;
        } else {
            $template = parent::getTemplate($template_name);
        }
        return $template;
    }
}
