<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsMenu.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsCalculatorsController extends ModuleAdminController
{
    /*
        protected $colorOnBackground = true;
        protected $color_on_background = true;
    */
    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->class_name = 'AdminMxbToolsCalculatorsController';
        // $this->class_name = Mxb_Tools::getControllerName('Calculators', null, true);
        $this->tab = 'front_office_features';
        $this->templateFile = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/calculators.tpl';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('MXB Tools: calculators');
    }

    public function renderList()
    {
        $output = '';
        if (Tools::getValue('submitAddmxb_tools_calculators')) {
            $output .= $this->processForm();
        }
        $smarty = Context::getContext()->smarty;
        /*
        $smarty->assign('languages', Language::getLanguages(false));
        $smarty->assign('defaultFormLanguage', (int) $this->context->language->id);
        $smarty->assign('uri', MxbToolsUtils::getModulePathUri());
        $smarty->assign('module_dir', _PS_MODULE_DIR_ . 'mxb_tools'); // $this->_path);
        */

        // add config values to smarty
        $smarty->assign($this->getConfigFormValues());
        $smarty->assign(MxbToolsUtils::getCommonAdminTplValues($this->context));
        $smarty->assign('form_action', $this->context->link->getAdminLink('AdminMxbToolsCalculators'));

        // add header menu
        $smarty->assign('mxb_tools_menu', MxbToolsMenu::getMxbToolsMenu($this->context));

        $output .= $this->context->smarty->fetch($this->templateFile);
        return $output; // .$this->renderForm();
    }

    protected function getValue($key, $default = null, $type = null, $save = false)
    {
        return MxbToolsUtils::getConfigValue($key, $default, $type, $save);
        /*
          if (Tools::getIsset($key)) {
            if ($save)
              $this->setValue($key, Tools::getValue($key), $type);
              $ret = $this->setValueType(Tools::getValue($key), $type);
              return $ret;
          }
          $ret = Configuration::getGlobalValue($key);
          if ($default !== null && $ret === false)
            $ret = $default;
          if ($save && $default !== null)
            $this->setValue($key, $ret, $type);
          return $this->setValueType($ret, $type);
        */
    }

    protected function setValue($key, $value, $type = null)
    {
        MxbToolsUtils::setConfigValue($key, $value, $type);
        // Configuration::updateGlobalValue($key, $this->setValueType($value, $type));
    }
    /*
        protected function setValueType($value, $type = null) {
            if ($type == 'string')
              return $value;
            elseif ($type == 'int')
              return (int) $value;
            elseif ($type == 'bool')
              return (int) $value;
            return $value;
        }
    */

    /**
     * Get values for the inputs.
     */
    public function getConfigFormValues()
    {
        return MxbToolsUtils::getConfigFormValues();
    }

    public function processForm()
    {
        $smarty = Context::getContext()->smarty;
        $form_error = [];
        $form_success = [];
        $form_values = $this->getConfigFormValues();

        // validation: not needed
        if (count($form_error)) {
            return $this->module->displayError(
                $this->l('Unable to save configuration for the following reasons:')
                    . '<br>.' . implode('<br>. ', $form_error)
            );
        }

        foreach (array_keys($form_values) as $key) {
            Configuration::updateGlobalValue($key, $form_values[$key]);
        }

        if (count($form_success)) {
            return $this->module->displayConfirmation(
                $this->l('Configuration saved:')
                    . '<br>.' . implode('<br>. ', $form_success)
            );
        }
        return $this->module->displayConfirmation($this->l('Configuration saved'));
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);
        return $return;
    }
}
