<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsMenu.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsConfigurationController extends ModuleAdminController
{
    /*
        protected $colorOnBackground = true;
        protected $color_on_background = true;
    */
    public $customCssFolder = null;
    public $customJsFolder = null;

    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->class_name = 'AdminMxbToolsConfigurationController';
        $this->tab = 'front_office_features';
        $this->customCssFolder = 'customcss';
        $this->customJsFolder = 'customjs';
        /*
          $this->name = 'gadgetcarddecorationcontroller';
          $this->displayName = 'Gadget Card configuration';
          $this->table = 'gadgetcard_decoration_method';
          $this->className = 'Decoration';
          $this->lang = true;
          // $this->list_no_link = true;
          $this->module = 'gadgetcard';
        */

        $this->templateFile = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/configuration.tpl';
        $this->multishop_context = Shop::CONTEXT_ALL;
        /*
            $this->_orderBy = 'id_gadgetcard_decoration_method';
            $this->_orderWay = 'DESC';
        */
        $this->bootstrap = true;
        // $this->addWaitingFlows();
        parent::__construct();

        $this->displayName = $this->l('MXB Tools: configuration');
        /*
        if (Tools::getValue('action') && Tools::getValue('action')=='MxbToolsTools') {
            // Tools::redirectAdmin('?controller=AdminMxbToolsTools&token='.$this->token.'&m=0');
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminMxbToolsTools').'&m=0');
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='Mxb_ToolsCalculators') {
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminMxbToolsCalculators').'&m=0');
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='Mxb_ToolsImport') {
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminMxbToolsImport').'&m=0');
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='MxbToolsExport') {
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminMxbToolsExport').'&m=0');
        }
        */
        if (Tools::getValue('action') && Tools::getValue('action') == 'updateMxbToolsProcedures') {
            $this->updateMxbToolsProcedures();
        }
    }

    /*
        protected function getPathUri() {
            return $this->context->link->getPageLink('',true) . 'modules/mxb_tools' . DIRECTORY_SEPARATOR;
        }
        protected function getPath() {
            return _PS_MODULE_DIR_ . 'gadgetcard' . DIRECTORY_SEPARATOR;
        }
    */

    public function renderList()
    {
        // MxbToolsDebug::writeLog($this->name, $this->class_name, 'renderList()');
        $output = '';
        if (Tools::getValue('submitAddmxb_tools_configuration')) {
            $output .= $this->processForm();
        }

        // @TODO Add box to display ajax cron call
        $smarty = Context::getContext()->smarty;

        // add config values to smarty
        $smarty->assign($this->getConfigFormValues());
        $smarty->assign(MxbToolsUtils::getCommonAdminTplValues($this->context));
        $smarty->assign('form_action', $this->context->link->getAdminLink('AdminMxbToolsConfiguration'));
        $smarty->assign(
            'module_name_and_version',
            $this->module->displayName . ' ver. ' . $this->module->version . ' (dev. ' . $this->module->dev_version . ')'
        );

        // add header menu
        $smarty->assign('mxb_tools_menu', MxbToolsMenu::getMxbToolsMenu($this->context));

        // add link for CSS sample
        $smarty->assign('default_global_css_link', Tools::getHttpHost(true) . __PS_BASE_URI__
            . '/modules/mxb_tools/views/css/_global_default.css');

        // add common modules list
        $common_modules = MxbToolsMenu::getCommonModuleList($this->context, false, true);
        if ($common_modules) {
            $smarty->assign('common_modules', $common_modules);
        }

        // add mxb modules list
        $mxb_modules = MxbToolsMenu::getMxbModuleList($this->context, false, true);
        // if ($mxb_modules) {
        //     $smarty->assign('mxb_modules', $mxb_modules);
        // }

        // add mxb modules download list
        $online_modules = MxbToolsMenu::getMxbModuleDownloadList($this->context, false);
        // MxbToolsDebug::writeLog($this->name, $this->class_name, 'getMxbModuleDownloadList > online_modules: ' . print_r($online_modules, true));
        if ($online_modules && $mxb_modules) {
            // check version for upgrade
            foreach ($online_modules as &$mod) {
                if (isset($mxb_modules[$mod['name']])) {
                    if (isset($mxb_modules[$mod['name']]['version']) && isset($mod['version'])) {
                        if (version_compare($mxb_modules[$mod['name']]['version'], $mod['version'], '<')) {
                            $mod['upgrade_available'] = 1;
                            $mxb_modules[$mod['name']]['upgrade_available'] = 1;
                        }
                    }
                }
            }
        }
        if ($online_modules) {
            $smarty->assign('mxb_modules_download', $online_modules);
        }
        if ($mxb_modules) {
            $smarty->assign('mxb_modules', $mxb_modules);
        }

        // add cms.css content
        $content = MxbToolsUtils::readModuleFile('views/css/cms.css');
        $smarty->assign('cms_csv_content', $content);

        // add cms-sample.html content
        $content = MxbToolsUtils::readModuleFile('docs/cms-sample.html');
        $smarty->assign('cms_html_content', $content);
        $output .= $this->context->smarty->fetch($this->templateFile);

        return $output; // .$this->renderForm();
    }

    protected function getValue($key, $default = null, $type = null, $save = false)
    {
        /*
          // dedicated management for following keys:
          if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_CSS' && !Tools::getIsset($key)) {
            $content = $this->readCssFile($this->customCssFolder.'/global.css', 'css/_global_default.css');
            return $content;
          }
        */
        return MxbToolsUtils::getConfigValue($key, $default, $type, $save);
        /*
          if (Tools::getIsset($key)) {
            if ($save)
              $this->setValue($key, Tools::getValue($key), $type);
              $ret = $this->setValueType(Tools::getValue($key), $type);
              return $ret;
          }
          $ret = Configuration::getGlobalValue($key);
          if ($default !== null && $ret === false)
            $ret = $default;
          if ($save && $default !== null)
            $this->setValue($key, $ret, $type);
          return $this->setValueType($ret, $type);
        */
    }

    protected function setValue($key, $value, $type = null)
    {
        MxbToolsUtils::setConfigValue($key, $value, $type);
    }
    /*
        protected function setValueType($value, $type = null) {
            if ($type == 'string')
                return $value;
            elseif ($type == 'int')
                return (int) $value;
            elseif ($type == 'bool')
                return (int) $value == 1;
            return $value;
        }
    */

    /**
     * Set values for the inputs.
     */
    public function getConfigFormValues()
    {
        return MxbToolsUtils::getConfigFormValues('config');
    }

    public function processForm()
    {
        $smarty = Context::getContext()->smarty;
        $form_error = [];
        $form_success = [];

        // validation
        if ((int) Tools::getValue('MXB_TOOLS_PRICE_NUMBER_OF_DIGITS') < 0) {
            $form_error[] = $this->l('Number of digits for prices must be equal or greater than 0.');
        }
        if ((int) Tools::getValue('MXB_TOOLS_PRICE_NUMBER_OF_DIGITS') > 4) {
            $form_error[] = $this->l('Number of digits for prices must be equal or lower than 4.');
        }

        // if (count($form_error) > 0)
        $form_values = $this->getConfigFormValues();

        // MxbToolsDebug::writeLog($this->name, $this->class_name, 'form_values=' . print_r($form_values, true));

        if ((int) $form_values['MXB_TOOLS_PRICE_NUMBER_OF_DIGITS'] !==
            (int) Configuration::getGlobalValue('MXB_TOOLS_PRICE_NUMBER_OF_DIGITS')) {
            $this->setPriceNumberOfDigits($form_values['MXB_TOOLS_PRICE_NUMBER_OF_DIGITS']);
        }
        // if mxbmemberprogram module is installed, let it manage these values
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            $form_values['MXB_TOOLS_BLOCK_EMAIL_ORDER_NEW_VOUCHER'] = 0;
            $form_values['MXB_TOOLS_REMOVE_SHOP_NAME_FROM_EMAIL_SUBJECT'] = 0;
            $form_values['MXB_TOOLS_BLOCK_EMAIL_ORDER_TRACKING_ADDED'] = 0;
            $form_values['MXB_TOOLS_NEW_CUSTOMER_EMAIL_ADDRESS'] = '';
            $form_values['MXB_TOOLS_CUSTOMER_REDIRECT_AFTER_LOGIN'] = 0;
            // $form_values['MXB_TOOLS_ENABLE_WAIT_LAYER'] = 0;
            $form_success[] = $this->l(
                'Some settings were reset because MXB Member Program was found and already manages that stuff.'
            );
        }

        if (count($form_error)) {
            return $this->module->displayError($this->l('Unable to save configuration for the following reasons:')
                . '<br>. ' . implode('<br>. ', $form_error));
        }

        if ((int) Tools::getValue('MXB_TOOLS_COVERJOB_LOGO_DELETE') === 1) {
            // delete all COVERJOB_LOGO icons
            /* lang true
                foreach (Language::getLanguages(false) as $lang) {
                  $file_name = Configuration::getGlobalValue('MXB_TOOLS_COVERJOB_LOGO', (int) $lang['id_lang']);
                  $form_values['MXB_TOOLS_COVERJOB_LOGO'][(int) $lang['id_lang']] = '';
                  if ($file_name && strlen($file_name) > 0) {
                    // remove file
                    if (is_file(MxbToolsUtils::getModulePath().'img'.DIRECTORY_SEPARATOR.$file_name)) {
                      if (unlink(MxbToolsUtils::getModulePath().'img'.DIRECTORY_SEPARATOR.$file_name))
                        $form_success[] = $this->l('Image deleted: ') . $file_name;
                      else
                        $form_error[] = $this->l('Error deleting image file: ') . $file_name;
                    }
                  }
                }
                // clear variabile
                Configuration::updateGlobalValue('MXB_TOOLS_COVERJOB_LOGO', $form_values['MXB_TOOLS_COVERJOB_LOGO']);
            */
            // lang false
            $file_name = Configuration::getGlobalValue('MXB_TOOLS_COVERJOB_LOGO');
            $form_values['MXB_TOOLS_COVERJOB_LOGO'] = '';
            if ($file_name && strlen($file_name) > 0) {
                // remove file
                // $path = MxbToolsUtils::getModulePath().'img'.DIRECTORY_SEPARATOR;
                $path = _PS_IMG_DIR_;

                if (is_file($path . $file_name)) {
                    if (unlink($path . $file_name)) {
                        $form_success[] = $this->l('Image deleted: ') . $file_name;
                    } else {
                        $form_error[] = $this->l('Error deleting image file: ') . $file_name;
                    }
                }
            }
        }

        // lang false
        if (isset($_FILES['MXB_TOOLS_COVERJOB_LOGO'])
            && isset($_FILES['MXB_TOOLS_COVERJOB_LOGO']['tmp_name'])
            && !empty($_FILES['MXB_TOOLS_COVERJOB_LOGO']['tmp_name'])
        ) {
            if ($error = ImageManager::validateUpload($_FILES['MXB_TOOLS_COVERJOB_LOGO'], 4000000)) {
                $form_error[] = $error;
            } else {
                $path = _PS_IMG_DIR_;
                $ext = substr(
                    $_FILES['MXB_TOOLS_COVERJOB_LOGO']['name'],
                    strrpos($_FILES['MXB_TOOLS_COVERJOB_LOGO']['name'], '.') + 1
                );
                $file_name = md5($_FILES['MXB_TOOLS_COVERJOB_LOGO']['name']) . '.' . $ext;

                if (!move_uploaded_file($_FILES['MXB_TOOLS_COVERJOB_LOGO']['tmp_name'], $path . $file_name)) {
                    $form_error[] = $this->trans(
                        'An error occurred while attempting to upload the file.',
                        [],
                        'Admin.Notifications.Error'
                    );
                } else {
                    $form_values['MXB_TOOLS_COVERJOB_LOGO'] = $file_name;
                    $form_success[] = $this->l('Image added: ') . $file_name;
                }
            }
        }

        foreach (array_keys($form_values) as $key) {
            if ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_CSS') {
                $this->saveCssFile($this->getValue($key), $this->customCssFolder . '/global.css');
            } elseif ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_JS') {
                $content = $this->getValue($key);
                if (!empty($content)) {
                    MxbToolsUtils::saveJsFile($this->getValue($key), $this->customJsFolder . '/custom.js');
                }
            /*
            } elseif ($key == 'MXB_TOOLS_GLOBAL_CUSTOM_FOOTER_JS') {
                $content = $this->getValue($key);
                if (!empty($content)) {
                    MxbToolsUtils::saveJsFile($this->getValue($key), $this->customJsFolder . '/custom_footer.js');
                }
            */
            } else {
                $ret = Configuration::updateGlobalValue($key, $form_values[$key]);
            }
        }

        // $this->_clearCache($this->templateFile);
        if (count($form_success)) {
            return $this->module->displayConfirmation(
                $this->l('Configuration saved') . '<br>. ' . implode('<br>. ', $form_success)
            );
        }

        return $this->module->displayConfirmation($this->l('Configuration saved'));
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);

        return $return;
    }

    protected function saveCssFile($content, $filename = 'css/global.css')
    {
        return MxbToolsUtils::saveCssFile($content, $filename);
    }

    protected function readCssFile($filename = 'customcss/global.css', $filename_default = 'css/_global_default.css')
    {
        return MxbToolsUtils::readCssFile($filename, $filename_default);
    }

    protected function setPriceNumberOfDigits($digits = 2)
    {
        // change digits in database
        $sql = '
            UPDATE ' . _DB_PREFIX_ . 'currency
            SET `precision`=' . (int) $digits . '
            WHERE `active`=1';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);

        // load active languages
        $sql = '
            SELECT locale
            FROM ' . _DB_PREFIX_ . 'lang
            WHERE active = 1';
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);

        // loop trough language to change translations/CLDR file if exists (files was removed from PS 1.7.6)
        if (isset($result[0])) {
            foreach ($result as $langcode) {
                $filename = $this->setPriceNumberOfDigitsForLanguage($digits, $langcode['locale']);
                if (isset($filename)) {
                    if ($filename === false) {
                        $this->errors[] = $this->l('No changes for language') . ' ' . $langcode['locale'];
                    } else {
                        $this->confirmations[] =
                            $this->l('Number of digits changed in "currency" table and "translations/CLDR" file.')
                            . ' ' . $this->l('Original backup file for language') . ' ' . $langcode['locale']
                            . ' : <a href="'
                            . MxbToolsUtils::path2url(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename)
                            // . $this->context->link->getPageLink('',true) . 'translations/cldr/' . $filename
                            . '" target="_new">' . _PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename . '</a><br />';
                    }
                } else {
                    $this->confirmations[] =
                        $this->l('"translations/CLDR" language file missing in this Prestashop version.')
                        . ' ' . $this->l('Changes was made only in "currency" table for language')
                        . ' ' . $langcode['locale'];
                }
            }
        }

        if (count($this->confirmations)) {
            $this->confirmations[] = $this->l(
                'If the number of decimals has not changed, please clear Prestashop cache and browser cache too.'
            );
        }
    }

    /**
     * change the "number of digits" string into the json file with locale definitions
     * files are here:
     * and are named as follow:
     *  main--it-IT--numbers
     *  main--en-US--numbers
     *  ... and so on, for each language using the iso code
     *
     * the value is in the json file, at this level:
     *  ['main']['it-IT']['numbers']['currencyFormats-numberSystem-latn']['standard']
     *
     * sample of value to edit:
     *  it-IT:  "#,##0.0000\u00a0\u00a4"
     *  en-US:  "\u00a4#,##0.00"
     */
    protected function setPriceNumberOfDigitsForLanguage($digits = 2, $iso = null)
    {
        if ($iso === null) {
            return;
        }
        /*
            // create the digits string to put into the json file
            // hases format part can have both dot or comma as separator
            // format of "standard" field:
            //  2 digits: \u00a4#.###0.000
            //  3 digits: \u00a4#,####0.0000
            //  4 digits: \u00a4#,#####0.00000
            // format of "accounting" filed:
            //  2 digits: \u00a4#.##0.00;(\u00a4#.##0.00)
        */
        // $stringRoot = '\u00a4';
        $newValue = '0';
        $newHashes = '#';
        if ((int) $digits > 0) {
            $newValue .= '.';
            $newHashes .= '.';

            for ($i = 0; $i < $digits; ++$i) {
                $newValue .= '0';
                $newHashes .= '#';
            }
        }

        // this is the name of the file to edit
        $filename = 'main--' . $iso . '--numbers';

        // if the target file does not exist, exit
        if (!file_exists(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename)) {
            return;
        }

        // read the file that contains the number of digits definition
        $jsonString = Tools::file_get_contents(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename);
        // and decode to array
        $data = json_decode($jsonString, true);

        // this variabile contains the number of digits definition
        $currencyString = $data['main'][$iso]['numbers']['currencyFormats-numberSystem-latn']['standard'];

        // searching the digits value into the string
        // digits definition can be just '0' so I cannot find it searching for 0.00 or 0. I have to divide the string
        // in 3 parts and then substitute the old value with the new one
        $s = strpos($currencyString, '#0') + 1;
        // starting from number of digits, to the end of the string
        $str = substr($currencyString, $s, strlen($currencyString) - $s);
        // save the first part of the string
        $first = substr($currencyString, 0, $s);
        $oldValue = '';
        // the last part of the string, that could be empty
        $last = '';
        $len = strlen($str);
        for ($i = 0; $i < $len; ++$i) {
            if ($str[$i] != '0' && $str[$i] != '.') {
                $last = substr($str, $i, strlen($str) - $i);
                break;
            } else {
                $oldValue .= $str[$i];
            }
        }

        // get hashes format to add / remove one or more
        $len = strlen($first);
        // $f1 = '';
        // $f2 = '';
        $useComma = false;
        $oldHashes = '';
        for ($i = 0; $i < $len; ++$i) {
            if ($first[$i] == '#' || $first[$i] == '.' || $first[$i] == ',') {
                $oldHashes .= $first[$i];
                if ($first[$i] == ',') {
                    $useComma = true;
                }
            }
        }
        if ($useComma) {
            $newHashes = str_replace('.', ',', $newHashes);
        }

        // if values are equal, no action is required
        if ($oldValue === $newValue) {
            return false;
        }

        // compose the new value with updated number of digits
        // ... use static method
        $currencyString = str_replace($oldValue, $newValue, $currencyString);
        $currencyString = str_replace($oldHashes, $newHashes, $currencyString);
        $data['main'][$iso]['numbers']['currencyFormats-numberSystem-latn']['standard'] = $currencyString;
        $newJsonString = json_encode($data);

        // backup of the original file
        $backup_filename = $filename . MxbToolsUtils::getCurrentDateTime(false, '-', '', ''); // '.BACKUP';
        if (!file_exists(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $backup_filename)) {
            copy(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename, _PS_TRANSLATIONS_DIR_ . 'cldr/' . $backup_filename);
        }

        // save the file
        file_put_contents(_PS_TRANSLATIONS_DIR_ . 'cldr/' . $filename, $newJsonString);

        // return the name of the original file
        return $backup_filename;
    }

    protected function updateMxbToolsProcedures()
    {
        $confirmations = [];
        $errors = [];
        $res = null;
        /*
            SAMPLES:
            $res = $this->module->registerHook('actionMailAlterMessageBeforeSend');
            if ((bool) $res) {
                $confirmations[] = 'SUCCESS REGISTERING HOOKS actionMailAlterMessageBeforeSend';
            }
            else {
                $errors[] = '*** ERROR *** REGISTERING HOOKS actionMailAlterMessageBeforeSend';
            }
        */
        Configuration::updateGlobalValue('MXB_TOOLS_VERSION', 1);
        if (count($errors)) {
            foreach ($errors as $err) {
                $this->errors[] = $err;
            }
        }
        if (count($confirmations)) {
            foreach ($confirmations as $cnf) {
                $this->confirmations[] = $cnf;
            }
        } else {
            $this->confirmations[] = $this->l('Nothing done ...');
        }
    }
}
