<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
// use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCodes.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsMenu.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsExport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsProduct.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCustomer.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsPrice.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCartRule.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsExportController extends ModuleAdminController
{
    /*
        protected $colorOnBackground = true;
        protected $color_on_background = true;
    */
    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->class_name = 'AdminMxbToolsExportController';
        $this->tab = 'front_office_features';
        $this->templateFile = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/export.tpl';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('MXB Tools: export utilities');

        // MANAGE REQUESTED OPERATION
        if (Tools::getValue('action') && Tools::getValue('action') == 'exportProductCSV') {
            $this->exportProductCSV();
        }
        if (Tools::getValue('action') && Tools::getValue('action') == 'exportOrderedProductsCSV') {
            // MxbToolsDebug::writeLog($this->name, $this->class_name . '::__construct()', '_POST=' . MxbToolsDebug::getAllPost());
            $this->exportOrderedProductsCSV(); // Tools::getValue('action_details'));
        }
        if (Tools::getValue('action') && Tools::getValue('action') == 'exportCustomersCSV') {
            $this->exportCustomersCSV();
        }
        if (Tools::getValue('action') && Tools::getValue('action') == 'exportVouchersCSV') {
            $this->exportVouchersCSV();
        }
    }

    public function renderList()
    {
        $output = '';
        if (Tools::getValue('submitAddmxb_tools_export')) {
            $output .= $this->processForm();
        }
        $smarty = Context::getContext()->smarty;

        // add config values to smarty
        $smarty->assign($this->getConfigFormValues());
        $smarty->assign(MxbToolsUtils::getCommonAdminTplValues($this->context));
        $smarty->assign('form_action', $this->context->link->getAdminLink('AdminMxbToolsExport'));

        // add header menu
        $smarty->assign('mxb_tools_menu', MxbToolsMenu::getMxbToolsMenu($this->context));

        // load lists for select elements
        $idLang = (int) $this->context->language->id;
        $smarty->assign('order_state_list', MxbToolsProduct::getOrderStateList($idLang));

        $output .= $this->context->smarty->fetch($this->templateFile);

        return $output; // . $this->renderForm();
    }

    protected function getValue($key, $default = null, $type = null, $save = false)
    {
        return MxbToolsUtils::getConfigValue($key, $default, $type, $save);
    }

    protected function setValue($key, $value, $type = null)
    {
        MxbToolsUtils::setConfigValue($key, $value, $type);
    }

    /**
     * Set values for the inputs.
     */
    public function getConfigFormValues()
    {
        return MxbToolsUtils::getConfigFormValues();
    }

    public function processForm()
    {
        $smarty = Context::getContext()->smarty;
        $form_error = [];
        $form_success = [];
        $form_values = $this->getConfigFormValues();
        // MxbToolsDebug::writeLog($this->name, $this->class_name . '::processForm()', 'form_values=' . print_r($form_values, true), false);

        // validation: not needed
        if (count($form_error)) {
            return $this->module->displayError($this->l('Unable to save configuration for the following reasons:') . '<br>.' . implode('<br>. ', $form_error));
        }
        foreach (array_keys($form_values) as $key) {
            Configuration::updateGlobalValue($key, $form_values[$key]);
        }
        if (count($form_success)) {
            return $this->module->displayConfirmation($this->l('Configuration saved:') . '<br>.' . implode('<br>. ', $form_success));
        }

        return $this->module->displayConfirmation($this->l('Configuration saved'));
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);
        return $return;
    }

    /**
     * EXPORT PRODUCT LIST
     */
    protected function exportProductCSV()
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        $force_debug = true;
        $id_lang = (int) $this->context->language->id;
        // $priceFormatter = new PriceFormatter();

        // save values
        $this->processForm();
        $data = [
            'export_last_record' => $this->getValue('export_last_record', 1, 'int', false),
            'id_customer' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_ID_CUSTOMER', 1, 'int', true),
            'quantity_columns_min' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_QUANTITY_COLUMNS_MIN', 4, 'int', true),
            'products_per_loop' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_PRODUCTS_LOOP', 500, 'int', true),
            'export_zero_price' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_ZERO_PRICE', 1, 'bool', true),
            'export_category_id' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_CATEGORY_ID', 1, 'int', true),
            'export_flag_show_category' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_CATEGORY', 1, 'bool', true),
            'export_flag_stock_quantity' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_STOCK_QUANTITY', 1, 'bool', true),
            'export_flag_product_price' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_PRODUCT_PRICE', 0, 'bool', true),
            'export_flag_wholesale_price' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_WHOLESALE_PRICE', 0, 'bool', true),
            'export_flag_gross_price' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_GROSS_PRICE', 0, 'bool', true),
            'export_flag_description' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_DESCRIPTION', 0, 'bool', true),
            'export_flag_description_short' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_DESCRIPTION_SHORT', 0, 'bool', true),
            'export_flag_tags' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_TAGS', 0, 'bool', true),
            'export_flag_price_charge' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_PRICE_CHARGE', 0, 'bool', true),
            'export_flag_discount' => $this->getValue('MXB_TOOLS_PRODUCT_LIST_FLAG_DISCOUNT', 0, 'bool', true),
            'export_material_text' => Configuration::getGlobalValue('MXB_TOOLS_MATERIAL_PRODUCT_TAG', $id_lang),
            'export_segment_text' => Configuration::getGlobalValue('MXB_TOOLS_SEGMENT_PRODUCT_TAG', $id_lang),
        ];

        $ret = MxbToolsExport::exportProducts($data);

        if (isset($ret['export_last_record'])) {
            $smarty = Context::getContext()->smarty;
            $smarty->assign('export_last_record', $ret['export_last_record']);
            $smarty->assign('export_tot_records', $ret['export_tot_records']);
            // $this->confirmations = $ret['success'];
            // $this->errors = $ret['errors'];
            // return;
        }
        if (count($ret['success'])) {
            $this->confirmations = $ret['success'];
        }
        if (count($ret['errors'])) {
            $this->errors = $ret['errors'];
        }
    }

    /**
     * EXPORT ORDERED PRODUCT LIST
     */
    /*
        ORDER ARRAY:
        [id_order] => 127
        [reference] => TAQJJNMQP
        [id_shop_group] => 1
        [id_shop] => 1
        [id_carrier] => 2
        [id_lang] => 1
        [id_customer] => 5
        [id_cart] => 209
        [id_currency] => 1
        [id_address_delivery] => 8
        [id_address_invoice] => 8
        [current_state] => 10
        [secure_key] => 85d4bc16935413f4af493e0e51516b0d
        [payment] => Bonifico bancario
        [conversion_rate] => 1.000000
        [module] => ps_wirepayment
        [recyclable] => 0
        [gift] => 0
        [gift_message] =>
        [mobile_theme] => 0
        [shipping_number] =>
        [total_discounts] => 0.000000
        [total_discounts_tax_incl] => 0.000000
        [total_discounts_tax_excl] => 0.000000
        [total_paid] => 832.410000
        [total_paid_tax_incl] => 832.410000
        [total_paid_tax_excl] => 682.300000
        [total_paid_real] => 0.000000
        [total_products] => 679.800000
        [total_products_wt] => 829.360000
        [total_shipping] => 3.050000
        [total_shipping_tax_incl] => 3.050000
        [total_shipping_tax_excl] => 2.500000
        [carrier_tax_rate] => 22.000
        [total_wrapping] => 0.000000
        [total_wrapping_tax_incl] => 0.000000
        [total_wrapping_tax_excl] => 0.000000
        [round_mode] => 2
        [round_type] => 2
        [invoice_number] => 0
        [delivery_number] => 0
        [invoice_date] => 0000-00-00 00:00:00
        [delivery_date] => 0000-00-00 00:00:00
        [valid] => 0
        [date_add] => 2020-02-26 16:41:22
        [date_upd] => 2020-02-26 16:41:23

        ORDER_DETAIL ARRAY:
        [id_order_detail] => 235
              [id_order] => 114
              [id_order_invoice] => 0
              [id_warehouse] => 0
              [id_shop] => 1
              [product_id] => 12809
              [product_attribute_id] => 30712
              [id_customization] => 0
              [product_name] => Borraccia in alluminio Oregon con moschettone - Color : verde mela
              [product_quantity] => 2
              [product_quantity_in_stock] => 2
              [product_quantity_refunded] => 0
              [product_quantity_return] => 0
              [product_quantity_reinjected] => 0
              [product_price] => 1.318500
              [reduction_percent] => 0.00
              [reduction_amount] => 0.000000
              [reduction_amount_tax_incl] => 0.000000
              [reduction_amount_tax_excl] => 0.000000
              [group_reduction] => 55.00
              [product_quantity_discount] => 0.000000
              [product_ean13] => 8713159038610
              [product_isbn] =>
              [product_upc] =>
              [product_reference] => 10000200
              [product_supplier_reference] =>
              [product_weight] => 86.000000
              [id_tax_rules_group] => 1
              [tax_computation_method] => 0
              [tax_name] =>
              [tax_rate] => 0.000
              [ecotax] => 0.000000
              [ecotax_tax_rate] => 0.000
              [discount_quantity_applied] => 0
              [download_hash] =>
              [download_nb] => 0
              [download_deadline] => 0000-00-00 00:00:00
              [total_price_tax_incl] => 3.220000
              [total_price_tax_excl] => 2.640000
              [unit_price_tax_incl] => 1.608570
              [unit_price_tax_excl] => 1.318500
              [total_shipping_price_tax_incl] => 0.000000
              [total_shipping_price_tax_excl] => 0.000000
              [purchase_supplier_price] => 1.050000
              [original_product_price] => 1.318500
              [original_wholesale_price] => 1.050000

        *
        * Danea XML documentation: https://www.danea.it/software/easyfatt/xml/documenti/
        * TAG <DocumentType>:
        *  codice corrispondente al tipo di documento; se questo tag viene omesso, il documento sar� considerato un ordine cliente.
        *   Elenco dei tipi ammessi:
        *     A: Avviso di parcella
        *     B: Vendita al banco
        *     C: Ordine cliente (tipo predefinito)
        *     D: Documento di trasporto
        *     E: Ordine fornitore
        *     F: Fattura accompagnatoria
        *     G: Rapporto d'intervento
        *     I: Fattura
        *     J: Fattura d'acconto
        *     L: Fattura pro-forma
        *     M: Autofattura
        *     N: Nota d'accredito
        *     O: Nota d'addebito
        *     P: Parcella
        *     Q: Preventivo
        *     R: Ricevuta fiscale
        *     S: Preventivo fornitore
        *     H: Arrivi merce fornitore

        https://www.imprenditoriamo.it/site/xml-caratteri-non-ammessi/
    */
    protected function exportOrderedProductsCSV()
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        $force_debug = false;

        // $gcodes = GadgetcardCodes::getCodes();
        $id_lang = (int) $this->context->language->id;
        // $priceFormatter = new PriceFormatter();

        // save values
        $this->processForm();

        // get export_last_record: if set, we are into the export loop, else this is the first loop
        // start_record : not used because there is not ajax multicalls functions: report will be created all at once
        //  protected function getValue($key, $default = null, $type = null, $save = false)
        // $start_record = $this->getValue('export_last_record', 1, 'int', false);
        // $products_per_loop = $this->getValue('MXB_TOOLS_PRODUCT_LIST_PRODUCTS_LOOP', 500, 'int', true);

        $data = [];
        $data['size_tag'] = Configuration::getGlobalValue('MXB_TOOLS_SIZE_PRODUCT_TAG', $id_lang);
        $data['color_tag'] = Configuration::getGlobalValue('MXB_TOOLS_COLOR_PRODUCT_TAG', $id_lang);
        $data['date_from'] = $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_DATE_FROM', MxbToolsUtils::getDateString('2020-01-01'), 'string', true);
        $data['date_to'] = $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_DATE_TO', MxbToolsUtils::getDateString('2030-01-01'), 'string', true);
        $data['id_from'] = (int) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_FROM', 0, 'int', true);
        $data['id_to'] = (int) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_TO', 0, 'int', true);
        $data['output_format'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_FORMAT', 'xml', 'string', true);  // "csv", "xml"
        $data['output_type'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_TYPE', 'orders', 'string', true);    // "orders", "ordered_products"
        $data['id_order_state'] = (int) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE', 0, 'int', true);
        $data['id_order_state_next'] = (int) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_NEXT', 0, 'int', true);
        $data['id_order_state_list'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST', '', 'string', true);
        $data['id_order_state_list_not'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST_NOT', '', 'string', true);
        $data['order_prefix_from'] = $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_PREFIX_FROM', '', 'string', true);
        $data['order_prefix_to'] = $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_PREFIX_TO', '', 'string', true);
        $data['force_order_numeric'] = (bool) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_FORCE_NUMERIC', false, 'bool', true);
        $data['export_invoice'] = (bool) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_INVOICE', false, 'bool', true);
        $data['export_delivery_slip'] = (bool) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_DELIVERY_SLIP', false, 'bool', true);
        $data['export_product_discount'] = (bool) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_PRODUCT_DISCOUNT', false, 'bool', true);
        $data['numbering'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_NUMBERING', '', 'string', true);
        $data['customer_prefix'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_CUSTOMER_PREFIX', '', 'string', true);
        // MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportOrderedProductsCSV()', 'date_from=' . $data['date_from'], $force_debug);

        // force international/database date format
        $data['date_from'] = MxbToolsUtils::getDateString($data['date_from']);
        $data['date_to'] = MxbToolsUtils::getDateString($data['date_to']);
        // MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportOrderedProductsCSV()', 'DATA=' . print_r($data, true), $force_debug);

        $ret = MxbToolsExport::exportOrders($data);

        // returned values was corrected if necessary, so we'll save it into configuration
        // if (isset($ret['confirmations']))
        //  $this->confirmations[] = $confirmation;
        if (isset($ret['errors'])) {
            $this->errors[] = $this->l('No orders available for the date range');
            return;
        }

        if (isset($ret['data'])) {
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_FROM', $ret['data']['id_from'], 'int');
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_TO', $ret['data']['id_to'], 'int');
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE', $ret['data']['id_order_state'], 'int');
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_NEXT', $ret['data']['id_order_state_next'], 'int');
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST', $ret['data']['id_order_state_list'], 'string');
            $this->setValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST_NOT', $ret['data']['id_order_state_list_not'], 'string');
        }

        if (isset($ret['download_link'])) {
            if ($data['output_type'] == 'orders') {
                $this->confirmations[] = 'CSV ' . $this->l('file is ready for download! Click here') . ': '
                    . '<a href="' . $ret['download_url'] . '" target="_new">' . $ret['download_url'] . '</a>';
                if (isset($ret['other_files'])) {
                    foreach ($ret['other_files'] as $info) {
                        $this->confirmations[] = '<br>CSV ' . $this->l('file is ready for download! Click here') . ': '
                        . '<a href="' . $info['download_url'] . '" target="_new">' . $info['download_url'] . '</a>';
                    }
                }
            } elseif ($data['output_type'] == 'ordered_products') {
                $this->confirmations[] = 'DANEA XML CSV ' . $this->l('file is ready for download! Click here') . ': '
                    . '<a href="' . $ret['download_url'] . '" target="_new">' . $ret['download_url'] . '</a>';
            }
        }
    }

    protected function exportCustomersCSV()
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        $force_debug = false;

        // $gcodes = GadgetcardCodes::getCodes();
        // $id_lang = (int) $this->context->language->id;
        // $priceFormatter = new PriceFormatter();

        // save values
        $this->processForm();

        $data = [];
        // $data['output_format'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_FORMAT', 'xml', 'string', true);  // "csv", "xml"
        // $data['output_type'] = (string) $this->getValue('MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_TYPE', 'orders', 'string', true);    // "orders", "ordered_products"
        $data['id_order_state_list'] = (string) $this->getValue('MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST', '', 'string', true);
        $data['id_order_state_list_not'] = (string) $this->getValue('MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST_NOT', '', 'string', true);
        $data['date_from'] = $this->getValue('MXB_TOOLS_CUSTOMER_LIST_DATE_FROM', MxbToolsUtils::getDateString('2020-01-01'), 'string', true);
        $data['date_to'] = $this->getValue('MXB_TOOLS_CUSTOMER_LIST_DATE_TO', MxbToolsUtils::getDateString('2030-01-01'), 'string', true);

        // force international/database date format
        $data['date_from'] = MxbToolsUtils::getDateString($data['date_from']);
        $data['date_to'] = MxbToolsUtils::getDateString($data['date_to']);

        MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportCustomersCSV()', 'START', $force_debug);

        $ret = MxbToolsExport::exportCustomers($data);
        MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportCustomersCSV()', 'exported, ret=' . print_r($ret, true), $force_debug);

        // returned values was corrected if necessary, so we'll save it into configuration
        // if (isset($ret['confirmations']))
        //  $this->confirmations[] = $confirmation;
        if (isset($ret['errors'])) {
            $this->errors[] = $this->l('No customers available for the date range');

            return;
        }
        if (isset($ret['data'])) {
            $this->setValue('MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST_NOT', $ret['data']['id_order_state_list_not'], 'string');
            $this->setValue('MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST', $ret['data']['id_order_state_list'], 'string');
        }
        if (isset($ret['download_link'])) {
            $this->confirmations[] = 'CSV ' . $this->l('file is ready for download! Click here') . ': '
                . '<a href="' . $ret['download_url'] . '" target="_new">' . $ret['download_url'] . '</a>';
        }
    }

    protected function exportVouchersCSV()
    {
        // force_debug (bypass_settings) must set to false for production (or delete all MxbToolsDebug calls
        $force_debug = false;

        // save values
        $this->processForm();

        $data = [];
        MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportVouchersCSV()', 'START', $force_debug);
        $ret = MxbToolsExport::exportVouchers($data);
        MxbToolsDebug::writeLog($this->name, $this->class_name . '::exportVouchersCSV()', 'exported, ret=' . print_r($ret, true), $force_debug);

        // returned values was corrected if necessary, so we'll save it into configuration
        // if (isset($ret['confirmations']))
        //  $this->confirmations[] = $confirmation;
        if (isset($ret['errors'])) {
            $this->errors[] = $this->l('No vouchers available for the date range');
            return;
        }
        if (isset($ret['download_link'])) {
            $this->confirmations[] = 'CSV ' . $this->l('file is ready for download! Click here') . ': '
              . '<a href="' . $ret['download_url'] . '" target="_new">' . $ret['download_url'] . '</a>';
        }
    }
}
