<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsMenu.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsProduct.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCustomerImport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCartRuleImport.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsTextureImport.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsImportController extends ModuleAdminController
{
    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->class_name = 'AdminMxbToolsImportController';
        $this->tab = 'front_office_features';
        $this->templateFile = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/import.tpl';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->bootstrap = true;
        $this->file_uploaded = null;
        $this->colors_to_hex_list = null;

        parent::__construct();

        $this->displayName = $this->l('MXB Tools: import utilities');

        // MANAGE REQUESTED OPERATION
        if (Tools::getValue('action') && Tools::getValue('action') == 'importColorTextures') {
            $this->importColorTextures();
        }

        if (Tools::getValue('action') && Tools::getValue('action') == 'importCsvColorTextures') {
            $this->importColorTextures(MxbToolsUtils::path2url(MxbToolsTextureImport::$definitions['texture_folder']));
        }

        if (Tools::getValue('mxb_tools-upload-csv-customer-name')
            && Tools::getValue('mxb_tools-upload-csv-customer-name') !== '') {
            if ($this->uploadCustomerCsvFile()) {
                $this->file_uploaded = 'customer';
            }
        }

        if (Tools::getValue('mxb_tools-upload-csv-cart_rule-name')
            && Tools::getValue('mxb_tools-upload-csv-cart_rule-name') !== '') {
            if ($this->uploadCartRuleCsvFile()) {
                $this->file_uploaded = 'cart_rule';
            }
        }

        if (Tools::getValue('mxb_tools-upload-csv-texture-name')
            && Tools::getValue('mxb_tools-upload-csv-texture-name') !== '') {
            if ($this->uploadTextureCsvFile()) {
                $this->file_uploaded = 'texture';
            }
        }

        if (Tools::getIsset('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO')) {
            MxbToolsUtils::setConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO', Tools::getValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO'));
        }
        if (Tools::getIsset('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO')) {
            MxbToolsUtils::setConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO', Tools::getValue('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO'));
        }
        if (Tools::getIsset('MXB_TOOLS_IMPORT_COLORS_ID_ATTRIBUTE_GROUP')) {
            MxbToolsUtils::setConfigValue('MXB_TOOLS_IMPORT_COLORS_ID_ATTRIBUTE_GROUP', Tools::getValue('MXB_TOOLS_IMPORT_COLORS_ID_ATTRIBUTE_GROUP'));
        }
    }

    public function renderList()
    {
        $output = '';
        if (Tools::getValue('submitAddmxb_tools_import')) {
            $output .= $this->processForm();
        }
        $smarty = Context::getContext()->smarty;

        // add config values to smarty
        $smarty->assign($this->getConfigFormValues());
        $smarty->assign(MxbToolsUtils::getCommonAdminTplValues($this->context));
        $smarty->assign('form_action', $this->context->link->getAdminLink('AdminMxbToolsTools'));

        // add header menu
        $smarty->assign('mxb_tools_menu', MxbToolsMenu::getMxbToolsMenu($this->context));

        // add used links
        $smarty->assign('sample_customer_csv_link', MxbToolsUtils::getModulePathUri() . 'csv/'
            . MxbToolsCustomerImport::$definitions['sample_filename']);
        $smarty->assign('sample_cart_rule_csv_link', MxbToolsUtils::getModulePathUri() . 'csv/'
            . MxbToolsCartRuleImport::$definitions['sample_filename']);
        $smarty->assign('sample_texture_csv_link', MxbToolsUtils::getModulePathUri() . 'csv/'
            . MxbToolsTextureImport::$definitions['sample_filename']);
        $smarty->assign('uploaded_texture_csv_link', MxbToolsUtils::getModulePathUri() . 'csv/'
            . MxbToolsTextureImport::$definitions['target_filename']);
        $smarty->assign(
            'texture_remote_folder',
            MxbToolsUtils::path2url(MxbToolsTextureImport::$definitions['texture_folder'])
        );
        $smarty->assign('color_list_csv_link', MxbToolsUtils::path2url(MxbToolsTextureImport::$definitions['color_list_folder'])
            . MxbToolsTextureImport::$definitions['color_list_filename']);

        // load color list for name to hex conversion
        $this->colors_to_hex_list = MxbToolsUtils::csvToArray(
            MxbToolsTextureImport::$definitions['color_list_folder'] . MxbToolsTextureImport::$definitions['color_list_filename'],
            MxbToolsTextureImport::$definitions['csv_delimeter']
        );

        // check customer import CSV file and set layout if exists
        $csv_table = $this->getCsvImportTable(
            MxbToolsCustomerImport::$definitions['target_folder']
                . MxbToolsCustomerImport::$definitions['target_filename'],
            MxbToolsCustomerImport::$definitions['csv_delimeter']
        );
        if ($csv_table) {
            $smarty->assign('customer_csv_content', $csv_table);
        }

        // check cart_rule import CSV file and set layout if exists
        $csv_table = $this->getCsvImportTable(
            MxbToolsCartRuleImport::$definitions['target_folder']
                . MxbToolsCartRuleImport::$definitions['target_filename'],
            MxbToolsCartRuleImport::$definitions['csv_delimeter']
        );
        if ($csv_table) {
            $smarty->assign('cart_rule_csv_content', $csv_table);
        }

        // check texture import CSV file and set layout if exists
        $csv_table = $this->getCsvImportTable(
            MxbToolsTextureImport::$definitions['target_folder']
                . MxbToolsTextureImport::$definitions['target_filename'],
            MxbToolsTextureImport::$definitions['csv_delimeter'],
            'filename', // filename column
            'hex_colors' // hex_colors column
        );
        if ($csv_table) {
            $smarty->assign('texture_csv_content', $csv_table);
        }
        if ($this->file_uploaded !== null) {
            $smarty->assign($this->file_uploaded . '_csv_uploaded', 1);
        }

        // populate language list for hex codes and generation and textures import
        $langArr = [];
        $languages = Language::getLanguages(false);
        foreach ($languages as &$lng) {
            $lng['full_name'] = $lng['iso_code'] . ' - ' . $lng['name'];
        }
        $compatibleLang = [];
        foreach ($this->colors_to_hex_list[0] as $lang_code => $lang_val) {
            if ($lang_code !== 'hex') {
                $compatibleLang[] = $lang_code;
                foreach ($languages as $lang) {
                    /*
                    MxbToolsDebug::writeLog(
                        $this->name,
                        $this->class_name . '::renderList',
                        'lang_code=' . $lang_code . ', iso_code=' . $lang['iso_code'],
                        true
                    );
                    */
                    if ($lang['iso_code'] == $lang_code) {
                        // $langArr[] = array('id' => $lang['id_lang'], 'iso_code' => $lang['iso_code'], 'name' => $lang['name'], 'full_name' => $lang['full_name']);
                        $langArr[] = $lang;
                    }
                }
            }
        }
        $smarty->assign('color_find_texture_lang_array', $languages);
        if (count($langArr)) {
            $smarty->assign('color_find_hex_lang_array', $langArr);
        }
        $smarty->assign('color_find_hex_lang_compatible_list', implode(', ', $compatibleLang));

        /*
        MxbToolsDebug::writeLog(
            $this->name,
            $this->class_name . '::renderList',
            'colors_to_hex_list=' . print_r($this->colors_to_hex_list, true),
            true
        );
        */

        // load color textures
        $textures = MxbToolsTextureImport::getColorAttributeList($this->colors_to_hex_list);
        if (count($textures)) {
            $smarty->assign('color_texture_array', $textures);
        }

        // load attribute group of "color" type
        $idLang = (int) $this->context->language->id;
        // $attributeGroup = AttributeGroup::getAttributesGroups($idLang);
        /*
        $attributeGroup = Db::getInstance()->executeS('
            SELECT DISTINCT agl.`name` as name, ag.`id_attribute_group` as id
            FROM `' . _DB_PREFIX_ . 'attribute_group` ag
            ' . Shop::addSqlAssociation('attribute_group', 'ag') . '
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group_lang` agl
                ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND `id_lang` = ' . $idLang . ')
            WHERE ag.is_color_group = 1
            ORDER BY `name` ASC
        ');
        */
        $attributeGroup = MxbToolsProduct::getAttributeGroupList($idLang);
        /*
        MxbToolsDebug::writeLog(
            $this->name,
            $this->class_name . '::renderList',
            'attributeGroup=' . print_r($attributeGroup, true),
            true
        );
        */
        if ($attributeGroup && count($attributeGroup)) {
            $smarty->assign('attribute_group_list', $attributeGroup);
        }

        $output .= $this->context->smarty->fetch($this->templateFile);
        return $output; // .$this->renderForm();
    }

    protected function getCsvImportTable($target_filename, $csv_delimeter, $texture_col = false, $hex_col = false)
    {
        $csv = MxbToolsUtils::csvToArray($target_filename, $csv_delimeter);
        if ($csv) {
            $csv_heading = '';
            $csv_rows = '';
            foreach ($csv as $rowkey => $row) {
                if ((int) $rowkey === 0) {
                    $csv_heading .= '<tr>';
                }
                $csv_rows .= '<tr>';
                $img_name = null;
                foreach ($row as $key => $value) {
                    if ((int) $rowkey === 0) {
                        $csv_heading .= '<th>' . $key . '</th>';
                    }
                    if ($texture_col == $key) {
                        $img_name = MxbToolsUtils::path2url(MxbToolsTextureImport::$definitions['texture_folder']) . $value;
                    }
                    if ($hex_col == $key) {
                        if (empty($value)) {
                            // try to calculate hex
                            $value = MxbToolsTextureImport::convertColorNameToHex($row['name'], $this->colors_to_hex_list);
                        }
                    }
                    $csv_rows .= '<td>' . $value . '</td>';
                }
                if ($img_name !== null) {
                    if ((int) $rowkey === 0) {
                        $csv_heading .= '<th>texture</th>';
                    }
                    $csv_rows .= '<td><img src="' . $img_name . '" style="width:40px;height:40px;border:1px solid grey;" /></td>';
                }
                if ((int) $rowkey === 0) {
                    $csv_heading .= '</tr>';
                }
                $csv_rows .= '</tr>';
            }
            return '<table class="csv-table">' . $csv_heading . $csv_rows . '</table>';
        }
        return null;
    }

    protected function getValue($key, $default = null, $type = null, $save = false)
    {
        return MxbToolsUtils::getConfigValue($key, $default, $type, $save);
    }

    protected function setValue($key, $value, $type = null)
    {
        MxbToolsUtils::setConfigValue($key, $value, $type);
    }

    public function getConfigFormValues()
    {
        return MxbToolsUtils::getConfigFormValues();
    }

    public function processForm()
    {
        $smarty = Context::getContext()->smarty;
        $form_error = [];
        $form_success = [];
        $form_values = $this->getConfigFormValues();

        // validation: not needed
        if (count($form_error)) {
            return $this->module->displayError(
                $this->l('Unable to save configuration for the following reasons:')
                . '<br>.' . implode('<br>. ', $form_error)
            );
        }

        foreach (array_keys($form_values) as $key) {
            Configuration::updateGlobalValue($key, $form_values[$key]);
        }

        if (count($form_success)) {
            return $this->module->displayConfirmation(
                $this->l('Configuration saved:')
                . '<br>.' . implode('<br>. ', $form_success)
            );
        }

        return $this->module->displayConfirmation($this->l('Configuration saved'));
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);

        return $return;
    }

    protected function importColorTextures($remote_addr = null)
    {
        $res = MxbToolsTextureImport::importFromUrl($remote_addr);

        MxbToolsDebug::writeLog(
            $this->name,
            $this->class_name . '::importColorTextures',
            'importFromUrl response=' . print_r($res, true),
            true
        );

        if (isset($res['errors'])) {
            $this->errors = $res['errors'];
        }
        if (isset($res['success'])) {
            $this->confirmations = $res['success'];
        }
    }

    protected function uploadCustomerCsvFile()
    {
        if (!Tools::getValue('mxb_tools-upload-csv-customer-name')) {
            $this->errors[] = $this->trans('No file provided.', [], 'Admin.Notifications.Error');
            return false;
        }
        if (MxbToolsCustomerImport::uploadCsvFile()) {
            $this->confirmations[] = $this->l('File CSV successfully loaded') . ': '
                . $this->l('check the data to import in the table below and start to import customers.');
        } else {
            $this->errors[] = $this->trans(
                'An error occurred while attempting to upload the file.',
                [],
                'Admin.Notifications.Error'
            );

            return false;
        }

        return true;
    }

    protected function uploadCartRuleCsvFile()
    {
        if (!Tools::getValue('mxb_tools-upload-csv-cart_rule-name')) {
            $this->errors[] = $this->trans('No file provided.', [], 'Admin.Notifications.Error');
            return false;
        }
        if (MxbToolsCartRuleImport::uploadCsvFile()) {
            $this->confirmations[] = $this->l('File CSV successfully loaded') . ': '
                . $this->l('check the data to import in the table below and start to import cart_rule.');
        } else {
            $this->errors[] = $this->trans(
                'An error occurred while attempting to upload the file.',
                [],
                'Admin.Notifications.Error'
            );

            return false;
        }

        return true;
    }

    protected function uploadTextureCsvFile()
    {
        if (!Tools::getValue('mxb_tools-upload-csv-texture-name')) {
            $this->errors[] = $this->trans('No file provided.', [], 'Admin.Notifications.Error');
            return false;
        }

        // load color list for name to hex conversion
        if ($this->colors_to_hex_list === null) {
            $this->colors_to_hex_list = MxbToolsUtils::csvToArray(
                MxbToolsTextureImport::$definitions['color_list_folder'] . MxbToolsTextureImport::$definitions['color_list_filename'],
                MxbToolsTextureImport::$definitions['csv_delimeter']
            );
        }
        $res = MxbToolsTextureImport::uploadCsvFile($this->colors_to_hex_list);
        if ($res === true) {
            $this->confirmations[] = $this->l('File CSV successfully loaded') . ': '
                . $this->l('check the data to import in the table below and start to import textures.');
        } else {
            $this->errors[] =
                $this->trans(
                    'An error occurred while attempting to upload the file',
                    [],
                    'Admin.Notifications.Error'
                )
                . ($res === false ? '.' : ': ' . $res);

            return false;
        }

        return true;
    }
}
