<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
/*
* MANAGE COMBINATION ATTRIBUTES
*/
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsManageAttributes.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_CLASS_DIR_ . 'Product.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsManageAttributesController extends ModuleAdminController
{
    public function __construct()
    {
        $this->name = 'AdminMxbToolsManageAttributes';
        $this->table = 'attribute';
        $this->className = 'MxbToolsManageAttributes';
        $this->lang = false;
        // $this->list_no_link = true;
        $this->module = 'mxb_tools';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->_orderBy = 'id_attribute';
        $this->_orderWay = 'ASC';
        $this->identifier = 'id_attribute';
        $this->list_no_link = true;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Attributes usage');

        $this->errors[] = $this->l('This module shows existing product attributes (color, size...) with the number
            of combinations that uses it. You can delete attributes with usage = 0 (not used anymore by
            combinations) or force-delete attributes used by combinations. In this case the relationship with
            combinatin and attribute will be deleted too.');

        if (Tools::getIsset('deleted')) {
            $deleted_records = Tools::getValue('deleted');
            $this->confirmations[] = $this->l('Attributes deleted') . ': ' . $deleted_records;
            // $output .= $this->module->displayConfirmation($this->l('Attributes deleted') . ': ' . $deleted_records );
        }

        $this->default_form_language = $this->context->language->id;
        $this->_select = '
            ag.`id_attribute_group`,
            agl.`name` AS group_name,
            al.`name` AS attribute_name,
            (SELECT COUNT(`id_attribute`)
            FROM `' . _DB_PREFIX_ . 'product_attribute_combination`
            WHERE a.`id_attribute` = `id_attribute`) AS counter
        ';

        $this->_join .= '
            LEFT JOIN `' . _DB_PREFIX_
            . 'attribute_lang` al ON (a.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '
            . $this->context->language->id . ')
            LEFT JOIN `' . _DB_PREFIX_
            . 'attribute_group` ag ON ag.`id_attribute_group` = a.`id_attribute_group`
            LEFT JOIN `' . _DB_PREFIX_
            . 'attribute_group_lang` agl ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = '
            . $this->context->language->id . ')';
        // $this->addRowAction('edit');
        $this->addRowAction('delete');

        $this->addRowAction('forcedelete');
        /*
        $this->addRowAction('viewproduct');
        $this->addRowAction('editproduct');
        $this->addRowAction('editattribute');
        */

        $this->bulk_actions = [
            'delete' => [
                'text' => $this->trans('Delete selected', [], 'Admin.Actions'),
                'confirm' => $this->l('Delete attributes from combinations ? Note: you cannot rollback !!'),
                'icon' => 'icon-trash',
            ],
        ];

        $this->_use_found_rows = false;
        $this->fields_list = [
            'id_attribute' => [
                'title' => $this->l('ID attribute'),
                'filter_key' => 'id_attribute',
                'align' => 'left',
                'width' => 60,
            ],
            'attribute_name' => [
                'title' => $this->l('attribute name'),
                'filter_key' => 'al!name',
                'align' => 'left',
                'width' => 120,
            ],
            'id_attribute_group' => [
                'title' => $this->l('ID attribute group'),
                'filter_key' => 'ag!id_attribute_group',
                'align' => 'left',
                'width' => 60,
            ],
            'group_name' => [
                'title' => $this->l('Attribute group name'),
                'filter_key' => 'agl!name',
                'align' => 'left',
                'width' => 120,
            ],
            'counter' => [
                'title' => $this->l('Usage'),
                'filter_key' => 'counter',
                'align' => 'left',
                'width' => 120,
            ],
        ];
        $this->renderForm();
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);
        return $return;
    }

    /**
     * @param: $id = 'caption' field of query -> 'id_product'-'id_product_attribute'-'id_attribute'
     */
    public function displayDeleteLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('delete', self::$cache_lang)) {
            self::$cache_lang['delete'] = $this->l('Delete attribute');
            self::$cache_lang['delete_confirm'] = $this->l('Delete attribute ? Note: you cannot rollback !!');
        }

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $this->context->link->getAdminLink($this->name) // self::$currentIndex
                // . '&'.$this->identifier.'=' . $id
                // simulated: give a new name to discern this id_product from the id_product that identifies the print option
                . '&caption=' . $id
                // . '&id_product_attribute=' . $params[0]
                // . '&id_attribute=' . $params[1]
                . '&delete' . $this->table,
            'action' => self::$cache_lang['delete'],
            'first' => true,
            'js_confirmation' => self::$cache_lang['delete_confirm'],
            'target' => '',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/delete.tpl'
        );
    }

    public function displayForcedeleteLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('forcedelete', self::$cache_lang)) {
            self::$cache_lang['forcedelete'] = $this->l('Force delete attribute');
            self::$cache_lang['forcedelete_confirm'] = $this->l('Delete attributes AND RELATED ATTRIBUTE
                COMBINATIONS ? Note: you have selected the FORCE DELETE option and attributes will
                be deleted elseif used in combinations. Related attribute_combinations will be deleted
                too. You cannot rollback !!');
        }

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $this->context->link->getAdminLink($this->name) // self::$currentIndex
                // . '&' . $this->identifier . '=' . $id
                // simulated: give new name to discern this id_product from the one that identifies the print option
                . '&caption=' . $id
                // . '&id_product_attribute=' . $params[0]
                // . '&id_attribute='.$params[1]
                . '&forcedelete' . $this->table,
            'action' => self::$cache_lang['forcedelete'],
            'first' => true,
            'js_confirmation' => self::$cache_lang['forcedelete_confirm'],
            'target' => '',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/delete.tpl'
        );
    }

    public function postProcess()
    {
        if (Tools::getIsset('delete' . $this->table)) {
            // get id_product_attribute and id_attribute from param $id
            // $caption = Tools::getValue('caption');
            // $params = explode('-', $caption);
            $deleted = $this->deleteAttribute(Tools::getValue('caption'));
            if ($deleted >= 0) {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&deleted=' . $deleted);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&deleted=0 : '
                    . (string) ($deleted * (-1)) . ' '
                    . $this->l('attribute was not deleted because used in one or more combinations !'));
            }
        } elseif (Tools::getIsset('forcedelete' . $this->table)) {
            // MxbToolsDebug::writeLog($this->name.'Controller', 'postProcess()', 'LINK='.$_SERVER['QUERY_STRING']);
            $deleted = $this->deleteAttribute(Tools::getValue('caption'), true);
            if ($deleted >= 0) {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&deleted=' . $deleted);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&deleted='
                    . (string) ($deleted * (-1)) . ' '
                    . $this->l('attribute used in combinations was deleted !'));
            }
        } elseif (Tools::getIsset('submitBulkdelete' . $this->table)) {
            // intercept bulk delete to get IDs and delete gadgetcard_special_discount_product records
            $deleted = 0;
            $not_deleted = 0;
            $row_array = Tools::getValue($this->table . 'Box');
            if (count($row_array)) {
                foreach ($row_array as $row) {
                    // MxbToolsDebug::writeLog($this->name.'Controller', 'postProcess()', 'BUILK DELETE ID='.$row);
                    $del = $this->deleteAttribute($row);
                    if ($del >= 0) {
                        $deleted += $del;
                    } else {
                        $not_deleted += ($del * (-1));
                    }
                }
            }
            $str = (string) $deleted . ($not_deleted > 0
                    ? ', ' . $this->l('not delete because used in combinations:') . ' ' . $not_deleted
                    : '');
            Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&deleted=' . $str);
        }
        return parent::postProcess();
    }

    protected function deleteAttribute($caption, $force = false)
    {
        $values = $this->getCaptionValues($caption);
        if (!is_null($values)) {
            $result = Db::getInstance()->executeS('
                SELECT pac.*
                FROM `' . _DB_PREFIX_ . 'product_attribute_combination` pac
                WHERE pac.id_attribute = ' . $values['id_attribute']
            );

            // if any record found, don't delete attribute !
            $delete = !(is_array($result) && count($result) > 1);

            if ($delete || $force) {
                // delete
                // $id_product_attribute = $params[0];
                // $id_attribute = $params[1];
                Db::getInstance()->execute('
                    DELETE FROM `' . _DB_PREFIX_ . 'attribute`
                    WHERE id_attribute=' . $values['id_attribute']
                );
                Db::getInstance()->execute('
                    DELETE FROM `' . _DB_PREFIX_ . 'attribute_shop`
                    WHERE id_attribute=' . $values['id_attribute']
                );
                Db::getInstance()->execute('
                    DELETE FROM `' . _DB_PREFIX_ . 'attribute_lang`
                    WHERE id_attribute=' . $values['id_attribute']
                );
                if ($force) {
                    Db::getInstance()->execute('
                        DELETE FROM `' . _DB_PREFIX_ . 'product_attribute_combination`
                        WHERE id_attribute=' . $values['id_attribute']
                    );
                }

                return $delete ? 1 : -1;
            } else {
                return -1;
            }
        }

        return 0;
    }

    protected function getCaptionValues($caption)
    {
        // need only id_attribute ...
        return ['id_attribute' => $caption];
        /*
            $params = explode('-', $caption);

            if (is_array($params) && count($params) > 1) {
              return array('id_product' => $params[0],
                'id_product_attribute' => $params[1],
                'id_attribute_group' => $params[2],
                'id_attribute' => $params[3],
                );
            }
            return null;
        */
    }

    protected function cacheSetValue($value, $key = 'MXB_TOOLS_MCA_MODE')
    {
        $this->context->cookie->__set($key, $value);
    }

    protected function cacheGetValue($key = 'MXB_TOOLS_MCA_MODE')
    {
        switch ($key) {
            case 'id_product_attribute':
                return (int) $this->context->cookie->id_product_attribute;
            default:
                return (int) $this->context->cookie->MXB_TOOLS_MCA_MODE;
        }

        return null;
    }
}
