<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

/*
    MANAGE COMBINATION ATTRIBUTES
*/
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsManageCombinationAttributes.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCodes.php';
require_once _PS_CLASS_DIR_ . 'Product.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsManageCombAttributesController extends ModuleAdminController
{
    public function __construct()
    {
        $this->name = 'AdminMxbToolsManageCombAttributes';
        $this->table = 'product_attribute';
        $this->className = 'MxbToolsManageCombinationAttributes';
        $this->lang = false;
        // $this->list_no_link = true;
        $this->module = 'mxb_tools';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->_orderBy = 'id_product_attribute';
        $this->_orderWay = 'ASC';
        $this->identifier = 'caption';
        $this->list_no_link = true;
        $this->bootstrap = true;

        parent::__construct();
        // $this->displayName = 'Combination attributes';

        if (Tools::getIsset('m')) {
            $this->mode = (int) Tools::getValue('m');
        } elseif ($this->cacheGetValue() !== null) {
            $this->mode = $this->cacheGetValue();
        } else {
            $this->mode = 0;
        }
        $this->cacheSetValue($this->mode);

        if ($this->mode == 0) {
            // list all attributes
            $this->errors[] = $this->l('This module lists all attributes of all products combinations. You can delete the desired attribute(s) of any combination, but keep in mind that there is no rollback. Deleting the wrong or all attributes of a combination, you will loose it !');
            $this->displayName = $this->l('Combination attributes');
        } else {
            // list only duplicated
            $this->displayName = $this->l('Duplicated combination attributes');
            $this->errors[] = $this->l('This module lists only duplicated attributes (different id_attribute but same id_attribute_group) of all products combinations. You can delete the desired duplicated attribute(s) of any combination, but keep in mind that there is no rollback. Deleting the wrong or all attributes of a combination, you will loose it !');
        }

        if (Tools::getIsset('deleted')) {
            $deleted_records = Tools::getValue('deleted');
            $this->confirmations[] = $this->l('Attributes deleted') . ': ' . $deleted_records;
            // $output .= $this->module->displayConfirmation($this->l('Attributes deleted') . ': ' . $deleted_records );
        }

        $this->default_form_language = $this->context->language->id;
        $this->_select = '
            ag.`id_attribute_group`,
            agl.`name` AS group_name,
            att.`id_attribute` as id_attribute,
            al.`name` AS attribute_name,
            att.`position`,
            CONCAT(a.`id_product`, "-", a.`id_product_attribute`, "-", ag.`id_attribute_group`, "-", att.`id_attribute`) AS caption
        ';
        $this->_join .= '
            LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute_combination` pac ON pac.`id_product_attribute` = a.`id_product_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute` att ON att.`id_attribute` = pac.`id_attribute`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_lang` al ON (att.`id_attribute` = al.`id_attribute` AND al.`id_lang` = ' . $this->context->language->id . ')
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group` ag ON ag.`id_attribute_group` = att.`id_attribute_group`
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group_lang` agl ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND agl.`id_lang` = ' . $this->context->language->id . ')
        ';
        $this->_where = ' AND' . MxbToolsCodes::sqlFilterProductsOnly('a');

        if ($this->mode == 1) {
            $this->_where .= '
                AND  1 < (SELECT COUNT(c_att.id_attribute) as result
                FROM `' . _DB_PREFIX_ . 'attribute` c_att
                LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute_combination` c_pac ON (c_pac.`id_attribute` = c_att.`id_attribute`)
                WHERE c_pac.`id_product_attribute` = a.`id_product_attribute`
                AND c_att.`id_attribute_group` = ag.`id_attribute_group`
                GROUP BY c_att.`id_attribute_group`
                ORDER BY result DESC
                LIMIT 1)';
        }
        // $this->addRowAction('edit');
        $this->addRowAction('delete');
        $this->addRowAction('forcedelete');

        $this->addRowAction('viewproduct');
        $this->addRowAction('editproduct');
        $this->addRowAction('editattribute');

        $this->bulk_actions = [
            'delete' => [
                'text' => $this->trans('Delete selected', [], 'Admin.Actions'),
                'confirm' => $this->l('Delete attributes from combinations ? Note: you cannot rollback !!'),
                'icon' => 'icon-trash',
              ],
        ];

        $this->_use_found_rows = false;
        $this->fields_list = [
            'id_product' => [
                'title' => $this->l('Product ID'),
                'align' => 'left',
                'width' => 60,
            ],
            'id_product_attribute' => [
                'title' => $this->l('Product attribute ID'),
                'align' => 'left',
                'width' => 60,
            ],
            'reference' => [
                'title' => $this->l('Reference'),
                // 'title' => $this->l('ID'),
                'align' => 'left',
                'width' => 120,
            ],
            'id_attribute_group' => [
                'title' => $this->l('ID attribute group'),
                'filter_key' => 'ag!id_attribute_group',
                'align' => 'left',
                'width' => 60,
            ],
            'group_name' => [
                'title' => $this->l('Attribute group name'),
                'filter_key' => 'agl!name',
                'align' => 'left',
                'width' => 120,
            ],
            'id_attribute' => [
                'title' => $this->l('ID attribute'),
                'filter_key' => 'att!id_attribute',
                'align' => 'left',
                'width' => 60,
            ],
            'attribute_name' => [
                'title' => $this->l('attribute name'),
                'filter_key' => 'al!name',
                'align' => 'left',
                'width' => 120,
            ],
        ];
        $this->renderForm();
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);
        return $return;
    }

    /**
     * @param: $id = 'caption' field of query -> 'id_product'-'id_product_attribute'-'id_attribute'
     */
    public function displayDeleteLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('delete', self::$cache_lang)) {
            self::$cache_lang['delete'] = $this->l('Delete attribute');
            self::$cache_lang['delete_confirm'] = $this->l('Delete attributes from combinations ? Note: you cannot rollback !!');
        }

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $this->context->link->getAdminLink('AdminMxbToolsManageCombAttributes') // self::$currentIndex
                // '&' . $this->identifier . '=' . $id
                // simulated: give a new name to discern this id_product from the id_product that identifies the print option
                . '&caption=' . $id
                //  '&id_product_attribute=' . $params[0]
                //  '&id_attribute=' . $params[1]
                . '&delete' . $this->table
                . '&m=' . $this->mode,
            'action' => self::$cache_lang['delete'],
            'first' => true,
            'js_confirmation' => self::$cache_lang['delete_confirm'],
            'target' => '',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);

        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/delete.tpl'
        );
    }

    public function displayForcedeleteLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('forcedelete', self::$cache_lang)) {
            self::$cache_lang['forcedelete'] = $this->l('Force delete attribute');
            self::$cache_lang['forcedelete_confirm'] = $this->l('Delete attributes from combinations ? Note: you have selected the FORCE DELETE option and attributes will be deleted elseif unique. You cannot rollback !!');
        }

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $this->context->link->getAdminLink('AdminMxbToolsManageCombAttributes') // self::$currentIndex.
                // '&' . $this->identifier . '=' . $id
                // simulated: give a new name to discern this id_product from the id_product that identifies the print option
                . '&caption=' . $id
                //  '&id_product_attribute=' . $params[0]
                //  '&id_attribute=' . $params[1]
                . '&forcedelete' . $this->table
                . '&m=' . $this->mode,
            'action' => self::$cache_lang['forcedelete'],
            'first' => true,
            'js_confirmation' => self::$cache_lang['forcedelete_confirm'],
            'target' => '',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/delete.tpl'
        );
    }

    public function displayViewproductLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('viewproduct', self::$cache_lang)) {
            self::$cache_lang['viewproduct'] = $this->l('View product');
        }

        $values = $this->getCaptionValues($id);
        $product_url = $this->context->link->getProductLink($values['id_product'], null, null, null, null, null, $values['id_product_attribute'], Configuration::get('PS_REWRITING_SETTINGS'), false, true);

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $product_url,
            // '&caption=' . $id.
            // '&viewproduct' . $this->table.
            // '&m=' . $this->mode,
            'action' => self::$cache_lang['viewproduct'],
            'first' => false,
            'js_confirmation' => '',
            'target' => '_new',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/view.tpl'
        );
    }

    public function displayEditproductLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('editproduct', self::$cache_lang)) {
            self::$cache_lang['editproduct'] = $this->l('Edit product');
        }

        $values = $this->getCaptionValues($id);
        // https://amogadget.com/Backoffice/index.php/sell/catalog/products/9835?_token=-.....
        $product_url = $this->context->link->getAdminLink('AdminProducts', false, ['id_product' => $values['id_product']], ['id_product' => $values['id_product']]);

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $product_url,
            // '&caption=' . $id.
            // '&viewproduct' . $this->table.
            // '&m=' . $this->mode,
            'action' => self::$cache_lang['editproduct'],
            'first' => false,
            'js_confirmation' => '',
            'target' => '_new',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/edit.tpl'
        );
    }

    public function displayEditattributeLink($token = null, $id = null, $name = null)
    {
        if (!array_key_exists('editattribute', self::$cache_lang)) {
            self::$cache_lang['editattribute'] = $this->l('Edit attribute');
        }

        $values = $this->getCaptionValues($id);
        // https://amogadget.com/Backoffice/index.php?controller=AdminAttributesGroups&id_attribute_group=140&viewattribute_group&id_attribute_group=140&viewattribute_group&updateattribute=&id_attribute=1049&token=03fbaf7a652558e707ba749882ea7bc2
        $product_url = $this->context->link->getAdminLink('AdminAttributesGroups', true, [], ['id_attribute_group' => $values['id_attribute_group'], 'viewattribute_group' => 1, 'updateattribute' => 1, 'id_attribute' => $values['id_attribute']]);

        // get id_product_attribute and id_attribute from param $id
        // $params = explode('-', $id);
        // if (is_array($params) && count($params) > 1) {
        $this->context->smarty->assign([
            'module_dir' => __PS_BASE_URI__ . 'modules/mxb_tools/',
            'href' => $product_url,
            // '&caption=' . $id.
            // '&viewproduct' . $this->table.
            // '&m=' . $this->mode,
            'action' => self::$cache_lang['editattribute'],
            'first' => false,
            'js_confirmation' => '',
            'target' => '_new',
            'classes' => '',
        ]);
        // }
        // $link = $this->context->link->getAdminLink('AdminProducts');
        // Tools::redirectAdmin($link);
        return $this->context->smarty->fetch(
            _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/list_action/edit.tpl'
        );
    }

    /*
        public function displayEditLink($token = null, $id = null)
        {
            return false;
        }
        public function getEditLink($token = null, $id = null)
        {
            return false;
        }
    */

    public function postProcess()
    {
        if (Tools::getIsset('delete' . $this->table)) {
            // get id_product_attribute and id_attribute from param $id
            // $caption = Tools::getValue('caption');
            // $params = explode('-', $caption);
            // if (is_array($values) && count($values) > 0) {
            $deleted = $this->deleteAttributeCombination(Tools::getValue('caption'));
            if ($deleted >= 0) {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name) . '&m=' . $this->mode . '&deleted=' . $deleted);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name)
                    . '&m=' . $this->mode . '&deleted=0 : '
                    . (string) ($deleted * (-1)) . ' '
                    . $this->l('attribute was not deleted because unique !'));
            }
        } elseif (Tools::getIsset('forcedelete' . $this->table)) {
            $deleted = $this->deleteAttributeCombination(Tools::getValue('caption'), true);
            if ($deleted >= 0) {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name)
                    . '&m=' . $this->mode . '&deleted=' . $deleted);
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink($this->name)
                    . '&m=' . $this->mode . '&deleted='
                    . (string) ($deleted * (-1)) . ' '
                    . $this->l('unique attribute was deleted !'));
            }
        }

        // intercept bulk delete to get IDs and delete gadgetcard_special_discount_product records
        elseif (Tools::getIsset('submitBulkdelete' . $this->table)) {
            $deleted = 0;
            $not_deleted = 0;
            $row_array = Tools::getValue($this->table . 'Box');
            if (count($row_array)) {
                foreach ($row_array as $row) {
                    // MxbToolsDebug::writeLog($this->name . 'Controller', 'postProcess()', 'BUILK DELETE ID=' . $row);
                    $del = $this->deleteAttributeCombination($row);
                    if ($del >= 0) {
                        $deleted += $del;
                    } else {
                        $not_deleted += ($del * (-1));
                    }
                    // CategoryGalleryModel::deleteGallery((int) $id_gallery);
                }
                // exit;
            }
            $str = (string) $deleted . ($not_deleted > 0
                ? ', ' . $this->l('not delete because unique:') . ' ' . $not_deleted
                : '');
            Tools::redirectAdmin($this->context->link->getAdminLink($this->name)
                . '&m=' . $this->mode . '&deleted=' . $str);
        }
        return parent::postProcess();
    }

    protected function deleteAttributeCombination($caption, $force = false)
    {
        $values = $this->getCaptionValues($caption);
        if (!is_null($values)) {
            $result = Db::getInstance()->executeS('
                SELECT pac.*
                FROM `' . _DB_PREFIX_ . 'product_attribute_combination` pac
                LEFT JOIN `' . _DB_PREFIX_ . 'attribute` a ON (a.id_attribute = pac.id_attribute)
                WHERE pac.id_product_attribute = ' . $values['id_product_attribute'] . '
                AND a.id_attribute_group = ' . $values['id_attribute_group']
            );
            $delete = (is_array($result) && count($result) > 1);

            if ($delete || $force) {
                // delete
                // $id_product_attribute = $params[0];
                // $id_attribute = $params[1];
                Db::getInstance()->execute('
                    DELETE FROM `' . _DB_PREFIX_ . 'product_attribute_combination`
                    WHERE id_product_attribute=' . $values['id_product_attribute'] . '
                    AND  id_attribute=' . $values['id_attribute']);

                return $delete ? 1 : -1;
            } else {
                return -1;
            }
        }

        return 0;
    }

    protected function getCaptionValues($caption)
    {
        $params = explode('-', $caption);
        if (is_array($params) && count($params) > 1) {
            return [
                'id_product' => $params[0],
                'id_product_attribute' => $params[1],
                'id_attribute_group' => $params[2],
                'id_attribute' => $params[3],
            ];
        }

        return null;
    }

    protected function cacheSetValue($value, $key = 'MXB_TOOLS_MCA_MODE')
    {
        // if (is_null($key))
        $this->context->cookie->__set($key, $value);
    }

    protected function cacheGetValue($key = 'MXB_TOOLS_MCA_MODE')
    {
        switch ($key) {
            case 'id_product_attribute':
                return (int) $this->context->cookie->id_product_attribute;
            default:
                return (int) $this->context->cookie->MXB_TOOLS_MCA_MODE;
        }
        return null;
    }
}
