<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

// use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsMenu.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsProduct.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCartRule.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminMxbToolsToolsController extends ModuleAdminController
{
    /*
      protected $colorOnBackground = true;
      protected $color_on_background = true;
    */
    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->class_name = 'AdminMxbToolsToolsController';
        $this->tab = 'front_office_features';
        $this->templateFile = _PS_MODULE_DIR_ . 'mxb_tools/views/templates/admin/tools.tpl';
        $this->multishop_context = Shop::CONTEXT_ALL;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('MXB Tools: utilities');

        // MANAGE REQUESTED OPERATION
        if (Tools::getValue('action')) {
            // NOTE:
            // set an hidden input into tools.tpl, named 'action' and with the function name as value
            $action = Tools::getValue('action');
            if (method_exists($this, $action)) {
                $this->{$action}();
            }
        }

        /*
        // MANAGE REQUESTED OPERATION
        if (Tools::getValue('action') && Tools::getValue('action')=='setAllCategoriesVisibleToAllGroups') {
            $this->setAllCategoriesVisibleToAllGroups();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='manageCombinationAttributes') {
            $this->manageCombinationAttributes();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='manageAttributes') {
            $this->manageAttributes();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='manageDuplicatedCombinationAttributes') {
            $this->manageDuplicatedCombinationAttributes();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='changeStatusCartRules') {
            $this->changeStatusCartRules();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='createCartRules') {
            $this->createCartRules();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='generateOrderPdf') {
            $this->generateOrderPdf();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='generateCoverjobPdf') {
            $this->generateCoverjobPdf();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='enableEmptyCategories') {
            $this->enableEmptyCategories();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='disableEmptyCategories') {
            $this->disableEmptyCategories();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='enableEmptyManufacturers') {
            $this->enableEmptyManufacturers();
        }
        if (Tools::getValue('action') && Tools::getValue('action')=='disableEmptyManufacturers') {
            $this->disableEmptyManufacturers();
        }
        */
    }

    public function renderList()
    {
        $output = '';
        if (Tools::getValue('submitAddmxb_tools_tools')) {
            $output .= $this->processForm();
        }
        $smarty = Context::getContext()->smarty;

        // add config values to smarty
        $smarty->assign($this->getConfigFormValues());
        $smarty->assign(MxbToolsUtils::getCommonAdminTplValues($this->context));
        $smarty->assign('form_action', $this->context->link->getAdminLink('AdminMxbToolsTools'));

        // add header menu
        $smarty->assign('mxb_tools_menu', MxbToolsMenu::getMxbToolsMenu($this->context));

        // load lists for select elements
        $idLang = (int) $this->context->language->id;
        $smarty->assign('category_list', MxbToolsProduct::getRecursiveCategoriesList($idLang, null, 0, '&nbsp;&nbsp;'));
        $smarty->assign('group_list', MxbToolsProduct::getGroupList($idLang));

        if (Tools::getValue('target')) {
            $smarty->assign('target_menu', Tools::getValue('target'));
        }

        $output .= $this->context->smarty->fetch($this->templateFile);
        return $output; // .$this->renderForm();
    }

    /*
      protected function getCsvImportTable($target_filename, $csv_delimeter)
      {
        $csv = MxbToolsUtils::csvToArray($target_filename, $csv_delimeter);

            if ($csv) {
              $csv_heading = '';
              $csv_rows = '';

              foreach ($csv as $rowkey=>$row) {
                if ((int) $rowkey === 0)
                  $csv_heading .= '<tr>';
                $csv_rows .= '<tr>';
                foreach ($row as $key=>$value) {
                  if ((int) $rowkey === 0)
                    $csv_heading .= '<th>' . $key . '</th>';
                  $csv_rows .= '<td>' . $value . '</td>';
                }
                if ((int) $rowkey === 0)
                  $csv_heading .= '</tr>';
                $csv_rows .= '</tr>';
              }

              return '<table class="csv-table">' . $csv_heading . $csv_rows . '</table>';
            }
            return null;
      }
    */

    protected function getValue($key, $default = null, $type = null, $save = false)
    {
        return MxbToolsUtils::getConfigValue($key, $default, $type, $save);
    }

    protected function setValue($key, $value, $type = null)
    {
        MxbToolsUtils::setConfigValue($key, $value, $type);
    }

    /**
     * Set values for the inputs.
     */
    public function getConfigFormValues()
    {
        return MxbToolsUtils::getConfigFormValues();
    }

    public function processForm()
    {
        $smarty = Context::getContext()->smarty;
        $form_error = [];
        $form_success = [];
        $form_values = $this->getConfigFormValues();
        // MxbToolsDebug::writeLog($this->name, $this->class_name . '::processForm()', 'form_values=' . print_r($form_values, true), false);

        // validation: not needed
        if (count($form_error)) {
            return $this->module->displayError(
                $this->l('Unable to save configuration for the following reasons:')
                . '<br>.' . implode('<br>. ', $form_error)
            );
        }
        foreach (array_keys($form_values) as $key) {
            Configuration::updateGlobalValue($key, $form_values[$key]);
        }
        if (count($form_success)) {
            return $this->module->displayConfirmation(
                $this->l('Configuration saved:') . '<br>.' . implode('<br>. ', $form_success)
            );
        }
        return $this->module->displayConfirmation($this->l('Configuration saved'));
    }

    public function initToolbar()
    {
        $return = parent::initToolbar();
        $this->toolbar_title[] = $this->module->l($this->displayName);
        return $return;
    }

    protected function setAllCategoriesVisibleToAllGroups()
    {
        $groups = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT DISTINCT c.*
            FROM `' . _DB_PREFIX_ . 'group` c
        ');

        $categories = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT DISTINCT c.*
            FROM `' . _DB_PREFIX_ . 'category` c
            LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl
            ON (c.`id_category` = cl.`id_category` AND cl.`id_lang` = '
            . (int) Context::getContext()->language->id . ')
        ');

        foreach ($groups as $group) {
            foreach ($categories as $category) {
                $exists = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                    SELECT id_category, id_group
                    FROM ' . _DB_PREFIX_ . 'category_group
                    WHERE id_category = "' . $category['id_category'] . '"
                    AND id_group = "' . $group['id_group'] . '"');

                if (!$exists) {
                    $categoryObj = new Category($category['id_category'], (int) Context::getContext()->language->id);
                    $categoryObj->addGroups([$group['id_group']]);
                    $this->confirmations[] = $this->l('group') . ' ' . $group['id_group'] . ' '
                        . $this->l('added to category') . ' '
                        . $category['id_category'] . '<br />';
                } else {
                    $this->errors[] = $this->l('group') . ' ' . $group['id_group'] . ' '
                        . $this->l('ALREADY added to category') . ' '
                        . $category['id_category'] . '<br />';
                }
            }
        }
    }

    protected function changeStatusCartRules()
    {
        // protected function getValue($key, $default = null, $type = null, $save = false)
        $id_cart_rule_start = (int) $this->getValue('MXB_TOOLS_CHANGE_STATUS_CART_RULES_START_ID', 0, 'number', true);
        $id_cart_rule_end = (int) $this->getValue('MXB_TOOLS_CHANGE_STATUS_CART_RULES_END_ID', 0, 'number', true);
        $check_expired = (bool) $this->getValue('MXB_TOOLS_CHANGE_STATUS_CART_RULES_CHECK_DEADLINE', false, 'bool', true);
        $cart_rule_new_status = (int) $this->getValue(
            'MXB_TOOLS_CHANGE_STATUS_CART_RULES_TARGET_STATUS',
            1,
            'number',
            true
        );
        $delete_from_cart = (bool) $this->getValue(
            'MXB_TOOLS_CHANGE_STATUS_CART_RULES_DELETE_FROM_CART',
            false,
            'bool',
            true
        );

        // status to active, no need to delete cart rules from carts ...
        if ($cart_rule_new_status === 1) {
            $delete_from_cart = 0;
            $this->setValue('MXB_TOOLS_CHANGE_STATUS_CART_RULES_DELETE_FROM_CART', $delete_from_cart, 'bool');
        }

        $filters = '';
        if ($id_cart_rule_start > 0) {
            $filters .= ' AND id_cart_rule >= ' . $id_cart_rule_start;
        }
        if ($id_cart_rule_end > 0) {
            $filters .= ' AND id_cart_rule <= ' . $id_cart_rule_end;
        }
        if ($check_expired) {
            if ((int) $cart_rule_new_status == 1) {
                $filters .= ' AND date_to > "' . date('Y-m-d 23:59:59') . '"';
            } else {
                $filters .= ' AND date_to < "' . date('Y-m-d 23:59:59') . '"';
            }
        }

        /*
          $res = Db::getInstance()->execute('
            UPDATE `' . _DB_PREFIX_ . 'cart_rule` cr
            SET active = ' . (int) $cart_rule_new_status . '
            WHERE 1 ' . $filters);
        */

        $cart_rules = Db::getInstance()->executeS('
            SELECT id_cart_rule, active
            FROM `' . _DB_PREFIX_ . 'cart_rule`
            WHERE active <> ' . (int) $cart_rule_new_status . $filters);

        if (!count($cart_rules)) {
            $this->errors[] = ': ' . $this->l('NO VOUCHERS FOUND with status different from')
                . ' ' . $cart_rule_new_status
                . ' ' . $this->l('and from cart rule ID=')
                . $id_cart_rule_start
                . ' ' . $this->l('to ID=') . $id_cart_rule_end . '<br />';
            return;
        }

        $this->confirmations[] = $this->l('Vouchers found:') . ' ' . count($cart_rules);

        // change to "disabled": need to delete vouchers from user carts
        if ($delete_from_cart && (int) $cart_rule_new_status === 0) {
            foreach ($cart_rules as $cr) {
                $cart_with_rule = Db::getInstance()->executeS('
                    SELECT DISTINCT id_cart, id_cart_rule
                    FROM `' . _DB_PREFIX_ . 'cart_cart_rule`
                    WHERE id_cart_rule = ' . (int) $cr['id_cart_rule'] . '
                    ORDER BY id_cart');

                // delete cart rules from cart
                if (count($cart_with_rule)) {
                    foreach ($cart_with_rule as $cwr) {
                        $cart = new Cart((int) $cwr['id_cart']);
                        if (!Validate::isLoadedObject($cart)) {
                            continue;
                        }
                        $cart->removeCartRule((int) $cwr['id_cart_rule']);
                        $cart->update();
                        $this->confirmations[] = '<br>'
                            . $this->l('Removed voucher ID')
                            . ' ' . $cwr['id_cart_rule']
                            . ' ' . $this->l('from cart ID')
                            . ' ' . $cwr['id_cart'];
                    }
                }
            }
        }

        // change status of cart rules
        $res = Db::getInstance()->execute('
            UPDATE `' . _DB_PREFIX_ . 'cart_rule` cr
            SET active = ' . (int) $cart_rule_new_status . '
            WHERE 1 ' . $filters);

        if (!$res) {
            $this->errors[] = ': '
                . $this->l('error setting vouchers status to')
                . ' ' . $cart_rule_new_status
                . ' ' . $this->l('from ID=') . ' ' . $id_cart_rule_start
                . ' ' . $this->l('to ID=') . ' ' . $id_cart_rule_end . '<br />';
        } else {
            $this->confirmations[] = '<br>'
                . $this->l('Vouchers status set to')
                . ' ' . $cart_rule_new_status
                . ' ' . $this->l('from ID=')
                . ' ' . $id_cart_rule_start
                . ' ' . $this->l('to ID=')
                . ' ' . $id_cart_rule_end
                . '<br />';
        }
    }

    protected function createCartRules()
    {
        //   protected function getValue($key, $default = null, $type = null, $save = false)
        $cart_rule_code = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CODE', '', 'string', true);
        $cart_rule_name = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_NAME', '', 'string', true);
        $cart_rule_description = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_DESCRIPTION', '', 'string', true);
        $random_code = (bool) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CODE_RND', true, 'bool', true);
        $code_len = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CODE_LEN', 12, 'number', true);
        $quantity_to_create = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_QUANTITY_TO_CREATE', 1, 'number', true);
        $quantity = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_QUANTITY', 1, 'number', true);
        $quantity_per_user = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_QUANTITY_PER_USER', 1, 'number', true);
        $priority = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_PRIORITY', 1, 'number', true);
        $id_customer = abs((int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CUSTOMER_ID', 0, 'string', true));
        $group_ids = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_GROUP_IDS', '', 'string', true);
        $category_ids = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CATEGORY_IDS', '', 'string', true);
        $partial_use = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_PARTIAL_USE', true, 'bool', true);
        $free_shipping = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_FREE_SHIPPING', true, 'bool', true);
        $highlight = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_HIGHLIGHT', true, 'bool', true);
        $reduction_percent = (float) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_REDUCTION_PERCENT', 0, 'number', true);
        $reduction_amount = (float) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_REDUCTION_AMOUNT', 0, 'number', true);
        $reduction_tax = (int) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_REDUCTION_TAX', true, 'bool', true);
        $date_from = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_DATE_FROM', '', 'string', true);
        $date_to = (string) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_DATE_TO', '', 'string', true);
        $check_active = (bool) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CHECK_ACTIVE', false, 'bool', true);
        $check_inStock = (bool) $this->getValue('MXB_TOOLS_CREATE_CART_RULE_CHECK_IN_STOCK', false, 'bool', true);
        $check_includeGeneric = (bool) $this->getValue(
            'MXB_TOOLS_CREATE_CART_RULE_CHECK_INCLUDE_GENERIC',
            false,
            'bool',
            true
        );

        $ret = MxbToolsCartRule::createCartRule(
            $cart_rule_code,    // used as prefix if random_code is true
            $random_code,       // bool: true=generate a random code with "cart_rule_code" as prefix
            $code_len,          // length of code (max 254 chars)
            $quantity_to_create,// quantity of rules to create
            $quantity,          // quantity of single cart rule
            $quantity_per_user, // quantity of single cart rule per user
            $priority,          // priority of the coupon (1 is better)
            $cart_rule_name,    // STRING !! (1 language) // array/string. if empty: name will be cart_rule_code
            $cart_rule_description, // string
            $id_customer,       // id_customer: restrict to one customer
            $group_ids,         // array or ID string of groups for restrinctions.
            $category_ids,      // array or ID string of categories for restrinctions.
            $partial_use,       // can cart rule be used partially ?
            $highlight,         // does the voucher be listed in user cart voucher list ?
            $free_shipping,     // 1 = free
            $reduction_percent, // your percentage or 0.00;
            $reduction_amount,  // your_amount or 0.00;
            $reduction_tax,     // 1 = tax included;
            $date_from,         // start date / time = date('Y-m-d H:i:s');
            $date_to,           // end date / time = date('Y-m-d H:i:s'););
            false,              // $overwrite if true, existing voucher code will be overwritten
            1,                  // $create_csv 1 = create new / 2 = append
            // $unique_for_customer  // if true, voucher will be created only if the customer has no other vouchers
            (bool) $check_active,  // if true, voucher will be created only if the customer has no other vouchers
            (bool) $check_inStock, // if true, voucher will be created only if the customer has no other vouchers
            (bool) $check_includeGeneric // if true, voucher will be created only if the customer has no other vouchers
        );

        if (isset($ret['error'])) {
            $this->errors[] = $ret['error'];
            return;
        }

        if (isset($ret['download_link'])) {
            $this->confirmations[] = $this->l('Cart rules created! CSV file is ready to download! Click here') . ': '
                . '<a href="' . $ret['download_url'] . '" target="_new">' . $ret['download_url'] . '</a>';
        } else {
            $this->confirmations[] = 'Cart rules created<br />';
        }
    }

    protected function manageCombinationAttributes()
    {
        Tools::redirectAdmin(
            Context::getContext()->link->getAdminLink('AdminMxbToolsManageCombAttributes') . '&m=0'
        );
    }

    protected function manageDuplicatedCombinationAttributes()
    {
        Tools::redirectAdmin(
            Context::getContext()->link->getAdminLink('AdminMxbToolsManageCombAttributes') . '&m=1'
        );
    }

    protected function manageAttributes()
    {
        Tools::redirectAdmin(
            Context::getContext()->link->getAdminLink('AdminMxbToolsManageAttributes') . '&m=0'
        );
    }

    protected function generateOrderPdf()
    {
        if (Tools::getIsset('id_order')) {
            $id_order = Tools::getValue('id_order');
        } else {
            $id_order = $this->getValue('MXB_TOOLS_GENERATE_PDF_ORDER_ID', 0, 'int', true);
        }
        if ((int) $id_order <= 0) {
            $this->errors[] = $this->l('Please insert a valid order ID.');

            return;
        }
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            $this->errors[] = $this->l('The order cannot be found within your database.');

            return;
        }
        require_once _PS_MODULE_DIR_ . $this->name . '/classes/pdf/HTMLTemplateOrderPdf.php';
        // header('Content-type: application/pdf');
        // header('Content-Disposition: attachment; filename=order.pdf');

        $pdf = new PDF($order, 'OrderPdf', Context::getContext()->smarty);
        // $pdf->render();
        $ret = MxbToolsUtils::tryRenderPdf($pdf);
        if (is_string($ret)) {
            exit($ret);
        }
    }

    protected function generateCoverjobPdf()
    {
        if (Tools::getIsset('id_order')) {
            $id_order = Tools::getValue('id_order');
        } else {
            $id_order = $this->getValue('MXB_TOOLS_GENERATE_PDF_ORDER_ID', 0, 'int', true);
        }
        if ((int) $id_order <= 0) {
            $this->errors[] = $this->l('Please insert a valid order ID.');

            return;
        }
        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            $this->errors[] = $this->l('The order cannot be found within your database.');

            return;
        }
        require_once _PS_MODULE_DIR_ . $this->name . '/classes/pdf/HTMLTemplateCoverjobPdf.php';
        // header('Content-type: application/pdf');
        // header('Content-Disposition: attachment; filename=order.pdf');

        $pdf = new PDF($order, 'CoverjobPdf', Context::getContext()->smarty);
        // $pdf->render();
        $ret = MxbToolsUtils::tryRenderPdf($pdf);
        if (is_string($ret)) {
            exit($ret);
        }
    }

    protected function enableEmptyCategories()
    {
        $this->setConfirmationsFromResult($this->toggleEntityStatus('cat', true));
    }

    protected function disableEmptyCategories()
    {
        $this->setConfirmationsFromResult($this->toggleEntityStatus('cat', false));
    }

    protected function enableEmptyManufacturers()
    {
        $this->setConfirmationsFromResult($this->toggleEntityStatus('man', true));
    }

    protected function disableEmptyManufacturers()
    {
        $this->setConfirmationsFromResult($this->toggleEntityStatus('man', false));
    }

    protected function modifyAdvancedimporterDatabase()
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute('
            ALTER TABLE `' . _DB_PREFIX_ . 'advancedimporter_block`
            MODIFY `block` LONGTEXT
        ');
        if ($result === true) {
            $this->confirmations[] = 'Table `' . _DB_PREFIX_ . 'advancedimporter_block` > field block successfully modified !<br />';
        } else {
            $this->errors[] = 'Error during database operation: ' . $result . '<br />';
        }
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute('
            ALTER TABLE `' . _DB_PREFIX_ . 'advancedimporter_template`
            MODIFY `xslt` MEDIUMTEXT
        ');
        if ($result === true) {
            $this->confirmations[] = 'Table `' . _DB_PREFIX_ . 'advancedimporter_template` > field xslt successfully modified !<br />';
        } else {
            $this->errors[] = 'Error during database operation: ' . $result . '<br />';
        }
    }

    protected function setConfirmationsFromResult($result)
    {
        if (!$result) {
            return;
        }
        if (!empty($result['error'])) {
            $this->errors = $result['error'];
        }
        if (!empty($result['success'])) {
            $this->confirmations = $result['success'];
        }
    }

    protected function toggleEntityStatus($entity, $enable = true)
    {
        if (!$entity) {
            return ['error' => ['ERROR: no entity specified!']];
        }
        if ($entity == 'cat' || $entity == 'category') {
            $name = [
                'entity' => 'category',
                'entities' => 'categories',
                'id_entity' => 'id_category',
                'isEmpty' => 'MxbToolsUtils::isCategoryEmpty',
            ];
        } elseif ($entity == 'man' || $entity == 'manufacturer') {
            $name = [
                'entity' => 'manufacturer',
                'entities' => 'manufacturers',
                'id_entity' => 'id_manufacturer',
                'isEmpty' => 'MxbToolsUtils::isManufacturerEmpty',
            ];
        } else {
            return ['error' => ['ERROR: wrong entity (' . $entity . ')!']];
        }

        $sql = '
            SELECT ' . $name['id_entity'] . '
            FROM ' . _DB_PREFIX_ . $name['entity'] . '
            WHERE active=' . (int) !$enable;
        $element_array = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        $disabled_cnt = 0;
        $element_cnt = 0;

        $actionText = ($enable ? 'enable non empty' : 'disable empty');
        $success = [];
        $error = [];

        if ($element_array) {
            $success[] = $actionText . ' ' . $name['entities'] . ': ' . count($element_array) . ' ' . ($enable ? 'disabled' : 'enabled') . ' ' . $name['entities'] . ' collected<br>';

            foreach ($element_array as $el) {
                $empty = $name['isEmpty']($el[$name['id_entity']]);
                if (($enable && !$empty) || (!$enable && $empty)) {
                    $sql = '
                        UPDATE ' . _DB_PREFIX_ . $name['entity'] . '
                        SET active=' . (int) $enable . '
                        WHERE ' . $name['id_entity'] . '=' . (int) $el[$name['id_entity']];
                    $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->execute($sql);

                    ++$disabled_cnt;
                    $success[] = $actionText . ' ' . $name['entity'] . ' id: ' . (int) $el[$name['id_entity']] . '<br>';
                }
                ++$element_cnt;
            }
            if ($disabled_cnt === 0) {
                $error[] = 'No ' . $name['entity'] . ' needs to be ' . ($enable ? 'enabled' : 'disabled') . '<br>';
            }
        } else {
            $error[] = 'No ' . ($enable ? 'disabled' : 'enabled') . ' ' . $name['entities'] . 'found<br>';
        }
        if ($disabled_cnt > 0) {
            $success[] = '<br>' . $name['entities'] . ' ' . ($enable ? 'enabled' : 'disabled') . ': ' . $disabled_cnt . '<br>';
        }

        $ret = [];
        if (count($error)) {
            $ret['error'] = $error;
        }
        if (count($success)) {
            $ret['success'] = $success;
        }

        return $ret;
    }
}
