<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
/*
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
*/

require_once _PS_MODULE_DIR_ . 'mxb_tools/mxb_tools.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class Mxb_ToolsAjaxModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        // MxbToolsDebug::writeLog('Gadgetcard', 'GadgetcardAjaxModuleFrontController::initContent', 'START - action:'.Tools::getValue('action'));
        $module = new mxb_tools();

        $return = [
            'status' => 1,
            'action' => '',
            'success' => '',
            'error' => '',
        ];

        // You may should do some security work here, like checking an hash from your module
        if (Tools::isSubmit('action')) {
            $return['action'] = Tools::getValue('action');

            // Usefull vars derivated from getContext
            $context = Context::getContext();
            $cart = $context->cart;
            $cookie = $context->cookie;
            $customer = $context->customer;
            $id_lang = $cookie->id_lang;

            // Default response with translation from the module
            $response = ['status' => false, 'message' => $module->l('Nothing here.')];

            switch (Tools::getValue('action')) {
                case 'emptyCart':
                    try {
                        // collect cart products in array by reference
                        $products = $cart->getProducts(true);

                        MxbToolsDebug::WriteLog(
                            'mxb_tools',
                            'ajax::emptyCart',
                            'id_cart=' . $cart->id
                        );

                        foreach ($products as $product) {
                            if (!$cart->deleteProduct((int) $product['id_product'], (int) $product['id_product_attribute'], (int) $product['id_customization'])) {
                                // error deleting product
                                $return['error'] .= (empty($return['error']) ? 'error deleting product(s) from cart [id_product-id_product_attribute]: ' : '')
                                    . '[' . (int) $product['id_product'] . '-' . (int) $product['id_product_attribute'] . '] ';
                            } else {
                                $return['error'] .= (empty($return['error']) ? 'product(s) deleted from cart [id_product-id_product_attribute]: ' : '')
                                    . '[' . (int) $product['id_product'] . '-' . (int) $product['id_product_attribute'] . '] ';
                            }
                        }

                        $return['success'] .= 'cart is now empty';
                    } catch (Exception $e) { // try
                        $return['error'] .= ' ' . $e->getMessage();
                        MxbToolsDebug::WriteLog('mxb_tools', 'ajax::emptyCart', 'exception: ' . $e->getMessage());
                        MxbToolsDebug::WriteLog('mxb_tools', 'ajax::emptyCart', 'exception: ' . $e->getMessage(), true, 'error.log');
                    }
                    // Tools::redirect($this->context->link->getPageLink('cart'));

                    break;

                default:
                    break;
            }
        }

        if (empty($return['success'])) {
            $return['success'] = 'nothing to do for action ' . $return['action'];
        }

        MxbToolsDebug::WriteLog(
            'mxb_tools',
            'ajax::emptyCart',
            'return=' . print_r($return, true)
        );

        ob_end_clean();
        header('Content-Type: application/json');
        exit(json_encode($return));
    }
}
