<?php
/**
 * 2018-2023 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2018-2023 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class Mxb_Tools extends Module
{
    const MODULE_NAME = 'mxb_tools';
    const ROOT_CONTROLLER = 'AdminMxbTools';
    public $documentationUrl = 'https://www.mxbinformatica.it/prestashop-mxb-tools/';
    public $thumbnailUrl = 'https://www.mxbinformatica.it/ps_modules/thumbs/mxb_tools.jpg';
    /*
    the following version variable is for development, used do update databases and other stuff
      without reinstalling the module.
    Please check checkForUpdates() function
    use int value !
    */
    const MXB_DEV_VERSION = 6;

    protected $config_form = false;
    protected $tabs = [];

    public $dev_version = 0;
    public $customCssFolder = null;
    public $customJsFolder = null;
    public $rootController = null;
    
    public $cache_update_additional_cost = false;
    public $cache_saving_dress_size = false;

    // TRANSLATIONS
    public static $l = [];

    // use static array for translations from static classes where is:
    //    $module = Module::getInstanceByName('mxb_tools');
    //    $module->l(string)
    public function isUsingNewTranslationSystem()
    {
        // force using old translation system: wait for 1.7.8 to use new translation,
        // when module translations will be exportable.
        return false;
    }

    public function initTranslation()
    {
        if (self::isInstalled($this->name)) {
            self::$l = [
                // MxbToolsExport.php
                'Your discount' => $this->l('Your discount'),
                'DIS' => $this->l('DIS'), // discount, abbrev
                'Shipping cost' => $this->l('Shipping cost'),

                // riferimento ord / fatt / bolla
                'Ref.' => $this->l('Ref.'),
                'of' => $this->l('of'),         // [number] "of" [date]
                'order ' => $this->l('order'),  // order, abbrev
                'DS' => $this->l('DS'),         // delivery slip, abbrev
                'inv' => $this->l('inv'),       // invoice, abbrev

                'Please wait untill the end of the export loop' => $this->l('Please wait untill the end of the export loop'),
                'exported products' => $this->l('exported products'),
                'file is ready for download! Click here' => $this->l('file is ready for download! Click here'),
                'No products available for given parameters. Please check customer and category IDs' => $this->l('No products available for given parameters. Please check customer and category IDs'),

                // export orders
                'ordered products list' => $this->l('ordered products list'),
                'order list' => $this->l('order list'),
                'orders total per customer group' => $this->l('orders total per customer group'),
                'Date' => $this->l('Date'),
                'Reference' => $this->l('Reference'),
                'Order state' => $this->l('Order state'),
                'Total paid tax excl' => $this->l('Total paid tax excl'),
                'Total paid tax incl' => $this->l('Total paid tax incl'),
                'Shipping tax excl' => $this->l('Shipping tax excl'),
                'Shipping tax %' => $this->l('Shipping tax %'),
                'Payment' => $this->l('Payment'),
                'Customer company' => $this->l('Customer company'),
                'Address' => $this->l('Address'),
                'Country' => $this->l('Country'),
                'VAT number' => $this->l('VAT number'),
                'Siret' => $this->l('Siret'),
                'Phone' => $this->l('Phone'),
                'E-mail' => $this->l('E-mail'),
                'Pec' => $this->l('Pec'),
                'Sdi' => $this->l('Sdi'),
                'Delivery to' => $this->l('Delivery to'),
                'Product reference' => $this->l('Product reference'),
                'Product name' => $this->l('Product name'),
                'Quantity' => $this->l('Quantity'),
                'Unit' => $this->l('Unit'),
                'Price tax excl' => $this->l('Price tax excl'),
                'Discount %' => $this->l('Discount %'),
                'Tax %' => $this->l('Tax %'),
                'Total tax excl' => $this->l('Total tax excl'),
                'Total tax incl' => $this->l('Total tax incl'),
                'Full name' => $this->l('Full name'),
                'Category' => $this->l('Category'),
                'Unit price' => $this->l('Unit price'),
                'Total price' => $this->l('Total price'),

                // orders_by_group
                'Total discount' => $this->l('Total discount'),
                'Total discount on products' => $this->l('Total discount on products'),
                'Total discount on shipping' => $this->l('Total discount on shipping'),
                'Total order tax excl' => $this->l('Total order tax excl'),
                'Total products' => $this->l('Total products'),
                'Total shipping tax excl' => $this->l('Total shipping tax excl'),
                'Total wrapping tax excl' => $this->l('Total wrapping tax excl'),

                // export customers
                'customer list' => $this->l('customer list'),
                'My main address' => $this->l('My main address'),
                'ID customer' => $this->l('ID customer'),
                'Customer code' => $this->l('Customer code'),
                'Firstname' => $this->l('Firstname'),
                'Lastname' => $this->l('Lastname'),

                'Privacy check' => $this->l('Privacy check'),
                'Is active' => $this->l('Is active'),
                'Since' => $this->l('Since'),
                'User group' => $this->l('User group'),
                'Newsletter' => $this->l('Newsletter'),
                'Website' => $this->l('Website'),
                'Birthday' => $this->l('Birthday'),
                'Note' => $this->l('Note'),
                'Partner special offer' => $this->l('Partner special offer'),
                'Postcode' => $this->l('Postcode'),
                'City' => $this->l('City'),
                'State' => $this->l('State'),

                'Remaining amount' => $this->l('Remaining amount'),
                'Amount spent' => $this->l('Amount spent'),
                'Missing amount' => $this->l('Missing amount'),
                'Orders' => $this->l('Orders'),
                'Vouchers' => $this->l('Vouchers'),
                'available' => $this->l('available'),
                'used' => $this->l('used'),
                'disabled' => $this->l('disabled'),

                // export vouchers
                'voucher list' => $this->l('voucher list'),
                'ID voucher' => $this->l('ID voucher'),
                'date_add' => $this->l('date_add'),
                'date_upd' => $this->l('date_upd'),
                'description' => $this->l('description'),
                'code' => $this->l('code'),
                'date_from' => $this->l('date_from'),
                'date_to' => $this->l('date_to'),
                'quantity' => $this->l('quantity'),
                'quantity_per_user' => $this->l('quantity_per_user'),
                'priority' => $this->l('priority'),
                'partial_use' => $this->l('partial_use'),
                'minimum_amount' => $this->l('minimum_amount'),
                'minimum_amount_tax' => $this->l('minimum_amount_tax'),
                'minimum_amount_currency' => $this->l('minimum_amount_currency'),
                'minimum_amount_shipping' => $this->l('minimum_amount_shipping'),
                'country_restriction' => $this->l('country_restriction'),
                'carrier_restriction' => $this->l('carrier_restriction'),
                'group_restriction' => $this->l('group_restriction'),
                'cart_rule_restriction' => $this->l('cart_rule_restriction'),
                'product_restriction' => $this->l('product_restriction'),
                'shop_restriction' => $this->l('shop_restriction'),
                'free_shipping' => $this->l('free_shipping'),
                'highlight' => $this->l('highlight'),
                'reduction_percent' => $this->l('reduction_percent'),
                'reduction_amount' => $this->l('reduction_amount'),
                'reduction_tax' => $this->l('reduction_tax'),
                'reduction_currency' => $this->l('reduction_currency'),
                'reduction_product' => $this->l('reduction_product'),
                'reduction_exclude_special' => $this->l('reduction_exclude_special'),
                'gift_product' => $this->l('gift_product'),
                'gift_product_attribute' => $this->l('gift_product_attribute'),
                'highlight' => $this->l('highlight'),
                'active' => $this->l('active'),
                'status' => $this->l('status'),
                'amount' => $this->l('amount'),
                'groups' => $this->l('groups'),
                'categories' => $this->l('categories'),
                'name' => $this->l('name'),

                // export products
                'product list' => $this->l('product list'),
                'all categories' => $this->l('all categories'),
                'qty' => $this->l('qty'),
                'col' => $this->l('col'),
                'price' => $this->l('price'),
                'public price' => $this->l('public price'),
                'wholesale price' => $this->l('wholesale price'),
                'charge' => $this->l('charge'),
                'discount' => $this->l('discount'),
                'category' => $this->l('category'),
                'model' => $this->l('model'),
                'reference' => $this->l('reference'),
                // 'name' => $this->l('name'),
                'color' => $this->l('color'),
                'size' => $this->l('size'),
                'other' => $this->l('other'),
                'material' => $this->l('material'),
                'brand' => $this->l('brand'),
                'ean 13' => $this->l('ean 13'),
                'segment' => $this->l('segment'),
                'for sale' => $this->l('for sale'),
                'stock qty' => $this->l('stock qty'),
                'short description' => $this->l('short description'),
                // 'description' => $this->l('description'),
                'tags' => $this->l('tags'),
                'wholesale' => $this->l('wholesale'),
                'gross' => $this->l('gross'),
                'order id' => $this->l('order id'),

                // coverjob PDF: address-tab
                'Delivery date' => $this->l('Delivery date'),
                'Event date' => $this->l('Event date'),
                'Program date' => $this->l('Program date'),
                'Job' => $this->l('Job'),
                'Job date' => $this->l('Job date'),
                'Job name' => $this->l('Job name'),
                'DDT number' => $this->l('DDT number'),
                'Invoice number' => $this->l('Invoice number'),
                'Customer' => $this->l('Customer'),
                'Order n.' => $this->l('Order n.'),

                // coverjob PDF: add-data-tab
                'Weight' => $this->l('Weight'),
                'Volume' => $this->l('Volume'),
                'Package q.ty' => $this->l('Package q.ty'),
                'Pallets' => $this->l('Pallets'),
                'Aesthetic check' => $this->l('Aesthetic check'),
                'Functionality check' => $this->l('Functionality check'),
                'Print tightness control' => $this->l('Print tightness control'),
                'Print type' => $this->l('Print type'),
                'Embroidery' => $this->l('Embroidery'),
                'Frameworks' => $this->l('Frameworks'),
                'Unpacking' => $this->l('Unpacking'),
                'Packaging' => $this->l('Packaging'),
                'Subpacking' => $this->l('Subpacking'),
                'Packing' => $this->l('Packing'),
                'Label' => $this->l('Label'),
                'Scotch tape' => $this->l('Scotch tape'),
                'Transport' => $this->l('Transport'),
                'Courier' => $this->l('Courier'),
                'Shipping times' => $this->l('Shipping times'),

                // Product PDF: product.tpl
                'Variants' => $this->l('Variants'),
                'Product sheet' => $this->l('Product sheet'),

                // mxb_tools menu (MxbToolsMenu.php)
                'Configuration' => $this->l('Configuration'),
                'Tools' => $this->l('Tools'),
                'Calculators' => $this->l('Calculators'),
                'Import' => $this->l('Import'),
                'Export' => $this->l('Export'),
                'Modules' => $this->l('Modules'),
                'Common modules' => $this->l('Common modules'),
            ];
        }
    }

    public static function configVaues()
    {
        return [
            ['name' => 'MXB_TOOLS_VERSION',                            'type' => 'int',    'default' => 0,       'lang' => false],
            ['name' => 'MXB_TOOLS_DEBUG',                              'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_ENABLE_GLOBAL_CUSTOM_CSS',           'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_GLOBAL_CUSTOM_CSS',                  'type' => 'string', 'default' => '',      'lang' => false],
            ['name' => 'MXB_TOOLS_PRICE_NUMBER_OF_DIGITS',             'type' => 'number', 'default' => 2,       'lang' => false],
            ['name' => 'MXB_TOOLS_ENABLE_GLOBAL_CUSTOM_JS',            'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_GLOBAL_CUSTOM_JS',                   'type' => 'string', 'default' => '',      'lang' => false],
            // ['name' => 'MXB_TOOLS_GLOBAL_CUSTOM_FOOTER_JS',            'type' => 'string', 'default' => '',      'lang' => false],
            ['name' => 'MXB_TOOLS_COLOR_TEXTURES_REMOTE_ADDRESS',      'type' => 'string', 'default' => '',      'lang' => false],
            ['name' => 'MXB_TOOLS_EXCLUDE_DISCOUNT_FOR_FREE_SHIPPING', 'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_SCRIPTS_ALLOWED_IP_ADDRESS',         'type' => 'string', 'default' => '',      'lang' => false],

            // send registration email to admin
            ['name' => 'MXB_TOOLS_NEW_CUSTOMER_EMAIL_ADDRESS',         'type' => 'string', 'default' => '',      'lang' => false],
            ['name' => 'MXB_TOOLS_BLOCK_EMAIL_ORDER_NEW_VOUCHER',      'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_REMOVE_SHOP_NAME_FROM_EMAIL_SUBJECT', 'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_REMOVE_ORDER_STATUS_NUMBERING',      'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_BLOCK_EMAIL_ORDER_TRACKING_ADDED',   'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_CUSTOMER_REDIRECT_AFTER_LOGIN',      'type' => 'bool',   'default' => false,   'lang' => false],

            ['name' => 'MXB_TOOLS_PAGINATION_LIST_ENABLED',            'type' => 'bool',   'default' => false,   'lang' => false],

            // wait layer
            ['name' => 'MXB_TOOLS_ENABLE_WAIT_LAYER',                  'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_WAIT_LAYER_USE_LOGO',                'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_WAIT_LAYER_USE_PRODUCT_IMAGE',       'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_WAIT_LAYER_ROTATE_LOGO',             'type' => 'bool',   'default' => false,   'lang' => false],
            ['name' => 'MXB_TOOLS_WAIT_LAYER_BG_COLOR',                'type' => 'string', 'default' => '#FFFFFF', 'lang' => false],
            ['name' => 'MXB_TOOLS_WAIT_LAYER_OPACITY',                 'type' => 'number', 'default' => 1,       'lang' => false],

            // PDF product sheet
            ['name' => 'MXB_TOOLS_ENABLE_PRODUCT_PDF',                 'type' => 'bool',   'default' => false,  'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_PDF_SHOW_PRICES',            'type' => 'bool',   'default' => false,  'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_PDF_EXCLUDE_GROUP_IDS',      'type' => 'string', 'default' => '',     'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_PDF_BTN_TEXT',               'type' => 'string', 'default' => '',     'lang' => true],

            // coverjob PDF
            ['name' => 'MXB_TOOLS_COVERJOB_LOGO',                      'type' => 'string',  'default' => '',    'lang' => false],
            ['name' => 'MXB_TOOLS_COVERJOB_TYPE',                      'type' => 'bool',    'default' => false, 'lang' => false],

            // cart
            ['name' => 'MXB_TOOLS_ENABLE_EMPTY_CART',                  'type' => 'bool',    'default' => false, 'lang' => false],

            /*
             * MXB_TOOLS_DELIVERY_SLIP_ADVANCED is disabled
             * because pdf templates are stored in theme folder and this module avoid theme and overrides.
             * Also AdminPdfController.php has no hooks for delivery slip !
             */
            ['name' => 'MXB_TOOLS_DELIVERY_SLIP_ADVANCED',             'type' => 'bool',    'default' => false, 'lang' => false],

            ['name' => 'MXB_TOOLS_GOOGLE_TAG_MANAGER_CODE',            'type' => 'string',  'default' => '',    'lang' => false],
            ['name' => 'MXB_TOOLS_GOOGLE_ANALYTICS_CODE',              'type' => 'string',  'default' => '',    'lang' => false],
            ['name' => 'MXB_TOOLS_REDIRECT_404_URL',                   'type' => 'string',  'default' => '',    'lang' => false],
            ['name' => 'MXB_TOOLS_REDIRECT_SEARCH_NOT_FOUND_URL',      'type' => 'string',  'default' => '',    'lang' => false],
            ['name' => 'MXB_TOOLS_REDIRECT_ADD_TO_CART_URL',           'type' => 'string',  'default' => '',    'lang' => false],
        ];
    }

    public static function toolsVaues()
    {
        return [
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_ID_CUSTOMER',                'type' => 'int',     'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_QUANTITY_COLUMNS_MIN',       'type' => 'int',     'default' => 4,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_PRODUCTS_LOOP',              'type' => 'int',     'default' => 500,     'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_ZERO_PRICE',                 'type' => 'bool',    'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_CATEGORY_ID',                'type' => 'int',     'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_CATEGORY',         'type' => 'bool',    'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_STOCK_QUANTITY',   'type' => 'bool',    'default' => 1,       'lang' => false],

            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_PRODUCT_PRICE',     'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_WHOLESALE_PRICE',   'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_GROSS_PRICE',       'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_DESCRIPTION',       'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_DESCRIPTION_SHORT', 'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_SHOW_TAGS',              'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_DISCOUNT',               'type' => 'bool',   'default' => 1,       'lang' => false],
            ['name' => 'MXB_TOOLS_PRODUCT_LIST_FLAG_PRICE_CHARGE',           'type' => 'bool',   'default' => 1,       'lang' => false],

            ['name' => 'MXB_TOOLS_MATERIAL_PRODUCT_TAG',                     'type' => 'string', 'default' => 'Material',  'lang' => true],
            ['name' => 'MXB_TOOLS_COLOR_PRODUCT_TAG',                        'type' => 'string', 'default' => 'Color',    'lang' => true],
            ['name' => 'MXB_TOOLS_SIZE_PRODUCT_TAG',                         'type' => 'string', 'default' => 'Size',    'lang' => true],

            ['name' => 'MXB_TOOLS_SEGMENT_PRODUCT_TAG',                      'type' => 'string', 'default' => 'Segment',    'lang' => true],
            ['name' => 'MXB_TOOLS_DRESS_SIZE_LIST',                          'type' => 'string', 'default' => '3XS;XXXS;XXS;XS;S;M;L;XL;XXL;XXXL;3XL;4XL;5XL;4XLP;5XLP;116;128;104;140;152;164',      'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_DATE_FROM',           'type' => 'string', 'default' => MxbToolsUtils::getDateString('2020-01-01'),  'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_DATE_TO',             'type' => 'string', 'default' => MxbToolsUtils::getDateString('2030-01-01'),  'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_FROM',             'type' => 'int',    'default' => 0,        'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_TO',               'type' => 'int',    'default' => 0,        'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_FORMAT',       'type' => 'string', 'default' => 'csv',    'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_TYPE',         'type' => 'string', 'default' => 'orders', 'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE',      'type' => 'int',    'default' => 0,        'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_NEXT', 'type' => 'int',    'default' => 0,        'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST', 'type' => 'string', 'default' => '0',      'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ID_ORDER_STATE_LIST_NOT', 'type' => 'string', 'default' => '0',  'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_PREFIX_FROM',   'type' => 'int',    'default' => '',       'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_PREFIX_TO',     'type' => 'int',    'default' => '',       'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_FORCE_NUMERIC', 'type' => 'bool',   'default' => false,    'lang' => false],

            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_INVOICE',      'type' => 'bool',   'default' => false,    'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_DELIVERY_SLIP', 'type' => 'bool',  'default' => false,    'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_EXPORT_PRODUCT_DISCOUNT', 'type' => 'bool', 'default' => false,  'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_ORDER_NUMBERING',     'type' => 'string', 'default' => '',       'lang' => false],
            ['name' => 'MXB_TOOLS_ORDERED_PRODUCT_LIST_CUSTOMER_PREFIX',     'type' => 'string', 'default' => '',       'lang' => false],

            ['name' => 'MXB_TOOLS_CUSTOMER_LIST_DATE_FROM',                  'type' => 'string', 'default' => MxbToolsUtils::getDateString('2020-01-01'), 'lang' => false],
            ['name' => 'MXB_TOOLS_CUSTOMER_LIST_DATE_TO',                    'type' => 'string', 'default' => MxbToolsUtils::getDateString('2030-01-01'), 'lang' => false],
            ['name' => 'MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST',        'type' => 'string', 'default' => '0', 'lang' => false],
            ['name' => 'MXB_TOOLS_CUSTOMER_LIST_ID_ORDER_STATE_LIST_NOT',    'type' => 'string', 'default' => '0', 'lang' => false],

            ['name' => 'MXB_TOOLS_CHANGE_STATUS_CART_RULES_START_ID',        'type' => 'number', 'default' => 0,   'lang' => false],
            ['name' => 'MXB_TOOLS_CHANGE_STATUS_CART_RULES_END_ID',          'type' => 'number', 'default' => 0,   'lang' => false],
            ['name' => 'MXB_TOOLS_CHANGE_STATUS_CART_RULES_TARGET_STATUS',   'type' => 'number', 'default' => 1,   'lang' => false],
            ['name' => 'MXB_TOOLS_CHANGE_STATUS_CART_RULES_DELETE_FROM_CART', 'type' => 'bool',   'default' => false, 'lang' => false],
            ['name' => 'MXB_TOOLS_CHANGE_STATUS_CART_RULES_CHECK_DEADLINE',  'type' => 'bool',   'default' => false, 'lang' => false],

            // create cart rules
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CODE', 'default' => '', 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_NAME', 'default' => '', 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_DESCRIPTION', 'default' => '', 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CODE_RND', 'default' => true, 'type' => 'bool', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CODE_LEN', 'default' => 12, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_QUANTITY_TO_CREATE', 'default' => 1, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_QUANTITY', 'default' => 1, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_QUANTITY_PER_USER', 'default' => 1, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_PRIORITY', 'default' => 1, 'type' => 'number', 'lang' => false],

            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CUSTOMER_ID', 'default' => '', 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_GROUP_IDS', 'default' => '', 'type' => 'string', 'lang' => false],

            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CATEGORY_IDS', 'default' => '', 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_PARTIAL_USE', 'default' => true, 'type' => 'bool', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_FREE_SHIPPING', 'default' => true, 'type' => 'bool', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_HIGHLIGHT', 'default' => true, 'type' => 'bool', 'lang' => false],

            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_REDUCTION_PERCENT', 'default' => 0, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_REDUCTION_AMOUNT', 'default' => 0, 'type' => 'number', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_REDUCTION_TAX', 'default' => true, 'type' => 'bool', 'lang' => false],

            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_DATE_FROM', 'default' => MxbToolsUtils::getDateString('2020-01-01 01:00:00'), 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_DATE_TO', 'default' => MxbToolsUtils::getDateString('2030-01-01 23:59:59'), 'type' => 'string', 'lang' => false],

            ['name' => 'MXB_TOOLS_GENERATE_PDF_ORDER_ID', 'default' => 0, 'type' => 'int',   'lang' => false],
            // ['name' => 'MXB_TOOLS_CREATE_CART_RULE_UNIQUE_FOR_CUSTOMER', 'default' => false, 'type' => 'bool',  'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CHECK_ACTIVE', 'default' => false, 'type' => 'bool', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CHECK_IN_STOCK', 'default' => false, 'type' => 'bool', 'lang' => false],
            ['name' => 'MXB_TOOLS_CREATE_CART_RULE_CHECK_INCLUDE_GENERIC', 'default' => false, 'type' => 'bool', 'lang' => false],

            ['name' => 'MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO', 'default' => Context::getContext()->language->iso_code, 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO', 'default' => Context::getContext()->language->iso_code, 'type' => 'string', 'lang' => false],
            ['name' => 'MXB_TOOLS_IMPORT_COLORS_ID_ATTRIBUTE_GROUP', 'default' => 0, 'type' => 'int', 'lang' => false],
        ];
    }

    public function __construct()
    {
        $this->name = 'mxb_tools';
        $this->tab = 'administration';
        $this->dev_version = self::MXB_DEV_VERSION;
        $this->version = '1.0.6';
        $this->author = 'MXB Informatica';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = ['min' => '1.6', 'max' => _PS_VERSION_];
        /* Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6) */
        $this->bootstrap = true;
        $this->customCssFolder = 'customcss';
        $this->customJsFolder = 'customjs';

        parent::__construct();

        $this->displayName = $this->l('MXB Tools');
        $this->description = $this->l('Adds some usefull tools');
        $this->rootController = self::ROOT_CONTROLLER;

        $this->tabs = [
            [
                'name' => 'MXB Tools',
                'position' => 10,
                'class_name' => $this->rootController,
                'visible' => true,
                'parent_class_name' => 'DEFAULT_MTR',
            ],
            [
                'name' => $this->l('Configuration'),
                'position' => 20,
                'class_name' => $this->rootController . 'Configuration',
                'visible' => true,
                'parent_class_name' => $this->rootController,
            ],
            [
                'name' => $this->l('Tools'),
                'position' => 30,
                'class_name' => $this->rootController . 'Tools',
                'visible' => true,
                'parent_class_name' => $this->rootController,
            ],
            [
                'name' => $this->l('Calculators'),
                'position' => 40,
                'class_name' => $this->rootController . 'Calculators',
                'visible' => true,
                'parent_class_name' => $this->rootController,
            ],
            [
                'name' => $this->l('Import tools'),
                'position' => 50,
                'class_name' => $this->rootController . 'Import',
                'visible' => true,
                'parent_class_name' => $this->rootController,
            ],
            [
                'name' => $this->l('Export tools'),
                'position' => 60,
                'class_name' => $this->rootController . 'Export',
                'visible' => true,
                'parent_class_name' => $this->rootController,
            ],
            // invisible tabs
            [
                'name' => $this->l('Manage combination attributes'),
                'position' => 110,
                'class_name' => $this->rootController . 'ManageCombAttributes',
                'visible' => false,
                'parent_class_name' => $this->rootController,
            ],
            [
                'name' => $this->l('Manage attributes'),
                'position' => 120,
                'class_name' => $this->rootController . 'ManageAttributes',
                'visible' => false,
                'parent_class_name' => $this->rootController,
            ],
        ];
        $this->checkForUpdates();

        // check configuration values and set the not existing ones
        $this->setModuleValues();

        // DEBUG: remove for production!
        // $this->createAdminTabs();
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     *
     * LIST OF HOOKS PS1.7
     * http://build.prestashop.com/news/new-updated-hooks-1-7-1-0/
     */
    public function install()
    {
        include dirname(__FILE__) . '/sql/install.php';

        $res = parent::install();
        $this->createAdminTabs();
        $this->setModuleValues();
        $this->installModuleFiles();

        // register hooks
        $res &= $this->registerHook('header');
        // $res &= $this->registerHook('footer');
        // $res &= $this->registerHook('backOfficeHeader');
        $res &= $this->registerHook('displayAdminAfterHeader');
        $res &= $this->registerHook('displayTop');

        // register javascript
        $res &= $this->registerHook('actionAdminControllerSetMedia');
        $res &= $this->registerHook('actionFrontControllerSetMedia');

        // manage emails on new account event or filter it for new voucher and so on
        $res &= $this->registerHook('actionMailAlterMessageBeforeSend');
        $res &= $this->registerHook('actionCustomerAccountAdd');
        $res &= $this->registerHook('sendMailAlterTemplateVars');
        $res &= $this->registerHook('actionEmailSendBefore');

        // redirect to index
        $res &= $this->registerHook('actionAuthentication');

        // display "no search results" and "page 404 not found"
        $res &= $this->registerHook('displayNotFound');

        // for google analytics and tag manager
        $res &= $this->registerHook('displayAfterBodyOpeningTag');

        // manipulate cms content
        $res &= $this->registerHook('filterCmsCategoryContent');

        return (bool) $res;
    }

    public function uninstall()
    {
        $this->unsetModuleValues();
        include dirname(__FILE__) . '/sql/uninstall.php';

        // uninstall tabs for version <= 1.7.0.0
        if (!MxbToolsUtils::checkTabDontInstall()) {
            foreach ($this->tabs as $tab) {
                MxbToolsUtils::uninstallTab($tab['class_name']);
            }
        }

        return parent::uninstall();
    }

    protected function setModuleValues()
    {
        foreach (Mxb_Tools::configVaues() as $value) {
            if (Configuration::hasKey($value['name']) === false) {
                Configuration::updateGlobalValue(
                    $value['name'],
                    $this->setValueType($value['default'], $value['type'])
                );
            }
        }
        foreach (Mxb_Tools::toolsVaues() as $value) {
            if (Configuration::hasKey($value['name']) === false) {
                Configuration::updateGlobalValue(
                    $value['name'],
                    $this->setValueType($value['default'], $value['type'])
                );
            }
        }
    }

    protected function unsetModuleValues()
    {
        // if (true) return; // TODO: production version: remove this code
        foreach (Mxb_Tools::configVaues() as $value) {
            Configuration::deleteByName($value['name']);
        }
        foreach (Mxb_Tools::toolsVaues() as $value) {
            Configuration::deleteByName($value['name']);
        }
    }

    protected function setValueType($value, $type = null)
    {
        if ($type == 'string') {
            return $value;
        } elseif ($type == 'int') {
            return (int) $value;
        } elseif ($type == 'bool') {
            return (int) $value;
        }

        return $value;
    }

    protected function installModuleFiles()
    {
        // $this->installCustomFile('views'.DIRECTORY_SEPARATOR.'css'.DIRECTORY_SEPARATOR, 'product.css');

        return true;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        return Tools::redirectAdmin($this->context->link->getAdminLink($this->rootController . 'Configuration'));
    }

    // create tabs for version <= 1.7.0.0
    public function createAdminTabs()
    {
        if (MxbToolsUtils::checkTabDontInstall()) {
            return true;
        }

        foreach ($this->tabs as $tab) {
            if ($tab['visible']) {
                // ensure compatibility with version < 1.7
                $pcl = $tab['parent_class_name'];
                MxbToolsUtils::installTab($tab['class_name'], $tab['name'], $pcl, $tab['position']);
            } else {
                MxbToolsUtils::installTab($tab['class_name'], $tab['name'], -1, $tab['position']);
            }
        }

        return true;
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    /* instead, hookActionAdminControllerSetMedia() is used
        public function hookBackOfficeHeader()
        {
            if (Tools::getValue('module_name') == $this->name) {
              $this->context->controller->addJS($this->_path . 'views/js/back.js');
              $this->context->controller->addCSS($this->_path . 'views/css/back.css');
            }
        }
    */

    /*
    public function hookFooter($params) {}
    public function hookDisplayFooter($params) {}
    */
   

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . 'views/js/mxb_tools.js');

        // $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');

        /*
        if ((bool) Configuration::getGlobalValue('MXB_TOOLS_ENABLE_WAIT_LAYER')) {
            $this->context->controller->addJS($this->_path . '/views/js/wait_layer.js');
        }
        if ((bool) Configuration::get('MXB_TOOLS_ENABLE_GLOBAL_CUSTOM_JS')) {
            $this->context->controller->addJS($this->_path . '/views/' . $this->customJsFolder . '/custom.js');
        }
        */
        if ((bool) Configuration::get('MXB_TOOLS_ENABLE_GLOBAL_CUSTOM_CSS')) {
            $this->context->controller->addCSS($this->_path . '/views/' . $this->customCssFolder . '/global.css');
        }
        $this->context->controller->addCSS($this->_path . '/views/css/mxb_tools.css');
        $this->context->controller->addCSS($this->_path . '/views/css/cms.css');

        // google tags
        $analytics = Configuration::getGlobalValue('MXB_TOOLS_GOOGLE_ANALYTICS_CODE');
        $tagManager = Configuration::getGlobalValue('MXB_TOOLS_GOOGLE_TAG_MANAGER_CODE');
        $tagsArray = [];
        if (!empty($analytics)) {
            $tagsArray['tag_analytics'] = $analytics;
        }
        if (!empty($tagManager)) {
            $tagsArray['tag_manager'] = $tagManager;
        }

        if (count($tagsArray)) {
            $this->context->smarty->assign('mxbtGoogleTags', $tagsArray);
            return $this->display(__FILE__, 'front/mxb_tools-header-tags.tpl');
        }
    }

    public function hookActionAdminControllerSetMedia()
    {
        /*
        MxbToolsDebug::writeLog(
            $this->name,
            'hookActionAdminControllerSetMedia',
            'START'
        );
        */
        $this->context->controller->addJS($this->_path . 'views/js/mxb_tools.js');
        $this->context->controller->addJS($this->_path . 'views/js/back.js');
        $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        $this->context->controller->addJqueryUI('ui.datepicker');
    }

    // https://devdocs.prestashop.com/1.7/themes/getting-started/asset-management/
    public function hookActionFrontControllerSetMedia($params)
    {
        /*
        $this->context->controller->registerJavascript(
            'module-mxb_tools-js',
            $this->_path . '/views/js/mxb_tools.js',
            [
                'priority' => 200,
                'attribute' => 'async',
            ]
        );
        */
        /*
        // On every pages
        $this->context->controller->registerJavascript(
            'google-analytics',
            'modules/'.$this->name.'/ga.js',
            [
                'position' => 'head',
                'inline' => true,
                'priority' => 10,
            ]
        );
        */

        $this->context->controller->registerJavascript(
            'module-mxb_tools-js',
            $this->_path . '/views/js/mxb_tools.js',
            [
                'priority' => 200,
                'attribute' => 'async',
            ]
        );

        $this->context->controller->registerJavascript(
            'module-mxb_tools-frontjs',
            $this->_path . '/views/js/front.js',
            [
                'priority' => 200,
                'attribute' => 'async',
            ]
        );

        if ((bool) Configuration::get('MXB_TOOLS_ENABLE_WAIT_LAYER')) {
            $this->context->controller->registerJavascript(
                'module-mxb_tools-wait_layerjs',
                $this->_path . '/views/js/wait_layer.js',
                [
                    'priority' => 200,
                    'attribute' => 'async',
                ]
            );
        }

        if ((bool) Configuration::get('MXB_TOOLS_ENABLE_GLOBAL_CUSTOM_JS')) {
            $this->context->controller->registerJavascript(
                'module-mxb_tools-customjs',
                $this->_path . '/views/' . $this->customJsFolder . '/custom.js',
                [
                    'priority' => 200,
                    'attribute' => 'async',
                ]
            );
        }
        
    }

    /**
     * set usefull values into smarty to make it available for js
     * hidden input fields must be created in view/templates/hook/mxb_tools-header.tpl
     * to be available to JS
     * Located in:
     *  /themes/classic/templates/_partials/header.tpl
     *  /themes/classic/templates/checkout/_partials/header.tpl
     */
    public function hookDisplayTop()
    {
        return $this->setDefaultSmartyValuesAndTemplate(
            false,
            'front/mxb_tools-header.tpl',
            MxbToolsUtils::getShopValuesForSmarty()
        );
    }

    public function hookDisplayAdminAfterHeader()
    {
        return $this->setDefaultSmartyValuesAndTemplate(
            true,
            'admin/mxb_tools-admin-header.tpl',
            MxbToolsUtils::getShopValuesForSmarty()
        );
    }

    public function prepareArrayForSmarty($values)
    {
        foreach ($values as $key => &$cf) {
            if ($key == 'MXB_TOOLS_PRODUCT_PDF_EXCLUDE_GROUP_IDS' || $key == 'MXB_TOOLS_DRESS_SIZE_LIST') {
                $cf = MxbToolsUtils::idStringToSmartyString($cf);
            } elseif ($key == 'MXB_TOOLS_PRODUCT_PDF_BTN_TEXT' && is_array($cf)) {
                if (isset($cf[(int) $this->context->language->id])) {
                    $cf = $cf[(int) $this->context->language->id];
                } else {
                    // $cf = json_encode($cf);
                    $cf = reset($cf); // get first
                }
            } else {
                if ($cf === true) {
                    $cf = 1;
                } elseif ($cf === false) {
                    $cf = 0;
                } elseif (is_array($cf)) {
                    $cf = htmlentities(json_encode($cf));
                }
            }
        }
        /*
        MxbToolsDebug::writeLog(
            $this->name,
            'prepareArrayForSmarty',
            'values=' . print_r($values, true)
        );
        */
        return $values;
    }

    /**
     * push in smarty config values of the module and set a .tpl template
     */
    public function setDefaultSmartyValuesAndTemplate(
        $admin = false,
        $tpl = null,
        $add_values_array = null,
        $other_values_array = null
    ) {
        if (Tools::getValue('controller') == $this->rootController
            || Tools::getValue('controller') == $this->rootController . 'Configuration') {
            $config_values = [];
        } else {
            $config_values = MxbToolsUtils::getConfigValues($admin);
        }
        $messages = null;
        if (!$admin) {
            $messages = [
                'MXB_TOOLS_MSG_EMPTY_CART_BTN' => $this->l('Empty cart'),
                'MXB_TOOLS_MSG_EMPTY_CART_CONFIRM' => $this->l('Remove all products from cart ?'),
            ];
        }

        if ($messages) {
            $config_values = array_merge($config_values, $messages);
        }
        /*
        MxbToolsDebug::writeLog(
            $this->name,
            'setDefaultSmartyValuesAndTemplate',
            'config_values.  ' . print_r($config_values, true)
        );
        */

        if ($tpl !== null) {
            $config_values = $this->prepareArrayForSmarty($config_values);
            if ($add_values_array !== null) {
                $add_values_array = $this->prepareArrayForSmarty($add_values_array);
                $this->context->smarty->assign('mxbtConfigValues', array_merge($config_values, $add_values_array));
            } else {
                $this->context->smarty->assign('mxbtConfigValues', $config_values);
            }
            if (!empty($other_values_array)) {
                $other_values_array = $this->prepareArrayForSmarty($other_values_array);
                $this->context->smarty->assign('mxbtValues', $other_values_array);
            }

            return $this->display(__FILE__, $tpl);
        } else {
            return $config_values;
        }
    }

    public function addConfigValuesToSmarty($admin = false)
    {
        $config_values = $this->setDefaultSmartyValuesAndTemplate($admin);
        $this->context->smarty->assign('mxbtConfigValues', $config_values);
    }

    /**
     * edit outgoing emails to change subject
     * called by classes/Mail.php :: send()
     *  // Hook to alter Swift Message before sending mail
     *          Hook::exec('actionMailAlterMessageBeforeSend', [
     *              'message' => &$message,
     *          ]);
     *
     * $message class is in Prestashop\vendor\swiftmailer\swiftmailer\lib\classes\Swift\Mime\SimpleMessage.php !!!
     */
    public function hookActionMailAlterMessageBeforeSend(&$hookParams)
    {
        // if mxbmemberprogram module is installed, let it manage this hook
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            return;
        }
        $subject = $hookParams['message']->getSubject();
        $site = '';

        // remove website info from subject
        $pos = strpos($subject, '] ');
        if ($pos !== false && strpos($subject, '[') !== false) {
            $site = substr($subject, 0, $pos + 2);
            // remove site name from subject
            $subject = substr($subject, $pos + 2);
        }

        // remove enumerator from subject for "status" email ( "1) ....." )
        if (Configuration::getGlobalValue('MXB_TOOLS_REMOVE_ORDER_STATUS_NUMBERING') == 0) {
            $pos = strpos($subject, ')');
            if ($pos !== false && (int) $pos < 5) {
                $subject = trim(substr($subject, $pos + 1));
            }
        }

        // mantain the site info ?
        if (Configuration::getGlobalValue('MXB_TOOLS_REMOVE_SHOP_NAME_FROM_EMAIL_SUBJECT') == 0) {
            $subject = $site . $subject;
        }

        $hookParams['message']->setSubject($subject);
    }

    /**
     * new customer: send an email to dedicated account
     *
     * params: array('newCustomer' => (object) Customer object);
     *
     * $hookParams['newCustomer'] :
     *  [id] => 57
     *  [id_shop] => 1
     *  [id_shop_group] => 1
     *  [secure_key] => c63b2320b57a8acc19ce23abe2ca0546
     *  [note] =>
     *  [id_gender] => 1
     *  [id_default_group] => 3
     *  [id_lang] => 1
     *  [lastname] => cinque
     *  [firstname] => test
     *  [birthday] =>
     *  [email] => test5@rino-graficaestampa.it
     *  [newsletter] =>
     *  [ip_registration_newsletter] =>
     *  [newsletter_date_add] =>
     *  [optin] =>
     *  [website] =>
     *  [company] => aziendaTest5
     *  [siret] => ABCDEFG
     *  [ape] =>
     *  [outstanding_allow_amount] => 0
     *  [show_public_prices] => 0
     *  [id_risk] =>
     *  [max_payment_days] => 0
     *  [passwd] => $2y$10$72muqx1kVvh/Mru7X3WfzeaA3KuAkiu44L13l23a6cvkGc3/9YYxC
     *  [last_passwd_gen] => 2020-09-09 12:11:22
     *  [active] => 1
     *  [is_guest] => 0
     *  [deleted] => 0
     *  [date_add] => 2020-09-09 18:11:22
     *  [date_upd] => 2020-09-09 18:11:22
     *  [years] =>
     *  [days] =>
     *  [months] =>
     *  [geoloc_id_country] =>
     *  [geoloc_id_state] =>
     *  [geoloc_postcode] =>
     *  [logged] => 1
     *  [id_guest] =>
     *  [groupBox] =>
     *  [reset_password_token] =>
     *  [reset_password_validity] =>
     */
    public function hookActionCustomerAccountAdd($hookParams)
    {
        // if mxbmemberprogram module is installed, let it manage this hook
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            return;
        }

        // get new customer infos
        $customer = $hookParams['newCustomer'];
        $customer_data = [];
        if (isset($customer->siret)) {
            if (strlen((string) $customer->siret) > 0) {
                $customer_data['dni'] = $customer->siret;
            }
        }
        if (isset($customer->dni)) {
            if (strlen((string) $customer->dni) > 0) {
                $customer_data['dni'] = $customer->dni;
            }
        }

        // siret is used for fiscal code (16 char, db field modifyed by module installation)
        if (!isset($customer_data['dni'])) {
            if (Tools::getIsset('siret')) {
                $customer_data['dni'] = Tools::getValue('siret');
            } elseif (Tools::getIsset('dni')) {
                $customer_data['dni'] = Tools::getValue('dni');
            }
        }

        if (Tools::getIsset('vat_number')) {
            $customer_data['vat_number'] = Tools::getValue('vat_number');
        } elseif (isset($customer_data['dni'])) {
            $customer_data['vat_number'] = $customer_data['dni'];
        } else {
            $customer_data['vat_number'] = '';
        }

        // no recipient email address: don't send the email
        if (Configuration::getGlobalValue('MXB_TOOLS_NEW_CUSTOMER_EMAIL_ADDRESS') != '') {
            $rec_str = str_replace(' ', '', Configuration::getGlobalValue('MXB_TOOLS_NEW_CUSTOMER_EMAIL_ADDRESS'));

            // get all recipients
            $recipients = explode(',', str_replace(';', ',', $rec_str));

            // send a message per every recipient
            foreach ($recipients as $recipient) {
                // send an email for new registration
                /*
                   public static function send(
                        $idLang,
                        $template,
                        $subject,
                        $templateVars,
                        $to,
                        $toName = null,
                        $from = null,
                        $fromName = null,
                        $fileAttachment = null,
                        $mode_smtp = null,
                        $templatePath = _PS_MAIL_DIR_,
                        $die = false,
                        $idShop = null,
                        $bcc = null,
                        $replyTo = null,
                        $replyToName = null
                    )
                */
                if (Validate::isEmail($recipient)) {
                    Mail::Send(
                        (int) Configuration::get('PS_LANG_DEFAULT'), // defaut language id
                        'adm_new_customer_account', // email template file to be use
                        $this->l('New user'), // email subject
                        [
                            '{email}' => Configuration::get('PS_SHOP_EMAIL'), // sender email address
                            '{message}' => 'Hello world', // email content
                            '{vat_number}' => $customer_data['vat_number'],
                        ],
                        $recipient, // receiver email address
                        // Configuration::get('PS_SHOP_EMAIL'), // receiver email address
                        null, // receiver name
                        null, // from email address
                        null, // from name
                        null, // file attachment
                        null, // mode smtp
                        '/modules/' . $this->name . '/mails' // custom template path
                    );
                }
            }
        }
    }

    /**
     * new customer: send an email to dedicated account: add customer data to template
     * hook called by classes/email.php
     *
     * params: array( [
     *              'template' => $template,
     *              'template_vars' => &$templateVars,
     *          ] );
     *
     * params values:
     *     [template] => adm_new_account
     *  [template_vars] => Array
     *      (
     *          [{email}] => info@rino-graficaestampa.it
     *          [{message}] => Hello world
     *      )
     *
     *  [cookie] => Cookie Object
     *      (
     *          [_content:protected] => Array
     *              (
     *                  [id_lang] => 1
     *                  [id_currency] => 1
     *                  [date_add] => 2020-08-16 14:54:09
     *                  [id_guest] => 420
     *                  [id_connections] => 735
     *                  [id_customer] => 23
     *                  [customer_lastname] => testtre
     *                  [customer_firstname] => testtre
     *                  [passwd] => $2y$10$MBSWY6ZDpnOufr/evIiQ/uTJGBRYEju9AUKxUSQCgD7VyaxWPYJl.
     *                  [logged] => 1
     *                  [email] => testtre@mxbinformatica.it
     *                  [is_guest] =>
     *                  [id_cart] => 123
     *              )
     *              .......
     */
    public function hookSendMailAlterTemplateVars(&$hookParams)
    {
        // if mxbmemberprogram module is installed, let it manage this hook
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            return;
        }

        if ($hookParams['template'] == 'adm_new_customer_account') {
            $customer = new Customer((int) $hookParams['cookie']->id_customer);
            if (isset($customer->vat_number)) {
                $vat = $customer->vat_number;
            } elseif (isset($customer->siret)) {
                $vat = $customer->siret;
            } elseif (isset($customer->dni)) {
                $vat = $customer->dni;
            } elseif (isset($hookParams['cookie']->vat_number)) {
                $vat = $hookParams['cookie']->vat_number;
            } elseif (isset($hookParams['cookie']->siret)) {
                $vat = $hookParams['cookie']->siret;
            } elseif (isset($hookParams['cookie']->dni)) {
                $vat = $hookParams['cookie']->dni;
            } else {
                $vat = '';
            }
            $hookParams['template_vars']['{firstname}'] = $hookParams['cookie']->customer_firstname;
            $hookParams['template_vars']['{lastname}'] = $hookParams['cookie']->customer_lastname;
            $hookParams['template_vars']['{email}'] = $hookParams['cookie']->email;
            $hookParams['template_vars']['{company}'] = $customer->company;
            $hookParams['template_vars']['{vat_number}'] = $vat; // for next version, ioncluding vat in some way
        }
    }

    /**
     * modify or block email messages to users
     * - block "new voucher" message
     * hook called by classes/email.php
     *
     * params: array([
     *      'idLang' => &$idLang,
     *      'template' => &$template,
     *      'subject' => &$subject,
     *      'templateVars' => &$templateVars,
     *      'to' => &$to,
     *      'toName' => &$toName,
     *      'from' => &$from,
     *      'fromName' => &$fromName,
     *      'fileAttachment' => &$fileAttachment,
     *      'mode_smtp' => &$mode_smtp,
     *      'templatePath' => &$templatePath,
     *      'die' => &$die,
     *      'idShop' => &$idShop,
     *      'bcc' => &$bcc,
     *      'replyTo' => &$replyTo,
     *  ]);
     */
    public function hookActionEmailSendBefore($hookParams)
    {
        // if mxbmemberprogram module is installed, let it manage this hook
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            return;
        }

        if (Configuration::getGlobalValue('MXB_TOOLS_BLOCK_EMAIL_ORDER_NEW_VOUCHER') == 1
            && ($hookParams['template'] == 'voucher' || $hookParams['template'] == 'in_transit')
        ) {
            return false;
        }

        return true;
    }

    public function hookActionAuthentication($hookParams)
    {
        // if mxbmemberprogram module is installed, let it manage this hook
        if (MxbToolsUtils::isModuleEnabled('mxbmemberprogram')) {
            return;
        }
        // redirect to home page after the log-in
        if ((bool) Configuration::getGlobalValue('MXB_TOOLS_CUSTOMER_REDIRECT_AFTER_LOGIN')) {
            Tools::redirect('index.php');
        }
    }

    public function hookDisplayAfterBodyOpeningTag($hookParams = null)
    {
        $analytics = Configuration::getGlobalValue('MXB_TOOLS_GOOGLE_ANALYTICS_CODE');
        $tagManager = Configuration::getGlobalValue('MXB_TOOLS_GOOGLE_TAG_MANAGER_CODE');
        $tagsArray = [];
        if (!empty($analytics)) {
            $tagsArray['tag_analytics'] = $analytics;
        }
        if (!empty($tagManager)) {
            $tagsArray['tag_manager'] = $tagManager;
        }
        if (count($tagsArray)) {
            $this->context->smarty->assign('mxbtGoogleTags', $tagsArray);

            return $this->display(__FILE__, 'front/mxb_tools-body-top-tags.tpl');
        }
    }

    public function hookDisplayNotFound($hookParams = null)
    {
        // LogDebug::WriteLog('Gadgetcard', 'hookDisplayNotFound', 'START !');
        $url = null;
        if (Tools::getIsset('s') && !empty(Tools::getValue('s'))) {
            // manage search not found
            $url = Configuration::getGlobalValue('MXB_TOOLS_REDIRECT_SEARCH_NOT_FOUND_URL');
        } else {
            $url = Configuration::getGlobalValue('MXB_TOOLS_REDIRECT_404_URL');
        }
        if (!$url) {
            return;
        }
        $this->context->smarty->assign('mxbtNotFoundUrl', ['url' => $url]);

        return $this->display(__FILE__, 'front/mxb_tools-404.tpl');
    }

    /*
    [object] => Array
        (
            [cms_category] => Array
                (
                    [id_parent] => 1
                    [position] => 0
                    [level_depth] => 2
                    [date_add] => 2024-01-29 13:24:30
                    [date_upd] => 2024-01-29 13:24:30
                    [name] => News
                    [link_rewrite] => news
                    [description] => 
                    [meta_title] => 
                    [meta_description] => 
                    [meta_keywords] => 
                    [id] => 2
                )

            [sub_categories] => Array
                (
                )

        [cms_pages] => Array
                    (
                        [9] => Array
                            (
                                [id_cms] => 9
                                [id_cms_category] => 2
                                [position] => 0
                                [active] => 1
                                [indexation] => 0
                                [id_lang] => 1
                                [id_shop] => 1
                                [meta_title] => Aiuta il pianeta con un gadget personalizzabile green
                                [head_seo_title] => 
                                [meta_description] => 
                                [meta_keywords] => 
                                [content] => ... content
                                [link_rewrite] => aiuta-il-pianeta-con-un-gadget-personalizzabile-green
                                [link] => https://www.amogadget.com/content/9-aiuta-il-pianeta-con-un-gadget-personalizzabile-green
                                
                                // and this function will add:
                                [content_preview] =>
    */
    public function hookFilterCmsCategoryContent($hookParams)
    {
        if (!empty($hookParams['object']['cms_pages'])) {
            // try to find cms-preview-content
            foreach ($hookParams['object']['cms_pages'] as &$page) {
                $contentPreview = $page['content'];
                $pos = strpos(strtolower($contentPreview), '<div class="cms-preview-content');
                if ($pos !== false) {
                    $contentPreview = substr($page['content'], $pos);
                    $pos = strpos(strtolower($contentPreview), '</div>');
                    if ($pos !== false) {
                        $pos += strlen('</div>');
                        $contentPreview = substr($contentPreview, 0, $pos);
                        // replace any empty link with page link
                        $contentPreview = str_replace('<a>', '<a href="' . $page['link'] . '">', $contentPreview);

                        // find article-date
                        $pos = strpos(strtolower($contentPreview), 'cmsdate-');
                        $articleDate = null;
                        if ($pos !== false) {
                            $articleDate = substr($contentPreview, $pos + strlen('cmsdate-'));
                            $page['content_date'] = substr($articleDate, 0, strpos($articleDate, '-cmsdate'));
                        }
                        $page['content_preview'] = str_replace('cms-preview-content', 'cms-preview-content-show', $contentPreview);
                        MxbToolsDebug::writeLog($this->name, 'hookFilterCmsCategoryContent', 'content_preview: ' .  $page['content_preview']);
                        
                        $html = '&raquo; '
                            . ($articleDate ? $page['content_date'] . ': ' : '')
                            . $page['meta_title'] . ' '
                            . $page['content_preview']
                            . '<div class="clearfix"></div>';
                        $html = str_replace('<', '[(', $html);
                        $html = str_replace('>', ')]', $html);
                        $html = str_replace('&', '[ecom]', $html);
                        $page['meta_title'] = $html;
                    }
                }
            }
        }
        MxbToolsDebug::writeLog($this->name, 'hookFilterCmsCategoryContent', 'hookParams: ' .  print_r($hookParams, true));

        return $hookParams;
    }

    protected function checkForUpdates()
    {
        // check version
        $version = (int) Configuration::getGlobalValue('MXB_TOOLS_VERSION');
        if ($version === 0) {
            // module was fresh of installation: no updates are needed: update current version and exit
            Configuration::updateGlobalValue('MXB_TOOLS_VERSION', self::MXB_DEV_VERSION);
            return false;
        }
        if ((int) self::MXB_DEV_VERSION <= (int) $version) {
            /*
            MxbToolsDebug::writeLog(
                $this->name,
                'checkForUpdates',
                'no update needed: already aligned with current version  ' . $version
            );
            */
            return false;
        }
        $confirmations = [];
        $errors = [];

        for ($i = $version + 1; $i <= self::MXB_DEV_VERSION; ++$i) {
            $version = $i;
            switch ($version) {
                case 2:
                    // ready to make any upgrade !
                    break;
                case 3:
                    // ready to make any upgrade !
                    $res = $this->registerHook('displayAfterBodyOpeningTag');

                    break;
                case 4:
                    // ready to make any upgrade !
                    $res = $this->registerHook('displayNotFound');
                    $this->setModuleValues();

                    break;
                case 5:
                    $res = $this->registerHook('filterCmsCategoryContent');
                    if (!$res) {
                        $errors[] = '<br>*** ERROR *** add new hook filterCmsCategoryContent for MXB_CUSTOMBLOCK_VERSION='.$version;
                    } else {
                        $confirmations[] .= '<br>SUCCESS add new hook filterCmsCategoryContent for MXB_CUSTOMBLOCK_VERSION='.$version;
                    }

                    break;
                case 6:
                    /*
                    $res = $this->registerHook('footer');
                    if (!$res) {
                        $errors[] = '<br>*** ERROR *** add new hook footer for MXB_CUSTOMBLOCK_VERSION='.$version;
                    } else {
                        $confirmations[] .= '<br>SUCCESS add new hook footer for MXB_CUSTOMBLOCK_VERSION='.$version;
                    }
                    */

                    break;
            }
        }

        // update current version
        Configuration::updateGlobalValue('MXB_TOOLS_VERSION', $version);

        MxbToolsDebug::writeLog(
            $this->name,
            'checkForUpdates',
            'updated to version ' . $version
            . PHP_EOL . '> confirmations=' . print_r($confirmations, true)
            . PHP_EOL . '> errors=' . print_r($errors, true),
            true,
            'update.log'
        );
       

        return true;
    }
}
