/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

var CRLF = "\r\n";
var CR = "\n";

(function(window) {
    if (!!window.mxb_tools) {
        return window.mxb_tools;
    }
    var document = window.document;
    var mxb_tools = (function() {

        var theme = 'supervan'; // modern, light, dark, material, bootstrap, supervan

        function infoMessage(title, message, function_onConfirm) {
            // https://craftpip.github.io/jquery-confirm/
            // return alert(message);
            // alert(message);
            $.alert({
                title: title,
                content: message,
                containerFluid: true,
                boxWidth: '50%',
                useBootstrap: false,
                type: 'blue',
                typeAnimated: true,
                animation: 'zoom',
                closeAnimation: 'scale',
                    //icon: 'glyphicon glyphicon-heart',
                buttons: {
                    ok: {
                        btnClass: 'btn-blue',
                        keys: ['enter', 'esc'],
                        action: function_onConfirm
                    }
                }
            });
        }


        function confirmMessage(dialogType, title, message, function_onConfirm, function_onCancel) {
            //alert ("_confirmMessage ::: " + title + ", " + message);
            //return confirm(message);
            var _type;
            switch (dialogType) {
                case 'alert':
                    _type = 'red';
                    break;
                case 'normal':
                case 'blue':
                default:
                    _type = 'blue';
                    break;
            }

            $.confirm({
                title: title,
                content: message,
                containerFluid: true,
                boxWidth: '50%',
                useBootstrap: false,
                type: _type,
                typeAnimated: true,
                animation: 'zoom',
                closeAnimation: 'scale',
                columnClass: 'col-md-4 col-md-offset-8 col-xs-4 col-xs-offset-8',
                icon: 'fa fa-question-circle',
                theme: theme, // modern, light, dark, material, bootstrap, supervan
                    //icon: 'glyphicon glyphicon-heart',
                buttons: {
                    ok: {
                        btnClass: 'btn-blue',
                        keys: ['enter'],
                        action: function_onConfirm
                    },
                    cancel: {
                        //btnClass: 'btn-grey',
                        keys: ['esc'],
                        action: function_onCancel
                    }
                }
            });
        }

        function alertMessage(title, message, function_onConfirm) {
            // https://craftpip.github.io/jquery-confirm/
            // return alert(message);
            $.alert({
                title: title,
                content: message,
                containerFluid: true,
                boxWidth: '50%',
                useBootstrap: false,
                type: 'red',
                typeAnimated: true,
                animation: 'zoom',
                closeAnimation: 'scale',
                columnClass: 'col-md-4 col-md-offset-8 col-xs-4 col-xs-offset-8',
                icon: 'fa fa-warning',
                theme: theme, // modern, light, dark, material, bootstrap, supervan
                    //icon: 'glyphicon glyphicon-heart',
                buttons: {
                    ok: {
                        btnClass: 'btn-blue',
                        keys: ['enter', 'esc'],
                        action: function_onConfirm
                    }
                }
            });
            
        }

        /**
         * this function is called by buttons or elements that have the very next element as an hidden div containing the message
         * example:
         *  <button class="btn btn-primary" onClick="mxb_tools.infoMessage('{l s="Order a sample" mod="mxb_tools"}',mxb_tools.getMessageString(this)); return false;">
         *    {l s='Order a Sample' mod='mxb_tools'}
         *  </button>
         *  <div class="ghidden">{l s="msg-order-a-sample" mod="mxb_tools"}</div>
         *
         */
        function getMessageString(element) {
            var msg_el = element.nextElementSibling;
            if (msg_el) {
                //alert("getMessageString : " + msg_el.id + " - " + msg_el.innerHTML);
                return str2html(msg_el.innerHTML);
            }
            return "no message";
        }


        /*     UTILITY FUNCTIONS */
        function str2html (str, excludeTags) {
            if (isEmpty(str))
                return str;
            if (excludeTags === undefined)
                excludeTags = false;

            var ret = str.replaceAll("&", "&amp;");
            ret = ret.replaceAll("<br>", CRLF);
            ret = ret.replaceAll("<BR>", CRLF);
            if (!excludeTags) {
                ret = ret.replaceAll("<", "&lt;");
                ret = ret.replaceAll(">", "&gt;");
                ret = ret.replaceAll('"', "&quot;");
            }
            ret = ret.replaceAll(CRLF, "<br>");
            ret = ret.replaceAll(CR, "<br>");
            ret = ret.replaceAll("[BR]", "<br>");
            ret = ret.replaceAll("ù", "&ugrave;");
            ret = ret.replaceAll("È", "&egrave;");
            ret = ret.replaceAll("É", "&eacute;");
            ret = ret.replaceAll("è", "&Egrave;");
            ret = ret.replaceAll("é", "&Eacute;");
            ret = ret.replaceAll("ê", "&ecirc;");
            ret = ret.replaceAll("ë", "&euml;");
            ret = ret.replaceAll("ò", "&ograve;");
            ret = ret.replaceAll("ó", "&oacute;");
            ret = ret.replaceAll("Ò", "&Ograve;");
            ret = ret.replaceAll("Ó", "&Oacute;");
            ret = ret.replaceAll("à", "&agrave;");
            ret = ret.replaceAll("á", "&aacute;");
            ret = ret.replaceAll("À", "&Agrave;");
            ret = ret.replaceAll("Á", "&Aacute;");
            ret = ret.replaceAll("â", "&acirc;");
            ret = ret.replaceAll("â", "&acirc;");
            ret = ret.replaceAll("ä", "&auml;");
            ret = ret.replaceAll("æ", "&aelig;");
            ret = ret.replaceAll("Æ", "&AElig;");
            ret = ret.replaceAll("ì", "&igrave;");
            ret = ret.replaceAll("í", "&iacute;");
            ret = ret.replaceAll("Ì", "&Igrave;");
            ret = ret.replaceAll("Í", "&Iacute;");


            return ret;
        }

        function isEmpty(obj) {
            if (obj === undefined)
                return true;
            if (obj === null)
                return true;
            if (obj.length == 0)
                return true;
            return false;
        }

        function isObject(obj)     {     return (typeof obj !== "undefined"); }

        // somevar.constructor.name == "Array"
        function isArray(obj)     {    return ( Object.prototype.toString.call( obj ) === '[object Array]' ); }

        function isString(obj)     {    return ( typeof obj === 'string' );    }


        function isNumeric(n) {      return !isNaN(parseFloat(n)) && isFinite(n); }



        function parseIntOrZero(n) {
            try {
                if (isNumeric(n))
                    return parseInt(n);
            } catch (err) {
            }
            return 0;
        }


        function parseFloatOrZero(n) {
            try {
                if (isNumeric(n))
                    return parseFloat(n);
                var n1 = parseFloatWithComma(n.match(/[\d.,]+/));
                if (isNumeric(n1))
                    return n1;
            } catch (err) {
            }
            return 0;
        }



        function addChargePerc(price, charge) {
            price = parseFloatOrZero(price);
            charge = parseFloatOrZero(charge);
            if (price === 0 || charge === 0)
                return 0;
            return price + (price * charge / 100);
        }

        function applyDiscountPerc(price, discount) {
            price = parseFloatOrZero(price);
            discount = parseFloatOrZero(discount);
            if (price === 0 || discount === 0)
                return 0;
            return price - (price * discount / 100);
        }


        function addProfitPerc(price, profit) {
            price = parseFloatOrZero(price);
            profit = parseFloatOrZero(profit);
            if (price === 0 || profit === 0)
                return 0;
            return price/((100-profit)/100);
        }

        function calculateCharge(lowPrice, highPrice) {
            //((lordo-netto)/netto)*100
            if (lowPrice == 0)
                return 0;
            return ((highPrice-lowPrice)/lowPrice)*100;
        }

        function calculateDiscount(lowPrice, highPrice) {
            //100-((netto/lordo)*100)
            if (highPrice == 0)
                return 0;
            return 100-((lowPrice/highPrice)*100);
        }


        function charge2profit(charge_perc) {
            var wholesale = 10;
            var price = mxb_tools.addChargePerc(wholesale, charge_perc);
            var profit = price - wholesale;
            return 100-((wholesale*100)/price);        // 100 - ( (PA€ * 100) / PV€)
        }


        function profit2charge(profit_perc) {
            var wholesale = 10;
            var price = mxb_tools.addProfitPerc(wholesale, profit_perc);
            //var profit = price - wholesale;
            return ((price-wholesale) * 100)/wholesale; // ( (PV€ - PA€) * 100) / PA€

        }


        function roundDecimal(number, decimals) {
            if (decimals === undefined)
                decimals = 2;
            if (decimals === 0)
                return Math.round(number);
            var div = Math.pow(10, decimals);

            return Math.round(number * div) / div;
        }

        function cloneHtml(html, prefix) {
            if (prefix == undefined)
                prefix = 'clone';
            return html.replaceAll('id="', 'id="'+prefix);
        }

        function htmlDecode(input) {
          var e = document.createElement('textarea');
          e.innerHTML = input;
          // handle case of empty input
          var ret = e.childNodes.length === 0 ? "" : e.childNodes[0].nodeValue;
          e.remove();
          return ret;
        }

        function domReadyEvent_check(container_name, callback) {
            var el_name = container_name;
            if (el_name.charAt(0)=="." || el_name.charAt(0)=="#") {
                el_name = el_name.substring(1);
            }
            //alert("domReadyEvent_check - container_name=" + container_name);
            setTimeout(function() {
                // check if the hidden input field added to the "listener" element stills alive.
                // if so, the dom was not reloaded yet, so keep looping to test it within this timeout event.
                // if "dom ready element" is not present, the element was reloaded by ajax,
                // so we need to renew the javascript events and settings
                if ( $(container_name + ' input[name="mxb_' + el_name + '_ready"]').length ) {
                    domReadyEvent_check(container_name, callback);
                    return;
                }
                // alert("domReadyEvent_check STOP - container_name=" + container_name);
                if (callback !== undefined) {
                    setTimeout(function() {
                        callback();
                    }, 1000);
                }
            }, 100);
        }

        /**
         * container_name (jquery format with # or .): the "domReadyEvent_check" element will be created after the element: "container_name + ' ' + dom_element_name" (if dom_element_name is set)
         * dom_element_name (jquery format with # or .): the "domReadyEvent_check" element will be created after the element: "container_name + ' ' + dom_element_name" (if dom_element_name is set)
         * listener_element_name: can be string in jquery format (with # or .) or a jQuery element.
         *        if string, actin will be bind to "container_name + ' ' + listener_element_name" (if listener_element_name is set)
         *        if jQuery element, actin will be bind to the element itself
         * action (string): action on listener_element_name to start domReadyEvent_check
         */
        function domReadyEvent_add(container_name, dom_element_name, listener_element_name, callback, action, custon_action_name) {
            //alert("domReadyEvent_add - container_name=" + container_name);

            // add dom ready elements
            /*
            if (callback_param === undefined) {
                callback_param = 10;
            }
            */
            var el_name = container_name;
            if (el_name.charAt(0)=="." || el_name.charAt(0)=="#") {
                el_name = el_name.substring(1);
            }
            el_name = 'mxb_' + el_name + '_ready';
            if (action === undefined)
                action = 'change';
            if (custon_action_name === undefined)
                custon_action_name = 'mxbtdomcustom';

            //alert("domReadyEvent_add - el_name=" + el_name);
            if (!$('#' + el_name).length) {
                $('<input>').attr({
                    type: 'hidden',
                    id: el_name,
                    name: el_name
                }).appendTo(container_name + (dom_element_name != null ? ' ' + dom_element_name : ''));
            }


            if (!listener_element_name || isString(listener_element_name)) {
                $(container_name + (listener_element_name != null ? ' ' + listener_element_name : '')).unbind(action + "." + custon_action_name).bind(action + "." + custon_action_name, function(event) {
                    //alert("change");
                    mxb_tools.domReadyEvent_check(container_name, callback);
                });
            }
            else {
                //alert(container_name + ": bind element");
                listener_element_name.unbind(action + "." + custon_action_name).bind( action + "." + custon_action_name, function(event) {
                    //alert("change");
                    mxb_tools.domReadyEvent_check(container_name, callback);
                });
            }
            /*
            if (action_element !== undefined && action_element !== null) {
                $(action_element).unbind( "click.mxbtdomcustom").bind( "change.mxbtdomcustom", function(event) {
                    //alert("change");
                    domReadyEvent_check(container_name, callback);
                });
            }
            else {
                // add event listener for updating reloaded dom
                //alert($(container_name + ' ' + listener_element_name).length);
                $(container_name + (listener_element_name != null ? ' ' + listener_element_name : '')).unbind( "change.mxbtcustom").bind( "change.mxbtcustom", function(event) {
                    //alert("change");
                    domReadyEvent_check(container_name, callback);
                });
            }
            */
        }


        function addItemToForm(formId, inputName, inputType, value) {
            if (value === null || value === undefined)
                value = '';
                //return false;

            // remove tabs
            value = value.toString().replace(/\t/g, '');
            // remove line breaks
            value = value.toString().replace(/(\r\n|\n|\r)/gm,"");

            var el = document.createElement('input');
            el.type = inputType;
            el.name = inputName;
            el.value = value;
            document.getElementById(formId).appendChild(el);
            return true;
        }


        function strContains(a, b) {
            try {
                return (a.indexOf(b) !=-1);
            } catch (err) {
            }
            return false;
        }

        function zeroFill(baseText, text) {
            var tl = (""+text).length;
            var bl = (""+baseText).length;
            if (bl <= tl)
                return text;
            return baseText.substring(0, bl - tl) + text;
        }

        /* convert date (yyyy-MM-dd) to any format
         * or date-time yyyy-mm-dd hh:mm:ss
         * dateFormat('2021-12-10', 'MM-dd-yyyy');
         * It’s important to note that the second argument must be a valid date format
         * containing "dd", "MM", and "yy" or "yyyy".
         *
         */
        function dateFormat(inputDate, format) {
            const hasTime = (inputDate.length > 12);
            //parse the input date
            const date = new Date(inputDate);

            //extract the parts of the date
            const day = date.getDate();
            const month = date.getMonth() + 1;
            const year = date.getFullYear();

            const hours = date.getHours();
            const minutes = date.getMinutes();
            const seconds = date.getSeconds();

            // check hours
            if (format.indexOf("hh") > -1 && format.indexOf(" ") > -1) {
                var f = format.split(' ');
                //replace time
                f[1] = f[1].replace("hh", hours.toString().padStart(2,"0"));
                f[1] = f[1].replace("mm", minutes.toString().padStart(2,"0"));
                f[1] = f[1].replace("ss", seconds.toString().padStart(2,"0"));
                format = f[0] + ' ' + f[1];
            }

            //replace the month
            //format = format.replace("MM", month.toString().padStart(2,"0"));
            // compatibliw also with lowercase month "mm"
            format = format.toLowerCase().replace("mm", month.toString().padStart(2,"0"));

            //replace the year
            if (format.indexOf("yyyy") > -1) {
                format = format.replace("yyyy", year.toString());
            } else if (format.indexOf("yy") > -1) {
                format = format.replace("yy", year.toString().substr(2,2));
            }

            //replace the day
            format = format.replace("dd", day.toString().padStart(2,"0"));

            return format;
        }

        /* KEYBOARD MANAGEMENT */
        function getKeyPressed(evt) {
            return evt.key;
            /*
            evt = evt || window.event;
            var charCode = evt.keyCode || evt.which;
            var charStr = String.fromCharCode(charCode);
            return charStr;
            */
        }

        // tasti che si usano per cancellare, spostarsi etc
        // NOTA: si passa charCode !!
        function isTypingKey(charCode) {
            //alert("isTypingKey - charCode: " + charCode);
            if ((charCode > 7 && charCode < 32)) { // DEL (backspace), BS, TAB - excluding "space"
                //|| charCode == 45 // insert
                //|| charCode == 46 // canc
                //|| (charCode > 32 && charCode < 41) // page up, down, end, home, arrows
                return true;
            }
            return false;
        }

        function isEnterKey(evt, callback) {
            var charCode = (evt.which) ? evt.which : evt.keyCode;
            //alert("isEnterKey: "+charCode);
            if (charCode == 13) {
                if (callback) {
                    callback();
                }
                return true;
            }
            return false;
        }

        function isNumberKey(evt) {
            var charCode = (evt.which) ? evt.which : evt.keyCode;
            if (isTypingKey(charCode)) {
                return true;
            }
            if ((charCode > 47 && charCode < 58)) {
                // numbers
                return true;
            }
            return false;
        }

        // aggiunge punto e meno
        function isDecimalKey(evt) {
            var ok = isNumberKey(evt); // typing key already tested here
            
            // check other allowed chars
            if (!ok) {
                var charCode = (evt.which) ? evt.which : evt.keyCode;
                switch (charCode) {
                    case 45: // -
                    case 46: // .
                    //case 189: // -
                    //case 190: // .
                        return true;
                }
            }
            return ok;
        }

        function frontAjaxCall(link, params, on_success_callback)
        {
            if (typeof link === "undefined" || link === null) {
                if (!$("#mxb_tools_ajax_link").length) {
                    // alert('mxb_tools.frontAjaxCall - no link!');
                    return false;
                }
                link = $("#mxb_tools_ajax_link").val();
            }

            // alert('mxb_tools.frontAjaxCall - link: ' + link);
            // $.post(link, params, function(data) {
            $.getJSON(link, params, function(data) {
                // alert('mxb_tools.frontAjaxCall - back from call');
                if (typeof data.status !== "undefined") {
                    // alert('mxb_tools.frontAjaxCall - message: ' + data.message);
                    // console.log(data);
                    if (on_success_callback !== undefined) {
                        on_success_callback(data);
                    }
                }
            });
        }

        function copyContentToClipboard(el_id)
        {
            
            if (el_id == undefined) {
                // alert (el_id);
                return;
            }
            var el = document.getElementById(el_id);
            if (!el) {
                el = document.getElementsByClassName(el_id);
                if (el && el.length) {
                    el = el[0];
                }
            }
            if (!el) {
                // alert ('no el ' + el_id);
                return;
            }

            // element type
            var is_input = (el.tagName.toLowerCase() == "input");

            // Get the text field
            // var copyText = document.getElementById("myInput");

            // Select the text field
            if (is_input) {
                // el.select();
                // el.setSelectionRange(0, 99999); // For mobile devices
                var txt = el.value;
            } else {
                var txt = el.innerHTML;
            }
            
            // alert (txt);

            // Copy the text inside the text field
            navigator.clipboard.writeText(txt);
        }


        function smoothScroll(id_or_class) {
            document.querySelector(getIdOrClassQuery(id_or_class)).scrollIntoView({
                behavior: 'smooth'
            });
        }

        /*
        goToCollapsedDiv
            div_id: main div that contains the target element to scroll into, or a secondary div
            target_id: target element to scroll into (a form, a div, the title we want to scroll to)
            sub_div_id: a sub-div containing the target element, in case of nested collapsable div
        */
        function goToCollapsedDiv(div_id, target_id, sub_div_id) {
            var el = document.getElementById(target_id);
            if (!el) {
                return;
            }
            if (div_id !== null) {
                if ($('#' + div_id).length) {
                    // check if div is closed
                    if ($('#' + div_id).is(':hidden')) {
                        $('#' + div_id).css('height','auto');
                        $('#' + div_id).addClass('in');
                    }
                }
            }
            if (sub_div_id) {
                if ($('#' + sub_div_id).length) {
                    // check if div is closed
                    if ($('#' + sub_div_id).is(':hidden')) {
                        $('#' + sub_div_id).css('height','auto');
                        $('#' + sub_div_id).addClass('in');
                    }
                }
            }
            el.scrollIntoView();
        }

        function getIdOrClassQuery(id_or_class) {
            var type = '';
            if (!id_or_class.startsWith(".") && !id_or_class.startsWith("#")) {
                type = ($('#' . id_or_class).length ? '#' : '.');
                return type + id_or_class;
            }

            return id_or_class;
        }

        function blinkElement(id_or_class, seconds, callback) {
            var el_id = getIdOrClassQuery(id_or_class)
            $(el_id).addClass('blink');
            if (!seconds) {
                return;
            }
            var millis = seconds*1000;
            setTimeout(function() {
                $(el_id).removeClass('blink');
                if (callback !== undefined) {
                    callback();
                }
            }, millis);
        }

        function submitForm(form_id, use_button) {
            if (use_button === undefined) {
                use_button = true;
            }
            if (use_button) {
                if ($('#' + form_id + ' input[type="submit"]').length) {
                    $('#' + form_id + ' input[type="submit"]').click();
                }
                return;
            }
            var theForm = document.getElementById(form_id);
            if (theForm) {
                theForm.submit();
            }
        }

        function confirmExecuteCommand(message) {
            if (message === undefined) {
                message = 'Execute command ?';
            }
            if (confirm('Execute command ?')) {
                return true;
            }
            event.stopPropagation();
            event.preventDefault();
            return false;
        }

        const startObservingNewDOMElement = (domNode, elementIdToLookFor, callback) => {
            const observer = new MutationObserver(mutations => {
                mutations.forEach(function (mutation) {
                  // console.log(Array.from(mutation.addedNodes));

                    const elementAdded = Array.from(mutation.addedNodes).some(
                        element => {
                            if (element.id !== undefined && element.id == elementIdToLookFor) {
                                // alert(element.id + ' added!');
                                return true;
                            }
                            return false;
                        },
                    );

                    if (elementAdded) {
                        console.log('The element was added to the DOM');
                        if (callback !== undefined) {
                            callback();
                        }
                        //alert('element added!');
                    }
                });
            });
            observer.observe(domNode, {
                childList: true,
                attributes: true,
                characterData: true,
                subtree: true,
            });

          return observer;
        };


        /* EXPORTS */
        var exports = {};
        exports.infoMessage             = infoMessage;
        exports.alertMessage            = alertMessage;
        exports.confirmMessage          = confirmMessage;

        exports.copyContentToClipboard  = copyContentToClipboard;
        exports.getMessageString        = getMessageString;
        exports.parseFloatOrZero        = parseFloatOrZero;
        exports.parseIntOrZero          = parseIntOrZero;
        exports.isEmpty                 = isEmpty;
        exports.isObject                = isObject;
        exports.isArray                 = isArray;
        exports.isString                = isString;
        exports.isNumeric               = isNumeric;
        exports.roundDecimal            = roundDecimal;
        exports.strContains             = strContains;
        exports.str2html                = str2html;
        exports.addChargePerc           = addChargePerc;
        exports.addProfitPerc           = addProfitPerc;
        exports.applyDiscountPerc       = applyDiscountPerc;
        exports.calculateCharge         = calculateCharge;
        exports.calculateDiscount       = calculateDiscount;
        exports.charge2profit           = charge2profit;
        exports.profit2charge           = profit2charge;
        exports.cloneHtml               = cloneHtml;
        exports.domReadyEvent_add       = domReadyEvent_add;
        exports.domReadyEvent_check     = domReadyEvent_check;
        exports.addItemToForm           = addItemToForm;
        exports.htmlDecode              = htmlDecode;
        exports.zeroFill                = zeroFill;
        exports.dateFormat              = dateFormat;

        exports.isTypingKey             = isTypingKey;
        exports.isEnterKey              = isEnterKey;
        exports.isNumberKey             = isNumberKey;
        exports.isDecimalKey            = isDecimalKey;

        exports.frontAjaxCall           = frontAjaxCall;

        exports.submitForm              = submitForm;
        exports.confirmExecuteCommand   = confirmExecuteCommand;
        exports.smoothScroll            = smoothScroll;
        exports.goToCollapsedDiv        = goToCollapsedDiv;
        exports.getIdOrClassQuery       = getIdOrClassQuery;
        exports.blinkElement            = blinkElement;
        exports.startObservingNewDOMElement = startObservingNewDOMElement;

        return exports;
    })();

      window.mxb_tools = mxb_tools;
      return mxb_tools;
})(this);









String.prototype.replaceAll = function (find, replace) {
    var str = this;
    return str.replace(new RegExp(find.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&'), 'g'), replace);
};